/*
 * Cashew platform setup
 *
 * Copyright (C) 2014 Google, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/of_address.h>
#include <linux/of_fdt.h>
#include <linux/of_platform.h>

#include <asm/cacheflush.h>
#include <asm/dma-coherence.h>
#include <asm/fw/fw.h>
#include <asm/gcmpregs.h>
#include <asm/mips-boards/generic.h>
#include <asm/mips-cm.h>
#include <asm/mips-cpc.h>
#include <asm/prom.h>
#include <asm/smp-ops.h>
#include <asm/traps.h>

#include <asm/mips-boards/cashew.h>
#include <asm/mach-cashew/irq.h>
#include <asm/mach-generic/irq.h>

extern void cashew_be_init(void);
extern int cashew_be_handler(struct pt_regs *regs, int is_fixup);

const char *get_system_type(void)
{
	return "IMG Cashew SoC";
}

static int __init plat_enable_iocoherency(void)
{
	int supported = 0;

	if (mips_cm_numiocu() != 0) {
		/* Nothing special needs to be done to enable coherency */
		pr_info("CMP IOCU detected\n");

		// TODO - Setting hw_coherentio causes dma_alloc_coherent() to return
		// cached addresses for all peripherals, but only the GPU is connected
		// to the IOCU. For now, IOCU support is disabled. To re-enable it,
		// get_dma_ops() will need to return different dma ops for different
		// devices. arch_setup_dma_ops will need to be implemented for MIPS,
		// such that it sets the device archdata dma_ops. This should get
		// called automatically with bool coherent set as appropriate based on
		// each device tree node's dma-coherent property.
		pr_info("CMP IOCU support disabled\n");
		//supported = 1;
	}
	hw_coherentio = supported;
	return supported;
}

static void __init plat_setup_iocoherency(void)
{
#ifdef CONFIG_DMA_NONCOHERENT
	/*
	 * Kernel has been configured with software coherency
	 * but we might choose to turn it off and use hardware
	 * coherency instead.
	 */
	if (plat_enable_iocoherency()) {
		if (coherentio == 0)
			pr_info("Hardware DMA cache coherency disabled\n");
		else
			pr_info("Hardware DMA cache coherency enabled\n");
	} else {
		if (coherentio == 1)
			pr_info("Hardware DMA cache coherency unsupported, but enabled from command line!\n");
		else
			pr_info("Software DMA cache coherency enabled\n");
	}
#else
	if (!plat_enable_iocoherency())
		panic("Hardware DMA cache coherency not supported!");
#endif
}


#ifdef CONFIG_EVA
extern unsigned int mips_cca;

void __init plat_eva_setup(void)
{
	unsigned int val;

#ifdef CONFIG_EVA_OLD_MALTA_MAP

#ifdef CONFIG_EVA_3GB
	val = ((MIPS_SEGCFG_UK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (2 << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
	write_c0_segctl0(val);

	val = ((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
#else /* !CONFIG_EVA_3G */
	val = ((MIPS_SEGCFG_MK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
	write_c0_segctl0(val);

	val = ((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (2 << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
#endif /* CONFIG_EVA_3G */
#ifdef CONFIG_SMP
	val |= (((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
#else
	val |= (((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(4 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
#endif
	write_c0_segctl1(val);

	val = ((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(6 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(4 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);

#else /* !CONFIG_EVA_OLD_MALTA_MAP */

#ifdef CONFIG_EVA_3GB
	val = ((MIPS_SEGCFG_UK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (2 << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
	write_c0_segctl0(val);

	val = ((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(6 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(5 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
	write_c0_segctl1(val);

	val = ((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(3 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(1 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
#else /* !CONFIG_EVA_3G */
	val = ((MIPS_SEGCFG_MK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
	write_c0_segctl0(val);

	val = ((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (2 << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
	write_c0_segctl1(val);

	val = ((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(2 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT));
	val |= (((MIPS_SEGCFG_MUSUK << MIPS_SEGCFG_AM_SHIFT) |
		(0 << MIPS_SEGCFG_PA_SHIFT) | (mips_cca << MIPS_SEGCFG_C_SHIFT) |
		(1 << MIPS_SEGCFG_EU_SHIFT)) << 16);
#endif /* CONFIG_EVA_3G */

#endif /* CONFIG_EVA_OLD_MALTA_MAP */

	write_c0_segctl2(val);
	back_to_back_c0_hazard();

	val = read_c0_config5();
	write_c0_config5(val|MIPS_CONF5_K|MIPS_CONF5_CV|MIPS_CONF5_EVA);
	back_to_back_c0_hazard();

	printk("Enhanced Virtual Addressing (EVA) active\n");
}

extern int gcmp_present;
void BEV_overlay_segment(void);
#endif

#ifdef CONFIG_SYS_SUPPORTS_UDDR
unsigned long cashew_highmem_start;
unsigned long cashew_page_offset;
EXPORT_SYMBOL(cashew_page_offset);
unsigned long cashew_uncac_base;
#endif

void __init plat_mem_setup(void)
{
	void *fdtp;

	if (fw_arg0 == -2 && fw_arg1) {
		/* External DTB is setup. */
		fdtp = (void *)fw_arg1;
		fdtp = phys_to_virt(__pa(fdtp));
		__dt_setup_arch(fdtp);
		pr_info("Boot from external DTB at 0x%lx\n",
		        (unsigned long)fdtp);

		if (&__dtb_start != &__dtb_end) {
			pr_warn("External DTB overriding internal one\n");
		}
	} else {
#ifdef CONFIG_BUILTIN_DTB
		/* Use a built-in DTB */
		fdtp = &__dtb_start;
		fdtp = phys_to_virt(__pa(fdtp));
		__dt_setup_arch(fdtp);
		pr_info("Boot from internal DTB at 0x%lx\n",
		        (unsigned long)fdtp);
#endif
	}

	if (!initial_boot_params)
		panic("Device-tree not present");

	/* Copy over device tree cmdline */
	strlcpy(arcs_cmdline, boot_command_line, COMMAND_LINE_SIZE);
	strlcat(arcs_cmdline, " ", COMMAND_LINE_SIZE);

	/* Append legacy bootloader cmdline */
	fw_init_cmdline();

#ifdef CONFIG_SYS_SUPPORTS_UDDR
	/* Establish the memory map if using UDDR capability. */
	/* Using UDDR if memory doesn't start below 0x80000000. */
	if (boot_mem_map.map[0].addr >> 31) {
		cashew_highmem_start = 0x90000000UL;
		cashew_page_offset = 0;
		cashew_uncac_base = 0x20000000UL;
	} else {
		cashew_highmem_start = 0x10000000UL;
		cashew_page_offset = 0x80000000UL;
		cashew_uncac_base = 0xa0000000UL;
	}
#endif

#ifdef CONFIG_EVA
	if (gcmp_present)
		BEV_overlay_segment();

	if ((cpu_has_segments) && (cpu_has_eva))
		plat_eva_setup();
	else {
	    printk("cpu_has_segments=%ld cpu_has_eva=%ld\n",cpu_has_segments,cpu_has_eva);
	    printk("Kernel is built for EVA support but EVA or segment control registers are not found\n");
	    panic("EVA absent");
	}
#endif

	plat_setup_iocoherency();

	board_be_init = cashew_be_init;
	board_be_handler = cashew_be_handler;
}

#ifdef CONFIG_SERIAL_8250_CONSOLE
static void __init console_config(void)
{
	char console_string[40];

	sprintf(console_string," console=ttyS0,115200n81");
	strcat(fw_getcmdline(), console_string);
}
#endif


static void __init mips_nmi_setup(void)
{
	void *base;
	extern char except_vec_nmi;

	base = cpu_has_veic ?
		(void *)(CAC_BASE + 0xa80) :
		(void *)(CAC_BASE + 0x380);
	memcpy(base, &except_vec_nmi, 0x80);
	flush_icache_range((unsigned long)base,
			   (unsigned long)base + 0x80);
}

static void __init mips_ejtag_setup(void)
{
	void *base;
	extern char except_vec_ejtag_debug;

	base = cpu_has_veic ?
		(void *)(CAC_BASE + 0xa80) :
		(void *)(CAC_BASE + 0x380);
	memcpy(base, &except_vec_ejtag_debug, 0x80);
	flush_icache_range((unsigned long)base,
			   (unsigned long)base + 0x80);
}


void __init prom_mem_check(int niocu);
extern struct plat_smp_ops msmtc_smp_ops;

phys_addr_t mips_cpc_default_phys_base(void)
{
	return CPC_BASE_ADDR;
}

void __init prom_init(void)
{
	set_io_port_base(0);

	board_nmi_handler_setup = mips_nmi_setup;
	board_ejtag_handler_setup = mips_ejtag_setup;

#ifdef CONFIG_SERIAL_8250_CONSOLE
	console_config();
#endif
	/* Early detection of CMP support */
	mips_cm_probe();
	mips_cpc_probe();

	if (!register_cps_smp_ops())
		return;
	if (!register_cmp_smp_ops())
		return;
	if (!register_vsmp_smp_ops())
		return;

#ifdef CONFIG_MIPS_MT_SMTC
	register_smp_ops(&msmtc_smp_ops);
#endif
}

void __init prom_free_prom_memory(void)
{
}

void __init device_tree_init(void)
{
	if (!initial_boot_params)
		return;

	unflatten_and_copy_device_tree();
}

static int __init plat_of_setup(void)
{
	if (!of_have_populated_dt())
		panic("Device tree not present");

	if (of_platform_populate(NULL, of_default_bus_match_table, NULL, NULL))
		panic("Failed to populate DT");

	return 0;
}
arch_initcall(plat_of_setup);
