/*
 * Copyright 2016 Imagination Technologies, Ltd
 *
 * This program is free software; you can redistribute	it and/or modify it
 * under  the terms of	the GNU General	 Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <asm/asm.h>
#include <asm/cacheops.h>
#include <asm/pm.h>
#include <asm/regdef.h>

/* Base addresses */
#define PDC_BASE				0xb0080000
#define CR_ALWAYS_ON_BASE			0xb0090000
#define CR_TOP_BASE				0xb00a0000
#define DDRC0_BASE				0xb0194000
#define DDRP0_BASE				0xb0195000
#define DDRC1_BASE				0xb0196000
#define DDRP1_BASE				0xb0197000

/* PDC registers */
#define PDC_SOC_SW_PROT0			0x20
#define PDC_SOC_SW_PROT1			0x24
#define PDC_SOC_SW_PROT2			0x28
#define PDC_SOC_SW_PROT3			0x2C
#define PDC_SOC_SW_PROT4			0x30
#define PDC_SOC_SW_PROT5			0x34
#define PDC_SOC_SW_PROT6			0x38
#define PDC_SOC_SW_PROT7			0x3C
#define PDC_SOC_SW_PROT8			0x40
#define PDC_SOC_SW_PROT9			0x44
#define PDC_SOC_POWER				0x300
#define PDC_SOC_WAKEUP_RTC_CONFIG		0x320
#define PDC_SOC_WAKEUP_IR_CONFIG		0x324
#define PDC_SOC_WAKEUP_WD_CONFIG		0x328
#define PDC_SOC_SYS_WAKE_CONFIG0		0x334
#define PDC_SOC_SYS_WAKE_CONFIG7		0x36c

/* PDC register field values */
#define PDC_SOC_POWER_SWITCHED_TOP		0x1
#define PDC_SOC_POWER_CPU0			0x2

/* Always on registers */
#define CR_ALWAYS_ON_DDR_PAD_RETENTION		0x98

/* Top registers */
#define CR_TOP_CLOCK_SYS_PRIMARY		0x000
#define CR_TOP_CLOCK_NOC_HS			0x004
#define CR_PLL0_CONTROL_A			0x144
#define CR_PLL3_CONTROL_A			0x168

/* Top register fields */
#define CR_TOP_CS_PLL0BYPASSCLK_0_SW		(1 << 8)
#define CR_TOP_CS_PLL3BYPASSCLK_0_SW		(1 << 14)
#define CR_PLL_CONTROL_A_PD			(1 << 25)

/* DDRC registers */
#define	DDRC_STAT				0x4
#define	DDRC_PWRCTL				0x30
#define	DDRC_PSTAT				0x3fc
#define	DDRC_PCTRL_0				0x490
#define	DDRC_PCTRL_1				0x540

/* DDRP registers */
#define DDRP_PLLCR				0x080
#define DDRP_ZQ0DR				0x688
#define DDRP_ZQ1DR				0x698
#define DDRP_ZQ2DR				0x6A8
#define DDRP_ZQ3DR				0x6B8

/* DDRC register field masks */
#define	DDRC_PWRCTL_SELFREF_SW_MASK		0x20
#define	DDRC_STAT_SELFREF_TYPE_MASK		0x30

/* DDRP register fields */
#define	DDRP_PLLCR_GSHIFT			(1 << 11)
#define	DDRP_PLLCR_RGSHIFT			(1 << 18)
#define	DDRP_PLLCR_RGVINT			(1 << 21)

/* Protected registers */
#define BOOTMODE_REG				(PDC_BASE + PDC_SOC_SW_PROT0)
#define RESUME_FUNC_REG				(PDC_BASE + PDC_SOC_SW_PROT1)

/* Protected register field values */
#define BOOTMODE_NORMAL				0x0
#define BOOTMODE_DEEP_RESUME			0x1
#define BOOTMODE_LIGHT_RESUME			0x2

/* SFC0 Registers */
#define SFC0_CLOCK_CONTROL			0xb0170010
#define SFC0_CLOCK_CONTROL_ENABLE		0x1

/* CR_PERIP Registers */
#define CR_PERIP_CLKEN				0xb0100008
#define CR_PERIP_CLKEN_TIMER_EN			0x2

	.text
	.set noreorder

LEAF(cashew_sleep)

	/* Retain irqwake_set argument in t9 */
	add	t9, a0, zero

	/* Save CPU state */
	SUSPEND_SAVE

	/* Set up PDC protected registers for resume */
	PTR_LI	t0, BOOTMODE_REG
	add	t1, t9, 1
	PTR_S	t1, (t0)
	PTR_LI	t0, RESUME_FUNC_REG
	PTR_LA  t1, cashew_sleep_wakeup
	PTR_S	t1, (t0)

	/* Retain pre-retention DDR PHY ZQnDR values */
	PTR_LI	t0, (DDRP0_BASE + DDRP_ZQ0DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT2)
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRP0_BASE + DDRP_ZQ1DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT3)
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRP0_BASE + DDRP_ZQ2DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT4)
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRP0_BASE + DDRP_ZQ3DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT5)
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRP1_BASE + DDRP_ZQ0DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT6)
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRP1_BASE + DDRP_ZQ1DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT7)
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRP1_BASE + DDRP_ZQ2DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT8)
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRP1_BASE + DDRP_ZQ3DR)
	PTR_L	t1, (t0)
	PTR_LI	t0, (PDC_BASE + PDC_SOC_SW_PROT9)
	PTR_S	t1, (t0)

	/* Enable SFC0 and timer clocks, as bootram requires these to be on */
	PTR_LI	t0, SFC0_CLOCK_CONTROL
	PTR_LI	t1, SFC0_CLOCK_CONTROL_ENABLE
	PTR_S	t1, (t0)
	PTR_LI	t0, CR_PERIP_CLKEN
	PTR_L	t1, (t0)
	or	t1, t1, CR_PERIP_CLKEN_TIMER_EN
	PTR_S	t1, (t0)

	/* Load the start and end addresses of the remaining instructions */
	/* Fetch an extra 64 bytes to allow for pre-fetching */
	PTR_LA	t0, ddr_to_self_refresh
	PTR_LA	t1, (post_suspend + 64)

	/* Align start address to the start of the cache line */
	and	t0, t0, ~31

	/*
	 * Prefetch the remaining instructions into the cache, as RAM will soon
	 * be inaccessible
	 */
fill_icache:
	cache	Fill, 0(t0)
	addiu	t0, t0, 32
	bltu	t0, t1, fill_icache
	nop
	sync

	/* Put DDR into self refresh */
ddr_to_self_refresh:
	/* Write 0 to PCTRL_0 to block AXI port 0 */
	PTR_LI	t0, (DDRC0_BASE + DDRC_PCTRL_0)
	PTR_S	zero, (t0)
	PTR_LI	t0, (DDRC1_BASE + DDRC_PCTRL_0)
	PTR_S	zero, (t0)
	/* Write 0 to PCTRL_1 to block AXI port 1 */
	PTR_LI	t0, (DDRC0_BASE + DDRC_PCTRL_1)
	PTR_S	zero, (t0)
	PTR_LI	t0, (DDRC1_BASE + DDRC_PCTRL_1)
	PTR_S	zero, (t0)
	/* Poll for PSTAT == 0 to indicate all AXI ports are idle */
	PTR_LI	t0, (DDRC0_BASE + DDRC_PSTAT)
ddr0_pstat_poll:
	PTR_L	t1, (t0)
	bnez	t1, ddr0_pstat_poll
	nop
	PTR_LI	t0, (DDRC1_BASE + DDRC_PSTAT)
ddr1_pstat_poll:
	PTR_L	t1, (t0)
	bnez	t1, ddr1_pstat_poll
	nop
	/* Set PWRCTL_SELFREF_SW to put the DDR into self-refresh */
	PTR_LI	t0, (DDRC0_BASE + DDRC_PWRCTL)
	PTR_L	t1, (t0)
	or	t1, t1, DDRC_PWRCTL_SELFREF_SW_MASK
	PTR_S	t1, (t0)
	PTR_LI	t0, (DDRC1_BASE + DDRC_PWRCTL)
	PTR_L	t1, (t0)
	or	t1, t1, DDRC_PWRCTL_SELFREF_SW_MASK
	PTR_S	t1, (t0)
	/* Poll STAT_SELFREF_TYPE to indicate self-refresh state is entered */
	PTR_LI	t0, (DDRC0_BASE + DDRC_STAT)
ddr0_self_refresh_poll:
	PTR_L	t1, (t0)
	and	t1, t1, DDRC_STAT_SELFREF_TYPE_MASK
	bne	t1, 0x20, ddr0_self_refresh_poll
	nop
	PTR_LI	t0, (DDRC1_BASE + DDRC_STAT)
ddr1_self_refresh_poll:
	PTR_L	t1, (t0)
	and	t1, t1, DDRC_STAT_SELFREF_TYPE_MASK
	bne	t1, 0x20, ddr1_self_refresh_poll
	nop

	/*
	 * If any non-PDC peripherals have been configured as wake sources,
	 * switched-top stays on ('light sleep' mode)
	 */
	bnez	t9, light_sleep
	nop

deep_sleep:
	/* Place DDR pads into retention mode */
	PTR_LI	t0, (CR_ALWAYS_ON_BASE + CR_ALWAYS_ON_DDR_PAD_RETENTION)
	PTR_LI  t1, 0x3
	PTR_S	t1, (t0)

	b	power_down
	nop

light_sleep:
	/* Put DDR PHY into refclk_stop mode */
	PTR_LI	t0, (DDRP0_BASE + DDRP_PLLCR)
	PTR_L	t1, (t0)
	PTR_LI	t2, (DDRP1_BASE + DDRP_PLLCR)
	PTR_L	t3, (t2)

	/* Set RGVINT */
	or	t1, t1, DDRP_PLLCR_RGVINT
	PTR_S	t1, (t0)
	or	t3, t3, DDRP_PLLCR_RGVINT
	PTR_S	t3, (t2)

	/* Switch NoC HS to XTAL1 */
	PTR_LI	t4, (CR_TOP_BASE + CR_TOP_CLOCK_NOC_HS)
	PTR_LI	t5, 0x18
	PTR_S	t5, (t4)

	/* Set GSHIFT and RGSHIFT */
	or	t4, t1, (DDRP_PLLCR_GSHIFT | DDRP_PLLCR_RGSHIFT)
	PTR_S	t4, (t0)
	or	t4, t3, (DDRP_PLLCR_GSHIFT | DDRP_PLLCR_RGSHIFT)
	PTR_S	t4, (t2)

	/* Clear GSHIFT and RGSHIFT */
	PTR_S	t1, (t0)
	PTR_S	t3, (t2)

	/* Bypass and power off unused PLL0 (CPU0) and PLL3 (NoC HS) */
	PTR_LI	t4, (CR_TOP_BASE + CR_TOP_CLOCK_SYS_PRIMARY)
	PTR_L	t5, (t4)
	and	t5, t5, ~CR_TOP_CS_PLL0BYPASSCLK_0_SW
	and	t5, t5, ~CR_TOP_CS_PLL3BYPASSCLK_0_SW
	PTR_S	t5, (t4)

	PTR_LI	t4, (CR_TOP_BASE + CR_PLL0_CONTROL_A)
	PTR_L	t5, (t4)
	or	t5, t5, CR_PLL_CONTROL_A_PD
	PTR_S	t5, (t4)

	PTR_LI	t4, (CR_TOP_BASE + CR_PLL3_CONTROL_A)
	PTR_L	t5, (t4)
	or	t5, t5, CR_PLL_CONTROL_A_PD
	PTR_S	t5, (t4)

power_down:
	/* Configure PDC to restore the current value of SOC_POWER on wake */
	PTR_LI	t0, (PDC_BASE + PDC_SOC_POWER)
	PTR_L	t1, (t0)
	PTR_LI	t2, (PDC_BASE + PDC_SOC_WAKEUP_RTC_CONFIG)
wakeup_config:
	PTR_S	t1, (t2)
	bne	t2, (PDC_BASE + PDC_SOC_WAKEUP_WD_CONFIG), wakeup_config
	PTR_ADD	t2, t2, 0x4
	PTR_LI	t2, (PDC_BASE + PDC_SOC_SYS_WAKE_CONFIG0)
sys_wake_config:
	PTR_S	t1, (t2)
	bne	t2, (PDC_BASE + PDC_SOC_SYS_WAKE_CONFIG7), sys_wake_config
	PTR_ADD	t2, t2, 0x8

	/* In light sleep mode, switched-top stays on */
	bnez	t9, power_down_cpu0
	nop
	and	t1, t1, ~PDC_SOC_POWER_SWITCHED_TOP

power_down_cpu0:
	and	t1, t1, ~PDC_SOC_POWER_CPU0
	PTR_S	t1, (t0)

	/* Loop until the CPU powers down */
wait_forever:
	b	wait_forever
	nop

post_suspend:

END(cashew_sleep)

	/*
	 * This is the entry point after resume.
	 * Clear PDC protected registers, reload the CPU state and return.
	 */
LEAF(cashew_sleep_wakeup)

	PTR_LI	t0, BOOTMODE_REG
	PTR_LI  t1, BOOTMODE_NORMAL
	PTR_S	t1, (t0)
	PTR_LI	t0, RESUME_FUNC_REG
	PTR_S	zero, (t0)

	RESUME_RESTORE_RETURN

END(cashew_sleep_wakeup)
