#ifndef __DRM_PDP_DRV_H__
#define __DRM_PDP_DRV_H__

#include <linux/clk.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/wait.h>

#include <drm/drmP.h>
#include <drm/drm_crtc.h>
#include <drm/drm_crtc_helper.h>
#include <drm/drm_fb_helper.h>
#include <drm/drm_mm.h>

#include "pdp_gem.h"
#include "pdp_plane.h"
#include "pdp_tiler.h"

#define DRM_PDP_NUM_PLANES 8
#define PDP_MAX_TAPS 8
#define PDP_MAX_INTERPOLATION_POINTS 16

struct pdp_private {
	struct drm_device *dev;
	struct drm_fbdev_cma *fbdev;

	struct drm_atomic_state *suspend_state;

	struct clk *pdp_clk;
	struct clk *pixel_clk;

	int plane_nr;
	/* irq_lock to lock irq */
	spinlock_t irq_lock;
	int irq;

	struct pdp_plane planes[DRM_PDP_NUM_PLANES];
	struct pdp_crtc *crtc;
	struct drm_connector *connector;
	struct drm_encoder *encoder;

	void __iomem *reg;

	struct pdp_tiler tiler;
	bool carveout;

	struct {
		struct drm_property *alpha;
		struct drm_property *zpos;
		struct drm_property *porter_duff_mode;
		struct drm_property *porter_duff_type;
		struct drm_property *colourkey_r_property;
		struct drm_property *colourkey_g_property;
		struct drm_property *colourkey_b_property;
		struct drm_property *colourkey_r_mask_property;
		struct drm_property *colourkey_g_mask_property;
		struct drm_property *colourkey_b_mask_property;
		struct drm_property *colourkey_enable_property;
		struct drm_property *colourkey_mode_property;
		struct drm_property *gamma_enable_property;
	} props;

	struct {
		wait_queue_head_t wait;
		u32 pending;
	} commit;

	bool auto_kick;
};

u32 pdp_read(struct drm_device *dev, unsigned int reg);
void pdp_write(struct drm_device *dev, unsigned int reg_offs,
	       unsigned int data);
u32 pdp_read_mmu(struct drm_device *dev, unsigned int reg);
void pdp_write_mmu(struct drm_device *dev, unsigned int reg_offs,
		   unsigned int data);

#endif /* !defined(__DRM_PDP_DRV_H__) */
