#ifndef _PDP_GEM_H_
#define _PDP_GEM_H_

#include <linux/dma-buf.h>
#include <linux/reservation.h>

#include <drm/drmP.h>
#include <drm/drm_gem.h>

#include "pdp_tiler.h"

struct pdp_gem_object {
	struct drm_gem_object		obj;

	dma_addr_t			paddr;
	struct page			**pages;
	struct sg_table			*sgt;

	struct pdp_tiler_domain		*domain; /* iommu domain associated */
	struct drm_mm_node		*mm; /* node for iova */
	size_t				size; /* Size of iommu mapped area */
	bool				tiled;
	uint32_t			tile_stride; /* tiling stride */

	struct reservation_object	resv;
};

extern const struct vm_operations_struct pdp_gem_vm_ops;

static inline struct pdp_gem_object *
to_pdp_gem_object(struct drm_gem_object *gem)
{
	return container_of(gem, struct pdp_gem_object, obj);
}

void pdp_gem_free_object(struct drm_gem_object *obj);
int pdp_gem_get_pages(struct drm_device *dev, struct pdp_gem_object *pobj);

/* dumb buffer*/
int pdp_gem_dumb_create(struct drm_file *file, struct drm_device *dev,
			struct drm_mode_create_dumb *args);
int pdp_gem_dumb_map_offset(struct drm_file *file, struct drm_device *dev,
			    uint32_t handle, uint64_t *offset);
int pdp_gem_dumb_destroy(struct drm_file *file, struct drm_device *dev,
			 uint32_t handle);

/* Prime import/export */
struct dma_buf *pdp_gem_prime_export(struct drm_device *dev,
				     struct drm_gem_object *obj,
				     int flags);
struct drm_gem_object *pdp_gem_prime_import(struct drm_device *dev,
					    struct dma_buf *buf);

/* ioctls */
int pdp_gem_create_ioctl(struct drm_device *dev, void *data,
			 struct drm_file *file_priv);
int pdp_gem_mmap_ioctl(struct drm_device *dev, void *data,
		       struct drm_file *file_priv);

#endif /* _PDP_GEM_H_ */
