#ifndef _PDP_PLANE_H_
#define _PDP_PLANE_H_

#define	PDPFB_NO_REG	1

#include <drm/drm_crtc.h>

#include "pdp_drv.h"

struct pdp_plane_regs {
	u16 surf;
	u16 blend2_r;
	u16 blend2_gb;
	u16 ctrl;
	u16 uctrl;
	u16 stride;
	u16 size;
	u16 posn;
	u16 galpha;
	u16 ckey_r;
	u16 ckey_gb;
	u16 interleave_ctrl;
	u16 base_addr;
	u16 u_base_addr;
	u16 v_base_addr;
	u16 memctrl;
	u16 memthresh;
	u16 skip_ctrl;
	u16 postskip_ctrl;
	u16 decimate_ctrl;
	u16 scale_ctrl;
	u16 vsinit;
	u16 first_vscoeff;
	u16 last_vscoeff;
	u16 hsinit;
	u16 first_hscoeff;
	u16 last_hscoeff;
	u16 scale_size;
	u16 lumakey_alpha_trans_opaque;
	u16 lumakey_luma_max_min;
	u16 lumakey_c_rg;
	u16 lumakey_c_b;
};

struct pdp_plane_caps {
	u32 has_scaler;
	u32 vs_taps;
	u32 hs_taps;
	u32 vs_interpolation_points;
	u32 hs_interpolation_points;
	u32 max_scaler_input_width;
	u32 has_doubler; /* Has pixel / line doubling */
	u32 has_decimator; /* Has pixel halving / line decimation */
	u32 has_cosited_chroma_control;
	u32 has_csc;
};

enum plane_formats {
	PDP_FORMAT_GRPH,
	PDP_FORMAT_VIDEO,
};

enum colourkey_mode {
	PDP_COLOURKEY_MODE_PREVIOUS = 0,
	PDP_COLOURKEY_MODE_SELF = 1,
};

enum porter_duff_mode {
	PDP_BLENDMODE_F_OVER_B = 0,
	PDP_BLENDMODE_F_IN_B = 1,
	PDP_BLENDMODE_F_OUT_B = 2,
	PDP_BLENDMODE_F_ATOP_B = 3,
	PDP_BLENDMODE_F_XOR_B = 4,
	PDP_BLENDMODE_F_PLUS_B = 5,
	PDP_BLENDMODE_B_OVER_F = 6,
	PDP_BLENDMODE_B_IN_F = 7,
	PDP_BLENDMODE_B_OUT_F = 8,
	PDP_BLENDMODE_B_ATOP_F = 9,
	PDP_BLENDMODE_B_XOR_F = 10,
	PDP_BLENDMODE_CLEAR = 11,
	PDP_BLENDMODE_F = 12,
};

enum porter_duff_type {
	PDP_BLENDTYPE_CLIP_TO_SELF = 0,
	PDP_BLENDTYPE_NORMAL = 1,
};

struct pdp_plane_conf {
	enum drm_plane_type type;
	struct pdp_plane_regs regs;
	struct pdp_plane_caps caps;
	enum plane_formats format;
};

struct pdp_plane {
	struct drm_plane plane;
	struct pdp_private *dev;
	struct pdp_plane_conf *conf;
	int pos;
	bool stale_coeffs;
};

struct pdp_plane_state {
	struct drm_plane_state state;

	unsigned int alpha;
	unsigned int zpos;
	enum porter_duff_type porter_duff_type;
	enum porter_duff_mode porter_duff_mode;
	u16 colourkey_r;
	u16 colourkey_g;
	u16 colourkey_b;
	u16 colourkey_r_mask;
	u16 colourkey_g_mask;
	u16 colourkey_b_mask;
	enum colourkey_mode colourkey_mode;
	bool colourkey_enable;
	bool gamma_enable;

	unsigned int hskip;
};

static inline struct pdp_plane *to_pdp_plane(struct drm_plane *plane)
{
	return container_of(plane, struct pdp_plane, plane);
}

static inline
struct pdp_plane_state *to_pdp_plane_state(struct drm_plane_state *state)
{
	return container_of(state, struct pdp_plane_state, state);
}

int pdp_planes_init(struct drm_device *dev);

#endif
