#ifndef _IMG_IOMMU_H_
#define _IMG_IOMMU_H_

#include <linux/img-iommu.h>

/* MMU register offsets */
#define IMG_MMU_CTRL0		0x0000
#define IMG_MMU_CTRL1		0x0008
#define MMU_CTRL1_RESET		BIT(28)
#define MMU_CTRL1_INVALIDATE_CACHE3	BIT(11)
#define MMU_CTRL1_INVALIDATE_CACHE2	BIT(10)
#define MMU_CTRL1_INVALIDATE_CACHE1	BIT(9)
#define MMU_CTRL1_INVALIDATE_CACHE0	BIT(8)
#define MMU_CTRL1_FLUSH_CACHE3		BIT(3)
#define MMU_CTRL1_FLUSH_CACHE2		BIT(2)
#define MMU_CTRL1_FLUSH_CACHE1		BIT(1)
#define MMU_CTRL1_FLUSH_CACHE0		BIT(0)
#define IMG_MMU_BANK_INDEX	0x0010
#define IMG_MMU_PRIORITY_EN	0x0018
#define IMG_MMU_LIMIT_THU	0x001C
#define IMG_MMU_DIR_BASE_ADDR	0x0020
#define IMG_MMU_TILE_CFG	0x0040
#define IMG_MMU_TILE_MIN	0x0050
#define IMG_MMU_TILE_MAX	0x0060
#define IMG_MMU_ADDR_CTRL	0x0070
#define IMG_MMU_CONFIG0		0x0080
#define IMG_MMU_CONFIG1		0x0084
#define IMG_MMU_STATUS0		0x0088
#define MMU_STATUS0_ADDR_MASK	0xFFFFF000
#define IMG_MMU_STATUS1		0x008C
#define IMG_MMU_MEM_REQ		0x0090
#define IMG_MMU_FAULT_SEL	0x00A0
#define IMG_PROTOCOL_FAULT	0x00A8
#define IMG_MMU_VERSION		0x01D0

#define NUM_DT_ENTRIES (1024)
#define NUM_PT_ENTRIES (1024)

#define SPAGE_ORDER 12
#define SPAGE_SIZE (BIT(SPAGE_ORDER))

#define IMG_IOMMU_PAGES (SZ_4K)

#define IMG_IOMMU_NUM_DOMAINS_MASK 0xf
#define IMG_TILE_MASK 0x1

struct img_iommu;

#define EXT_ADDR_RANGE 4

struct img_iommu_domain {
	struct iommu_domain domain;	/* base domain*/
	int index;		/* index of domain when registered to iommu */
	struct img_iommu *iommu;	/* link to registered iommu*/
	struct img_iommu_tile_cfg tile_cfg; /* memory tiling configuration */
};

struct img_iommu_suspend_reg {
	u32 ctrl0;
	u32 ctrl1;
	u32 addr_ctrl;
	u32 dir_base_addr[4];
	u32 tile_cfg[4];
	u32 tile_min[4];
	u32 tile_max[4];
};

struct img_iommu {
	struct device *dev;
	void __iomem *base;
	struct list_head node;

	struct img_iommu_suspend_reg suspend_reg;

	/* NB Hardware supports a directory table for each domain but we
	 * dont have a use-case so for now use only one here.
	 */
	u32 *dt_vaddr;
	dma_addr_t dt_bus;
	/* lock for dt access */
	spinlock_t dt_lock;

	/* The hardware supports 4 distinct domains. Each of these can have
	 * tiling modes enabled or not. The bitmap keeps track of currently
	 * allocated domains. When a domain has no buffers it can be remapped
	 * with a different tiling configuration
	 */
	struct iommu_domain *domain[4];
	u32 domain_bitmap;

	/* Current tiling scheme iommu is using */
	u8 tile_mode;
};

#endif /* _IMG_IOMMU_H_ */
