/*
 * Copyright 2016 Imagination Technologies.
 *
 * This program is free software; you may redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/**
 * Generates an m width bitmask starting at n.
 * @param m	width
 * @param n shift
 * This is also equal to GENMASK((n), ((n)+(l)))
 */
#define MSK(m, n)		(((1 << (m)) - 1) << (n))

/* HDMI registers */
/** Register address: setup control */
#define RA_HDMI_SETUP_CTRL      (0x000UL)
/** Hot plug detect signaled */
#define	HOT_PLUG_DETECT	 BIT(0)
#define HOT_PLUG_DETECT_INPUT_A BIT(24)
/** Register address: override control */
#define RA_HDMI_OVR_CTRL        (0x004UL)
/** Register address: timer control */
#define RA_HDMI_TIMERA_CTRL     (0x008UL)
/** Register address: resistor override */
#define RA_HDMI_RES_OVR         (0x010UL)
/** Register address: resistor status */
#define RA_HDMI_RES_STS         (0x014UL)
/** Register address: TMDS PLL control */
#define RA_HDMI_PLL_CTRL        (0x018UL)
/** Register address: TMDS PLL frequency range */
#define RA_HDMI_PLL_FRQSET1     (0x01CUL)
/** Register address: TMDS PLL frequency range */
#define RA_HDMI_PLL_FRQSET2     (0x020UL)
/** Register address: TMDS PLL PCP and ICP range */
#define RA_HDMI_PLL_PAR1        (0x024UL)
/** Register address: TMDS PLL PCP and ICP range */
#define RA_HDMI_PLL_PAR2        (0x028UL)
/** Register address: TMDS PLL KOSC and CCOLF range */
#define RA_HDMI_PLL_PAR3        (0x02CUL)
/** Register address: PLL post lock filter */
#define RA_HDMI_PLL_LCK_STS     (0x030UL)
#define PLL_LOCKED BIT(0)
/** Register address: PLL clock control */
#define RA_HDMI_CLK_CTRL        (0x034UL)
/** Register address: PCB diversity control */
#define RA_HDMI_PCB_CTRL        (0x038UL)
/** Input selector */
#define INPUT_SELECT	 BIT(16)
#define SEL_PIXCLKSRC	MSK(2, 18)
/** Register address: phase control */
#define RA_HDMI_PHS_CTRL        (0x040UL)
/** Register address: used phases */
#define RA_HDMI_PHS_USD         (0x044UL)
/** Register address: miscellaneous operations control */
#define RA_HDMI_MISC_CTRL       (0x048UL)
/** Register address: EQ offset calibration */
#define RA_HDMI_EQOFF_CTRL      (0x04CUL)
/** Register address: EQ gain control */
#define RA_HDMI_EQGAIN_CTRL     (0x050UL)
/** Register address: EQ status */
#define RA_HDMI_EQCAL_STS       (0x054UL)
/** Register address: EQ results */
#define RA_HDMI_EQRESULT        (0x058UL)
/** Register address: EQ measurement control */
#define RA_HDMI_EQ_MEAS_CTRL    (0x05CUL)
/** Register address: HDMI mode recover */
#define RA_HDMI_MODE_RECOVER    (0x080UL)
/** Register address: HDMI error protection */
#define RA_HDMI_ERROR_PROTECT  (0x084UL)
/** Register address: validation and production test */
#define RA_HDMI_ERD_STS         (0x088UL)
/** Register address: video output sync signals control */
#define RA_HDMI_SYNC_CTRL       (0x090UL)
/** VS polarity adjustment */
#define VS_POL_ADJ_MODE	MSK(2, 3)
/** HS polarity adjustment automatic */
#define VS_POL_ADJ_AUTO	(2)
/** HS polarity adjustment */
#define HS_POL_ADJ_MODE	MSK(2, 1)
/** HS polarity adjustment automatic inversion */
#define HS_POL_ADJ_AUTO	(2)
/** Register address: clock measurement */
#define RA_HDMI_CKM_EVLTM       (0x094UL)
/** Evaluation period */
#define EVAL_TIME MSK(12, 4)
/** Max tmds_clk variation for clk_chg interrupt */
#define CLK_HYST MSK(3, 16)
/** active wait period for TMDS stabilisation */
#define TMDS_STABLE_TIMEOUT	(30)
/** Register address: legal clock count */
#define RA_HDMI_CKM_F	(0x098UL)
/** Maximum count for legal count */
#define	MAXFREQ_CKM		MSK(16, 16)
/** Minimum count for legal count */
#define	MINFREQ_CKM		MSK(16, 0)
/** Register address: measured clock results */
#define RA_HDMI_CKM_RESULT      (0x09CUL)
/** Measured clock is stable */
#define	CLOCK_IN_RANGE	 BIT(17)
/** Measured clock rate in bits */
#define	CLKRATE		MSK(16, 0)
/** Register address: sub-sampling control */
#define RA_HDMI_RESMPL_CTRL     (0x0A4UL)
/** Register address: deep color mode control */
#define RA_HDMI_DCM_CTRL        (0x0A8UL)
/** Register address: video output mute configuration */
#define RA_HDMI_VM_CFG_CH_0_1   (0x0B0UL)
/** Register address: video output mute configuration */
#define RA_HDMI_VM_CFG_CH2      (0x0B4UL)
/** Register address: spare */
#define RA_HDMI_SPARE	(0x0B8UL)
/** Register address: HDMI status */
#define RA_HDMI_STS	(0x0BCUL)
/** Current deep color mode */
#define DCM_CURRENT_MODE	MSK(4, 28)
/** Deep color mode, 24 bit */
#define DCM_CURRENT_MODE_24b	(4)
/** Deep color mode, 30 bit */
#define DCM_CURRENT_MODE_30b	(5)
/** Deep color mode, 36 bit */
#define DCM_CURRENT_MODE_36b	(6)
/** Deep color mode, 48 bit */
#define DCM_CURRENT_MODE_48b	(7)

#define RA_DMI_HDCP_CTRL	(0x0C0UL)
/*
 * Video Mode registers
 */
/** Register address: video mode control */
#define RA_MD_HCTRL1	(0x140UL)
/** Register address: video mode control */
#define RA_MD_HCTRL2	(0x144UL)
/** Register address: horizontal sync */
#define RA_MD_HT0	(0x148UL)
/** Horizontal total length in imodetclk cycles averaged over 32 lines*/
#define	HTOT32_CLK	MSK(16, 16)
/** Horizontal sync pulse duration in imodetclk cycles */
#define	HS_CLK	MSK(16, 0)

/** Register address: horizontal offset */
#define RA_MD_HT1	(0x14CUL)
/** Horizontal total length */
#define	HTOT_PIX	MSK(16, 16)
/** Horizontal offset length */
#define	HOFS_PIX	MSK(16, 0)
/** Register address: horizontal active length */
#define RA_MD_HACT_PX	(0x150UL)
/** Horizontal active length */
#define	HACT_PIX	MSK(16, 0)
/** Register address: horizontal active time */
#define RA_MD_HACT_PXA          (0x154UL)
/** Register address: vertical control */
#define RA_MD_VCTRL	(0x158UL)
/** Register address: vertical timing - sync pulse duration */
#define RA_MD_VSC	(0x15CUL)
/** Vertical sync pulse duration in imodetclk cycles */
#define VS_CLK		MSK(16, 0)
/** Register address: vertical timing - frame duration */
#define RA_MD_VTC	(0x160UL)
/** Frame duration */
#define VTOT_CLK		(~0)
/** Register address: vertical offset length */
#define RA_MD_VOL	(0x164UL)
/** Vertical offset length */
#define	VOFS_LIN	MSK(16, 0)
/** Register address: vertical active length */
#define RA_MD_VAL	(0x168UL)
/** Vertical active length */
#define	VACT_LIN	MSK(16, 0)
/** Register address: vertical timing */
#define RA_MD_VTH	(0x16CUL)
/** Register address: vertical total length */
#define RA_MD_VTL	(0x170UL)
/** Vertical total length */
#define	VTOT_LIN	MSK(16, 0)
/** Register address: skew measurement trigger */
#define RA_MD_IL_CTRL	(0x174UL)
/** Register address: VS and HS skew */
#define RA_MD_IL_SKEW	(0x178UL)
/** Register address: V&H skew and filed detection */
#define RA_MD_IL_POL	(0x17CUL)
/** Register address: video mode status */
#define RA_MD_STS	(0x180UL)
/** Interlace active status */
#define	ILACE	 BIT(3)
/*
 * Audio registers
 */
/** Register address: audio mode control */
#define RA_AUD_CTRL	(0x200UL)
/** Register address: audio PLL control */
#define RA_AUD_PLL_CTRL         (0x208UL)
#define PLL_LOCK_STABLE BIT(31)
/** Register address: audio PLL lock */
#define RA_AUD_PLL_LOCK         (0x20CUL)
/** Register address: DDS audio clock control */
#define RA_AUD_PLL_RESET        (0x210UL)
/** Register address: audio clock control */
#define RA_AUD_CLK_CTRL         (0x214UL)
/** Register address: ASP sync intervals */
#define RA_AUD_CLK_MASP         (0x218UL)
/** Register address: audio sync interval */
#define RA_AUD_CLK_MAUD         (0x21CUL)
/** Register address: sync interval reset */
#define RA_AUD_FILT_CTRL1       (0x220UL)
/** Register address: phase filter control */
#define RA_AUD_FILT_CTRL2       (0x224UL)
/** Register address: manual CTS control */
#define RA_AUD_CTS_MAN          (0x228UL)
/** Register address: manual N control */
#define RA_AUD_N_MAN	(0x22CUL)
/** Register address: audio clock status */
#define RA_AUD_CLK_STS          (0x23CUL)
/** Register address: audio FIFO control */
#define RA_AUD_FIFO_CTRL        (0x240UL)
/** Audio FIFO reset */
#define AFIF_INIT BIT(0)
#define AFIF_SUBPACKETS BIT(16)
/** Register address: audio FIFO threshold */
#define RA_AUD_FIFO_TH          (0x244UL)
/** Register address: audio FIFO fill */
#define RA_AUD_FIFO_FILL_S      (0x248UL)
/** Register address: audio FIFO fill minimum/maximum */
#define RA_AUD_FIFO_CLRA_MM     (0x24CUL)
/** Register address: audio FIFO fill status */
#define RA_AUD_FIFO_FILLSTS     (0x250UL)
/** Register address: audio output interface configuration */
#define RA_AUD_CHEXTRA_CTRL     (0x254UL)
/** Register address: audio mute control */
#define RA_AUD_MUTE_CTRL        (0x258UL)
/** Manual/automatic audio mute control */
#define AUD_MUTE_SEL	MSK(2, 5)
/** Force unmute (overrules all) */
#define AUD_MUTE_FORCE_UN		(0)
/** Automatic mute when FIFO thresholds are exceeded */
#define AUD_MUTE_FIFO_TH		(1)
/** Automatic mute when FIFO over/underflows */
#define AUD_MUTE_FIFO_FL		(2)
/** Force mute (overrules all) */
#define AUD_MUTE_FORCE	(3)
/** Register address: serial audio output control */
#define RA_AUD_SAO_CTRL         (0x260UL)
#define SPDIF_DISABLE		MSK(4, 1)
#define I2S_DISABLE		MSK(4, 5)
/** Register address: parallel audio output control */
#define RA_AUD_PAO_CTRL         (0x264UL)
#define AUDIO_FMT_SEL BIT(0)
#define PAO_DISABLE BIT(12)
#define PAO_RATE		MSK(2, 16)
/** Register address: audio FIFO status */
#define RA_AUD_FIFO_STS         (0x27CUL)
/** Register address: lock detector threshold */
#define RA_CI_PLLAUDIO_5        (0x28CUL)
/** Register address: test mode selection */
#define RA_CI_PLLAUDIO_4        (0x290UL)
/** Register address: bypass divider control */
#define RA_CI_PLLAUDIO_3        (0x294UL)
/** Register address: monitoring */
#define RA_CI_PLLAUDIO_2        (0x298UL)
/** Register address: control */
#define RA_CI_PLLAUDIO_1        (0x29CUL)
/** Register address: SNPS PHY GEN3 control - starting version 1.30a */
#define RA_SNPS_PHYG3_CTRL	(0x2C0UL)

/*
 * Packet Decoder and FIFO Control registers
 */
/** Register address: packet decoder and FIFO control */
#define RA_PDEC_CTRL	(0x300UL)
/** Packet FIFO store filter enable */
#define PFIFO_STORE_FILTER_EN	BIT(31)
/** Packet FIFO store packet */
#define PFIFO_STORE_PACKET BIT(16)
/** Packet FIFO clear min/max information */
#define PD_FIFO_FILL_INFO_CLR	BIT(8)
/** Packet FIFO skip one packet */
#define PD_FIFO_SKIP	 BIT(6)
/** Packet FIFO clear */
#define PD_FIFO_CLR BIT(5)
/** Packet FIFO write enable */
#define PD_FIFO_WE BIT(4)
/** Packet error detection/correction enable */
#define PDEC_BCH_EN BIT(0)
/** Register address: packet decoder and FIFO configuration */
#define RA_PDEC_FIFO_CFG        (0x304UL)
/** Register address: packet decoder and FIFO status */
#define RA_PDEC_FIFO_STS        (0x308UL)
/** Register address: packet decoder and FIFO byte data */
#define RA_PDEC_FIFO_DATA       (0x30CUL)
/** Register address: packet decoder and FIFO debug control */
#define RA_PDEC_DBG_CTRL        (0x310UL)
/** Register address: packet decoder and FIFO measured timing gap */
#define RA_PDEC_DBG_TMAX        (0x314UL)
/** Register address: CTS loop */
#define RA_PDEC_DBG_CTS         (0x318UL)
/** Register address: ACP frequency count */
#define RA_PDEC_DBG_ACP         (0x31CUL)
/** Register address: signal errors in data island packet */
#define RA_PDEC_DBG_ERRA_CORR   (0x320UL)
/** Register address: CTS reset measurement control */
#define RA_PDEC_ACRM_CTRL       (0x330UL)
/** Register address: maximum CTS div N value */
#define RA_PDEC_ACRM_MAX        (0x334UL)
/** Register address: minimum CTS div N value */
#define RA_PDEC_ACRM_MIN        (0x338UL)
/** Register BCH/Checksum Error Filter Handling Register */
#define RA_PDEC_ERR_FILTER        (0x33CUL)
/** Register address: audio sub packet control */
#define RA_PDEC_ASP_CTRL        (0x340UL)
/** Automatic mute all video channels */
#define AUTO_VMUTE BIT(6)
/** Automatic mute audio sub packets */
#define AUTO_SPFLAT_MUTE		MSK(4, 2)
/** Register address: audio sub packet errors */
#define RA_PDEC_ASP_ERR         (0x344UL)
/** Register address: packet decoder status, see packet interrupts */
#define RA_PDEC_STS	(0x360UL)
/** Register address: general control packet AV mute */
#define RA_PDEC_GCP_AVMUTE      (0x380UL)
/** CD3 CD2 CD1 CD0, color depth for deep color mode support */
#define PKTDEC_GCP_CD		MSK(4, 4)
/** Register address: audio clock regeneration */
#define RA_PDEC_ACR_CTS        (0x390UL)
/** Audio clock regeneration, CTS parameter */
#define CTS_DECODED	MSK(20, 0)
/** Register address: audio clock regeneration */
#define RA_PDEC_ACR_N		(0x394UL)
/** Audio clock regeneration, N parameter */
#define N_DECODED	MSK(20, 0)
/** Register address: auxiliary video information info frame */
#define RA_PDEC_AVI_HB		(0x3A0UL)
/** PR3-0, pixel repetition factor */
#define	PIX_REP_FACTOR	MSK(4, 24)
/** CN1-0 AVI IT CONTENT TYPE */
#define IT_CONTENT_TYPE	MSK(2, 28)
/** Register address: auxiliary video information info frame */
#define RA_PDEC_AVI_PB		(0x3A4UL)
/** VIC7-0, video mode identification code */
#define	VID_IDENT_CODE	MSK(8, 24)
/** ITC, IT content */
#define IT_CONTENT BIT(23)
/** EC2-0, extended colorimetry */
#define EXT_COLORIMETRY	MSK(3, 20)
/** Q1-0, RGB quantization range */
#define RGB_QUANT_RANGE	MSK(2, 18)
/** SC1-0, non-uniform scaling information */
#define NON_UNIF_SCALE	MSK(2, 16)
/** C1-0, colorimetry information */
#define COLORIMETRY	MSK(2, 14)
/** M1-0, picture aspect ratio */
#define PIC_ASPECT_RATIO		MSK(2, 12)
/** R3-0, active format aspect ratio */
#define ACT_ASPECT_RATIO		MSK(4, 8)
/** Y1-0, video format */
#define VIDEO_FORMAT	MSK(2, 5)
/** A0, active format information present */
#define ACT_INFO_PRESENT BIT(4)
/** B1-0, bar valid information */
#define BAR_INFO_VALID	MSK(2, 2)
/** S1-0, scan information from packet extraction */
#define SCAN_INFO	MSK(2, 0)
/** Register address: auxiliary video information info frame */
#define RA_PDEC_AVI_TBB		(0x3A8UL)
/** Line number to start of bottom bar */
#define LIN_ST_BOT_BAR	MSK(16, 16)
/** Line number to end of top bar */
#define LIN_END_TOP_BAR	MSK(16, 0)
/** Register address: auxiliary video information info frame */
#define RA_PDEC_AVI_LRB		(0x3ACUL)
/** Pixel number of start right bar */
#define PIX_ST_RIG_BAR	MSK(16, 16)
/** Pixel number of end left bar */
#define PIX_END_LEF_BAR	MSK(16, 0)
/** Register address: special audio layout control for multi-channel audio */
#define RA_PDEC_AIF_CTRL	(0x3C0UL)
/** Register address: audio info frame */
#define RA_PDEC_AIF_HB		(0x3C4UL)
/** Register address: audio info frame */
#define RA_PDEC_AIF_PB0		(0x3C8UL)
/** CA7-0, channel/speaker allocation */
#define	CH_SPEAK_ALLOC	MSK(8, 24)
/** CTX, coding extension */
#define	AIF_DATA_BYTE_3	MSK(8, 16)
/** SF2-0, sample frequency */
#define	SAMPLE_FREQ	MSK(3, 10)
/** SS1-0, sample size */
#define	SAMPLE_SIZE	MSK(2, 8)
/** CT3-0, coding type */
#define	CODING_TYPE	MSK(4, 4)
/** CC2-0, channel count */
#define	CHANNEL_COUNT	MSK(3, 0)
/** Register address: audio info frame */
#define RA_PDEC_AIF_PB1	(0x3CCUL)
/** DM_INH, down-mix inhibit */
#define	DWNMIX_INHIBIT	 BIT(7)
/** LSV3-0, level shift value */
#define	LEVEL_SHIFT_VAL	MSK(4, 3)
/** Register address: gamut sequence number */
#define RA_PDEC_GMD_HB		(0x3D0UL)
/** Register address: gamut meta data */
#define RA_PDEC_GMD_PB		(0x3D4UL)
/** Register address: CEA Video Mode Configuration */
#define RA_CEAVID_CONFIG	(0x400UL)
/** Reset cea video output interface */
#define CEAVID_RST BIT(31)
/** HDMI2.0 and SCDC rgisters */
#define	RA_HDMI20_CONTROL	(0x800UL)
#define SCRAMBEN_SEL		MSK(2, 0)
#define	RA_SCDC_I2CCONFIG	(0x804UL)
#define	RA_SCDC_CONFIG	(0x808UL)
#define	RA_CHLOCK_CONFIG	(0x80CUL)
#define	RA_HDCP22_CONTROL	(0x81CUL)
#define	RA_SCDC_REGS0	(0x820UL)
#define	RA_SCDC_REGS1	(0x824UL)
#define	RA_SCDC_REGS2	(0x828UL)
#define	RA_SCDC_REGS3	(0x82CUL)
#define	RA_SCDC_MANSPEC0	(0x840UL)
#define	RA_SCDC_MANSPEC1	(0x844UL)
#define	RA_SCDC_MANSPEC2	(0x848UL)
#define	RA_SCDC_MANSPEC3	(0x84CUL)
#define	RA_SCDC_MANSPEC4	(0x850UL)
#define	RA_SCDC_WRDATA0	(0x860UL)
#define	RA_SCDC_WRDATA1	(0x864UL)
#define	RA_SCDC_WRDATA2	(0x868UL)
#define	RA_SCDC_WRDATA3	(0x86CUL)
#define	RA_SCDC_WRDATA4	(0x870UL)
#define	RA_SCDC_WRDATA5	(0x874UL)
#define	RA_SCDC_WRDATA6	(0x878UL)
#define	RA_SCDC_WRDATA7	(0x87CUL)

/*
 * DTL Interface registers
 */
/** Register address: dummy register for testing */
#define RA_DUMMY_IP_REG		(0xF00UL)
/*
 * Packet Decoder Interrupt registers
 */
/** Register address: packet decoder interrupt clear enable */
#define RA_PDEC_IEN_CLR		(0xF78UL)
/** Register address: packet decoder interrupt set enable */
#define RA_PDEC_IEN_SET		(0xF7CUL)
/** Register address: packet decoder interrupt status */
#define RA_PDEC_ISTS		(0xF80UL)
/** Register address: packet decoder interrupt enable */
#define RA_PDEC_IEN		(0xF84UL)
/** Register address: packet decoder interrupt clear status */
#define RA_PDEC_ICLR		(0xF88UL)
/** Register address: packet decoder interrupt set status */
#define RA_PDEC_ISET		(0xF8CUL)
#define AUD_TYPE_CHG BIT(29)
/** DVI detection status */
#define DVIDET	 BIT(28)
/* Vender Specific Infoframe packet checksum changed */
#define	VSI_CKS_CHG BIT(27)
/** AIF checksum changed */
#define AIF_CKS_CHG BIT(25)
/** AVI checksum changed */
#define AVI_CKS_CHG BIT(24)
/** N value from ACR packet changed */
#define ACR_N_CHG BIT(23)
/** ACP_AV_MUTE, the packet comes before any audio change */
#define GCP_AV_MUTE_CHG BIT(21)
/** Packet FIFO new entry */
#define PD_FIFO_NEW_ENTRY BIT(8)
/** Packet FIFO overflow */
#define PD_FIFO_OVERFL	 BIT(4)
/** Packet FIFO underflow */
#define PD_FIFO_UNDERFL	 BIT(3)
/*
 * Audio Clock Interrupt registers
 */
/** Register address: audio clock interrupt clear enable */
#define RA_AUD_CEC_IEN_CLR	(0xF90UL)
/** Register address: audio clock interrupt set enable */
#define RA_AUD_CEC_IEN_SET	(0xF94UL)
/** Register address: audio clock interrupt status */
#define RA_AUD_CEC_ISTS		(0xF98UL)
/** Register address: audio clock interrupt enable */
#define RA_AUD_CEC_IEN		(0xF9CUL)
/** Register address: audio clock interrupt clear status */
#define RA_AUD_CEC_ICLR		(0xFA0UL)
/** Register address: audio clock interrupt set status */
#define RA_AUD_CEC_ISET		(0xFA4UL)
/*
 * Audio FIFO Interrupt registers
 */
/** Register address: audio FIFO interrupt clear enable */
#define RA_AUD_FIFO_IEN_CLR	(0xFA8UL)
/** Register address: audio FIFO interrupt set enable */
#define RA_AUD_FIFO_IEN_SET	(0xFACUL)
/** Register address: audio FIFO interrupt status */
#define RA_AUD_FIFO_ISTS	(0xFB0UL)
/** Register address: audio FIFO interrupt enable */
#define RA_AUD_FIFO_IEN		(0xFB4UL)
/** Register address: audio FIFO interrupt clear status */
#define RA_AUD_FIFO_ICLR	(0xFB8UL)
/** Register address: audio FIFO interrupt set status */
#define RA_AUD_FIFO_ISET	(0xFBCUL)
/** Audio FIFO overflow interrupt */
#define AFIF_OVERFL BIT(4)
/** Audio FIFO underflow interrupt */
#define AFIF_UNDERFL	 BIT(3)
/*
 * Mode Detection Interrupt registers
 */
/** Register address: mode detection interrupt clear enable */
#define RA_MD_IEN_CLR		(0xFC0UL)
/** Register address: mode detection interrupt set enable */
#define RA_MD_IEN_SET		(0xFC4UL)
/** Register address: mode detection interrupt status */
#define RA_MD_ISTS		(0xFC8UL)
/** Register address: mode detection interrupt enable */
#define RA_MD_IEN		(0xFCCUL)
/** Register address: mode detection interrupt clear status */
#define RA_MD_ICLR		(0xFD0UL)
/** Register address: mode detection interrupt set status */
#define RA_MD_ISET		(0xFD4UL)
/** Video mode interrupts */
#define VIDEO_MODE (MSK(3, 9) | MSK(1, 6) | BIT(3))
/*
 * HDMI Interrupt registers
 */
/** Register address: HDMI interrupt clear enable */
#define RA_HDMI_IEN_CLR		(0xFD8UL)
/** Register address: HDMI interrupt set enable */
#define RA_HDMI_IEN_SET		(0xFDCUL)
/** Register address: HDMI interrupt status */
#define RA_HDMI_ISTS		(0xFE0UL)
/** Register address: HDMI interrupt enable */
#define RA_HDMI_IEN		(0xFE4UL)
/** Register address: HDMI interrupt clear status */
#define RA_HDMI_ICLR		(0xFE8UL)
/** Register address: HDMI interrupt set status */
#define RA_HDMI_ISET		(0xFECUL)
/** AKSV receive interrupt */
#define	AKSV_RCV BIT(25)
/** Deep color mode change interrupt */
#define DCM_CURRENT_MODE_CHG	BIT(16)
#define	SCDCTMDSCFGCHANGE	BIT(19)
#define	SCDCSCSTATUSCHANGE	BIT(18)
#define	SCDCCFGCHANGE BIT(17)

/** Clock change interrupt */
#define	CLK_CHANGE	MSK(2, 5)
/*
 * DMI registers
 */
/** Register address: DMI software reset */
#define RA_DMI_SW_RST          (0xFF0UL)
#define MAIN_SWRESET	BIT(0)
#define VID_SWRESET BIT(7)
/** Register address: DMI disable interface */
#define RA_DMI_DISABLE_IF      (0xFF4UL)
/** Register address: DMI module ID */
#define RA_DMI_MODULE_ID       (0xFFCUL)
