/*! 
******************************************************************************
 @file   : fixedpoint_sinetable.h

 @brief  

 @Author Tom Whalley

 @date   07/01/2005
 
         <b>Copyright 2003 by Imagination Technologies Limited.</b>\n
         All rights reserved.  No part of this software, either
         material or conceptual may be copied or distributed,
         transmitted, transcribed, stored in a retrieval system
         or translated into any human or computer language in any
         form by any means, electronic, mechanical, manual or
         other-wise, or disclosed to third parties without the
         express written permission of Imagination Technologies
         Limited, Unit 8, HomePark Industrial Estate,
         King's Langley, Hertfordshire, WD4 8LZ, U.K.

 <b>Description:</b>\n
         Lookup table for fixed point trig functions.

 <b>Platform:</b>\n
	     Platform Independent 

 @Version	
	    -	1.0	1st Release

******************************************************************************/

/* 
******************************************************************************
 Modifications :-

 $Log: fixedpoint_sinetable.h,v $
******************************************************************************/

#if !defined (__FIXEDPOINTSINETABLE_H__)
#define __FIXEDPOINTSINETABLE_H__

#if defined (__cplusplus)
extern "C" {
#endif

/* Table should be referenced using a 1.8 fixed point index, in the range 0->PI/2 	*/
/* Values of Sine (x) are in 1.30 fixed point format.								*/
img_uint32	FIXEDPT_aui32FixedPointSineTable [] =
{
	0x00000000, /* 0) Sine 0.000000 = 0.000000 */
	0x003ffff5, /* 1) Sine 0.003906 = 0.003906 */
	0x007fffaa, /* 2) Sine 0.007813 = 0.007812 */
	0x00bffee0, /* 3) Sine 0.011719 = 0.011718 */
	0x00fffd55, /* 4) Sine 0.015625 = 0.015624 */
	0x013ffaca, /* 5) Sine 0.019531 = 0.019530 */																										
	0x017ff700, /* 6) Sine 0.023438 = 0.023435 */																										
	0x01bff1b6, /* 7) Sine 0.027344 = 0.027340 */																										
	0x01ffeaaa, /* 8) Sine 0.031250 = 0.031245 */																										
	0x023fe1a0, /* 9) Sine 0.035156 = 0.035149 */																										
	0x027fd658, /* 10) Sine 0.039063 = 0.039053 */																										
	0x02bfc88c, /* 11) Sine 0.042969 = 0.042956 */																										
	0x02ffb804, /* 12) Sine 0.046875 = 0.046858 */																										
	0x033fa478, /* 13) Sine 0.050781 = 0.050759 */																										
	0x037f8db0, /* 14) Sine 0.054688 = 0.054660 */																										
	0x03bf7368, /* 15) Sine 0.058594 = 0.058560 */																										
	0x03ff555c, /* 16) Sine 0.062500 = 0.062459 */																										
	0x043f3358, /* 17) Sine 0.066406 = 0.066357 */																										
	0x047f0d10, /* 18) Sine 0.070313 = 0.070255 */																										
	0x04bee248, /* 19) Sine 0.074219 = 0.074151 */																										
	0x04feb2c8, /* 20) Sine 0.078125 = 0.078046 */																										
	0x053e7e40, /* 21) Sine 0.082031 = 0.081939 */																										
	0x057e4480, /* 22) Sine 0.085938 = 0.085832 */																										
	0x05be0540, /* 23) Sine 0.089844 = 0.089723 */																										
	0x05fdc040, /* 24) Sine 0.093750 = 0.093613 */																										
	0x063d7548, /* 25) Sine 0.097656 = 0.097501 */																										
	0x067d2408, /* 26) Sine 0.101563 = 0.101388 */																										
	0x06bccc58, /* 27) Sine 0.105469 = 0.105273 */																										
	0x06fc6de0, /* 28) Sine 0.109375 = 0.109157 */																										
	0x073c0870, /* 29) Sine 0.113281 = 0.113039 */																										
	0x077b9bc8, /* 30) Sine 0.117188 = 0.116919 */																										
	0x07bb27a0, /* 31) Sine 0.121094 = 0.120798 */																										
	0x07faabb8, /* 32) Sine 0.125000 = 0.124675 */																										
	0x083a27e0, /* 33) Sine 0.128906 = 0.128550 */																										
	0x08799bc0, /* 34) Sine 0.132813 = 0.132422 */																										
	0x08b90730, /* 35) Sine 0.136719 = 0.136293 */																										
	0x08f869f0, /* 36) Sine 0.140625 = 0.140162 */																										
	0x0937c3b0, /* 37) Sine 0.144531 = 0.144029 */																										
	0x09771430, /* 38) Sine 0.148438 = 0.147893 */																										
	0x09b65b40, /* 39) Sine 0.152344 = 0.151755 */																										
	0x09f59890, /* 40) Sine 0.156250 = 0.155615 */																										
	0x0a34cc00, /* 41) Sine 0.160156 = 0.159472 */																										
	0x0a73f520, /* 42) Sine 0.164063 = 0.163327 */																										
	0x0ab313e0, /* 43) Sine 0.167969 = 0.167180 */																										
	0x0af227e0, /* 44) Sine 0.171875 = 0.171030 */																										
	0x0b313100, /* 45) Sine 0.175781 = 0.174877 */																										
	0x0b702ee0, /* 46) Sine 0.179688 = 0.178722 */																										
	0x0baf2150, /* 47) Sine 0.183594 = 0.182564 */																										
	0x0bee0810, /* 48) Sine 0.187500 = 0.186403 */																										
	0x0c2ce2f0, /* 49) Sine 0.191406 = 0.190240 */																										
	0x0c6bb190, /* 50) Sine 0.195313 = 0.194073 */																										
	0x0caa73d0, /* 51) Sine 0.199219 = 0.197904 */																										
	0x0ce92970, /* 52) Sine 0.203125 = 0.201731 */																										
	0x0d27d210, /* 53) Sine 0.207031 = 0.205555 */																										
	0x0d666d90, /* 54) Sine 0.210938 = 0.209377 */																										
	0x0da4fbb0, /* 55) Sine 0.214844 = 0.213195 */																										
	0x0de37c20, /* 56) Sine 0.218750 = 0.217010 */																										
	0x0e21eec0, /* 57) Sine 0.222656 = 0.220821 */																										
	0x0e605330, /* 58) Sine 0.226563 = 0.224629 */																										
	0x0e9ea940, /* 59) Sine 0.230469 = 0.228434 */																										
	0x0edcf0b0, /* 60) Sine 0.234375 = 0.232235 */																										
	0x0f1b2940, /* 61) Sine 0.238281 = 0.236033 */																										
	0x0f5952c0, /* 62) Sine 0.242188 = 0.239827 */																										
	0x0f976ce0, /* 63) Sine 0.246094 = 0.243617 */																										
	0x0fd57770, /* 64) Sine 0.250000 = 0.247404 */																										
	0x10137220, /* 65) Sine 0.253906 = 0.251187 */																										
	0x10515cc0, /* 66) Sine 0.257813 = 0.254966 */																										
	0x108f3700, /* 67) Sine 0.261719 = 0.258741 */																										
	0x10cd00c0, /* 68) Sine 0.265625 = 0.262512 */																										
	0x110ab9c0, /* 69) Sine 0.269531 = 0.266280 */																										
	0x114861a0, /* 70) Sine 0.273438 = 0.270043 */																										
	0x1185f840, /* 71) Sine 0.277344 = 0.273802 */																										
	0x11c37d60, /* 72) Sine 0.281250 = 0.277557 */																										
	0x1200f0c0, /* 73) Sine 0.285156 = 0.281307 */																										
	0x123e5220, /* 74) Sine 0.289063 = 0.285054 */																										
	0x127ba120, /* 75) Sine 0.292969 = 0.288796 */																										
	0x12b8ddc0, /* 76) Sine 0.296875 = 0.292533 */																										
	0x12f607a0, /* 77) Sine 0.300781 = 0.296266 */																										
	0x13331ea0, /* 78) Sine 0.304688 = 0.299995 */																										
	0x13702240, /* 79) Sine 0.308594 = 0.303719 */																										
	0x13ad12a0, /* 80) Sine 0.312500 = 0.307439 */																										
	0x13e9ef40, /* 81) Sine 0.316406 = 0.311153 */																										
	0x1426b7e0, /* 82) Sine 0.320313 = 0.314863 */																										
	0x14636c80, /* 83) Sine 0.324219 = 0.318568 */																										
	0x14a00ca0, /* 84) Sine 0.328125 = 0.322269 */																										
	0x14dc9820, /* 85) Sine 0.332031 = 0.325964 */																										
	0x15190ec0, /* 86) Sine 0.335938 = 0.329654 */																										
	0x15557060, /* 87) Sine 0.339844 = 0.333340 */																										
	0x1591bca0, /* 88) Sine 0.343750 = 0.337020 */																										
	0x15cdf340, /* 89) Sine 0.347656 = 0.340695 */																										
	0x160a1420, /* 90) Sine 0.351563 = 0.344365 */																										
	0x16461f00, /* 91) Sine 0.355469 = 0.348030 */																										
	0x16821380, /* 92) Sine 0.359375 = 0.351689 */																										
	0x16bdf1a0, /* 93) Sine 0.363281 = 0.355343 */																										
	0x16f9b8e0, /* 94) Sine 0.367188 = 0.358992 */																										
	0x17356940, /* 95) Sine 0.371094 = 0.362635 */																										
	0x17710260, /* 96) Sine 0.375000 = 0.366273 */																										
	0x17ac8400, /* 97) Sine 0.378906 = 0.369905 */																										
	0x17e7ee00, /* 98) Sine 0.382813 = 0.373531 */																										
	0x18234020, /* 99) Sine 0.386719 = 0.377152 */																										
	0x185e7a20, /* 100) Sine 0.390625 = 0.380766 */																										
	0x18999ba0, /* 101) Sine 0.394531 = 0.384375 */																										
	0x18d4a4a0, /* 102) Sine 0.398438 = 0.387979 */																										
	0x190f94e0, /* 103) Sine 0.402344 = 0.391576 */																										
	0x194a6be0, /* 104) Sine 0.406250 = 0.395167 */																										
	0x198529c0, /* 105) Sine 0.410156 = 0.398753 */																										
	0x19bfce00, /* 106) Sine 0.414063 = 0.402332 */																										
	0x19fa5880, /* 107) Sine 0.417969 = 0.405905 */																										
	0x1a34c920, /* 108) Sine 0.421875 = 0.409472 */																										
	0x1a6f1f80, /* 109) Sine 0.425781 = 0.413032 */																										
	0x1aa95b60, /* 110) Sine 0.429688 = 0.416587 */																										
	0x1ae37ca0, /* 111) Sine 0.433594 = 0.420135 */																										
	0x1b1d8300, /* 112) Sine 0.437500 = 0.423676 */																										
	0x1b576e40, /* 113) Sine 0.441406 = 0.427211 */																										
	0x1b913e40, /* 114) Sine 0.445313 = 0.430740 */																										
	0x1bcaf280, /* 115) Sine 0.449219 = 0.434262 */																										
	0x1c048b20, /* 116) Sine 0.453125 = 0.437777 */																										
	0x1c3e07a0, /* 117) Sine 0.457031 = 0.441286 */																										
	0x1c7767e0, /* 118) Sine 0.460938 = 0.444788 */																										
	0x1cb0abc0, /* 119) Sine 0.464844 = 0.448283 */																										
	0x1ce9d2e0, /* 120) Sine 0.468750 = 0.451771 */																										
	0x1d22dd20, /* 121) Sine 0.472656 = 0.455253 */																										
	0x1d5bca40, /* 122) Sine 0.476563 = 0.458727 */																										
	0x1d9499e0, /* 123) Sine 0.480469 = 0.462195 */																										
	0x1dcd4c20, /* 124) Sine 0.484375 = 0.465655 */																										
	0x1e05e060, /* 125) Sine 0.488281 = 0.469109 */																										
	0x1e3e56c0, /* 126) Sine 0.492188 = 0.472555 */																										
	0x1e76aee0, /* 127) Sine 0.496094 = 0.475994 */																										
	0x1eaee880, /* 128) Sine 0.500000 = 0.479426 */																										
	0x1ee70360, /* 129) Sine 0.503906 = 0.482850 */																										
	0x1f1eff60, /* 130) Sine 0.507813 = 0.486267 */																										
	0x1f56dc60, /* 131) Sine 0.511719 = 0.489677 */																										
	0x1f8e99e0, /* 132) Sine 0.515625 = 0.493079 */																										
	0x1fc637e0, /* 133) Sine 0.519531 = 0.496473 */																										
	0x1ffdb620, /* 134) Sine 0.523438 = 0.499860 */																										
	0x20351480, /* 135) Sine 0.527344 = 0.503240 */																										
	0x206c5280, /* 136) Sine 0.531250 = 0.506611 */																										
	0x20a37000, /* 137) Sine 0.535156 = 0.509975 */																										
	0x20da6d00, /* 138) Sine 0.539063 = 0.513332 */																										
	0x21114940, /* 139) Sine 0.542969 = 0.516680 */																										
	0x21480440, /* 140) Sine 0.546875 = 0.520021 */																										
	0x217e9e00, /* 141) Sine 0.550781 = 0.523353 */																										
	0x21b51640, /* 142) Sine 0.554688 = 0.526678 */																										
	0x21eb6d00, /* 143) Sine 0.558594 = 0.529994 */																										
	0x2221a180, /* 144) Sine 0.562500 = 0.533303 */																										
	0x2257b400, /* 145) Sine 0.566406 = 0.536603 */																										
	0x228da440, /* 146) Sine 0.570313 = 0.539895 */																										
	0x22c37200, /* 147) Sine 0.574219 = 0.543179 */																										
	0x22f91cc0, /* 148) Sine 0.578125 = 0.546455 */																										
	0x232ea4c0, /* 149) Sine 0.582031 = 0.549722 */																										
	0x23640940, /* 150) Sine 0.585938 = 0.552981 */																										
	0x23994ac0, /* 151) Sine 0.589844 = 0.556231 */																										
	0x23ce6880, /* 152) Sine 0.593750 = 0.559473 */																										
	0x24036240, /* 153) Sine 0.597656 = 0.562707 */																										
	0x24383840, /* 154) Sine 0.601563 = 0.565931 */																										
	0x246cea00, /* 155) Sine 0.605469 = 0.569148 */																										
	0x24a17740, /* 156) Sine 0.609375 = 0.572355 */																										
	0x24d5dfc0, /* 157) Sine 0.613281 = 0.575554 */																										
	0x250a2380, /* 158) Sine 0.617188 = 0.578744 */																										
	0x253e4240, /* 159) Sine 0.621094 = 0.581925 */																										
	0x25723bc0, /* 160) Sine 0.625000 = 0.585097 */																										
	0x25a61000, /* 161) Sine 0.628906 = 0.588261 */																										
	0x25d9be40, /* 162) Sine 0.632813 = 0.591415 */																										
	0x260d46c0, /* 163) Sine 0.636719 = 0.594560 */																										
	0x2640a980, /* 164) Sine 0.640625 = 0.597697 */																										
	0x2673e5c0, /* 165) Sine 0.644531 = 0.600824 */																										
	0x26a6fb80, /* 166) Sine 0.648438 = 0.603942 */																										
	0x26d9ea80, /* 167) Sine 0.652344 = 0.607051 */																										
	0x270cb300, /* 168) Sine 0.656250 = 0.610150 */																										
	0x273f5440, /* 169) Sine 0.660156 = 0.613240 */																										
	0x2771ce40, /* 170) Sine 0.664063 = 0.616321 */																										
	0x27a420c0, /* 171) Sine 0.667969 = 0.619393 */																										
	0x27d64bc0, /* 172) Sine 0.671875 = 0.622455 */																										
	0x28084ec0, /* 173) Sine 0.675781 = 0.625507 */																										
	0x283a29c0, /* 174) Sine 0.679688 = 0.628550 */																										
	0x286bdc80, /* 175) Sine 0.683594 = 0.631583 */																										
	0x289d6700, /* 176) Sine 0.687500 = 0.634607 */																										
	0x28cec8c0, /* 177) Sine 0.691406 = 0.637621 */																										
	0x290001c0, /* 178) Sine 0.695313 = 0.640625 */																										
	0x293111c0, /* 179) Sine 0.699219 = 0.643620 */																										
	0x2961f880, /* 180) Sine 0.703125 = 0.646605 */																										
	0x2992b600, /* 181) Sine 0.707031 = 0.649580 */																										
	0x29c349c0, /* 182) Sine 0.710938 = 0.652544 */																										
	0x29f3b3c0, /* 183) Sine 0.714844 = 0.655499 */																										
	0x2a23f400, /* 184) Sine 0.718750 = 0.658444 */																										
	0x2a540a00, /* 185) Sine 0.722656 = 0.661379 */																										
	0x2a83f580, /* 186) Sine 0.726563 = 0.664304 */																										
	0x2ab3b6c0, /* 187) Sine 0.730469 = 0.667219 */																										
	0x2ae34d40, /* 188) Sine 0.734375 = 0.670123 */																										
	0x2b12b8c0, /* 189) Sine 0.738281 = 0.673018 */																										
	0x2b41f940, /* 190) Sine 0.742188 = 0.675902 */																										
	0x2b710e80, /* 191) Sine 0.746094 = 0.678775 */																										
	0x2b9ff840, /* 192) Sine 0.750000 = 0.681639 */																										
	0x2bceb640, /* 193) Sine 0.753906 = 0.684492 */																										
	0x2bfd48c0, /* 194) Sine 0.757813 = 0.687334 */																										
	0x2c2baf00, /* 195) Sine 0.761719 = 0.690166 */																										
	0x2c59e940, /* 196) Sine 0.765625 = 0.692988 */																										
	0x2c87f700, /* 197) Sine 0.769531 = 0.695799 */																										
	0x2cb5d840, /* 198) Sine 0.773438 = 0.698599 */																										
	0x2ce38d00, /* 199) Sine 0.777344 = 0.701389 */																										
	0x2d111480, /* 200) Sine 0.781250 = 0.704167 */																										
	0x2d3e6f40, /* 201) Sine 0.785156 = 0.706936 */																										
	0x2d6b9cc0, /* 202) Sine 0.789063 = 0.709693 */																										
	0x2d989c80, /* 203) Sine 0.792969 = 0.712440 */																										
	0x2dc56f00, /* 204) Sine 0.796875 = 0.715175 */																										
	0x2df21380, /* 205) Sine 0.800781 = 0.717900 */																										
	0x2e1e8a40, /* 206) Sine 0.804688 = 0.720614 */																										
	0x2e4ad2c0, /* 207) Sine 0.808594 = 0.723317 */																										
	0x2e76ed00, /* 208) Sine 0.812500 = 0.726009 */																										
	0x2ea2d8c0, /* 209) Sine 0.816406 = 0.728689 */																										
	0x2ece9600, /* 210) Sine 0.820313 = 0.731359 */																										
	0x2efa2440, /* 211) Sine 0.824219 = 0.734017 */																										
	0x2f2583c0, /* 212) Sine 0.828125 = 0.736665 */																										
	0x2f50b3c0, /* 213) Sine 0.832031 = 0.739301 */																										
	0x2f7bb4c0, /* 214) Sine 0.835938 = 0.741925 */																										
	0x2fa68640, /* 215) Sine 0.839844 = 0.744539 */																										
	0x2fd12800, /* 216) Sine 0.843750 = 0.747141 */																										
	0x2ffb99c0, /* 217) Sine 0.847656 = 0.749731 */																										
	0x3025dbc0, /* 218) Sine 0.851563 = 0.752311 */																										
	0x304fedc0, /* 219) Sine 0.855469 = 0.754878 */																										
	0x3079cf40, /* 220) Sine 0.859375 = 0.757435 */																										
	0x30a38040, /* 221) Sine 0.863281 = 0.759979 */																										
	0x30cd0080, /* 222) Sine 0.867188 = 0.762512 */																										
	0x30f65040, /* 223) Sine 0.871094 = 0.765034 */																										
	0x311f6ec0, /* 224) Sine 0.875000 = 0.767543 */																										
	0x31485c40, /* 225) Sine 0.878906 = 0.770042 */																										
	0x31711880, /* 226) Sine 0.882813 = 0.772528 */																										
	0x3199a340, /* 227) Sine 0.886719 = 0.775002 */																										
	0x31c1fc80, /* 228) Sine 0.890625 = 0.777465 */																										
	0x31ea2400, /* 229) Sine 0.894531 = 0.779916 */																										
	0x32121980, /* 230) Sine 0.898438 = 0.782355 */																										
	0x3239dcc0, /* 231) Sine 0.902344 = 0.784782 */																										
	0x32616e00, /* 232) Sine 0.906250 = 0.787197 */																										
	0x3288cd00, /* 233) Sine 0.910156 = 0.789600 */																										
	0x32aff940, /* 234) Sine 0.914063 = 0.791991 */																										
	0x32d6f2c0, /* 235) Sine 0.917969 = 0.794369 */																										
	0x32fdb980, /* 236) Sine 0.921875 = 0.796736 */																										
	0x33244d40, /* 237) Sine 0.925781 = 0.799091 */																										
	0x334aae00, /* 238) Sine 0.929688 = 0.801433 */																										
	0x3370db40, /* 239) Sine 0.933594 = 0.803763 */																										
	0x3396d540, /* 240) Sine 0.937500 = 0.806081 */																										
	0x33bc9b80, /* 241) Sine 0.941406 = 0.808387 */																										
	0x33e22e00, /* 242) Sine 0.945313 = 0.810680 */																										
	0x34078cc0, /* 243) Sine 0.949219 = 0.812961 */																										
	0x342cb780, /* 244) Sine 0.953125 = 0.815229 */																										
	0x3451ae00, /* 245) Sine 0.957031 = 0.817485 */																										
	0x34767040, /* 246) Sine 0.960938 = 0.819729 */																										
	0x349afdc0, /* 247) Sine 0.964844 = 0.821960 */																										
	0x34bf5700, /* 248) Sine 0.968750 = 0.824178 */																										
	0x34e37b40, /* 249) Sine 0.972656 = 0.826384 */																										
	0x35076ac0, /* 250) Sine 0.976563 = 0.828578 */																										
	0x352b2540, /* 251) Sine 0.980469 = 0.830758 */																										
	0x354eaa80, /* 252) Sine 0.984375 = 0.832926 */																										
	0x3571fa80, /* 253) Sine 0.988281 = 0.835082 */																										
	0x35951500, /* 254) Sine 0.992188 = 0.837224 */																										
	0x35b7fa00, /* 255) Sine 0.996094 = 0.839354 */																										
	0x35daa900, /* 256) Sine 1.000000 = 0.841471 */																										
	0x35fd2280, /* 257) Sine 1.003906 = 0.843575 */																										
	0x361f65c0, /* 258) Sine 1.007813 = 0.845666 */																										
	0x36417300, /* 259) Sine 1.011719 = 0.847745 */																										
	0x36634a00, /* 260) Sine 1.015625 = 0.849810 */																										
	0x3684eac0, /* 261) Sine 1.019531 = 0.851863 */																										
	0x36a654c0, /* 262) Sine 1.023438 = 0.853902 */																										
	0x36c78840, /* 263) Sine 1.027344 = 0.855928 */																										
	0x36e88500, /* 264) Sine 1.031250 = 0.857942 */																										
	0x37094ac0, /* 265) Sine 1.035156 = 0.859942 */																										
	0x3729d940, /* 266) Sine 1.039063 = 0.861929 */																										
	0x374a3100, /* 267) Sine 1.042969 = 0.863903 */																										
	0x376a5140, /* 268) Sine 1.046875 = 0.865864 */																										
	0x378a3a00, /* 269) Sine 1.050781 = 0.867812 */																										
	0x37a9eb40, /* 270) Sine 1.054688 = 0.869746 */																										
	0x37c96500, /* 271) Sine 1.058594 = 0.871667 */																										
	0x37e8a6c0, /* 272) Sine 1.062500 = 0.873575 */																										
	0x3807b0c0, /* 273) Sine 1.066406 = 0.875469 */																										
	0x382682c0, /* 274) Sine 1.070313 = 0.877351 */																										
	0x38451c80, /* 275) Sine 1.074219 = 0.879218 */																										
	0x38637e00, /* 276) Sine 1.078125 = 0.881073 */																										
	0x3881a700, /* 277) Sine 1.082031 = 0.882913 */																										
	0x389f97c0, /* 278) Sine 1.085938 = 0.884741 */																										
	0x38bd4fc0, /* 279) Sine 1.089844 = 0.886555 */																										
	0x38dacf00, /* 280) Sine 1.093750 = 0.888355 */																										
	0x38f81540, /* 281) Sine 1.097656 = 0.890142 */																										
	0x391522c0, /* 282) Sine 1.101563 = 0.891915 */																										
	0x3931f700, /* 283) Sine 1.105469 = 0.893675 */																										
	0x394e9240, /* 284) Sine 1.109375 = 0.895421 */																										
	0x396af400, /* 285) Sine 1.113281 = 0.897153 */																										
	0x39871c80, /* 286) Sine 1.117188 = 0.898872 */																										
	0x39a30b40, /* 287) Sine 1.121094 = 0.900576 */																										
	0x39bec080, /* 288) Sine 1.125000 = 0.902268 */																										
	0x39da3c00, /* 289) Sine 1.128906 = 0.903945 */																										
	0x39f57dc0, /* 290) Sine 1.132813 = 0.905609 */																										
	0x3a108580, /* 291) Sine 1.136719 = 0.907258 */																										
	0x3a2b5300, /* 292) Sine 1.140625 = 0.908894 */																										
	0x3a45e680, /* 293) Sine 1.144531 = 0.910516 */																										
	0x3a603fc0, /* 294) Sine 1.148438 = 0.912125 */																										
	0x3a7a5e80, /* 295) Sine 1.152344 = 0.913719 */																										
	0x3a9442c0, /* 296) Sine 1.156250 = 0.915299 */																										
	0x3aadec80, /* 297) Sine 1.160156 = 0.916865 */																										
	0x3ac75bc0, /* 298) Sine 1.164063 = 0.918418 */																										
	0x3ae09000, /* 299) Sine 1.167969 = 0.919956 */																										
	0x3af98980, /* 300) Sine 1.171875 = 0.921481 */																										
	0x3b1247c0, /* 301) Sine 1.175781 = 0.922991 */																										
	0x3b2acb40, /* 302) Sine 1.179688 = 0.924487 */																										
	0x3b431380, /* 303) Sine 1.183594 = 0.925969 */																										
	0x3b5b2040, /* 304) Sine 1.187500 = 0.927437 */																										
	0x3b72f200, /* 305) Sine 1.191406 = 0.928891 */																										
	0x3b8a8800, /* 306) Sine 1.195313 = 0.930330 */																										
	0x3ba1e2c0, /* 307) Sine 1.199219 = 0.931756 */																										
	0x3bb901c0, /* 308) Sine 1.203125 = 0.933167 */																										
	0x3bcfe500, /* 309) Sine 1.207031 = 0.934564 */																										
	0x3be68c40, /* 310) Sine 1.210938 = 0.935947 */																										
	0x3bfcf7c0, /* 311) Sine 1.214844 = 0.937315 */																										
	0x3c132780, /* 312) Sine 1.218750 = 0.938669 */																										
	0x3c291b00, /* 313) Sine 1.222656 = 0.940009 */																										
	0x3c3ed240, /* 314) Sine 1.226563 = 0.941334 */																										
	0x3c544d40, /* 315) Sine 1.230469 = 0.942645 */																										
	0x3c698c00, /* 316) Sine 1.234375 = 0.943942 */																										
	0x3c7e8e40, /* 317) Sine 1.238281 = 0.945224 */																										
	0x3c935440, /* 318) Sine 1.242188 = 0.946492 */																										
	0x3ca7dd80, /* 319) Sine 1.246094 = 0.947746 */																										
	0x3cbc2a00, /* 320) Sine 1.250000 = 0.948985 */																										
	0x3cd039c0, /* 321) Sine 1.253906 = 0.950209 */																										
	0x3ce40d00, /* 322) Sine 1.257813 = 0.951419 */																										
	0x3cf7a300, /* 323) Sine 1.261719 = 0.952615 */																										
	0x3d0afc40, /* 324) Sine 1.265625 = 0.953795 */																										
	0x3d1e1880, /* 325) Sine 1.269531 = 0.954962 */																										
	0x3d30f780, /* 326) Sine 1.273438 = 0.956114 */																										
	0x3d439940, /* 327) Sine 1.277344 = 0.957251 */																										
	0x3d55fdc0, /* 328) Sine 1.281250 = 0.958373 */																										
	0x3d682500, /* 329) Sine 1.285156 = 0.959481 */																										
	0x3d7a0f00, /* 330) Sine 1.289063 = 0.960575 */																										
	0x3d8bbb40, /* 331) Sine 1.292969 = 0.961654 */																										
	0x3d9d2a00, /* 332) Sine 1.296875 = 0.962718 */																										
	0x3dae5b40, /* 333) Sine 1.300781 = 0.963767 */																										
	0x3dbf4ec0, /* 334) Sine 1.304688 = 0.964801 */																										
	0x3dd00480, /* 335) Sine 1.308594 = 0.965821 */																										
	0x3de07c80, /* 336) Sine 1.312500 = 0.966827 */																										
	0x3df0b680, /* 337) Sine 1.316406 = 0.967817 */																										
	0x3e00b2c0, /* 338) Sine 1.320313 = 0.968793 */																										
	0x3e1070c0, /* 339) Sine 1.324219 = 0.969753 */																										
	0x3e1ff100, /* 340) Sine 1.328125 = 0.970700 */																										
	0x3e2f32c0, /* 341) Sine 1.332031 = 0.971631 */																										
	0x3e3e3680, /* 342) Sine 1.335938 = 0.972547 */																										
	0x3e4cfc40, /* 343) Sine 1.339844 = 0.973449 */																										
	0x3e5b8340, /* 344) Sine 1.343750 = 0.974335 */																										
	0x3e69cc40, /* 345) Sine 1.347656 = 0.975207 */																										
	0x3e77d6c0, /* 346) Sine 1.351563 = 0.976064 */																										
	0x3e85a2c0, /* 347) Sine 1.355469 = 0.976906 */																										
	0x3e933000, /* 348) Sine 1.359375 = 0.977734 */																										
	0x3ea07f00, /* 349) Sine 1.363281 = 0.978546 */																										
	0x3ead8f40, /* 350) Sine 1.367188 = 0.979343 */																										
	0x3eba60c0, /* 351) Sine 1.371094 = 0.980126 */																										
	0x3ec6f3c0, /* 352) Sine 1.375000 = 0.980893 */																										
	0x3ed347c0, /* 353) Sine 1.378906 = 0.981646 */																										
	0x3edf5d00, /* 354) Sine 1.382813 = 0.982383 */																										
	0x3eeb3340, /* 355) Sine 1.386719 = 0.983105 */																										
	0x3ef6cac0, /* 356) Sine 1.390625 = 0.983813 */																										
	0x3f022340, /* 357) Sine 1.394531 = 0.984505 */																										
	0x3f0d3cc0, /* 358) Sine 1.398438 = 0.985183 */																										
	0x3f181740, /* 359) Sine 1.402344 = 0.985845 */																										
	0x3f22b280, /* 360) Sine 1.406250 = 0.986493 */																										
	0x3f2d0ec0, /* 361) Sine 1.410156 = 0.987125 */																										
	0x3f372bc0, /* 362) Sine 1.414063 = 0.987742 */																										
	0x3f410980, /* 363) Sine 1.417969 = 0.988345 */																										
	0x3f4aa840, /* 364) Sine 1.421875 = 0.988932 */																										
	0x3f540780, /* 365) Sine 1.425781 = 0.989504 */																										
	0x3f5d2780, /* 366) Sine 1.429688 = 0.990061 */																										
	0x3f660800, /* 367) Sine 1.433594 = 0.990602 */																										
	0x3f6ea940, /* 368) Sine 1.437500 = 0.991129 */																										
	0x3f770ac0, /* 369) Sine 1.441406 = 0.991641 */																										
	0x3f7f2d40, /* 370) Sine 1.445313 = 0.992137 */																										
	0x3f871000, /* 371) Sine 1.449219 = 0.992619 */																										
	0x3f8eb340, /* 372) Sine 1.453125 = 0.993085 */																										
	0x3f961700, /* 373) Sine 1.457031 = 0.993536 */																										
	0x3f9d3b00, /* 374) Sine 1.460938 = 0.993972 */																										
	0x3fa41f80, /* 375) Sine 1.464844 = 0.994392 */																										
	0x3faac440, /* 376) Sine 1.468750 = 0.994798 */																										
	0x3fb12980, /* 377) Sine 1.472656 = 0.995188 */																										
	0x3fb74f00, /* 378) Sine 1.476563 = 0.995563 */																										
	0x3fbd34c0, /* 379) Sine 1.480469 = 0.995923 */																										
	0x3fc2dac0, /* 380) Sine 1.484375 = 0.996268 */																										
	0x3fc84140, /* 381) Sine 1.488281 = 0.996598 */																										
	0x3fcd67c0, /* 382) Sine 1.492188 = 0.996912 */																										
	0x3fd24e40, /* 383) Sine 1.496094 = 0.997211 */																										
	0x3fd6f540, /* 384) Sine 1.500000 = 0.997495 */																										
	0x3fdb5c40, /* 385) Sine 1.503906 = 0.997764 */																										
	0x3fdf8380, /* 386) Sine 1.507813 = 0.998017 */																										
	0x3fe36ac0, /* 387) Sine 1.511719 = 0.998255 */																										
	0x3fe71240, /* 388) Sine 1.515625 = 0.998478 */																										
	0x3fea79c0, /* 389) Sine 1.519531 = 0.998686 */																										
	0x3feda140, /* 390) Sine 1.523438 = 0.998879 */																										
	0x3ff08900, /* 391) Sine 1.527344 = 0.999056 */																										
	0x3ff330c0, /* 392) Sine 1.531250 = 0.999218 */																										
	0x3ff59880, /* 393) Sine 1.535156 = 0.999365 */																										
	0x3ff7c040, /* 394) Sine 1.539063 = 0.999497 */																										
	0x3ff9a800, /* 395) Sine 1.542969 = 0.999613 */																										
	0x3ffb5000, /* 396) Sine 1.546875 = 0.999714 */																										
	0x3ffcb800, /* 397) Sine 1.550781 = 0.999800 */																										
	0x3ffddfc0, /* 398) Sine 1.554688 = 0.999870 */																										
	0x3ffec7c0, /* 399) Sine 1.558594 = 0.999926 */																										
	0x3fff6fc0, /* 400) Sine 1.562500 = 0.999966 */																										
	0x3fffd780, /* 401) Sine 1.566406 = 0.999990 */																										
	0x3fffff80, /* 402) Sine 1.570313 = 1.000000 */																										
	0x40000000  /* 403) Final entry hard coded to 1.000000 */																										
};																					

#if defined (__cplusplus)
}
#endif

#endif	/* __FIXEDPOINTSINETABLE_H__ */

