/*!
******************************************************************************
 @file   : img_defs.h

 @brief

 @Author Various

 @date   03/06/2003

         <b>Copyright 2003 by Imagination Technologies Limited.</b>\n
         All rights reserved.  No part of this software, either
         material or conceptual may be copied or distributed,
         transmitted, transcribed, stored in a retrieval system
         or translated into any human or computer language in any
         form by any means, electronic, mechanical, manual or
         other-wise, or disclosed to third parties without the
         express written permission of Imagination Technologies
         Limited, Unit 8, HomePark Industrial Estate,
         King's Langley, Hertfordshire, WD4 8LZ, U.K.

 <b>Description:</b>\n
         This header file contains the Imagination Technologies platform
		 specific defintions.

 <b>Platform:</b>\n
	     Linux

 @Version
	     1.0

******************************************************************************/
/*
******************************************************************************
*/

#include "img_error.h"

#if !defined (__IMG_DEFS_H__)
#define __IMG_DEFS_H__


//#include <stddef.h>

  #include <linux/kernel.h>
  #include <linux/slab.h>
  #include "img_types.h"

#if defined (__cplusplus)
extern "C" {
#endif

typedef signed int		img_int32,	* img_pint32;
typedef int				img_result,	* img_presult;
typedef unsigned int	img_uint32, * img_puint32;
typedef void			* img_pvoid;
typedef void*			img_handle,	* img_phandle;
typedef void**			img_hvoid,	* img_phvoid;
typedef int				img_bool,	* img_pbool;
typedef unsigned char	img_uint8,	* img_puint8;
typedef float			img_float,	* img_pfloat;
typedef char			img_int8,	* img_pint8;
typedef char			img_char,	* img_pchar;
typedef unsigned char	img_byte,	* img_pbyte;
typedef unsigned short	img_uint16, * img_puint16;
typedef short			img_int16,  * img_pint16;
typedef unsigned long long img_uint64, * img_puint64;
typedef long long          img_int64,  * img_pint64;

/* Typedefs of void are synonymous with the void keyword in C, 
 * but not in C++. In order to support the use of img_void
 * in place of the void keyword to specify that a function takes no 
 * arguments, it must be a macro rather than a typedef.
 */
#define img_void void

#if 0//!defined(IMG_KERNEL_MODULE)
typedef uintptr_t		img_uintptr_t;
#endif

typedef		enum	img_tag_TriStateSwitch
{
	IMG_ON		=	0x00,
	IMG_OFF,
	IMG_IGNORE

} img_TriStateSwitch, * img_pTriStateSwitch;

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
#define	IMG_NULL 0
#define IMG_TRUE 1
#define IMG_FALSE 0


/* Linux sometimes already has endian macros defined in <endian.h> */
#if defined __BYTE_ORDER
#define IMG_LITTLE_ENDIAN	(__LITTLE_ENDIAN)
#define IMG_BIG_ENDIAN		(__BIG_ENDIAN)
#define	IMG_BYTE_ORDER		(__BYTE_ORDER)
#elif defined BYTE_ORDER
#define IMG_LITTLE_ENDIAN	(LITTLE_ENDIAN)
#define IMG_BIG_ENDIAN		(BIG_ENDIAN)
#define	IMG_BYTE_ORDER		(BYTE_ORDER)
#else
/* Default to litle endian */
#define IMG_LITTLE_ENDIAN	(1234)
#define IMG_BIG_ENDIAN		(4321)
#define IMG_BYTE_ORDER		(IMG_LITTLE_ENDIAN)
#endif

/* Maths functions - platform dependant						*/
#define		IMG_COS					cos
#define		IMG_SIN					sin
#define		IMG_POW(A, B)			pow		(A,B)
#define		IMG_SQRT(A)				sqrt	(A)
#define		IMG_FABS(A)				fabs	(A)
#define		IMG_EXP(A)				exp		(A)

/* General functions										*/
#if defined (CROSS_COPY) && defined (IMG_KERNEL_MODULE)
#include <asm/uaccess.h>
#define IMG_MEMCPY(A, B, C)   copy_to_user(A, B, C)
#else
#define	IMG_MEMCPY(A,B,C)	memcpy	(A,B,C)
#endif
#define	IMG_MEMSET(A,B,C)	memset	(A,B,C)
#define IMG_MEMCMP(A,B,C)	memcmp	(A,B,C)
#define IMG_MEMMOVE(A,B,C)	memmove	(A,B,C)

/* 64bit value prefix in printf - platform dependant		*/
#define IMG_INT64PRFX "ll"

/* Take a 64 bit unsigned int, check it is suitable and cast it as 32bit unsigned int */
#define IMG_UINT64_TO_UINT32(ui64Check) (IMG_ASSERT(((ui64Check) >> 32) == 0), (IMG_UINT32)(ui64Check))
/* Needed for historical reasons */
#define IMG_UINT64_TO_UNIT32 IMG_UINT64_TO_UINT32

/* Large File Seek and Tell functions */
/* These are non-POSIX standard API interfaces to support large files */
#ifndef IMG_FTELL64
	#define IMG_FTELL64 ftello64
#endif
#ifndef IMG_FSEEK64
	#define IMG_FSEEK64 fseeko64
#endif

#if defined (IMG_KERNEL_MODULE)

#if defined (IMG_CHECK_MEMORY_LEAKS)
extern IMG_ATOMIC	gsStrDupCnt;

#define IMG_STRDUP(A)		({					\
			IMG_VOID *ptr;						\
			ptr = kstrdup(A, GFP_ATOMIC);		\
			atomic_inc(&gsStrDupCnt);		\
			ptr;								\
		})

#else
#define IMG_STRDUP(A)		kstrdup(A, GFP_ATOMIC)
#endif

#else
#define IMG_STRDUP(A)		strdup(A)
#endif
#define IMG_STRCMP(A,B)		strcmp(A,B)
#define IMG_STRCPY(A,B)		strcpy(A,B)
#define IMG_STRLEN(A)		strlen(A)

/*#define IMG_ASSERT(A)		assert	(A)*/

#if 0 //!defined(IMG_KERNEL_MODULE)
	#ifndef EXIT_ON_ASSERT
		#if defined __RELEASE_RELEASE__ || defined NO_ASSERT
			#define IMG_ASSERT(A) (void)0
		#else  /* (__RELEASE_DEBUG__ or __RELEASE_TEST__) and not NO_ASSERT	*/
			#define IMG_ASSERT(A)		assert	(A)
			#define IMG_ASSERT_LIVE  /* Defined when IMG_ASSERT() does something.	*/
		#endif
	#else	/* EXIT_ON_ASSERT	*/
		#define IMG_ASSERT(exp) (void)( (exp) || (fprintf(stderr, "ERROR: Assert %s; File %s; Line %d\n", #exp, __FILE__, __LINE__), fflush(IMG_NULL), exit(-1), 0) )
		#define IMG_ASSERT_LIVE  /* Defined when IMG_ASSERT() does something.	*/
	#endif
#else	/*	IMG_KERNEL_MODULE	*/
	#ifndef EXIT_ON_ASSERT
		#if defined __RELEASE_RELEASE__ || defined NO_ASSERT
			#define IMG_ASSERT(A) (void)0
		#else  /* (__RELEASE_DEBUG__ or __RELEASE_TEST__) and not NO_ASSERT	*/
			#ifdef PIPE_ASSERT
				#define IMG_ASSERT(exp) ({												\
				extern IMG_CHAR pszAssertString[];							\
				extern IMG_RESULT LOG_WriteBinaryData(IMG_UINT32, IMG_UINT32, IMG_UINT32, IMG_UINT32, IMG_UINT32); \
				extern IMG_RESULT LOG_WriteString(IMG_CHAR *);				\
				(void)( (exp) || (IMG_MEMSET(pszAssertString, 0, 256)));	\
				(void)( (exp) || (sprintf(pszAssertString, "ERROR: Assert %s; File %s; Line %d\n", #exp, __FILE__, __LINE__))); \
				(void)( (exp) || (LOG_WriteBinaryData(0xFFFFFFFF, 0, 0, 0, 0))); \
				(void)( (exp) || (LOG_WriteString(pszAssertString)));		\
				(void)( (exp) || (printk(KERN_ERR "ERROR: Assert %s; File %s; Line %d\n", #exp, __FILE__, __LINE__))); \
			})
			#else
				#define IMG_ASSERT(exp) (void)( (exp) || (printk(KERN_ERR "ERROR: Assert %s; File %s; Line %d\n", #exp, __FILE__, __LINE__)) )
			#endif /* PIPE_ASSERT  */
	
		#define IMG_ASSERT_LIVE  /* Defined when IMG_ASSERT() does something.	*/
		#endif
	#else	/* EXIT_ON_ASSERT	*/
		#define IMG_ASSERT(exp) (void)( (exp) || (printk(KERN_ERR "ERROR: Assert %s; File %s; Line %d\n", #exp, __FILE__, __LINE__)) )
		#define IMG_ASSERT_LIVE  /* Defined when IMG_ASSERT() does something.	*/
	#endif
#endif
#define IMG_ASSERT_LIVE  /* Defined when IMG_ASSERT() does something.	*/

/* Takes any two signed values - return 'IMG_TRUE' if they have the same polarity */
#define	IMG_SAMESIGN(A,B)	(((((img_int32)A)^((img_int32)B))&0x80000000)==0)

/*
	Wide character definitions
*/
#if defined(UNICODE)
typedef unsigned short		TCHAR, *PTCHAR, *PTSTR;
#else	/* #if defined(UNICODE) */
typedef char				TCHAR, *PTCHAR, *PTSTR;
#endif	/* #if defined(UNICODE) */

#if defined (NO_INLINE_FUNCS)
	#define	INLINE
	#define	FORCE_INLINE
#else
#if defined(_UITRON_)
	#define	INLINE
	#define	FORCE_INLINE
	#define INLINE_IS_PRAGMA
#else
#if defined (__cplusplus)
	#define INLINE					inline
	#define	FORCE_INLINE			inline
#else
	#define	INLINE					__inline
#if defined(UNDER_CE) || defined(UNDER_XP) || defined(UNDER_VISTA)
	#define	FORCE_INLINE			__forceinline
#else
	#define	FORCE_INLINE			static __inline
#endif
#endif
#endif
#endif

/* Use this in any file, or use attributes under GCC - see below */
#ifndef PVR_UNREFERENCED_PARAMETER
#define	PVR_UNREFERENCED_PARAMETER(param) (param) = (param)
#endif

/* The best way to supress unused parameter warnings using GCC is to use a
 * variable attribute.  Place the unref__ between the type and name of an
 * unused parameter in a function parameter list, eg `int unref__ var'. This
 * should only be used in GCC build environments, for example, in files that
 * compile only on Linux. Other files should use UNREFERENCED_PARAMETER */
#ifdef __GNUC__
#define unref__ __attribute__ ((unused))
#else
#define unref__
#endif

#define IMG_CALLCONV
#define IMG_INTERNAL	__attribute__ ((visibility ("hidden")))
#define IMG_EXPORT
#define IMG_IMPORT

#if 0//!defined(IMG_KERNEL_MODULE)
#define IMG_MALLOC(A)       malloc(A)
#define IMG_CALLOC(A, B)    calloc(A, B)
#define IMG_REALLOC(A, B)   realloc(A, B)
#define IMG_FREE(A)         free(A)
#define IMG_MEMALIGN(A) memalign(getpagesize(), A)
#define IMG_ALIGN_FREE(A)		   free(A)
#else

#if defined (IMG_CHECK_MEMORY_LEAKS)
extern IMG_ATOMIC gsMallocCnt;
extern IMG_ATOMIC gsFreeCnt;

#define IMG_MALLOC(A) ({						\
		IMG_VOID *ptr;							\
		ptr = kmalloc(A, GFP_ATOMIC);			\
		atomic_inc(&gsMallocCnt);			\
		ptr;									\
		})
	
#define IMG_FREE(A)	 ({							\
		kfree(A);								\
		atomic_inc(&gsFreeCnt);				\
		})

#else
#define IMG_MALLOC(A)		kmalloc	(A, GFP_ATOMIC)
#define IMG_FREE(A)			kfree	(A)
#endif

#endif

#define IMG_ALIGN(byte)       __attribute__ ((aligned (byte)))
#define IMG_NORETURN          __attribute__ ((noreturn))

#define IMG_NORETURN		__attribute__ ((noreturn))


#if defined (__cplusplus)
	#define INLINE					inline
#else
#if !defined(INLINE)
	#define	INLINE					__inline
#endif
#endif

#define __TBILogF	consolePrintf

#if defined (__cplusplus)
}
#endif

#if !defined (__IMG_TYPES_H__)
#include "img_types.h"
#endif

#endif /* IMG_DEFS_H	*/
