#include <linux/kernel.h>
#include <linux/debugfs.h>
#include "vpp_csc.h"
#include "vpp_io.h"
const struct vpp_csc_coefs vpp_csc_presets[] = {
	[VPP_CSCPRESET_HDTV] = {
		.ry = 298,	.rv = 459,	.ru = 0,
		.gy = 298,	.gv = -137,	.gu = -55,
		.by = 298,	.bv = 0,	.bu = 541,
	},
	[VPP_CSCPRESET_SDTV] = {
		.ry = 298,	.rv = 409,	.ru = 0,
		.gy = 298,	.gv = -208,	.gu = -100,
		.by = 298,	.bv = 0,	.bu = 517,
	},
	[VPP_CSCPRESET_LEGACYHDTV] = {
		.ry = 298,	.rv = 459,	.ru = 0,
		.gy = 298,	.gv = -139,	.gu = -66,
		.by = 298,	.bv = 0,	.bu = 532,
	},
	[VPP_CSCPRESET_LEGACYSDTV] = {
		.ry = 298,	.rv = 409,	.ru = 0,
		.gy = 298,	.gv = -207,	.gu = -97,
		.by = 298,	.bv = 0,	.bu = 519,
	},
};

static void vpp_calc_procamp_coefs(const struct vpp_procamp_hsbc *hsbc,
								const u32 output_range,
								struct vpp_procamp_coefs *coefs,
								struct vpp_procamp_offsets *input_offsets,
								struct vpp_procamp_offsets *output_offsets);
#define DIVL(a, b)	fix_divl(VPP_PROCAMP_HSBC_FRACTIONAL_BITS, a, b)
#define MULL(a, b)	fix_mull(VPP_PROCAMP_HSBC_FRACTIONAL_BITS, a, b)
#define SIN(a)		fix_sin(VPP_PROCAMP_HSBC_FRACTIONAL_BITS, a)
#define COS(a)		fix_cos(VPP_PROCAMP_HSBC_FRACTIONAL_BITS, a)
#define FIX(a)		fix(VPP_PROCAMP_HSBC_FRACTIONAL_BITS, a)
static void vpp_fix_procamp_matrix_mul(const int matrix_a[3][3],
							const int matrix_b[3][3], int result_matrix[3][3])
{
	int i;
	int j;
	int k;
	int sum;
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {
			sum = 0;
			for (k = 0; k < 3; k++)
				sum += MULL(matrix_a[i][k], matrix_b[k][j]);
			result_matrix[i][j] = sum;
		}
	}
}


void vpp_csc_copy_coeffs(struct vpp_dev *idev, s32 *p_s32){
	
	idev->csc_config.coefs.ry = p_s32[0];	
	idev->csc_config.coefs.rv = p_s32[1];	
	idev->csc_config.coefs.ru = p_s32[2];
	idev->csc_config.coefs.gy = p_s32[3];	
	idev->csc_config.coefs.gv = p_s32[4];	
	idev->csc_config.coefs.gu = p_s32[5];
	idev->csc_config.coefs.by = p_s32[6];	
	idev->csc_config.coefs.bv = p_s32[7];	
	idev->csc_config.coefs.bu = p_s32[8];
	
}
int vpp_set_csc_config(struct vpp_dev *priv, struct vpp_csc_config *csc_config)
{
	if (csc_config->preset) {
		if (csc_config->preset < ARRAY_SIZE(vpp_csc_presets))
			csc_config->coefs = vpp_csc_presets[csc_config->preset];
		else
			return -EINVAL;
	} else {
		#define CSC_COEF_INVALID(x) (((x) >= (1 << 10)) || ((x) < (-1 << 10)))
		if (CSC_COEF_INVALID(csc_config->coefs.ry) ||
			CSC_COEF_INVALID(csc_config->coefs.rv) ||
			CSC_COEF_INVALID(csc_config->coefs.ru) ||
			CSC_COEF_INVALID(csc_config->coefs.gy) ||
			CSC_COEF_INVALID(csc_config->coefs.gv) ||
			CSC_COEF_INVALID(csc_config->coefs.gu) ||
			CSC_COEF_INVALID(csc_config->coefs.by) ||
			CSC_COEF_INVALID(csc_config->coefs.bv) ||
			CSC_COEF_INVALID(csc_config->coefs.bu)) {
			return -EINVAL;
		}
		#undef CSC_COEF_INVALID
	}

	priv->csc_config = *csc_config;
#if 0
	vpp_write(priv, VPP_CSCCOEFF0_OFFSET,
				PLACE_FIELD(VPP_CSCCOEFF0_CSCCOEFFRU, csc_config->coefs.ru)
				| PLACE_FIELD(VPP_CSCCOEFF0_CSCCOEFFRY, csc_config->coefs.ry));
	vpp_write(priv, VPP_CSCCOEFF1_OFFSET,
				PLACE_FIELD(VPP_CSCCOEFF1_CSCCOEFFGY, csc_config->coefs.gy)
				| PLACE_FIELD(VPP_CSCCOEFF1_CSCCOEFFRV, csc_config->coefs.rv));
	vpp_write(priv, VPP_CSCCOEFF2_OFFSET,
				PLACE_FIELD(VPP_CSCCOEFF2_CSCCOEFFGV, csc_config->coefs.gv)
				| PLACE_FIELD(VPP_CSCCOEFF2_CSCCOEFFGU, csc_config->coefs.gu));
	vpp_write(priv, VPP_CSCCOEFF3_OFFSET,
				PLACE_FIELD(VPP_CSCCOEFF3_CSCCOEFFBU, csc_config->coefs.bu)
				| PLACE_FIELD(VPP_CSCCOEFF3_CSCCOEFFBY, csc_config->coefs.by));
	vpp_write(priv, VPP_CSCCOEFF4_OFFSET,
				PLACE_FIELD(VPP_CSCCOEFF4_CSCCOEFFBV, csc_config->coefs.bv));

	vpp_flush_regs(priv);
#endif
	return 0;
}
void pdpfb_get_csc_config(struct vpp_dev *priv,
								struct vpp_csc_config *csc_config)
{
	*csc_config = priv->csc_config;
}
void vpp_get_procamp(struct vpp_dev *priv, struct vpp_procamp *procamp)
{
	*procamp = priv->procamp;
}

#define PROCAMP_COEF_INVALID(x) \
	(((x) >= (1 << VPP_CR_VPP_PROCAMP_C11C12_CR_PROCAMP_C11_LENGTH)) \
	|| ((x) < (-1 << VPP_CR_VPP_PROCAMP_C11C12_CR_PROCAMP_C11_LENGTH)))

#define PROCAMP_INPUT_OFFSET_INVALID(x) \
	(((x) >= (1 << VPP_CR_VPP_PROCAMP_INOFFSET_R_CR_PROCAMP_INOFF_R_LENGTH)) \
	|| ((x) < (-1 << VPP_CR_VPP_PROCAMP_INOFFSET_R_CR_PROCAMP_INOFF_R_LENGTH)))

#define OUTOFF_LENGTH VPP_CR_VPP_PROCAMP_OUTOFFSET_R_CR_PROCAMP_OUTOFF_R_LENGTH
#define PROCAMP_OUTPUT_OFFSET_INVALID(x) \
	(((x) >= (1 << OUTOFF_LENGTH)) || ((x) < (-1 << OUTOFF_LENGTH)))

#define PROCAMP_HSBC_INVALID(hsbc, min, max) \
	(((hsbc) > ((max) << VPP_PROCAMP_HSBC_FRACTIONAL_BITS)) \
	|| ((hsbc) < ((min) << VPP_PROCAMP_HSBC_FRACTIONAL_BITS)))

int vpp_set_procamp(struct vpp_dev *priv,
							struct vpp_procamp *procamp)
{
	u32 reg;

	if (procamp->enable) {
		if (procamp->output_range > VPP_PROCAMP_OUTPUT_RANGE_192_834)
			return -EINVAL;

		if (procamp->custom) {
			if (PROCAMP_COEF_INVALID(procamp->coefs.c11) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c12) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c13) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c21) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c22) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c23) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c31) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c32) ||
				PROCAMP_COEF_INVALID(procamp->coefs.c33)) {
				return -EINVAL;
			}
			if (PROCAMP_INPUT_OFFSET_INVALID(procamp->input_offsets.r) ||
				PROCAMP_INPUT_OFFSET_INVALID(procamp->input_offsets.g) ||
				PROCAMP_INPUT_OFFSET_INVALID(procamp->input_offsets.b)) {
				return -EINVAL;
			}
			if (PROCAMP_OUTPUT_OFFSET_INVALID(procamp->output_offsets.r) ||
				PROCAMP_OUTPUT_OFFSET_INVALID(procamp->output_offsets.g) ||
				PROCAMP_OUTPUT_OFFSET_INVALID(procamp->output_offsets.b)) {
				return -EINVAL;
			}
			priv->procamp = *procamp;
		} else {
			if (PROCAMP_HSBC_INVALID(procamp->hsbc.hue, -30, 30) ||
				PROCAMP_HSBC_INVALID(procamp->hsbc.saturation, 0, 2) ||
				PROCAMP_HSBC_INVALID(procamp->hsbc.brightness, -50, 50) ||
				PROCAMP_HSBC_INVALID(procamp->hsbc.contrast, 0, 2)) {
				return -EINVAL;
			}
			/*
			 * Retain the procamp setup for use by vpp_get_procamp before the
			 * coefs are overwritten 
			 */
			priv->procamp = *procamp;
			vpp_calc_procamp_coefs(&procamp->hsbc,
									procamp->output_range,
									&procamp->coefs,
									&procamp->input_offsets,
									&procamp->output_offsets);
		}

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_C11C12_CR_PROCAMP_C11,
							procamp->coefs.c11);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_C11C12_CR_PROCAMP_C12,
							procamp->coefs.c12);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_C11C12_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_C13C21_CR_PROCAMP_C13,
							procamp->coefs.c13);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_C13C21_CR_PROCAMP_C21,
							procamp->coefs.c21);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_C13C21_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_C22C23_CR_PROCAMP_C22,
							procamp->coefs.c22);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_C22C23_CR_PROCAMP_C23,
							procamp->coefs.c23);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_C22C23_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_C31C32_CR_PROCAMP_C31,
							procamp->coefs.c31);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_C31C32_CR_PROCAMP_C32,
							procamp->coefs.c32);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_C31C32_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_C33_CR_PROCAMP_EN, 1);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_C33_CR_PROCAMP_RANGE,
							procamp->output_range);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_C33_CR_PROCAMP_C33,
							procamp->coefs.c33);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_C33_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_INOFFSET_BG_CR_PROCAMP_INOFF_B,
							procamp->input_offsets.b);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_INOFFSET_BG_CR_PROCAMP_INOFF_G,
							procamp->input_offsets.g);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_INOFFSET_BG_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_INOFFSET_R_CR_PROCAMP_INOFF_R,
							procamp->input_offsets.r);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_INOFFSET_R_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_OUTOFFSET_BG_CR_PROCAMP_OUTOFF_B,
							procamp->output_offsets.b);
		SET_FIELD(reg, VPP_CR_VPP_PROCAMP_OUTOFFSET_BG_CR_PROCAMP_OUTOFF_G,
							procamp->output_offsets.g);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_OUTOFFSET_BG_OFFSET, reg);

		reg = PLACE_FIELD(VPP_CR_VPP_PROCAMP_OUTOFFSET_R_CR_PROCAMP_OUTOFF_R,
							procamp->output_offsets.r);
		vpp_write(priv, VPP_CR_VPP_PROCAMP_OUTOFFSET_R_OFFSET, reg);
	} else {
		vpp_write(priv, VPP_CR_VPP_PROCAMP_C33_OFFSET,
					PLACE_FIELD(VPP_CR_VPP_PROCAMP_C33_CR_PROCAMP_EN, 0));
	}

	
	vpp_flush_regs(priv);
	return 0;
}
#undef PROCAMP_COEF_INVALID
#undef PROCAMP_INPUT_OFFSET_INVALID
#undef OUTOFF_LENGTH
#undef PROCAMP_OUTPUT_OFFSET_INVALID
#undef PROCAMP_HSBC_INVALID

static void vpp_calc_procamp_coefs(const struct vpp_procamp_hsbc *hsbc,
								const u32 output_range,
								struct vpp_procamp_coefs *coefs,
								struct vpp_procamp_offsets *input_offsets,
								struct vpp_procamp_offsets *output_offsets)
{
	const int one_degree_in_radians = DIVL(31415927, (180*10000000));
	/* In x.25 format */
	const int rgb_to_yuv[3][3] = {	{0x00839581,  0x01020C4A,  0x00322D0E},
									{-0x004BC6A8, -0x0094FDF4,	0x00E0C49C},
									{0x00E0C49C,  -0x00BC6A7F, -0x00245A1D}	};
	/* In x.25 format */
	const int yuv_to_rgb[3][3] = {	{0x0253F7CF,  0x00000000,  0x033126E9},
									{0x0253F7CF, -0x00C83127, -0x01A04189},
									{0x0253F7CF,  0x0409374C,  0x00000000}	};
	int inverted_scale_factor;
	int output_offset;
	int hue_in_radians;
	int cos_hue;
	int sin_hue;
	int contrast_times_saturation;
	int result_matrix[2][3][3];

	switch (output_range) {
	case 0:
		inverted_scale_factor = FIX(1);
		output_offset = 0;
		break;
	case 1:
		inverted_scale_factor = DIVL(879, 1023);
		output_offset = fix(VPP_PROCAMP_OUTPUT_OFFSET_FRACTIONAL_BITS, 64);
		break;
	case 2:
	default:
		inverted_scale_factor = DIVL(642, 1023);
		output_offset = fix(VPP_PROCAMP_OUTPUT_OFFSET_FRACTIONAL_BITS, 192);
		break;
	}

	/*
	 * Cosine is symmetrical about 0 degrees, hence cos(theta)=cos(-theta)
	 * Sine is not symmetrical, however sin(theta)=-1 * sin(-theta)
	 */
	hue_in_radians = hsbc->hue;
	if (hue_in_radians < 0)
		hue_in_radians = hue_in_radians * -1;

	hue_in_radians = MULL(hue_in_radians, one_degree_in_radians);
	cos_hue = COS(hue_in_radians);
	sin_hue = SIN(hue_in_radians);

	if (hsbc->hue < 0)
		sin_hue = -1 * sin_hue;

	contrast_times_saturation = MULL(hsbc->contrast, hsbc->saturation);

	/*
	 * Create the HSBC matrix:
	 * [ con,		0,						0					]
	 * [ 0,			(con*sat*cos(hue)),		(con*sat*sin(hue))	]
	 * [ 0,			-(con*sat*sin(hue)),	(con*sat*cos(hue))	]
	 */
	result_matrix[0][0][0] = hsbc->contrast;
	result_matrix[0][0][1] = 0;
	result_matrix[0][0][2] = 0;

	result_matrix[0][1][0] = 0;
	result_matrix[0][1][1] = MULL(contrast_times_saturation, cos_hue);
	result_matrix[0][1][2] = MULL(contrast_times_saturation, sin_hue);

	result_matrix[0][2][0] = 0;
	result_matrix[0][2][1] = -1 * result_matrix[0][1][2];
	result_matrix[0][2][2] = result_matrix[0][1][1];

	/* Scale the HSBC matrix, if required */
	if (inverted_scale_factor != FIX(1)) {
		result_matrix[0][0][0] = MULL(result_matrix[0][0][0],
										inverted_scale_factor);
		result_matrix[0][1][1] = MULL(result_matrix[0][1][1],
										inverted_scale_factor);
		result_matrix[0][1][2] = MULL(result_matrix[0][1][2],
										inverted_scale_factor);
		result_matrix[0][2][1] = MULL(result_matrix[0][2][1],
										inverted_scale_factor);
		result_matrix[0][2][2] = MULL(result_matrix[0][2][2],
										inverted_scale_factor);
	}

	/* Multiply the HSBC matrix by the RGB to YUV matrix */
	vpp_fix_procamp_matrix_mul((const int (*)[3])result_matrix[0],
								rgb_to_yuv, result_matrix[1]);

	/* Multiply the result by the YUV to RGB matrix to get final coefs */
	vpp_fix_procamp_matrix_mul(yuv_to_rgb,
								(const int (*)[3])result_matrix[1],
								result_matrix[0]);

	#define HSBC_TO_COEFF(hsbc) \
		((hsbc) >> (VPP_PROCAMP_HSBC_FRACTIONAL_BITS \
			- VPP_PROCAMP_COEFFICIENT_FRACTIONAL_BITS))
	coefs->c11 = HSBC_TO_COEFF(result_matrix[0][0][0]);
	coefs->c12 = HSBC_TO_COEFF(result_matrix[0][0][1]);
	coefs->c13 = HSBC_TO_COEFF(result_matrix[0][0][2]);
	coefs->c21 = HSBC_TO_COEFF(result_matrix[0][1][0]);
	coefs->c22 = HSBC_TO_COEFF(result_matrix[0][1][1]);
	coefs->c23 = HSBC_TO_COEFF(result_matrix[0][1][2]);
	coefs->c31 = HSBC_TO_COEFF(result_matrix[0][2][0]);
	coefs->c32 = HSBC_TO_COEFF(result_matrix[0][2][1]);
	coefs->c33 = HSBC_TO_COEFF(result_matrix[0][2][2]);
	#undef HSBC_TO_COEFF

	input_offsets->r = input_offsets->g = input_offsets->b = 0;
	
	/*
	 * Add the brightness to the output offset, shifted for the correct number
	 * of fractional bits and to convert from 8 to 10 integer bits
	 */
	output_offset += hsbc->brightness >> (VPP_PROCAMP_HSBC_FRACTIONAL_BITS -
						VPP_PROCAMP_OUTPUT_OFFSET_FRACTIONAL_BITS - 2);
	output_offsets->r = output_offsets->g = output_offsets->b = output_offset;
}
#undef DIVL
#undef MULL
#undef SIN
#undef COS
#undef FIX