#ifndef _VPP_CSC_H
#define _VPP_CSC_H
#include <linux/ioctl.h>
#include <linux/types.h>


#define VPP_CSCPRESET_HDTV			0x1
#define VPP_CSCPRESET_SDTV			0x2	/* default */
#define VPP_CSCPRESET_LEGACYHDTV	0x3
#define VPP_CSCPRESET_LEGACYSDTV	0x4
#define VPP_PROCAMP_OUTPUT_RANGE_0_1023		0x0	/* 0 - 1023 */
#define VPP_PROCAMP_OUTPUT_RANGE_64_943		0x1	/* 64 - 943 */
#define VPP_PROCAMP_OUTPUT_RANGE_192_834	0x2	/* 192 - 834 */

#define VPP_PROCAMP_HSBC_FRACTIONAL_BITS	25


struct vpp_dev;
/* YUV->RGB colour space conversion */
struct vpp_csc_coefs {
	__s32 ry, rv, ru;
	__s32 gy, gv, gu;
	__s32 by, bv, bu;
};
struct vpp_csc_config {
	__u32 preset;	/* 0 for custom coefficients */

	struct vpp_csc_coefs coefs;
};


/* ProcAmp */
struct vpp_procamp_hsbc {
	__s32 hue;					/* Between -30 and +30 degrees (specified in x.25 signed fixed point format)	*/
	__s32 saturation;			/* Between 0 and 2 (specified in x.25 signed fixed point format)				*/
	__s32 brightness;			/* Between -50 and 50 (specified in x.25 signed fixed point format)				*/
	__s32 contrast;				/* Between 0 and 2 (specified in x.25 signed fixed point format)				*/
};

struct vpp_procamp_coefs {
	__s32 c11, c12, c13;
	__s32 c21, c22, c23;
	__s32 c31, c32, c33;
};

struct vpp_procamp_offsets {
	__s32 r, g, b;
};

struct vpp_procamp {
	__u32 enable;
	__u32 custom;
	struct vpp_procamp_hsbc hsbc;
	__u32 output_range;
	struct vpp_procamp_coefs coefs;				/* In 4.10 signed fixed point format */
	struct vpp_procamp_offsets input_offsets;		/* In 10.0 signed integer format */
	struct vpp_procamp_offsets output_offsets;	/* In 11.1 signed fixed point format */
};
void vpp_csc_copy_coeffs(struct vpp_dev *idev, s32 *p_s32);
void vpp_get_csc_config(struct vpp_dev *priv, struct vpp_csc_config *csc_config);

int vpp_set_csc_config(struct vpp_dev *priv,
								struct vpp_csc_config *csc_config);
void vpp_get_procamp(struct vpp_dev *priv,
							struct vpp_procamp *procamp);

int vpp_set_procamp(struct vpp_dev *priv,
							struct vpp_procamp *procamp);
static inline int fix(int p, int a)
{
	return a << p;
}

static inline int fix_div(int p, int n, int d)
{
	return (n << p) / d;
}

static inline int fix_divl(int p, int n, int d)
{
	return (int)(((u64)n << p) / d);
}

static inline int fix_mul(int p, int a, int b)
{
	return (a * b) >> p;
}

static inline int fix_mull(int p, int a, int b)
{
	return (int)(((u64)a * (u64)b) >> p);
}

static inline int fix_sin(int p, int x)
{
	/* tailor expansion */
	int pow;
	int sum;

	/* in [-pi,pi] */
	int pi2 = fix_divl(p, 6283185, 1000000);
	int pi = pi2 / 2;
	x = x % pi2;
	if (x > pi)
		x -= pi2;
	else if (x < -pi)
		x += pi2;

	sum = x;

	/* x represents x^2 from now on */
	x = fix_mull(p, x, x);

	/* -x^3/3! */
	pow = fix_mull(p, sum, x);
	sum -= pow / 6;
	/* +x^5/5! */
	pow = fix_mull(p, pow, x);
	sum += pow / 120;
	/* -x^7/7! */
	pow = fix_mull(p, pow, x);
	sum -= pow / 5040;
	/* +x^9/9! */
	pow = fix_mull(p, pow, x);
	sum += pow / 362880;

	return sum;
}

static inline int fix_cos(int p, int x)
{
	/* tailor expansion */
	int pow;
	int sum = fix(p, 1);

	/* in [-pi,pi] */
	int pi2 = fix_divl(p, 6283185, 1000000);
	int pi = pi2 / 2;
	x = x % pi2;
	if (x > pi)
		x -= pi2;
	else if (x < -pi)
		x += pi2;

	/* x represents x^2 from now on */
	x = fix_mull(p, x, x);

	/* -x^2/2! */
	pow = x;
	sum -= pow / 2;
	/* +x^4/4! */
	pow = fix_mull(p, pow, x);
	sum += pow / 24;
	/* -x^6/6! */
	pow = fix_mull(p, pow, x);
	sum -= pow / 720;
	/* +x^8/8! */
	pow = fix_mull(p, pow, x);
	sum += pow / 40320;
	/* -x^10/10! */
	pow = fix_mull(p, pow, x);
	sum -= pow / 3628800;

	return sum;
}

#endif /*_VPP_CSC_H*/