#include "vpp_io.h"
#include "linux/io.h"
#include "csc2/fixedpointmaths.h"
#include "linux/math64.h"
#define VPP_IN_SEL_REG_OFFSET 0x00778008
#define VPP_IN_SEL_REG_VPP_PIXELCLK_SW_MASK		(0x00000020)
#define VPP_IN_SEL_REG_VPP_PIXELCLK_SW_SHIFT		(5)
#define VPP_IN_SEL_REG_VPP_PIXELCLK_EN_MASK		(0x00000010)
#define VPP_IN_SEL_REG_VPP_PIXELCLK_EN_SHIFT		(4)
#define VPP_IN_SEL_REG_VPP_IN_MASK			(0x00000003)
#define VPP_IN_SEL_REG_VPP_IN_SHIFT			(0)
#define VPP_IN_SEL_REG_VPP_IN_LENGTH			(2)
enum {
CONFIG_DIFME_OFF = 0,		//00 : Off
CONFIG_DIFME_SLEEP,			//01 : Sleep
CONFIG_DIFME_ON,			//10 : On
CONFIG_DIFME_SOFTE_RESET,	//11 : Software reset
};

void vpp_write(struct vpp_dev *idev, u32 reg_offs, u32 data)
{
	iowrite32(data, idev->base + reg_offs);
}

unsigned int vpp_read(struct vpp_dev *idev, unsigned int reg_offs)
{
	return readl(idev->base + reg_offs);
}

void vpp_flush_regs(struct vpp_dev *idev)
{
#if 0
	u32 reg_upd_ctrl;
	reg_upd_ctrl = vpp_read(idev, VPP_REGISTER_UPDATE_CTRL_OFFSET);
	SET_FIELD(reg_upd_ctrl, VPP_REGISTER_UPDATE_CTRL_REGISTERS_VALID, 1);
	SET_FIELD(reg_upd_ctrl, VPP_REGISTER_UPDATE_CTRL_USE_VBLANK, 1);
	vpp_write(idev, VPP_REGISTER_UPDATE_CTRL_OFFSET, reg_upd_ctrl );
#endif
}
void vpp_mmu_write(struct vpp_dev *idev, unsigned int reg_offs, unsigned int data)
{
	iowrite32(data, idev->mmu_base + reg_offs);
}

unsigned int vpp_mmu_read(struct vpp_dev *idev, unsigned int reg_offs)
{
	return ioread32(idev->mmu_base + reg_offs);
}
int vpp_io_mmu_invalidate_cache(struct vpp_dev *idev)
{
	u32 reg;
	/* Invalidate the cache */
	reg = vpp_mmu_read(idev, VPP_MMU_CONTROL1_OFFSET);
	SET_FIELD(reg, VPP_MMU_CONTROL1_MMU_INVALDC0, 1);
	vpp_mmu_write(idev, VPP_MMU_CONTROL1_OFFSET, reg );
	SET_FIELD(reg, VPP_MMU_CONTROL1_MMU_INVALDC0, 0);
	vpp_mmu_write(idev, VPP_MMU_CONTROL1_OFFSET, reg );
	return 0;
}

int vpp_io_write_all_buffer_base_addresses(struct vpp_dev *idev, u32 dev_mem_start_addr, struct buf_offsets_strides *bofs)
{
	bool is_interleaved = bofs->is_interleaved;  //true: Interlaced; false:Progressive
	u32  y_offset = bofs->y_offset;
	u32  uv_offset = bofs->uv_offset;
	u32  y_odd_offset = bofs->y_odd_offset;
	u32  uv_odd_offset = bofs->uv_odd_offset;
	u32 valid_bits = 32 - VPP_YSTRIDE_ALIGN;

	if ((y_offset << valid_bits) || (uv_offset << valid_bits) ||
	    (y_odd_offset << valid_bits) || (uv_odd_offset << valid_bits))
		return -EINVAL;
	if ((!is_interleaved) && (y_odd_offset != 0 || uv_odd_offset != 0))
		return -EINVAL;
	PDEBUG("\nwriting 0x%x to BASE_ADDR_EVEN=0x%x  ",
	       dev_mem_start_addr + y_offset,
	       VPP_DIF_FRAME_BASE_ADDR_EVEN_OFFSET);
	vpp_write(idev, VPP_DIF_FRAME_BASE_ADDR_EVEN_OFFSET,
		  dev_mem_start_addr + y_offset);
	PDEBUG("\nwriting 0x%x to CHROMA_BASE_ADDR_EVEN=0x%x  ",
	       uv_offset ? (dev_mem_start_addr + uv_offset) : 0,
	       VPP_DIF_FRAME_CHROMA_BASE_ADDR_EVEN_OFFSET);
	vpp_write(idev, VPP_DIF_FRAME_CHROMA_BASE_ADDR_EVEN_OFFSET,
		  uv_offset ? (dev_mem_start_addr + uv_offset) : 0);
	PDEBUG("\nwriting 0x%x to BASE_ADDR_ODD=0x%x ",
	       y_odd_offset ? (dev_mem_start_addr + y_odd_offset) : 0,
	       VPP_DIF_FRAME_BASE_ADDR_ODD_OFFSET);
	vpp_write(idev, VPP_DIF_FRAME_BASE_ADDR_ODD_OFFSET,
		  y_odd_offset ? (dev_mem_start_addr + y_odd_offset) : 0);
	PDEBUG("\nwriting 0x%x to CHROMA_BASE_ADDR_ODD=0x%x  ",
	       uv_odd_offset ? (dev_mem_start_addr + uv_odd_offset) : 0,
	       VPP_DIF_FRAME_CHROMA_BASE_ADDR_ODD_OFFSET);
	vpp_write(idev, VPP_DIF_FRAME_CHROMA_BASE_ADDR_ODD_OFFSET,
		  uv_odd_offset ? (dev_mem_start_addr + uv_odd_offset) : 0);
	return 0;
}

int vpp_io_frame_buffers_base_addr_write(struct vpp_dev *idev,
					 enum VPP_BUFFERS buffer,
					 u32 buf_dev_address,
					 enum v4l2_field field)
{
	int error = -1;
	u32 offset = 0;
	/* Make sure that the first 4 bits are zero */
	PDEBUG("buffer =0x%08X   buf_dev_address=0x%08X ",
	       buffer, buf_dev_address);
	if (buf_dev_address &
	    ~VPP_DIF_FRAME_CHROMA_BASE_ADDR_EVEN_DIFFRCBASE_EVEN_MASK)
		return error;

	switch (buffer) {
	case VPP_BUF_ALL:
		PDEBUG("\n write 0x%X to BASE_ADDR_EVEN:0x%X and ODD:0x%X",
		       (u32)buf_dev_address,
		       (u32)(idev->base + VPP_DIF_FRAME_BASE_ADDR_EVEN_OFFSET),
		       (u32)(idev->base + VPP_DIF_FRAME_BASE_ADDR_ODD_OFFSET));
		vpp_write(idev, VPP_DIF_FRAME_BASE_ADDR_EVEN_OFFSET,
			  buf_dev_address);
		break;
	case VPP_BUF_ODD_INTERLACED_OR_CHROMA_PROGRESSIVE:
		/* Write to the register based on the field */
		offset = (field == V4L2_FIELD_NONE) ?
			 VPP_DIF_FRAME_CHROMA_BASE_ADDR_EVEN_OFFSET :
			 VPP_DIF_FRAME_BASE_ADDR_ODD_OFFSET;
		PDEBUG("\n writing buf_dev_address=0x%X  to 0x%X address",
		       (u32)buf_dev_address, (u32)(idev->base + offset));
		vpp_write(idev, offset, buf_dev_address);
		break;
	case VPP_BUF_INTERLACED_PLANAR_CHROMA_EVEN:
		PDEBUG("\n writing buf_dev_address=0x%X  to 0x%X address",
		       (u32)buf_dev_address, (u32)(idev->base +
		       VPP_DIF_FRAME_CHROMA_BASE_ADDR_EVEN_OFFSET));
		vpp_write(idev, VPP_DIF_FRAME_CHROMA_BASE_ADDR_EVEN_OFFSET,
			  buf_dev_address);
		break;
	case VPP_BUF_INTERLACED_PLANAR_CHROMA_ODD:
		PDEBUG("\n writing buf_dev_address=0x%X to 0x%X ",
		       (u32)buf_dev_address, (u32)(idev->base +
		       VPP_DIF_FRAME_CHROMA_BASE_ADDR_ODD_OFFSET));
		vpp_write(idev, VPP_DIF_FRAME_CHROMA_BASE_ADDR_ODD_OFFSET,
			  buf_dev_address);
		break;
	default:
		return error;
	}
	return 0;
}

int vpp_io_set_timing_registers(struct vpp_dev *idev, struct v4l2_dv_timings *timings)
{
	u32 h_active;
	u32 v_active;
	u32 h_inactive;
	u32 v_inactive;
	u32 reg;
	u32 ret = 0;
	bool isITU = false;
	
	//Make sure that the double-buffering is off, meaning that we are in healthy state.
	if ((timings==NULL) || (idev == NULL) || (vpp_io_is_double_buffer_enabled(idev))){
		return -EINVAL;
	}
	//HACT = (HSYNC + HBACKPORCH)-1
	h_active = timings->bt.hsync + timings->bt.hbackporch;
	//VACT = (VSYNC + VFBACKPORCH)-1 
	v_active = timings->bt.vsync + timings->bt.vbackporch;
	//Check that the values are not (badly) out of range
	if ((h_active 	> VPP_DIF_AA_H_ACTIVE_DIFAAHACTIVE_LSBMASK )   ||
		(v_active 	> VPP_DIF_AA_V_ACTIVE_DIFAAVACTIVE_LSBMASK ) 
		){
			printk ("\n vpp_io_set_timing_registers: out of range");
			return -EINVAL;
	}
	
	reg  = vpp_read(idev, VPP_DIF_AA_H_ACTIVE_OFFSET);
	SET_FIELD(reg, VPP_DIF_AA_H_ACTIVE_DIFAAHACTIVE, h_active);
	vpp_write(idev, VPP_DIF_AA_H_ACTIVE_OFFSET, reg );	
	
	
	reg  = vpp_read(idev, VPP_DIF_AA_V_ACTIVE_OFFSET);
	SET_FIELD(reg, VPP_DIF_AA_V_ACTIVE_DIFAAVACTIVE, v_active);
	vpp_write(idev, VPP_DIF_AA_V_ACTIVE_OFFSET, reg );	
	
	/* VPP_DIF_CONFIG_B1SDBTYP is set when the input mode has been set 
	 * if it has not been set by now, it will be set when we re-visit this function before start_streaming
	 */
	reg  = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	isITU = (bool) GET_FIELD(reg, VPP_DIF_CONFIG_B1SDBTYP);
	if (isITU){
		/* HINACT = (HSYNC+HBACKPORCH+WIDTH)-1 */
		h_inactive = timings->bt.hsync + timings->bt.hbackporch + timings->bt.width;
		/* VINACT = (VSYNC+VBACKPORCH+HEIGHT)-1 */
		v_inactive = timings->bt.vsync + timings->bt.vbackporch + timings->bt.height;

		if ((h_inactive	> VPP_DIF_AA_H_INACTIVE_DIFAAHACTIVEN_LSBMASK) ||
			(v_inactive > VPP_DIF_AA_V_INACTIVE_DIFAAVACTIVEN_LSBMASK)
			){
				printk ("\n vpp_io_set_timing_registers: out of range");
				return -EINVAL;
		}
		reg  = vpp_read(idev, VPP_DIF_AA_H_INACTIVE_OFFSET);
		SET_FIELD(reg, VPP_DIF_AA_H_INACTIVE_DIFAAHACTIVEN, h_inactive);
		vpp_write(idev, VPP_DIF_AA_H_INACTIVE_OFFSET, reg );	

		reg  = vpp_read(idev, VPP_DIF_AA_V_INACTIVE_OFFSET);
		SET_FIELD(reg, VPP_DIF_AA_V_INACTIVE_DIFAAVACTIVEN, v_inactive);
		vpp_write(idev, VPP_DIF_AA_V_INACTIVE_OFFSET, reg );	
	}
	
	//TODO: Now we need to use polarities to fill in the difhvspsl field of dif_config2 but for that we need another ioctl.
	
	//Set the input height
	reg  = vpp_read(idev, VPP_DIF_AA_LINE_COUNT_OFFSET);
	SET_FIELD(reg, VPP_DIF_AA_LINE_COUNT_DIFLINES, timings->bt.height);
	vpp_write(idev, VPP_DIF_AA_LINE_COUNT_OFFSET, reg );	
	return ret;
}

int vpp_io_set_input_polarity(struct vpp_dev *idev)
{
	u32 reg;
	PDEBUG("\n setting input polarity");
	reg  = vpp_read(idev, VPP_DIF_CONFIG2_OFFSET);
	SET_FIELD(reg, VPP_DIF_CONFIG2_DIFHVSPSL, idev->input_pol_ctrl->val);
	vpp_write(idev, VPP_DIF_CONFIG2_OFFSET, reg );
	
	return 0;
}
int vpp_io_set_input_mode(struct vpp_dev *idev)
{
	u32 reg;
	u32 vpp_sel_reg;
	bool is_ycc;
	bool is_ten_bit = false;
	u32 mode;
	u32 value = idev->csc_input_mode_ctrl->val;
	/* if the mode control value is 8 or 9, change it to 1 or 2 respectively but set the tenbit bit in config2*/
	if (value >= VPP_NO_OF_B1BMODE_MODE)
		return -EINVAL;
	switch (value)
	{
		case VPP_B1BMODE_MODE_1_444_DVI_RGB_3X10:
			mode = VPP_B1BMODE_MODE_1_444_DVI_RGB_3X8;
			is_ten_bit = true;
			break;
		case VPP_B1BMODE_MODE_2_444_YCBCR_3X10:
			mode = VPP_B1BMODE_MODE_2_444_YCBCR_3X8;
			is_ten_bit = true;
			break;
		default:
			mode = value;
			break;
	}
	is_ycc = ((value != VPP_B1BMODE_MODE_1_444_DVI_RGB_3X8) &&
		   (value != VPP_B1BMODE_MODE_1_444_DVI_RGB_3X10)) ?
		   true : false;
	/* Also set up the top level register CR_VPP_IN_SEL */
	vpp_sel_reg = ioread32(idev->vpp_sel_base);
	/* Set video input as YCbCr 422 */
	SET_FIELD(vpp_sel_reg, VPP_IN_SEL_REG_VPP_IN, 0x1);
	PDEBUG("\n setting input mode to 0x%X ",mode);
	reg  = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	SET_FIELD(reg, VPP_DIF_CONFIG_B1MODE, mode );
	SET_FIELD(reg, VPP_DIF_CONFIG_DIFYCCNOTRGB, is_ycc);
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg );
	PDEBUG("\n config_reg is 0x%X ", reg);
	/*Now set config2.tenbit if needed */
	reg  = vpp_read(idev, VPP_DIF_CONFIG2_OFFSET);
	SET_FIELD(reg, VPP_DIF_CONFIG2_TENBITIN, is_ten_bit );
	vpp_write(idev, VPP_DIF_CONFIG2_OFFSET, reg );
	
	reg  = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	switch (value)
	{
		case VPP_B1BMODE_MODE_1_444_DVI_RGB_3X8:
		case VPP_B1BMODE_MODE_2_444_YCBCR_3X8:
		case VPP_B1BMODE_MODE_1_444_DVI_RGB_3X10:
		case VPP_B1BMODE_MODE_2_444_YCBCR_3X10:
			SET_FIELD(reg, VPP_DIF_CONFIG_DIFOSL, 0x0 );
			SET_FIELD(reg, VPP_DIF_CONFIG_DCVSTR, 0x0 );
			SET_FIELD(reg, VPP_DIF_CONFIG_DCVFRM, 0x0 );
			/* Set video input as "YCbCr 444 or RGB" */
			SET_FIELD(vpp_sel_reg, VPP_IN_SEL_REG_VPP_IN, 0x0);
			break;
		case VPP_B1BMODE_MODE_3_422_YCBCR_2X8:
		case VPP_B1BMODE_MODE_4_422_YCBCR_2X10:
			SET_FIELD(reg, VPP_DIF_CONFIG_DIFOSL, 0x1 );
			SET_FIELD(reg, VPP_DIF_CONFIG_DCVSTR, 0x1 );
			SET_FIELD(reg, VPP_DIF_CONFIG_DCVFRM, 0x1 );
			break;
		case VPP_B1BMODE_MODE_5_422_YCBCR_1X8:
		case VPP_B1BMODE_MODE_6_422_YCBCR_1X10A:
		case VPP_B1BMODE_MODE_7_422_YCBCR_1X10B:
			SET_FIELD(reg, VPP_DIF_CONFIG_DIFOSL, 0x1 );
			SET_FIELD(reg, VPP_DIF_CONFIG_DCVSTR, 0x1 );
			// TODO: More info is needed to set VPP_DIF_CONFIG_DCVFRM. introduce a control
			//This field will be 0x2 if 656 data input with embedded TRS(Timing Reference Signal)
			//Otherwise it would be 0x3 representing 656 data input with HVF.
			//SET_FIELD(reg, VPP_DIF_CONFIG_DCVFRM, 0x1 );
			printk ("\n 656 not implemented");
			return -EINVAL;
			break;
		default:
			printk("\n input mode invalid");
			return -EINVAL;
	}
	/* Set to 16 the number of lines over which the DIF Horizontal Count data is averaged for Mode Recognition.
	 * Even though we are not using the feature in this driver.
	 */
	SET_FIELD(reg, VPP_DIF_CONFIG_DIF_LINEAV, 0x4 );
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg );
	iowrite32(vpp_sel_reg, idev->vpp_sel_base);
	return 0;

}
int vpp_io_set_input_itu_or_dvi(struct vpp_dev *idev)
{
	u32 reg;
	s32 value;
	if (idev == NULL || idev->csc_input_mode_ctrl == NULL)
		return -EINVAL;
	value =  idev->csc_input_mode_ctrl->val;
	value = ((value == VPP_B1BMODE_MODE_5_422_YCBCR_1X8 )||(value == VPP_B1BMODE_MODE_6_422_YCBCR_1X10A )||(value == VPP_B1BMODE_MODE_7_422_YCBCR_1X10B ) )? 1 : 0;
	reg  = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	SET_FIELD(reg, VPP_DIF_CONFIG_B1SDBTYP, value );
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg );
	PDEBUG("\n itu or dvi is %s config_reg is 0x%X ", value? "ITU":"DVI", reg);
	return 0;
}
int vpp_io_set_input_swap(struct vpp_dev *idev)
{
	u32 reg;
	bool value =  idev->sys_values.vpp_swap_lanes;
	u32 input_mode = idev->csc_input_mode_ctrl->val;
	reg  = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);

	SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G3, 0x03 );
	if (value)
	{
		/*swap group1 and group 2 */
		SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G2, 0x01 );
		SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G1, 0x02 );
	}	
	else 
	{
		SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G2, 0x02 );
		SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G1, 0x01 );
	}
	/* workaround for yuv44 */
	if (input_mode == VPP_B1BMODE_MODE_2_444_YCBCR_3X10 ||
	    input_mode == VPP_B1BMODE_MODE_2_444_YCBCR_3X8) {
		SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G3, 0x01);
		SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G2, 0x03);
		SET_FIELD(reg, VPP_DIF_CONFIG_B1B8G1, 0x02);
	}
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg);
	PDEBUG("group1 and group2 lanes %s config_reg is 0x%X\n",
	       value ? "swapped" : "not swapped", reg);

	return 0;
}
static unsigned long get_natural_stride(int width, int pixels_per_block, int bits_per_block, int linelen_align)
{
	const int linelen_align_in_bits = 8 << linelen_align;
	unsigned long length;

	/* round up to line length alignment */
	length = (width * bits_per_block) / pixels_per_block;
	length = (length + (linelen_align_in_bits - 1)) & -linelen_align_in_bits;
	length >>= 3;
	return length;
}
int vpp_io_set_output_format(struct vpp_dev *idev)
{
	/* Different bits of VPP_DIF_CONFIG and VPP_DIF_CONFIG2 must be set based on input format 
	 * See table 8 in "DIF Memory Format Configuration" Section of the TRM.
	*/ 
	struct v4l2_pix_format_mplane *pixel_mp;
	u32 reg;
	bool is_444 	= false;
	bool is_420 	= false;
	bool is_8bit	= false;
	bool is_planar	= false;
	bool is_interleaved = false;
	bool override_burst_page_alignment = false;
	int pixels_per_block;
	int bits_per_block;
	unsigned long 	natural_stride;
	if (idev == NULL)
		return -EINVAL;
	if (idev->format.type == V4L2_BUF_TYPE_VIDEO_CAPTURE ){
		printk("\nformat type V4L2_BUF_TYPE_VIDEO_CAPTURE not implemented yet");
		return -EINVAL;
	}
	if (idev->format.type != V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE	){
		printk("\nformat type is not V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE");
		return -EINVAL;
	}
	
	pixel_mp = (struct v4l2_pix_format_mplane *) &idev->format.fmt.pix_mp;
	if (pixel_mp->num_planes == 0 || pixel_mp->num_planes > VPP_MAX_NO_OF_PLANES_PER_FRAME){
		printk ("\n Invalid number of planes %d", pixel_mp->num_planes);
		return -EINVAL;
		}
	switch (pixel_mp->pixelformat){
		case V4L2_PIX_FMT_YUV30: //YUV101010
			pixels_per_block = 1;
			bits_per_block   = 32;
			is_444 = true;
			break;
		case V4L2_PIX_FMT_UYVY20: //UYVY10101010
			pixels_per_block = 1;
			bits_per_block   = 10;
			break;
		case V4L2_PIX_FMT_PLA422: //PL12Y10/422PL12UV10
			pixels_per_block = 3;
			bits_per_block   = 32;
			is_planar = true;
			break;
		case V4L2_PIX_FMT_PLA420:
			pixels_per_block = 3;
			bits_per_block   = 32;
			is_planar = true;
			is_420 = true;
			break;
		case V4L2_PIX_FMT_PL8422:
			pixels_per_block = 1;
			bits_per_block   = 8;
			is_planar = true;
			is_8bit   = true;
			break;
		case V4L2_PIX_FMT_PL8420:
			pixels_per_block = 1;
			bits_per_block   = 8;
			is_planar = true;
			is_420 	= true;
			is_8bit = true;
			break;
		default:
			printk ("\noutput format not supported. 0x%X", (u32) pixel_mp->pixelformat);
			return -EINVAL;
	}
	natural_stride = get_natural_stride(pixel_mp->width, pixels_per_block, bits_per_block, VPP_YSTRIDE_ALIGN);
	PDEBUG("\nvpp_io_set_output_format: natural_stride is %d stride is %d", (u32) natural_stride, (u32)pixel_mp->plane_fmt[0].bytesperline );
	if (natural_stride > pixel_mp->plane_fmt[0].bytesperline){
		printk("\nvpp_io_set_output_format: natural stride %d is higher than stride value %d",(u32) natural_stride, (u32)pixel_mp->plane_fmt[0].bytesperline );
		return -EINVAL;
	}
	else if (natural_stride < pixel_mp->plane_fmt[0].bytesperline){
		override_burst_page_alignment = true;
	}
	switch (pixel_mp->field){
		case V4L2_FIELD_NONE:
			/* Progressive */
			is_interleaved = false;
			PDEBUG("\n output field is progressive");
			break;
		case V4L2_FIELD_SEQ_TB:	
		case V4L2_FIELD_SEQ_BT:
		case V4L2_FIELD_ALTERNATE:
			is_interleaved = true;
			PDEBUG("\n output field is interleaved");
			break;
		default:
			/* Other options are not related to CAPTURE devices. I also do not allow V4L2_FIELD_ANY */
			printk ("\noutput format field not supported. 0x%X", (u32) pixel_mp->field);
			return -EINVAL;
	}
	PDEBUG("\n is_444= %d is_interleaved= %d  is_planar= %d is_420= %d",is_444, is_interleaved,  is_planar, is_420); 
	reg  = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	SET_FIELD(reg, VPP_DIF_CONFIG_DIFSEL422OUT, ~is_444 );
	SET_FIELD(reg, VPP_DIF_CONFIG_DIFITLV, 0);
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg );

	reg  = vpp_read(idev, VPP_DIF_CONFIG2_OFFSET);
	SET_FIELD(reg, VPP_DIF_CONFIG2_DEIGHTBITOUT, is_8bit );
	SET_FIELD(reg, VPP_DIF_CONFIG2_DPLANAR, is_planar);
	SET_FIELD(reg, VPP_DIF_CONFIG2_DEC420,is_420);
	SET_FIELD(reg, VPP_DIF_CONFIG2_OVERRIDEBURSTPAGEALIGN, override_burst_page_alignment);
	SET_FIELD(reg, VPP_DIF_CONFIG2_DIF_CEAFIELDDETECT, is_interleaved);
	vpp_write(idev, VPP_DIF_CONFIG2_OFFSET, reg );
	PDEBUG ("\nvpp_io_set_output_format: setting stride value %d",(u32)pixel_mp->plane_fmt[0].bytesperline );
	reg  = vpp_read(idev, VPP_DIF_FRAME_STRIDE_OFFSET);
	/*Write the stride value to the field in words not bytes*/
	SET_FIELD(reg, VPP_DIF_FRAME_STRIDE_DIFFRSTRIDE, pixel_mp->plane_fmt[0].bytesperline >> 4 );
	vpp_write(idev, VPP_DIF_FRAME_STRIDE_OFFSET, reg );
	/* Set output frame width and height */
	reg  = vpp_read(idev, VPP_DIF_FRAME_HEIGHT_OFFSET);
	SET_FIELD(reg, VPP_DIF_FRAME_HEIGHT_DIFFRHEIGHT, is_interleaved? (pixel_mp->height/2) -1 :pixel_mp->height - 1 );
	vpp_write(idev, VPP_DIF_FRAME_HEIGHT_OFFSET, reg );
	/* scale parameters are set right before start streaming */

	return 0;
}
u32 vpp_io_read_irq_enable(struct vpp_dev *idev)
{
	return (vpp_read(idev, VPP_DIF_IRQ_ENABLE_OFFSET) );
}
u32 vpp_io_read_irq_status(struct vpp_dev *idev)
{
	return (vpp_read(idev, VPP_DIF_IRQ_STATUS_OFFSET) );
}
void vpp_io_clear_interrupt(struct vpp_dev *idev, u32 mask)
{
	u32 reg;
	reg  = vpp_read(idev, VPP_DIF_IRQ_CLEAR_OFFSET);
	reg  |= mask;
	vpp_write(idev, VPP_DIF_IRQ_CLEAR_OFFSET, reg );
}
int vpp_io_disable_interrupt(struct vpp_dev *idev, u32 mask)
{	
	u32 reg;
	reg  = vpp_read(idev, VPP_DIF_IRQ_ENABLE_OFFSET);
	reg  &= ~mask;
	vpp_write(idev, VPP_DIF_IRQ_ENABLE_OFFSET, reg);
	return 0;
}
int vpp_io_enable_interrupt(struct vpp_dev *idev, u32 mask)
{
	
	u32 reg;
	reg  = vpp_read(idev, VPP_DIF_IRQ_ENABLE_OFFSET);
	reg  |= mask; 	
	vpp_write(idev, VPP_DIF_IRQ_ENABLE_OFFSET, reg );
	return 0;
}
static void wirte_to_csc_regs(struct vpp_dev * idev,u32 i, u32 j, u32 value)
{
	int reg;
	if (i < 1 || i > 3 || j < 1 || j > 3){
		printk("\n wirte_to_csc_regs: bad indices. i=%d j=%d",i,j);
		return;
	}
		switch (i)
		{
			case 1:
				switch (j)
				{
					case 1:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_1112_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_1112_DIFCOEFF11, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_1112_OFFSET, reg);
						break;
					case 2:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_1112_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_1112_DIFCOEFF12, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_1112_OFFSET, reg);
						break;
					case 3:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_1321_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_1321_DIFCOEFF13, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_1321_OFFSET, reg);
						break;
				}
				break;
			case 2:
				switch (j)
				{
					case 1:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_1321_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_1321_DIFCOEFF21, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_1321_OFFSET, reg);
						break;
					case 2:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_2223_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_2223_DIFCOEFF22, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_2223_OFFSET, reg);
						break;
					case 3:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_2223_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_2223_DIFCOEFF23, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_2223_OFFSET, reg);
						break;
				}
				break;
			case 3:
				switch (j)
				{
					case 1:
					reg = vpp_read(idev, VPP_DIF_CSC_COEFF_3132_OFFSET);
					SET_FIELD(reg, VPP_DIF_CSC_COEFF_3132_DIFCOEFF31, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_3132_OFFSET, reg);
						break;
					case 2:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_3132_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_3132_DIFCOEFF32, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_3132_OFFSET, reg);
						break;
					case 3:
						reg = vpp_read(idev, VPP_DIF_CSC_COEFF_33_OFFSET);
						SET_FIELD(reg, VPP_DIF_CSC_COEFF_33_DIFCOEFF33, value);
						vpp_write(idev,VPP_DIF_CSC_COEFF_33_OFFSET, reg);
						break;
				}
				break;
		}
}

int vpp_io_set_csc_regs(CSC_sConfiguration sResultantConfigurationData,
			struct vpp_dev *idev, bool swap)
{
	u32 i,j;
	u32 value;
	u32 reg;
	u32 coeff[3][3];
	for (j = 0; j < 3; j++) {
		coeff[j][0] = sResultantConfigurationData.sCoefficients.ai32Coefficients[j][swap ? 1 : 0];
		coeff[j][1] = sResultantConfigurationData.sCoefficients.ai32Coefficients[j][swap ? 2 : 1];
		coeff[j][2] = sResultantConfigurationData.sCoefficients.ai32Coefficients[j][swap ? 0 : 2];
	}

	PDEBUG("\n CSC coefficients:\n");
	for (i=0; i<3; i++){
		PDEBUG ("\n");
			for (j=0;j<3;j++){
				/* Change the fractional bits to VPP_CSC_MATRIX_FRACTIONAL_BITS, with rounding up */
				value = FIXEDPT_CONVERT_FRACTIONAL_BITS(coeff[i][j],
														CSC_FRACTIONAL_BITS, VPP_CSC_MATRIX_FRACTIONAL_BITS);
				/*
				shift = CSC_FRACTIONAL_BITS - VPP_CSC_MATRIX_FRACTIONAL_BITS - 1;
				value = sResultantConfigurationData.sCoefficients.ai32Coefficients[i][j]>> shift;
				value +=1;
				value = (value >>1)& (CSC_MATRIX_COEFFS_MIN + CSC_MATRIX_COEFFS_MAX);
				//value = sResultantConfigurationData.sCoefficients.ai32Coefficients[i][j]>>(CSC_FRACTIONAL_BITS - 8)) & 0x7FF)
				*/
				/*TODO: Find out if we need to swap the coloumns of the matrix */
				wirte_to_csc_regs(idev,i+1, j+1, value & (CSC_MATRIX_COEFFS_MIN + CSC_MATRIX_COEFFS_MAX));
				PDEBUG ("\n 0x%X ", value);
			}
		}
		PDEBUG("\n input offsets:\n");
		for (i=0; i<3; i++){
			PDEBUG ("\n 0x%X ", sResultantConfigurationData.ai32InputOffsets[i]);
		}
		/*Input offset is a signed integer value. Just write it to the register. */ 
		/*TODO: Revisit this during actual testing*/
		reg = vpp_read(idev, VPP_DIF_CSC_Y_INOFF_OFFSET);
		SET_FIELD(reg, VPP_DIF_CSC_Y_INOFF_DIFYI, sResultantConfigurationData.ai32InputOffsets[0]); 
		vpp_write(idev,VPP_DIF_CSC_Y_INOFF_OFFSET, reg);
		
		reg = vpp_read(idev, VPP_DIF_CSC_CBCR_INOFF_OFFSET);
		SET_FIELD(reg, VPP_DIF_CSC_CBCR_INOFF_DIFCBI, sResultantConfigurationData.ai32InputOffsets[1]); 
		SET_FIELD(reg, VPP_DIF_CSC_CBCR_INOFF_DIFCRI, sResultantConfigurationData.ai32InputOffsets[2]); 
		vpp_write(idev,VPP_DIF_CSC_CBCR_INOFF_OFFSET, reg);
		/* output offsets are in x.10 format but must be written to registers in 9.1 format. */
		/*TODO: Revisit this during actual testing*/
		PDEBUG("\n output offsets:\n");
		for (i=0; i<3; i++){
				PDEBUG ("\n 0x%X (x.10 --> 0x%X (9.1)", sResultantConfigurationData.ai32OutputOffsets[i] & CSC_OUTPUT_OFFSET_FRACTIONAL_BITS,
					FIXEDPT_CONVERT_FRACTIONAL_BITS_NO_ROUNDING(sResultantConfigurationData.ai32OutputOffsets[i],CSC_OUTPUT_OFFSET_FRACTIONAL_BITS,1 ));
				
		}
		reg = vpp_read(idev, VPP_DIF_CSC_Y_OUTOFF_OFFSET);
		SET_FIELD(reg, VPP_DIF_CSC_Y_OUTOFF_DIFYO, FIXEDPT_CONVERT_FRACTIONAL_BITS_NO_ROUNDING(sResultantConfigurationData.ai32OutputOffsets[0],CSC_OUTPUT_OFFSET_FRACTIONAL_BITS,1 )); 
		vpp_write(idev,VPP_DIF_CSC_Y_OUTOFF_OFFSET, reg);
		
		reg = vpp_read(idev, VPP_DIF_CSC_CBCR_OUTOFF_OFFSET);
		SET_FIELD(reg, VPP_DIF_CSC_CBCR_OUTOFF_DIFCBO, FIXEDPT_CONVERT_FRACTIONAL_BITS_NO_ROUNDING(sResultantConfigurationData.ai32OutputOffsets[1],CSC_OUTPUT_OFFSET_FRACTIONAL_BITS,1 )); 
		SET_FIELD(reg, VPP_DIF_CSC_CBCR_OUTOFF_DIFCRO, FIXEDPT_CONVERT_FRACTIONAL_BITS_NO_ROUNDING(sResultantConfigurationData.ai32OutputOffsets[2],CSC_OUTPUT_OFFSET_FRACTIONAL_BITS,1 )); 
		vpp_write(idev,VPP_DIF_CSC_CBCR_OUTOFF_OFFSET, reg);
	
	return 0;
}
int vpp_io_set_burst_size(struct vpp_dev *idev)
{
	u32 value;
	u32 reg;
	if (idev == NULL)
		return -EINVAL;
	
	switch (idev->sys_values.vpp_mem_burst_size)
	{
		case 32:
			value = 00;
			break;
		case 16:
			value = 01;
			break;
		case 8:
			value = 02;
			break;
		default:
			return -EINVAL;
	}
	reg = vpp_read(idev, VPP_DIF_BURST_SIZE_OFFSET);
	SET_FIELD(reg, VPP_DIF_BURST_SIZE_DIFBURSTSIZE, value); 
	vpp_write(idev,VPP_DIF_BURST_SIZE_OFFSET, reg);
	
	reg = vpp_read(idev, VPP_CORE_BURST_LENGTH_OFFSET);
	SET_FIELD(reg, VPP_CORE_BURST_LENGTH_CORE_BURST_LENGTH, idev->sys_values.vpp_mem_burst_length); 
	vpp_write(idev,VPP_CORE_BURST_LENGTH_OFFSET, reg);
	return 0;	
}
int vpp_io_set_h_scaler_coeffs(struct vpp_dev *idev)
{
	int i,j;
	u32 reg;
	if (idev == NULL)
		return -EINVAL;
	/* Number of coefficients = (T/2)*I + 1 */
			/* Number of coeff registers to write = ((T/2)*I)/4 + 1 */
			/* T = 8, so registers to write = I + 1 */
	for (i = 0; i < VPP_H_INTERPOLATION_POINTS; ++i) {
		reg =  (0xFF & idev->v.hscoeffs[i << 2])
				| ((0xFF & idev->v.hscoeffs[(i << 2) + 1]) << 8)
				| ((0xFF & idev->v.hscoeffs[(i << 2) + 2]) << 16)
				| ((0xFF & idev->v.hscoeffs[(i << 2) + 3]) << 24);
		vpp_write(idev,VPP_DIF_HSCALER_COEFF0_OFFSET + (i<< 2), reg);		
		PDEBUG("\n reg[%d]= 0x%08X ", i, reg);
	}
	reg = (idev->v.hscoeffs[i<<2]) & 0xFF;
	vpp_write(idev,VPP_DIF_HSCALER_COEFF0_OFFSET + (i<< 2), reg);	
	PDEBUG ("\n reg[%d]= 0x%08X",i,reg);
	
	for (i = 0; i < VPP_H_TAPS; ++i) {
		PDEBUG ("\n");
		for (j=0; j< VPP_H_INTERPOLATION_POINTS; j++){
		PDEBUG ("%02X ", (0xFF & (idev->v.hscoeffs[i*VPP_H_INTERPOLATION_POINTS +j])));
		}
	}
	return 0;
}
int vpp_io_set_v_scaler_coeffs(struct vpp_dev *idev)
{
	int i,j;
	u32 reg;
	if (idev == NULL)
		return -EINVAL;
	/* Number of coefficients = (T/2)*I + 1 */
			/* Number of coeff registers to write = ((T/2)*I)/4 + 1 */
			/* T = 8, so registers to write = I + 1 */
	for (i = 0; i < VPP_V_INTERPOLATION_POINTS/2; ++i) {
		reg =  (0xFF & idev->v.vscoeffs[i << 2])
				| ((0xFF & idev->v.vscoeffs[(i << 2) + 1]) << 8)
				| ((0xFF & idev->v.vscoeffs[(i << 2) + 2]) << 16)
				| ((0xFF & idev->v.vscoeffs[(i << 2) + 3]) << 24);
		vpp_write(idev,VPP_DIF_VSCALER_YCOEFF0_OFFSET + (i<< 2), reg);	
		vpp_write(idev,VPP_DIF_VSCALER_CCOEFF0_OFFSET + (i<< 2), reg);
		PDEBUG("\nvscale reg[%d]= 0x%08X ", i, reg);
	}
	reg = (idev->v.vscoeffs[i<<2]) & 0xFF;
	vpp_write(idev,VPP_DIF_VSCALER_YCOEFF0_OFFSET + (i<< 2), reg);	
	PDEBUG ("\nvscale reg[%d]= 0x%08X",i,reg);
	
	for (i = 0; i < VPP_V_TAPS; ++i) {
		PDEBUG ("\n");
		for (j=0; j< VPP_V_INTERPOLATION_POINTS; j++){
		PDEBUG ("%02X ", (0xFF & (idev->v.vscoeffs[i*VPP_V_INTERPOLATION_POINTS +j])));
		}
	}
	return 0;
}
int vpp_io_set_vpitch_vorder(struct vpp_dev *idev, u32 vpitch, u32 vorder)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_SCALECTRL_OFFSET);
	SET_FIELD(reg, VPP_DIF_SCALECTRL_DIFVPITCH, vpitch); 
	SET_FIELD(reg, VPP_DIF_SCALECTRL_DIFVORDER, vorder); 
	vpp_write(idev,VPP_DIF_SCALECTRL_OFFSET, reg);
	return 0;
}
int vpp_io_set_hpitch_hinit(struct vpp_dev *idev, u32 hpitch, u32 hinitial)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_HSCALER_P_I_OFFSET);
	SET_FIELD(reg, VPP_DIF_HSCALER_P_I_DIFHPITCH, hpitch); 
	SET_FIELD(reg, VPP_DIF_HSCALER_P_I_DIFHINITIAL, hinitial); 
	vpp_write(idev,VPP_DIF_HSCALER_P_I_OFFSET, reg);
	return 0;
}
int vpp_io_set_vinit(struct vpp_dev *idev, u32 vinit0, u32 vinit1)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_VSINIT_OFFSET);
	SET_FIELD(reg, VPP_DIF_VSINIT_DIFVINITIAL0, vinit0); 
	SET_FIELD(reg, VPP_DIF_VSINIT_DIFVINITIAL1, vinit1); 
	vpp_write(idev,VPP_DIF_VSINIT_OFFSET, reg);
	return 0;
}
int vpp_io_set_hscaled_size(struct vpp_dev *idev, u32 output_width_minus_one)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_HSCALED_SIZE_OFFSET);
	SET_FIELD(reg, VPP_DIF_HSCALED_SIZE_DIFSCALED_WIDTH, output_width_minus_one); 
	vpp_write(idev,VPP_DIF_HSCALED_SIZE_OFFSET, reg);
	return 0;
	
}
int vpp_io_set_vscaled_size(struct vpp_dev *idev, u32 output_height_minus_one)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_VSCALED_SIZE_OFFSET);
	SET_FIELD(reg, VPP_DIF_VSCALED_SIZE_DIFSCALED_HEIGHT, output_height_minus_one); 
	vpp_write(idev,VPP_DIF_VSCALED_SIZE_OFFSET, reg);
	return 0;
	
}
int vpp_io_set_hscale_bypass (struct vpp_dev *idev, bool hs_bp)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_SCALECTRL_OFFSET);
	SET_FIELD(reg, VPP_DIF_SCALECTRL_DIFHSCALEBP, hs_bp); 
	vpp_write(idev,VPP_DIF_SCALECTRL_OFFSET, reg);
	return 0;
}
int vpp_io_set_vscale_bypass (struct vpp_dev *idev, bool vs_bp)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_SCALECTRL_OFFSET);
	SET_FIELD(reg, VPP_DIF_SCALECTRL_DIFVSCALEBP, vs_bp); 
	vpp_write(idev,VPP_DIF_SCALECTRL_OFFSET, reg);
	return 0;
}
void vpp_io_set_hs_before_vs(struct vpp_dev *idev, bool hs_before_vs)
{
	u32 reg;
	reg = vpp_read(idev, VPP_DIF_SCALECTRL_OFFSET);
	SET_FIELD(reg, VPP_DIF_SCALECTRL_DIFHSBEFOREVS, hs_before_vs); 
	vpp_write(idev,VPP_DIF_SCALECTRL_OFFSET, reg);
}
int vpp_io_turn_on_hw(struct vpp_dev *idev)
{
	u32 reg_config;
	if (idev == NULL ){
		printk("\nvpp_io_turn_on_hw: input is NULL");
		return -EINVAL;
	}
	reg_config = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	SET_FIELD(reg_config, VPP_DIF_CONFIG_DIFME, CONFIG_DIFME_ON);
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg_config );
	return 0;
}
int vpp_io_turn_off_hw(struct vpp_dev *idev)
{
	u32 reg_config;
	if (idev == NULL )
		return -EINVAL;
	PDEBUG("\n_______ reg content before write 0x%X_____",vpp_read(idev, VPP_DIF_CONFIG_OFFSET));
	reg_config = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	SET_FIELD(reg_config, VPP_DIF_CONFIG_DIFME, CONFIG_DIFME_OFF);
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg_config );
	
	PDEBUG("\n_______ reg content after write 0x%X_____",vpp_read(idev, VPP_DIF_CONFIG_OFFSET));
	return 0;
}
int vpp_io_disable_memory(struct vpp_dev *idev)
{
	int ret =0;
	u32 reg_config;
	if (idev == NULL )
		return -EINVAL;
	reg_config = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	SET_FIELD(reg_config, VPP_DIF_CONFIG_DIFMEMWROFF, 0x0);
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg_config );
	PDEBUG("\n returning from vpp_io_disable_memory"); 
	return ret;
}
int vpp_io_enable_memory(struct vpp_dev *idev)
{
	int ret =0;
	u32 reg_config;
	PDEBUG("\n starting vpp_io_enable_memory"); 
	if (idev == NULL )
		return -EINVAL;
	reg_config = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	PDEBUG("\n  vpp_io_enable_memory:read"); 
	SET_FIELD(reg_config, VPP_DIF_CONFIG_DIFMEMWROFF, 0x1);
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg_config );
	PDEBUG("\n returning from vpp_io_enable_memory"); 
	return ret;
}
int vpp_io_soft_reset(struct vpp_dev *idev)
{
	int ret =0;
	u32 reg_config;
	if (idev == NULL )
		return -EINVAL;
	PDEBUG("\n_______ reg content before reset 0x%X_____",vpp_read(idev, VPP_DIF_CONFIG_OFFSET));
	reg_config = vpp_read(idev, VPP_DIF_CONFIG_OFFSET);
	SET_FIELD(reg_config, VPP_DIF_CONFIG_DIFME, CONFIG_DIFME_SOFTE_RESET);
	vpp_write(idev, VPP_DIF_CONFIG_OFFSET, reg_config );
	PDEBUG("\n_______ reg content after reset 0x%X______",vpp_read(idev, VPP_DIF_CONFIG_OFFSET));
	return ret;
}


bool vpp_io_is_double_buffer_enabled(struct vpp_dev *idev)
{
	return ( (vpp_read(idev,VPP_CORE_DOUBLE_BUFFER_CONTROL_OFFSET)) & VPP_CORE_DOUBLE_BUFFER_CONTROL_CORE_ENABLE_DOUBLE_BUFFERING_MASK );
}
void vpp_io_enable_double_buffering(struct vpp_dev *idev)
{
	u32 reg_double_buffering;
	reg_double_buffering = vpp_read(idev,VPP_CORE_DOUBLE_BUFFER_CONTROL_OFFSET);
	SET_FIELD (reg_double_buffering, VPP_CORE_DOUBLE_BUFFER_CONTROL_CORE_ENABLE_DOUBLE_BUFFERING, 0x1);
	vpp_write (idev, VPP_CORE_DOUBLE_BUFFER_CONTROL_OFFSET, reg_double_buffering );
}
void vpp_io_disable_double_buffering(struct vpp_dev *idev)
{
	u32 reg_double_buffering;
	reg_double_buffering = vpp_read(idev,VPP_CORE_DOUBLE_BUFFER_CONTROL_OFFSET);
	SET_FIELD (reg_double_buffering, VPP_CORE_DOUBLE_BUFFER_CONTROL_CORE_ENABLE_DOUBLE_BUFFERING, 0x0);
	vpp_write (idev, VPP_CORE_DOUBLE_BUFFER_CONTROL_OFFSET, reg_double_buffering );
}
#ifdef __VPP_TARGET_IS_SIMULATOR__
void vpp_io_kick_simulator(struct vpp_dev *idev)
{
	printk("\nstarting a new frame\n");
	vpp_write(idev,0x3FC, 0x1);
}
#endif
u32 vpp_io_read_signatures(struct vpp_dev *idev, u32 *chroma_sig)
{
	u32 reg;
	reg		  = vpp_read(idev, VPP_CORE_LUMA_SIGNATURE_OFFSET);
	*chroma_sig = vpp_read(idev, VPP_CORE_CHROMA_SIGNATURE_OFFSET);
	return reg;
	
}

void vpp_io_pixel_clock(struct vpp_dev *idev, bool enable)
{
	clear_bit(VPP_IN_SEL_REG_VPP_PIXELCLK_SW_SHIFT, idev->vpp_sel_base);
	if (enable)
		set_bit(VPP_IN_SEL_REG_VPP_PIXELCLK_EN_SHIFT,
			idev->vpp_sel_base);
	else
		clear_bit(VPP_IN_SEL_REG_VPP_PIXELCLK_EN_SHIFT,
			  idev->vpp_sel_base);
}

int mmu_tile_stride(u32 tiling, u32 stride)
{
	__u32 pwof2[] = {512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 131072};
	int j;
	if (tiling == VPP_TILING_NONE ||
	    (tiling == VPP_TILING_512_8 && stride <1024) ||
	    (tiling == VPP_TILING_256_16 && stride == 131072))
		return -EINVAL;
	for (j = 0; j < ARRAY_SIZE(pwof2); j++) {
		if (stride == pwof2[j])
			break;
	}
	if (j == ARRAY_SIZE(pwof2))
		return -EINVAL;
	return tiling == VPP_TILING_256_16 ? j : j - 1;
}

int vpp_io_mmu_set_tiling(struct vpp_dev *idev)
{
	u32 reg;
	u32 stride;
	u32 tile_stride;
	u32 tiling;
	u32 h;
	WARN_ON(!idev);
	tiling= v4l2_ctrl_g_ctrl(idev->tiling_ctrl);
	if (tiling == VPP_TILING_NONE) {
		reg = vpp_mmu_read(idev, VPP_MMU_TILE_CFG_OFFSET);
		SET_FIELD(reg, VPP_MMU_TILE_CFG_TILE_ENABLE, 0);
		vpp_mmu_write(idev, VPP_MMU_TILE_CFG_OFFSET, reg);
		return 0;
	}
	PDEBUG("\n%s: Setting tiling registers.", __func__);
	stride = (idev->format.type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE) ?
		  idev->format.fmt.pix_mp.plane_fmt[VPP_BUF_ALL].bytesperline :
		  idev->format.fmt.pix.bytesperline;
	h = (idev->format.type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE) ?
	     idev->format.fmt.pix_mp.height : idev->format.fmt.pix.height;
	/* Make sure that the line number is a multiple of 16 */
	if (h % 16) {
		dev_err(idev->dev, "\n%s: height is not a multiple of 16",
			__func__);
		return -EINVAL;
	}
	if ((idev->format.fmt.pix_mp.pixelformat == V4L2_PIX_FMT_PLA420 ||
		idev->format.fmt.pix_mp.pixelformat == V4L2_PIX_FMT_PL8420) &&
		(h % 32))
		WARN_ONCE(1, "\nheight isn't a mutiple of 32 for 420 format!");
	reg = vpp_mmu_read(idev, VPP_MMU_CONTROL0_OFFSET);
	SET_FIELD(reg, VPP_MMU_CONTROL0_MMU_TILING_SCHEME,
			  tiling == VPP_TILING_256_16 ? 0 : 1);
	vpp_mmu_write(idev, VPP_MMU_CONTROL0_OFFSET, reg);

	reg = vpp_mmu_read(idev, VPP_MMU_TILE_CFG_OFFSET);
	SET_FIELD(reg, VPP_MMU_TILE_CFG_TILE_ENABLE, 1);
	tile_stride = mmu_tile_stride(tiling, stride);
	if (tile_stride == -EINVAL) {
		dev_err(idev->dev, "\n%s: stride value incorrect for tiling",
			__func__);
		return -EINVAL;
	}
	SET_FIELD(reg, VPP_MMU_TILE_CFG_TILE_STRIDE, tile_stride);
	vpp_mmu_write(idev, VPP_MMU_TILE_CFG_OFFSET, reg);

	/* Set the min and max address. This means that all device virtual
	 * addresses are tiled.
	 */
	vpp_mmu_write(idev, VPP_MMU_TILE_MIN_ADDR_OFFSET, 0x0);
	vpp_mmu_write(idev, VPP_MMU_TILE_MAX_ADDR_OFFSET, 0xFFFFFFFF);

	return 0;
}

