/*
 * Copyright (c) 2011-2017 Imagination Technologies Ltd.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "core.h"
#include "utils.h"
#ifdef CONFIG_PM_SLEEP
#include <linux/pm_runtime.h>
#endif

int get_rate_prod(struct cmd_tx_ctrl *txcmd,
		     struct img_priv *priv)
{

	unsigned int index;

	if (priv->params->production_test != 1)
		return -80;

	index = 0;
	txcmd->rate_flags[index] =
		priv->params->fixed_rate_flags;

	if (priv->params->tx_fixed_mcs_indx != -1) {
		txcmd->rate[index] = 0x80;
		txcmd->rate[index] |=
		    (priv->params->tx_fixed_mcs_indx);
		txcmd->num_spatial_streams[index] =
		    priv->params->num_spatial_streams;
		txcmd->bcc_or_ldpc =
		    priv->params->fixed_rate_bcc_or_ldpc;
		txcmd->stbc_enabled =
		    priv->params->fixed_rate_stbc_enabled;
		update_mcs_packet_stat(
		    priv->params->tx_fixed_mcs_indx,
		    txcmd->rate_flags[index], priv);
	} else if (priv->params->production_test == 1 &&
		   priv->params->tx_fixed_rate != -1) {
		txcmd->rate[index] = 0x00;
		if (priv->params->tx_fixed_rate == 55)
			txcmd->rate[index] |=
			 ((priv->params->tx_fixed_rate) /
			  5);
		else
			txcmd->rate[index] |=
			  ((priv->params->tx_fixed_rate *
			    10) / 5);
		txcmd->num_spatial_streams[index] = 1;
		txcmd->bcc_or_ldpc = 0;
		txcmd->stbc_enabled = 0;
	} else {
		WARN_ON(1);
		rcu_read_unlock();
		return -90;
	}
	txcmd->num_rates = 1;
	txcmd->rate_retries[index] = 1;
	txcmd->rate_preamble_type[index] =
		priv->params->fixed_rate_preamble_type;

	return 0;
}


unsigned char bss_addr[6] = {72, 14, 29, 35, 31, 52};
void proc_bss_info_changed(unsigned char *mac_addr, int value)
{
		int temp = 0, i = 0, j = 0, ret = 0;

		get_random_bytes(&j, sizeof(j));
		for (i = 5; i > 0; i--) {
			j = j % (i+1);
			temp = bss_addr[i];
			bss_addr[i] = bss_addr[j];
			bss_addr[j] = temp;
			}
		CALL_RPU(rpu_prog_vif_bssid,
			  0,
			  mac_addr,
			  bss_addr);
prog_rpu_fail:
	return;
}

void packet_generation(unsigned long data)
{
		struct img_priv *priv = (struct img_priv *)data;
		unsigned char *mac_addr = priv->if_mac_addresses[0].addr;
		struct ieee80211_hdr hdr = {0};
		struct sk_buff *skb;
		unsigned char broad_addr[6] = {0xff, 0xff, 0xff,
					       0xff, 0xff, 0xff};
		u16 hdrlen = 26;

		/*LOOP_START*/
		/*PREPARE_SKB_LIST and SEND*/

		skb = alloc_skb(priv->params->payload_length + hdrlen,
				GFP_ATOMIC);
		img_ether_addr_copy(hdr.addr1, broad_addr);
		img_ether_addr_copy(hdr.addr2, mac_addr);
		img_ether_addr_copy(hdr.addr3, bss_addr);
		hdr.frame_control = cpu_to_le16(IEEE80211_FTYPE_DATA |
						IEEE80211_STYPE_QOS_DATA);
		memcpy(skb_put(skb, hdrlen), &hdr, hdrlen);
		memset(skb_put(skb, priv->params->payload_length), 0xAB,
			priv->params->payload_length);

		/*LOOP_END*/
		skb_queue_tail(&priv->tx.proc_tx_list[0], skb);
		rpu_proc_tx(priv);

}

int rpu_proc_tx(struct img_priv *priv)
{
	struct cmd_tx_ctrl *tx_cmd;
	struct sk_buff *tmp, *skb, *skb_first, *tx_cmd_skb;
	struct sk_buff_head *skb_list;
	struct ieee80211_hdr *mac_hdr;
	unsigned int descriptor_id = 0, queue = WLAN_AC_BE, pkt = 0;
	u16 hdrlen = 26;
	dma_addr_t phy_addr;
	unsigned int cmd_tx_size;

	spin_lock_bh(&priv->tx.lock);

	skb_list = &priv->tx.proc_tx_list[descriptor_id];

	skb_first = skb_peek(skb_list);

	if (!skb_first) {
		spin_unlock_bh(&priv->tx.lock);
		return -10;
	}

	cmd_tx_size = sizeof(struct cmd_tx_ctrl) +
			(skb_queue_len(skb_list) * MAX_GRAM_PAYLOAD_LEN);

	tx_cmd_skb = alloc_skb(cmd_tx_size, GFP_KERNEL);

	if (!tx_cmd_skb) {
		spin_unlock_bh(&priv->tx.lock);
		rcu_read_unlock();
		return -ENOMEM;
	}

	skb_put(tx_cmd_skb, cmd_tx_size);

	tx_cmd = kzalloc(cmd_tx_size, GFP_KERNEL);

	if (!tx_cmd) {
		spin_unlock_bh(&priv->tx.lock);
		rcu_read_unlock();
		kfree(tx_cmd_skb);
		return -ENOMEM;
	}

	/* HAL UMAC-LMAC HDR*/
	tx_cmd->hdr.id = RPU_CMD_TX;
	/* Keep the queue num and pool id in descriptor id */
	tx_cmd->hdr.descriptor_id = 0;
	tx_cmd->hdr.descriptor_id |= ((queue & 0x0000FFFF) << 16);
	tx_cmd->hdr.descriptor_id |= (descriptor_id & 0x0000FFFF);
	/* Not used anywhere currently */
	tx_cmd->hdr.length = sizeof(struct cmd_tx_ctrl);

	/* RPU_CMD_TX*/
	tx_cmd->if_index = 0;
	tx_cmd->queue_num = queue;
	tx_cmd->more_frms = 0;
	tx_cmd->descriptor_id = descriptor_id;
	tx_cmd->num_frames_per_desc = skb_queue_len(skb_list);
	tx_cmd->aggregate_mpdu = AMPDU_AGGR_DISABLED;

	tx_cmd->config_mac_hdr_len = hdrlen;

	get_rate_prod(tx_cmd, priv);

	pkt = 0;
	skb_queue_walk_safe(skb_list, skb, tmp) {
		struct ieee80211_tx_info *tx_info;

		if (!skb || (pkt > tx_cmd->num_frames_per_desc))
			break;

		tx_info = IEEE80211_SKB_CB(skb_first);
		mac_hdr = (struct ieee80211_hdr *)skb->data;
		/* Complete packet length*/
		tx_cmd->pkt_length[pkt] = skb->len;

		/* We move the 11hdr from skb to RPU_CMD_TX, this is part of
		 * online DMA changes, HW expects only data portion
		 * While DMA. Not requried for loopback
		 */
		memcpy((unsigned char *)tx_cmd+
			sizeof(struct cmd_tx_ctrl)+
			(pkt * MAX_GRAM_PAYLOAD_LEN),
			mac_hdr, hdrlen);

		skb_pull(skb, hdrlen);
		if (hal_ops.map_tx_buf(descriptor_id, pkt,
				       skb->data, skb->len, &phy_addr)) {
			spin_unlock_bh(&priv->tx.lock);
			kfree(tx_cmd_skb);
			kfree(tx_cmd);
			return -30;
		}
		pkt++;
	}
	memcpy(tx_cmd_skb->data, tx_cmd, cmd_tx_size);
		hal_ops.send((void *)tx_cmd_skb,
			     (void *)skb_list,
			     tx_cmd->descriptor_id);
	/* increment tx_cmd_send_count to keep track of number of
	 * tx_cmd send
	 */
	if (skb_queue_len(skb_list) == 1)
		priv->stats->tx_cmd_send_count_single++;
	else if (skb_queue_len(skb_list) > 1)
		priv->stats->tx_cmd_send_count_multi++;

	spin_unlock_bh(&priv->tx.lock);
	kfree(tx_cmd);
	return 0;
}

int start_prod_mode(struct img_priv *priv, unsigned int val)
{

	unsigned int pri_chnl_num = 0;
	unsigned int freq_band = IEEE80211_BAND_5GHZ;
	int center_freq = 0;
	int ret = -1;

	if (priv->params->production_test != 1) {
		pr_err("start_prod_mode: Can be set in only in production mode.\n");
		goto error;
	}

	if (priv->params->init_prod) {
		pr_err("Production Test is already initialized.\n");
		goto error;
	}

#ifdef CONFIG_PM_SLEEP
	mutex_lock(&priv->mutex);
	RPU_DEBUG_RUNTIME_PM("%s runtime_pm: waking up the chip\n", priv->name);

	ret = pm_runtime_get_sync(priv->dev);

	if (ret < 0 && ret != -EACCES) {
		ret = -ENODEV;
		pr_warn("%s: pm_runtime_get_sync failed: %d\n", __func__, ret);
		pm_runtime_put_sync(priv->dev);
	} else {
		RPU_DEBUG_RUNTIME_PM("%s: pm_runtime_get_sync succees: %d\n",
				     __func__, ret);
		ret = 0;
	}
	mutex_unlock(&priv->mutex);
#endif

	pri_chnl_num = val;
	priv->params->start_prod_mode = val;
	tasklet_init(&priv->proc_tx_tasklet, packet_generation,
		     (unsigned long)priv);
	if (pri_chnl_num < 15)
		freq_band = IEEE80211_BAND_2GHZ;
	else
		freq_band = IEEE80211_BAND_5GHZ;

	center_freq =
	ieee80211_channel_to_frequency(pri_chnl_num,
				       freq_band);

	if ((priv->params->fw_loading == 1) &&
	     load_fw(priv->hw)) {
		pr_err("%s: Firmware loading failed\n",
		       priv->name);
		goto error;
	}

	if (!rpu_core_init(priv, ftm)) {
		rpu_prog_vif_ctrl(0,
				priv->if_mac_addresses[0].addr,
				IF_MODE_STA_IBSS,
				IF_ADD);

		proc_bss_info_changed(
				priv->if_mac_addresses[0].addr,
				val);

		rpu_prog_channel(pri_chnl_num,
				 center_freq,
				    0,
				 /* It will be overwritten through procfs*/
				    0,
				    0,
				 freq_band);
		skb_queue_head_init(&priv->tx.proc_tx_list[0]);
		priv->params->init_prod = 1;
		priv->state = STARTED;
		uccp_reinit = 0;
	 } else {
		pr_err("RPU Initialization Failed\n");
		priv->params->init_prod = 0;
	}
	ret = 0;
error:
	return ret;
}

int stop_prod_mode(struct img_priv *priv, unsigned int val)
{
	int ret = -1;

	if (!priv->params->init_prod) {
		pr_err("Prod mode is not initialized\n");
		goto error;
	}

	tasklet_kill(&priv->proc_tx_tasklet);

	if (!uccp_reinit)
		stop(priv->hw);

	priv->params->start_prod_mode = 0;
	priv->params->pkt_gen_val = 1;
	priv->params->init_prod = 0;
	priv->params->init_pkt_gen = 0;
	ret = 0;
error:
	return -1;
}

int start_packet_gen(struct img_priv *priv, int sval)
{
	int ret = -1;

	if (!priv->params->init_prod) {
		pr_err("NEW Production Mode is not Initialized\n");
		goto error;
	}

	if (priv->params->init_pkt_gen) {
		pr_err("packet gen is already running\n");
		goto error;
	}

	if (priv->params->tx_fixed_mcs_indx == -1 &&
		priv->params->tx_fixed_rate == -1) {
		pr_err("Either tx_fixed_mcs_index Or tx_fixed_rate should be set, both can't be NULL.\n");
		goto error;
	}

	priv->params->init_pkt_gen = 1;

	priv->params->pkt_gen_val = sval;

	if (sval != 0)
		tasklet_schedule(&priv->proc_tx_tasklet);
	ret = 0;
error:
	return -1;
}

int stop_packet_gen(struct img_priv *priv, int sval)
{
	int ret = -1;

	if (!priv->params->init_prod) {
		pr_err("NEW Production Mode is not Initialized\n");
		goto error;
	}

	priv->params->pkt_gen_val = 1;
	priv->params->init_pkt_gen = 0;
	tasklet_kill(&priv->proc_tx_tasklet);
	ret = 0;
error:
	return -1;

}

void rpu_proc_tx_complete(struct rpu_event_tx_done *tx_done,
			     void *context)
{

	struct img_priv *priv = (struct img_priv *)context;
	struct sk_buff *skb, *tmp;
	struct sk_buff_head *tx_done_list;
	unsigned int pkt = 0;

	tx_done_list = &priv->tx.proc_tx_list[tx_done->descriptor_id];
	priv->stats->tx_done_recv_count++;
	update_aux_adc_voltage(priv, tx_done->pdout_voltage);
	skb_queue_walk_safe(tx_done_list, skb, tmp) {
		__skb_unlink(skb, tx_done_list);
		if (!skb)
			continue;
		hal_ops.unmap_tx_buf(tx_done->descriptor_id, pkt);
		dev_kfree_skb_any(skb);
		pkt++;
	}

	/*send NEXT packet list*/
	if ((priv->params->pkt_gen_val == -1) ||
	    (--priv->params->pkt_gen_val > 0))
		tasklet_schedule(&priv->proc_tx_tasklet);

	if (priv->params->pkt_gen_val == 0)
		priv->params->init_pkt_gen = 0;
}
