/*
 * Cashew-Palma SoC pinctrl driver
 *
 * Copyright (C) 2016 Imagination Technologies Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/of_device.h>
#include <linux/pinctrl/pinconf.h>
#include <linux/pinctrl/pinconf-generic.h>
#include <linux/pinctrl/pinctrl.h>

#include "pinctrl-cashew.h"

#define PAD_SENSOR_IF0_D0 0
#define PAD_SENSOR_IF0_D1 1
#define PAD_SENSOR_IF0_CLK 2
#define PAD_SENSOR_IF0_D2 3
#define PAD_SENSOR_IF0_D3 4
#define PAD_SENSOR_IF1_DO 5
#define PAD_SENSOR_IF1_D1 6
#define PAD_SENSOR_IF1_CLK0 7
#define PAD_SENSOR_IF1_D2 8
#define PAD_SENSOR_IF1_D3 9
#define PAD_SENSOR_IF1_D4 10
#define PAD_SENSOR_IF1_D5 11
#define PAD_SENSOR_IF1_CLK1 12
#define PAD_SENSOR_IF1_D6 13
#define PAD_SENSOR_IF1_D7 14
#define PAD_SENSOR_IF1_D8 15
#define PAD_SENSOR_IF1_D9 16
#define PAD_SENSOR_IF1_CLK2 17
#define PAD_SENSOR_IF1_D1O 18
#define PAD_SENSOR_IF1_D11 19
#define PAD_SENSOR_IF1_D12 20
#define PAD_SENSOR_IF1_D13 21
#define PAD_SENSOR_IF1_CLK3 22
#define PAD_SENSOR_IF1_D14 23
#define PAD_SENSOR_IF1_D15 24
#define PAD_I2S0_MCLK 25
#define PAD_I2S0_LRCLK 26
#define PAD_I2S0_BCLK 27
#define PAD_I2S0_DOUT0 28
#define PAD_I2S0_DOUT1 29
#define PAD_I2S0_DOUT2 30
#define PAD_I2S0_DOUT3 31
#define PAD_I2S0_DOUT4 32
#define PAD_I2S0_DOUT5 33
#define PAD_I2S0_LRCLK_IN 34
#define PAD_I2S0_BCLK_IN 35
#define PAD_I2S0_DIN0 36
#define PAD_I2S0_DIN1 37
#define PAD_I2S0_DIN2 38
#define PAD_I2S0_DIN3 39
#define PAD_I2S0_DIN4 40
#define PAD_I2S0_DIN5 41
#define PAD_I2S1_MCLK 42
#define PAD_I2S1_LRCLK 43
#define PAD_I2S1_BCLK 44
#define PAD_I2S1_DOUT0 45
#define PAD_I2S1_LRCLK_IN 46
#define PAD_I2S1_BCLK_IN 47
#define PAD_I2S1_DIN0 48
#define PAD_SPDIF_OUT 49
#define PAD_SPDIF_IN 50
#define PAD_GPIO0 51
#define PAD_GPIO1 52
#define PAD_GPIO2 53
#define PAD_GPIO3 54
#define PAD_GPIO4 55
#define PAD_GPIO5 56
#define PAD_GPIO6 57
#define PAD_GPIO7 58
#define PAD_GPIO8 59
#define PAD_GPIO9 60
#define PAD_GPIO10 61
#define PAD_GPIO11 62
#define PAD_GPIO12 63
#define PAD_GPIO13 64
#define PAD_GPIO14 65
#define PAD_GPIO15 66
#define PAD_I2C0_SDAT 67
#define PAD_I2C0_SCLK 68
#define PAD_I2C1_SDAT 69
#define PAD_I2C1_SCLK 70
#define PAD_I2C2_SDAT 71
#define PAD_I2C2_SCLK 72
#define PAD_I2C3_SDAT 73
#define PAD_I2C3_SCLK 74
#define PAD_I2C4_SDAT 75
#define PAD_I2C4_SCLK 76
#define PAD_I2C5_SDAT 77
#define PAD_I2C5_SCLK 78
#define PAD_I2C6_SDAT 79
#define PAD_I2C6_SCLK 80
#define PAD_I2C7_SDAT 81
#define PAD_I2C7_SCLK 82
#define PAD_UART0_TXD 83
#define PAD_UART0_RXD 84
#define PAD_UART0_CTS 85
#define PAD_UART0_RTS 86
#define PAD_UART1_TXD 87
#define PAD_UART1_RXD 88
#define PAD_UART1_CTS 89
#define PAD_UART1_RTS 90
#define PAD_PDM_PWD_0 91
#define PAD_PDM_PWD_1 92
#define PAD_PDM_PWD_2 93
#define PAD_PDM_PWD_3 94
#define PAD_IMG_JTAG_TCK 95
#define PAD_IMG_JTAG_TDI 96
#define PAD_IMG_JTAG_TDO 97
#define PAD_IMG_JTAG_TMS 98
#define PAD_IMG_JTAG_TRST 99
#define PAD_MIPS_JTAG_TCK 100
#define PAD_MIPS_JTAG_TDI 101
#define PAD_MIPS_JTAG_TDO 102
#define PAD_MIPS_JTAG_TMS 103
#define PAD_MIPS_JTAG_TRST 104
#define PAD_CLK_OUT0 105
#define PAD_CLK_OUT1 106
#define PAD_SPI0_CS0 107
#define PAD_SPI0_CS1 108
#define PAD_SPI0_CS2 109
#define PAD_SPI0_CS3 110
#define PAD_SPI0_CS4 111
#define PAD_SPI0_MCLK 112
#define PAD_SPI0_D0 113
#define PAD_SPI0_D1 114
#define PAD_SPI0_D2 115
#define PAD_SPI0_D3 116
#define PAD_SPI1_CS0 117
#define PAD_SPI1_CS1 118
#define PAD_SPI1_CS2 119
#define PAD_SPI1_CS3 120
#define PAD_SPI1_CS4 121
#define PAD_SPI1_MCLK 122
#define PAD_SPI1_D0 123
#define PAD_SPI1_D1 124
#define PAD_SPI1_D2 125
#define PAD_SPI1_D3 126
#define PAD_SDHOST_C0_CLK 127
#define PAD_SDHOST_C0_CMD 128
#define PAD_SDHOST_C0_DATA0 129
#define PAD_SDHOST_C0_DATA1 130
#define PAD_SDHOST_C0_DATA2 131
#define PAD_SDHOST_C0_DATA3 132
#define PAD_SDHOST_C0_DATA4 133
#define PAD_SDHOST_C0_DATA5 134
#define PAD_SDHOST_C0_DATA6 135
#define PAD_SDHOST_C0_DATA7 136
#define PAD_SDHOST_C0_VOL_SEL_0 137
#define PAD_SDHOST_C0_VOL_SEL_1 138
#define PAD_SDHOST_C0_VOL_SEL_2 139
#define PAD_SDHOST_C0_VOL_ON 140
#define PAD_SDHOST_C0_OD_PP 141
#define PAD_SDHOST_C0_CD_N 142
#define PAD_SDHOST_C0_WP 143
#define PAD_SDHOST_C0_RST_N 144
#define PAD_SDHOST_C1_CLK 145
#define PAD_SDHOST_C1_CMD 146
#define PAD_SDHOST_C1_DATA0 147
#define PAD_SDHOST_C1_DATA1 148
#define PAD_SDHOST_C1_DATA2 149
#define PAD_SDHOST_C1_DATA3 150
#define PAD_SDHOST_C1_VOL_SEL_0 151
#define PAD_SDHOST_C1_VOL_SEL_1 152
#define PAD_SDHOST_C1_VOL_SEL_2 153
#define PAD_SDHOST_C1_VOL_ON 154
#define PAD_SDHOST_C1_CD_N 155
#define PAD_SDHOST_C1_WP 156
#define PAD_SDHOST_C2_CLK 157
#define PAD_SDHOST_C2_CMD 158
#define PAD_SDHOST_C2_DATA0 159
#define PAD_SDHOST_C2_DATA1 160
#define PAD_SDHOST_C2_DATA2 161
#define PAD_SDHOST_C2_DATA3 162
#define PAD_SDHOST_C2_VOL_SEL_0 163
#define PAD_SDHOST_C2_VOL_SEL_1 164
#define PAD_SDHOST_C2_VOL_SEL_2 165
#define PAD_SDHOST_C2_VOL_ON 166
#define PAD_SDHOST_C2_CD_N 167
#define PAD_SDHOST_C2_WP 168
#define PAD_ETH_CLK_RX 169
#define PAD_ETH_CLK_TX 170
#define PAD_ETH_CLK 171
#define PAD_ETH_TXEN 172
#define PAD_ETH_TXER 173
#define PAD_ETH_TXD0 174
#define PAD_ETH_TXD1 175
#define PAD_ETH_TXD2 176
#define PAD_ETH_TXD3 177
#define PAD_ETH_CRS 178
#define PAD_ETH_COL 179
#define PAD_ETH_RXDV 180
#define PAD_ETH_RXER 181
#define PAD_ETH_RXD0 182
#define PAD_ETH_RXD1 183
#define PAD_ETH_RXD2 184
#define PAD_ETH_RXD3 185
#define PAD_ETH_MDC 186
#define PAD_ETH_MDIO 187
#define PAD_ETH_INT 188
#define PAD_NAND_RB 189
#define PAD_NAND_RE 190
#define PAD_NAND_WE 191
#define PAD_NAND_CLE 192
#define PAD_NAND_ALE 193
#define PAD_NAND_WP_L_0 194
#define PAD_NAND_CE 195
#define PAD_NAND_IO_DQ_0 196
#define PAD_NAND_IO_DQ_1 197
#define PAD_NAND_IO_DQ_2 198
#define PAD_NAND_IO_DQ_3 199
#define PAD_NAND_IO_DQ_4 200
#define PAD_NAND_IO_DQ_5 201
#define PAD_NAND_IO_DQ_6 202
#define PAD_NAND_IO_DQ_7 203
#define PAD_NAND_IO_DQ_8 204
#define PAD_NAND_IO_DQ_9 205
#define PAD_NAND_IO_DQ_10 206
#define PAD_NAND_IO_DQ_11 207
#define PAD_NAND_IO_DQ_12 208
#define PAD_NAND_IO_DQ_13 209
#define PAD_NAND_IO_DQ_14 210
#define PAD_NAND_IO_DQ_15 211
#define PAD_NAND_DQS 212
#define PAD_NAND_DQS_OUT 213
#define PAD_WLAN_TX_ON0 214
#define PAD_WLAN_RX_ON0 215
#define PAD_WLAN_RF_SHDN0 216
#define PAD_WLAN_RF_STBY0 217
#define PAD_WLAN_PA_ON0 218
#define PAD_WLAN_RX_HP0 219
#define PAD_WLAN_FASTWR0 220
#define PAD_WLAN_FASTWR1 221
#define PAD_WLAN_FASTWR2 222
#define PAD_WLAN_FASTWR3 223
#define PAD_WLAN_FASTWR4 224
#define PAD_WLAN_FASTWR5 225
#define PAD_WLAN_FASTWR6 226
#define PAD_WLAN_FASTWR7 227
#define PAD_WLAN_TX_ON1 228
#define PAD_WLAN_RX_ON1 229
#define PAD_WLAN_RF_SHDN1 230
#define PAD_WLAN_RF_STBY1 231
#define PAD_WLAN_PA_ON1 232
#define PAD_WLAN_RX_HP1 233
#define PAD_WLAN_ANT_SEL0_0 234
#define PAD_WLAN_ANT_SEL0_1 235
#define PAD_WLAN_ANT_SEL1_0 236
#define PAD_WLAN_ANT_SEL1_1 237
#define PAD_BT_MODEM_TO_BASEBAND_RX_CLK 238
#define PAD_BT_MODEM_TO_BASEBAND_RX_VALID 239
#define PAD_BT_MODEM_TO_BASEBAND_RX_DATA0 240
#define PAD_BT_MODEM_TO_BASEBAND_RX_DATA1 241
#define PAD_BT_MODEM_TO_BASEBAND_RX_DATA2 242
#define PAD_BT_BASEBAND_TO_MODEM_RATE0 243
#define PAD_BT_BASEBAND_TO_MODEM_RATE1 244
#define PAD_BT_BASEBAND_TO_MODEM_TX_EN 245
#define PAD_BT_BASEBAND_TO_MODEM_RX_EN 246
#define PAD_BT_BASEBAND_TO_MODEM_SYNC_P 247
#define PAD_BT_BASEBAND_TO_MODEM_TX_EDR 248
#define PAD_BT_BASEBAND_TO_MODEM_TX_VALID 249
#define PAD_BT_BASEBAND_TO_MODEM_TX_DATA0 250
#define PAD_BT_BASEBAND_TO_MODEM_TX_DATA1 251
#define PAD_BT_BASEBAND_TO_MODEM_TX_DATA2 252
#define PAD_BT_ENABLE_OSC 253
#define PAD_BT_ENABLE_RM 254
#define PAD_LTC_SPI_CLK 255
#define PAD_LTC_SPI_SS_RF 256
#define PAD_LTC_SPI_SS_MDM 257
#define PAD_LTC_SPI_MOSI 258
#define PAD_LTC_SPI_MISO 259
#define PAD_LTC_SPI_BIDIR_OEN 260
#define PAD_EXT_BT_MBSY 261
#define PAD_EXT_BT_RXNTX 262
#define PAD_EXT_BT_PTI0 263
#define PAD_EXT_BT_PTI1 264
#define PAD_EXT_BT_CONFIRM 265
#define PAD_EXT_BT_DENY 266
#define PAD_RF_SPI_CLK 267
#define PAD_RF_SPI_SS 268
#define PAD_RF_SPI_MOSI 269
#define PAD_RF_SPI_MISO 270
#define PAD_RF_SPI_BIDIR_OEN 271
#define PAD_XTAL3_IN 272
#define PAD_LVDS_TX_CLK 273
#define PAD_LVDS_TX_0 274
#define PAD_LVDS_TX_1 275
#define PAD_LVDS_TX_2 276
#define PAD_LVDS_TX_3 277
#define PAD_HDMI_TX_0_CEC 278
#define PAD_HDMI_TX_0_DDC_SCL 279
#define PAD_HDMI_TX_0_DDC_SDA 280
#define PAD_HDMI_TX_1_CEC 281
#define PAD_HDMI_TX_1_DDC_SCL 282
#define PAD_HDMI_TX_1_DDC_SDA 283
#define PAD_HDMI_RX_CEC 284
#define PAD_HDMI_RX_DDC_SCL 285
#define PAD_HDMI_RX_DDC_SDA 286
#define PAD_AVCLK_OUT0 287
#define PAD_AVCLK_OUT1 288
#define PAD_XTAL2_IN 289

#define F1(pin) ((pin > PAD_XTAL3_IN) ? (pin - 1) : (pin))
#define F2(pin) ((pin > PAD_LVDS_TX_3) ? (pin - 5) : (pin))

#define PADS_CR_SCHMITT_EN0 0x0
#define PADS_SCHMITT_EN_REG(pin) (PADS_CR_SCHMITT_EN0 + 0x4 * (F1(pin) / 32))
#define PADS_SCHMITT_EN_SHIFT(pin) (F1(pin) % 32)
#define PADS_SCHMITT_EN_MASK 0x1
#define PADS_SCHMITT_EN_NUM_REGS 9

#define PADS_CR_PU_PD0 0x50
#define PADS_PU_PD_REG(pin) (PADS_CR_PU_PD0 + 0x4 * (F1(F2(pin)) / 16))
#define PADS_PU_PD_SHIFT(pin) (2 * (F1(F2(pin)) % 16))
#define PADS_PU_PD_MASK 0x3
#define PADS_PU_PD_NUM_REGS 17

#define PADS_CR_SLEW_RATE0 0xEC
#define PADS_SLEW_RATE_REG(pin) (PADS_CR_SLEW_RATE0 + 0x4 * (F1(pin) / 32))
#define PADS_SLEW_RATE_SHIFT(pin) (F1(pin) % 32)
#define PADS_SLEW_RATE_MASK 0x1
#define PADS_SLEW_RATE_NUM_REGS 9

#define PADS_CR_DRIVE_STR0 0x114
#define PADS_DRIVE_STR_REG(pin) (PADS_CR_DRIVE_STR0 + 0x4 * (F2(pin) / 16))
#define PADS_DRIVE_STR_SHIFT(pin) (2 * (F2(pin) % 16))
#define PADS_DRIVE_STR_MASK 0x3
#define PADS_DRIVE_STR_NUM_REGS 17

#define MUX_GPIO15 27
#define MUX_I2C7 30
#define F3(pin) ((pin > MUX_GPIO15) ? (pin - 1) : (pin))

#define PADS_CR_SRC_SEL0 0x1E0
#define PADS_SRC_SEL_REG(pin) (PADS_CR_SRC_SEL0 + 0x4 * (F3(pin) / 10))
#define PADS_SRC_SEL_SHIFT(pin) (((pin) > MUX_GPIO15) && ((pin) <= MUX_I2C7) ? \
				((3 * ((pin) - MUX_GPIO15)) + 19) : \
				(3 * (F3(pin) % 10)))
#define PADS_SRC_SEL_MASK(pin) (((pin) == MUX_GPIO15) ? 0x1 : 0x7)
#define PADS_SRC_SEL_NUM_REGS 8

struct img_palma_pinctrl_ctx {
	u32 schmitt_en[PADS_SCHMITT_EN_NUM_REGS];
	u32 pu_pd[PADS_PU_PD_NUM_REGS];
	u32 slew_rate[PADS_SLEW_RATE_NUM_REGS];
	u32 drive_str[PADS_DRIVE_STR_NUM_REGS];
	u32 src_sel[PADS_SRC_SEL_NUM_REGS];
};

static const struct pinctrl_pin_desc palma_pins[] = {
	PINCTRL_PIN(PAD_SENSOR_IF0_D0, "SENSOR_IF0_D0"),
	PINCTRL_PIN(PAD_SENSOR_IF0_D1, "SENSOR_IF0_D1"),
	PINCTRL_PIN(PAD_SENSOR_IF0_CLK, "SENSOR_IF0_CLK"),
	PINCTRL_PIN(PAD_SENSOR_IF0_D2, "SENSOR_IF0_D2"),
	PINCTRL_PIN(PAD_SENSOR_IF0_D3, "SENSOR_IF0_D3"),
	PINCTRL_PIN(PAD_SENSOR_IF1_DO, "SENSOR_IF1_DO"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D1, "SENSOR_IF1_D1"),
	PINCTRL_PIN(PAD_SENSOR_IF1_CLK0, "SENSOR_IF1_CLK0"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D2, "SENSOR_IF1_D2"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D3, "SENSOR_IF1_D3"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D4, "SENSOR_IF1_D4"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D5, "SENSOR_IF1_D5"),
	PINCTRL_PIN(PAD_SENSOR_IF1_CLK1, "SENSOR_IF1_CLK1"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D6, "SENSOR_IF1_D6"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D7, "SENSOR_IF1_D7"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D8, "SENSOR_IF1_D8"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D9, "SENSOR_IF1_D9"),
	PINCTRL_PIN(PAD_SENSOR_IF1_CLK2, "SENSOR_IF1_CLK2"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D1O, "SENSOR_IF1_D1O"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D11, "SENSOR_IF1_D11"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D12, "SENSOR_IF1_D12"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D13, "SENSOR_IF1_D13"),
	PINCTRL_PIN(PAD_SENSOR_IF1_CLK3, "SENSOR_IF1_CLK3"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D14, "SENSOR_IF1_D14"),
	PINCTRL_PIN(PAD_SENSOR_IF1_D15, "SENSOR_IF1_D15"),
	PINCTRL_PIN(PAD_I2S0_MCLK, "I2S0_MCLK"),
	PINCTRL_PIN(PAD_I2S0_LRCLK, "I2S0_LRCLK"),
	PINCTRL_PIN(PAD_I2S0_BCLK, "I2S0_BCLK"),
	PINCTRL_PIN(PAD_I2S0_DOUT0, "I2S0_DOUT0"),
	PINCTRL_PIN(PAD_I2S0_DOUT1, "I2S0_DOUT1"),
	PINCTRL_PIN(PAD_I2S0_DOUT2, "I2S0_DOUT2"),
	PINCTRL_PIN(PAD_I2S0_DOUT3, "I2S0_DOUT3"),
	PINCTRL_PIN(PAD_I2S0_DOUT4, "I2S0_DOUT4"),
	PINCTRL_PIN(PAD_I2S0_DOUT5, "I2S0_DOUT5"),
	PINCTRL_PIN(PAD_I2S0_LRCLK_IN, "I2S0_LRCLK_IN"),
	PINCTRL_PIN(PAD_I2S0_BCLK_IN, "I2S0_BCLK_IN"),
	PINCTRL_PIN(PAD_I2S0_DIN0, "I2S0_DIN0"),
	PINCTRL_PIN(PAD_I2S0_DIN1, "I2S0_DIN1"),
	PINCTRL_PIN(PAD_I2S0_DIN2, "I2S0_DIN2"),
	PINCTRL_PIN(PAD_I2S0_DIN3, "I2S0_DIN3"),
	PINCTRL_PIN(PAD_I2S0_DIN4, "I2S0_DIN4"),
	PINCTRL_PIN(PAD_I2S0_DIN5, "I2S0_DIN5"),
	PINCTRL_PIN(PAD_I2S1_MCLK, "I2S1_MCLK"),
	PINCTRL_PIN(PAD_I2S1_LRCLK, "I2S1_LRCLK"),
	PINCTRL_PIN(PAD_I2S1_BCLK, "I2S1_BCLK"),
	PINCTRL_PIN(PAD_I2S1_DOUT0, "I2S1_DOUT0"),
	PINCTRL_PIN(PAD_I2S1_LRCLK_IN, "I2S1_LRCLK_IN"),
	PINCTRL_PIN(PAD_I2S1_BCLK_IN, "I2S1_BCLK_IN"),
	PINCTRL_PIN(PAD_I2S1_DIN0, "I2S1_DIN0"),
	PINCTRL_PIN(PAD_SPDIF_OUT, "SPDIF_OUT"),
	PINCTRL_PIN(PAD_SPDIF_IN, "SPDIF_IN"),
	PINCTRL_PIN(PAD_GPIO0, "GPIO0"),
	PINCTRL_PIN(PAD_GPIO1, "GPIO1"),
	PINCTRL_PIN(PAD_GPIO2, "GPIO2"),
	PINCTRL_PIN(PAD_GPIO3, "GPIO3"),
	PINCTRL_PIN(PAD_GPIO4, "GPIO4"),
	PINCTRL_PIN(PAD_GPIO5, "GPIO5"),
	PINCTRL_PIN(PAD_GPIO6, "GPIO6"),
	PINCTRL_PIN(PAD_GPIO7, "GPIO7"),
	PINCTRL_PIN(PAD_GPIO8, "GPIO8"),
	PINCTRL_PIN(PAD_GPIO9, "GPIO9"),
	PINCTRL_PIN(PAD_GPIO10, "GPIO10"),
	PINCTRL_PIN(PAD_GPIO11, "GPIO11"),
	PINCTRL_PIN(PAD_GPIO12, "GPIO12"),
	PINCTRL_PIN(PAD_GPIO13, "GPIO13"),
	PINCTRL_PIN(PAD_GPIO14, "GPIO14"),
	PINCTRL_PIN(PAD_GPIO15, "GPIO15"),
	PINCTRL_PIN(PAD_I2C0_SDAT, "I2C0_SDAT"),
	PINCTRL_PIN(PAD_I2C0_SCLK, "I2C0_SCLK"),
	PINCTRL_PIN(PAD_I2C1_SDAT, "I2C1_SDAT"),
	PINCTRL_PIN(PAD_I2C1_SCLK, "I2C1_SCLK"),
	PINCTRL_PIN(PAD_I2C2_SDAT, "I2C2_SDAT"),
	PINCTRL_PIN(PAD_I2C2_SCLK, "I2C2_SCLK"),
	PINCTRL_PIN(PAD_I2C3_SDAT, "I2C3_SDAT"),
	PINCTRL_PIN(PAD_I2C3_SCLK, "I2C3_SCLK"),
	PINCTRL_PIN(PAD_I2C4_SDAT, "I2C4_SDAT"),
	PINCTRL_PIN(PAD_I2C4_SCLK, "I2C4_SCLK"),
	PINCTRL_PIN(PAD_I2C5_SDAT, "I2C5_SDAT"),
	PINCTRL_PIN(PAD_I2C5_SCLK, "I2C5_SCLK"),
	PINCTRL_PIN(PAD_I2C6_SDAT, "I2C6_SDAT"),
	PINCTRL_PIN(PAD_I2C6_SCLK, "I2C6_SCLK"),
	PINCTRL_PIN(PAD_I2C7_SDAT, "I2C7_SDAT"),
	PINCTRL_PIN(PAD_I2C7_SCLK, "I2C7_SCLK"),
	PINCTRL_PIN(PAD_UART0_TXD, "UART0_TXD"),
	PINCTRL_PIN(PAD_UART0_RXD, "UART0_RXD"),
	PINCTRL_PIN(PAD_UART0_CTS, "UART0_CTS"),
	PINCTRL_PIN(PAD_UART0_RTS, "UART0_RTS"),
	PINCTRL_PIN(PAD_UART1_TXD, "UART1_TXD"),
	PINCTRL_PIN(PAD_UART1_RXD, "UART1_RXD"),
	PINCTRL_PIN(PAD_UART1_CTS, "UART1_CTS"),
	PINCTRL_PIN(PAD_UART1_RTS, "UART1_RTS"),
	PINCTRL_PIN(PAD_PDM_PWD_0, "PDM_PWD_0"),
	PINCTRL_PIN(PAD_PDM_PWD_1, "PDM_PWD_1"),
	PINCTRL_PIN(PAD_PDM_PWD_2, "PDM_PWD_2"),
	PINCTRL_PIN(PAD_PDM_PWD_3, "PDM_PWD_3"),
	PINCTRL_PIN(PAD_IMG_JTAG_TCK, "IMG_JTAG_TCK"),
	PINCTRL_PIN(PAD_IMG_JTAG_TDI, "IMG_JTAG_TDI"),
	PINCTRL_PIN(PAD_IMG_JTAG_TDO, "IMG_JTAG_TDO"),
	PINCTRL_PIN(PAD_IMG_JTAG_TMS, "IMG_JTAG_TMS"),
	PINCTRL_PIN(PAD_IMG_JTAG_TRST, "IMG_JTAG_TRST"),
	PINCTRL_PIN(PAD_MIPS_JTAG_TCK, "MIPS_JTAG_TCK"),
	PINCTRL_PIN(PAD_MIPS_JTAG_TDI, "MIPS_JTAG_TDI"),
	PINCTRL_PIN(PAD_MIPS_JTAG_TDO, "MIPS_JTAG_TDO"),
	PINCTRL_PIN(PAD_MIPS_JTAG_TMS, "MIPS_JTAG_TMS"),
	PINCTRL_PIN(PAD_MIPS_JTAG_TRST, "MIPS_JTAG_TRST"),
	PINCTRL_PIN(PAD_CLK_OUT0, "CLK_OUT0"),
	PINCTRL_PIN(PAD_CLK_OUT1, "CLK_OUT1"),
	PINCTRL_PIN(PAD_SPI0_CS0, "SPI0_CS0"),
	PINCTRL_PIN(PAD_SPI0_CS1, "SPI0_CS1"),
	PINCTRL_PIN(PAD_SPI0_CS2, "SPI0_CS2"),
	PINCTRL_PIN(PAD_SPI0_CS3, "SPI0_CS3"),
	PINCTRL_PIN(PAD_SPI0_CS4, "SPI0_CS4"),
	PINCTRL_PIN(PAD_SPI0_MCLK, "SPI0_MCLK"),
	PINCTRL_PIN(PAD_SPI0_D0, "SPI0_D0"),
	PINCTRL_PIN(PAD_SPI0_D1, "SPI0_D1"),
	PINCTRL_PIN(PAD_SPI0_D2, "SPI0_D2"),
	PINCTRL_PIN(PAD_SPI0_D3, "SPI0_D3"),
	PINCTRL_PIN(PAD_SPI1_CS0, "SPI1_CS0"),
	PINCTRL_PIN(PAD_SPI1_CS1, "SPI1_CS1"),
	PINCTRL_PIN(PAD_SPI1_CS2, "SPI1_CS2"),
	PINCTRL_PIN(PAD_SPI1_CS3, "SPI1_CS3"),
	PINCTRL_PIN(PAD_SPI1_CS4, "SPI1_CS4"),
	PINCTRL_PIN(PAD_SPI1_MCLK, "SPI1_MCLK"),
	PINCTRL_PIN(PAD_SPI1_D0, "SPI1_D0"),
	PINCTRL_PIN(PAD_SPI1_D1, "SPI1_D1"),
	PINCTRL_PIN(PAD_SPI1_D2, "SPI1_D2"),
	PINCTRL_PIN(PAD_SPI1_D3, "SPI1_D3"),
	PINCTRL_PIN(PAD_SDHOST_C0_CLK, "SDHOST_C0_CLK"),
	PINCTRL_PIN(PAD_SDHOST_C0_CMD, "SDHOST_C0_CMD"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA0, "SDHOST_C0_DATA0"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA1, "SDHOST_C0_DATA1"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA2, "SDHOST_C0_DATA2"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA3, "SDHOST_C0_DATA3"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA4, "SDHOST_C0_DATA4"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA5, "SDHOST_C0_DATA5"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA6, "SDHOST_C0_DATA6"),
	PINCTRL_PIN(PAD_SDHOST_C0_DATA7, "SDHOST_C0_DATA7"),
	PINCTRL_PIN(PAD_SDHOST_C0_VOL_SEL_0, "SDHOST_C0_VOL_SEL_0"),
	PINCTRL_PIN(PAD_SDHOST_C0_VOL_SEL_1, "SDHOST_C0_VOL_SEL_1"),
	PINCTRL_PIN(PAD_SDHOST_C0_VOL_SEL_2, "SDHOST_C0_VOL_SEL_2"),
	PINCTRL_PIN(PAD_SDHOST_C0_VOL_ON, "SDHOST_C0_VOL_ON"),
	PINCTRL_PIN(PAD_SDHOST_C0_OD_PP, "SDHOST_C0_OD_PP"),
	PINCTRL_PIN(PAD_SDHOST_C0_CD_N, "SDHOST_C0_CD_N"),
	PINCTRL_PIN(PAD_SDHOST_C0_WP, "SDHOST_C0_WP"),
	PINCTRL_PIN(PAD_SDHOST_C0_RST_N, "SDHOST_C0_RST_N"),
	PINCTRL_PIN(PAD_SDHOST_C1_CLK, "SDHOST_C1_CLK"),
	PINCTRL_PIN(PAD_SDHOST_C1_CMD, "SDHOST_C1_CMD"),
	PINCTRL_PIN(PAD_SDHOST_C1_DATA0, "SDHOST_C1_DATA0"),
	PINCTRL_PIN(PAD_SDHOST_C1_DATA1, "SDHOST_C1_DATA1"),
	PINCTRL_PIN(PAD_SDHOST_C1_DATA2, "SDHOST_C1_DATA2"),
	PINCTRL_PIN(PAD_SDHOST_C1_DATA3, "SDHOST_C1_DATA3"),
	PINCTRL_PIN(PAD_SDHOST_C1_VOL_SEL_0, "SDHOST_C1_VOL_SEL_0"),
	PINCTRL_PIN(PAD_SDHOST_C1_VOL_SEL_1, "SDHOST_C1_VOL_SEL_1"),
	PINCTRL_PIN(PAD_SDHOST_C1_VOL_SEL_2, "SDHOST_C1_VOL_SEL_2"),
	PINCTRL_PIN(PAD_SDHOST_C1_VOL_ON, "SDHOST_C1_VOL_ON"),
	PINCTRL_PIN(PAD_SDHOST_C1_CD_N, "SDHOST_C1_CD_N"),
	PINCTRL_PIN(PAD_SDHOST_C1_WP, "SDHOST_C1_WP"),
	PINCTRL_PIN(PAD_SDHOST_C2_CLK, "SDHOST_C2_CLK"),
	PINCTRL_PIN(PAD_SDHOST_C2_CMD, "SDHOST_C2_CMD"),
	PINCTRL_PIN(PAD_SDHOST_C2_DATA0, "SDHOST_C2_DATA0"),
	PINCTRL_PIN(PAD_SDHOST_C2_DATA1, "SDHOST_C2_DATA1"),
	PINCTRL_PIN(PAD_SDHOST_C2_DATA2, "SDHOST_C2_DATA2"),
	PINCTRL_PIN(PAD_SDHOST_C2_DATA3, "SDHOST_C2_DATA3"),
	PINCTRL_PIN(PAD_SDHOST_C2_VOL_SEL_0, "SDHOST_C2_VOL_SEL_0"),
	PINCTRL_PIN(PAD_SDHOST_C2_VOL_SEL_1, "SDHOST_C2_VOL_SEL_1"),
	PINCTRL_PIN(PAD_SDHOST_C2_VOL_SEL_2, "SDHOST_C2_VOL_SEL_2"),
	PINCTRL_PIN(PAD_SDHOST_C2_VOL_ON, "SDHOST_C2_VOL_ON"),
	PINCTRL_PIN(PAD_SDHOST_C2_CD_N, "SDHOST_C2_CD_N"),
	PINCTRL_PIN(PAD_SDHOST_C2_WP, "SDHOST_C2_WP"),
	PINCTRL_PIN(PAD_ETH_CLK_RX, "ETH_CLK_RX"),
	PINCTRL_PIN(PAD_ETH_CLK_TX, "ETH_CLK_TX"),
	PINCTRL_PIN(PAD_ETH_CLK, "ETH_CLK"),
	PINCTRL_PIN(PAD_ETH_TXEN, "ETH_TXEN"),
	PINCTRL_PIN(PAD_ETH_TXER, "ETH_TXER"),
	PINCTRL_PIN(PAD_ETH_TXD0, "ETH_TXD0"),
	PINCTRL_PIN(PAD_ETH_TXD1, "ETH_TXD1"),
	PINCTRL_PIN(PAD_ETH_TXD2, "ETH_TXD2"),
	PINCTRL_PIN(PAD_ETH_TXD3, "ETH_TXD3"),
	PINCTRL_PIN(PAD_ETH_CRS, "ETH_CRS"),
	PINCTRL_PIN(PAD_ETH_COL, "ETH_COL"),
	PINCTRL_PIN(PAD_ETH_RXDV, "ETH_RXDV"),
	PINCTRL_PIN(PAD_ETH_RXER, "ETH_RXER"),
	PINCTRL_PIN(PAD_ETH_RXD0, "ETH_RXD0"),
	PINCTRL_PIN(PAD_ETH_RXD1, "ETH_RXD1"),
	PINCTRL_PIN(PAD_ETH_RXD2, "ETH_RXD2"),
	PINCTRL_PIN(PAD_ETH_RXD3, "ETH_RXD3"),
	PINCTRL_PIN(PAD_ETH_MDC, "ETH_MDC"),
	PINCTRL_PIN(PAD_ETH_MDIO, "ETH_MDIO"),
	PINCTRL_PIN(PAD_ETH_INT, "ETH_INT"),
	PINCTRL_PIN(PAD_NAND_RB, "NAND_RB"),
	PINCTRL_PIN(PAD_NAND_RE, "NAND_RE"),
	PINCTRL_PIN(PAD_NAND_WE, "NAND_WE"),
	PINCTRL_PIN(PAD_NAND_CLE, "NAND_CLE"),
	PINCTRL_PIN(PAD_NAND_ALE, "NAND_ALE"),
	PINCTRL_PIN(PAD_NAND_WP_L_0, "NAND_WP_L_0"),
	PINCTRL_PIN(PAD_NAND_CE, "NAND_CE"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_0, "NAND_IO_DQ_0"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_1, "NAND_IO_DQ_1"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_2, "NAND_IO_DQ_2"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_3, "NAND_IO_DQ_3"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_4, "NAND_IO_DQ_4"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_5, "NAND_IO_DQ_5"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_6, "NAND_IO_DQ_6"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_7, "NAND_IO_DQ_7"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_8, "NAND_IO_DQ_8"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_9, "NAND_IO_DQ_9"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_10, "NAND_IO_DQ_10"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_11, "NAND_IO_DQ_11"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_12, "NAND_IO_DQ_12"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_13, "NAND_IO_DQ_13"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_14, "NAND_IO_DQ_14"),
	PINCTRL_PIN(PAD_NAND_IO_DQ_15, "NAND_IO_DQ_15"),
	PINCTRL_PIN(PAD_NAND_DQS, "NAND_DQS"),
	PINCTRL_PIN(PAD_NAND_DQS_OUT, "NAND_DQS_OUT"),
	PINCTRL_PIN(PAD_WLAN_TX_ON0, "WLAN_TX_ON0"),
	PINCTRL_PIN(PAD_WLAN_RX_ON0, "WLAN_RX_ON0"),
	PINCTRL_PIN(PAD_WLAN_RF_SHDN0, "WLAN_RF_SHDN0"),
	PINCTRL_PIN(PAD_WLAN_RF_STBY0, "WLAN_RF_STBY0"),
	PINCTRL_PIN(PAD_WLAN_PA_ON0, "WLAN_PA_ON0"),
	PINCTRL_PIN(PAD_WLAN_RX_HP0, "WLAN_RX_HP0"),
	PINCTRL_PIN(PAD_WLAN_FASTWR0, "WLAN_FASTWR0"),
	PINCTRL_PIN(PAD_WLAN_FASTWR1, "WLAN_FASTWR1"),
	PINCTRL_PIN(PAD_WLAN_FASTWR2, "WLAN_FASTWR2"),
	PINCTRL_PIN(PAD_WLAN_FASTWR3, "WLAN_FASTWR3"),
	PINCTRL_PIN(PAD_WLAN_FASTWR4, "WLAN_FASTWR4"),
	PINCTRL_PIN(PAD_WLAN_FASTWR5, "WLAN_FASTWR5"),
	PINCTRL_PIN(PAD_WLAN_FASTWR6, "WLAN_FASTWR6"),
	PINCTRL_PIN(PAD_WLAN_FASTWR7, "WLAN_FASTWR7"),
	PINCTRL_PIN(PAD_WLAN_TX_ON1, "WLAN_TX_ON1"),
	PINCTRL_PIN(PAD_WLAN_RX_ON1, "WLAN_RX_ON1"),
	PINCTRL_PIN(PAD_WLAN_RF_SHDN1, "WLAN_RF_SHDN1"),
	PINCTRL_PIN(PAD_WLAN_RF_STBY1, "WLAN_RF_STBY1"),
	PINCTRL_PIN(PAD_WLAN_PA_ON1, "WLAN_PA_ON1"),
	PINCTRL_PIN(PAD_WLAN_RX_HP1, "WLAN_RX_HP1"),
	PINCTRL_PIN(PAD_WLAN_ANT_SEL0_0, "WLAN_ANT_SEL0_0"),
	PINCTRL_PIN(PAD_WLAN_ANT_SEL0_1, "WLAN_ANT_SEL0_1"),
	PINCTRL_PIN(PAD_WLAN_ANT_SEL1_0, "WLAN_ANT_SEL1_0"),
	PINCTRL_PIN(PAD_WLAN_ANT_SEL1_1, "WLAN_ANT_SEL1_1"),
	PINCTRL_PIN(PAD_BT_MODEM_TO_BASEBAND_RX_CLK,
		    "BT_MODEM_TO_BASEBAND_RX_CLK"),
	PINCTRL_PIN(PAD_BT_MODEM_TO_BASEBAND_RX_VALID,
		    "BT_MODEM_TO_BASEBAND_RX_VALID"),
	PINCTRL_PIN(PAD_BT_MODEM_TO_BASEBAND_RX_DATA0,
		    "BT_MODEM_TO_BASEBAND_RX_DATA0"),
	PINCTRL_PIN(PAD_BT_MODEM_TO_BASEBAND_RX_DATA1,
		    "BT_MODEM_TO_BASEBAND_RX_DATA1"),
	PINCTRL_PIN(PAD_BT_MODEM_TO_BASEBAND_RX_DATA2,
		    "BT_MODEM_TO_BASEBAND_RX_DATA2"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_RATE0,
		    "BT_BASEBAND_TO_MODEM_RATE0"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_RATE1,
		    "BT_BASEBAND_TO_MODEM_RATE1"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_TX_EN,
		    "BT_BASEBAND_TO_MODEM_TX_EN"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_RX_EN,
		    "BT_BASEBAND_TO_MODEM_RX_EN"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_SYNC_P,
		    "BT_BASEBAND_TO_MODEM_SYNC_P"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_TX_EDR,
		    "BT_BASEBAND_TO_MODEM_TX_EDR"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_TX_VALID,
		    "BT_BASEBAND_TO_MODEM_TX_VALID"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_TX_DATA0,
		    "BT_BASEBAND_TO_MODEM_TX_DATA0"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_TX_DATA1,
		    "BT_BASEBAND_TO_MODEM_TX_DATA1"),
	PINCTRL_PIN(PAD_BT_BASEBAND_TO_MODEM_TX_DATA2,
		    "BT_BASEBAND_TO_MODEM_TX_DATA2"),
	PINCTRL_PIN(PAD_BT_ENABLE_OSC, "BT_ENABLE_OSC"),
	PINCTRL_PIN(PAD_BT_ENABLE_RM, "BT_ENABLE_RM"),
	PINCTRL_PIN(PAD_LTC_SPI_CLK, "LTC_SPI_CLK"),
	PINCTRL_PIN(PAD_LTC_SPI_SS_RF, "LTC_SPI_SS_RF"),
	PINCTRL_PIN(PAD_LTC_SPI_SS_MDM, "LTC_SPI_SS_MDM"),
	PINCTRL_PIN(PAD_LTC_SPI_MOSI, "LTC_SPI_MOSI"),
	PINCTRL_PIN(PAD_LTC_SPI_MISO, "LTC_SPI_MISO"),
	PINCTRL_PIN(PAD_LTC_SPI_BIDIR_OEN, "LTC_SPI_BIDIR_OEN"),
	PINCTRL_PIN(PAD_EXT_BT_MBSY, "EXT_BT_MBSY"),
	PINCTRL_PIN(PAD_EXT_BT_RXNTX, "EXT_BT_RXNTX"),
	PINCTRL_PIN(PAD_EXT_BT_PTI0, "EXT_BT_PTI0"),
	PINCTRL_PIN(PAD_EXT_BT_PTI1, "EXT_BT_PTI1"),
	PINCTRL_PIN(PAD_EXT_BT_CONFIRM, "EXT_BT_CONFIRM"),
	PINCTRL_PIN(PAD_EXT_BT_DENY, "EXT_BT_DENY"),
	PINCTRL_PIN(PAD_RF_SPI_CLK, "RF_SPI_CLK"),
	PINCTRL_PIN(PAD_RF_SPI_SS, "RF_SPI_SS"),
	PINCTRL_PIN(PAD_RF_SPI_MOSI, "RF_SPI_MOSI"),
	PINCTRL_PIN(PAD_RF_SPI_MISO, "RF_SPI_MISO"),
	PINCTRL_PIN(PAD_RF_SPI_BIDIR_OEN, "RF_SPI_BIDIR_OEN"),
	PINCTRL_PIN(PAD_XTAL3_IN, "XTAL3_IN"),
	PINCTRL_PIN(PAD_LVDS_TX_CLK, "LVDS_TX_CLK"),
	PINCTRL_PIN(PAD_LVDS_TX_0, "LVDS_TX_0"),
	PINCTRL_PIN(PAD_LVDS_TX_1, "LVDS_TX_1"),
	PINCTRL_PIN(PAD_LVDS_TX_2, "LVDS_TX_2"),
	PINCTRL_PIN(PAD_LVDS_TX_3, "LVDS_TX_3"),
	PINCTRL_PIN(PAD_HDMI_TX_0_CEC, "HDMI_TX_0_CEC"),
	PINCTRL_PIN(PAD_HDMI_TX_0_DDC_SCL, "HDMI_TX_0_DDC_SCL"),
	PINCTRL_PIN(PAD_HDMI_TX_0_DDC_SDA, "HDMI_TX_0_DDC_SDA"),
	PINCTRL_PIN(PAD_HDMI_TX_1_CEC, "HDMI_TX_1_CEC"),
	PINCTRL_PIN(PAD_HDMI_TX_1_DDC_SCL, "HDMI_TX_1_DDC_SCL"),
	PINCTRL_PIN(PAD_HDMI_TX_1_DDC_SDA, "HDMI_TX_1_DDC_SDA"),
	PINCTRL_PIN(PAD_HDMI_RX_CEC, "HDMI_RX_CEC"),
	PINCTRL_PIN(PAD_HDMI_RX_DDC_SCL, "HDMI_RX_DDC_SCL"),
	PINCTRL_PIN(PAD_HDMI_RX_DDC_SDA, "HDMI_RX_DDC_SDA"),
	PINCTRL_PIN(PAD_AVCLK_OUT0, "AVCLK_OUT0"),
	PINCTRL_PIN(PAD_AVCLK_OUT1, "AVCLK_OUT1"),
	PINCTRL_PIN(PAD_XTAL2_IN, "XTAL2_IN"),
};

static const unsigned afe_out_clk_0_pins[] = {PAD_LTC_SPI_BIDIR_OEN};
static const unsigned afe_out_clk_1_pins[] = {PAD_RF_SPI_BIDIR_OEN};
static const unsigned bt_baseband_to_modem_pins[] = {
	PAD_BT_BASEBAND_TO_MODEM_RATE0, PAD_BT_BASEBAND_TO_MODEM_RATE1,
	PAD_BT_BASEBAND_TO_MODEM_RX_EN, PAD_BT_BASEBAND_TO_MODEM_SYNC_P,
	PAD_BT_BASEBAND_TO_MODEM_TX_DATA0, PAD_BT_BASEBAND_TO_MODEM_TX_DATA1,
	PAD_BT_BASEBAND_TO_MODEM_TX_DATA2, PAD_BT_BASEBAND_TO_MODEM_TX_EDR,
	PAD_BT_BASEBAND_TO_MODEM_TX_EN, PAD_BT_BASEBAND_TO_MODEM_TX_VALID};
static const unsigned bt_modem_to_baseband_rx_pins[] = {
	PAD_BT_MODEM_TO_BASEBAND_RX_DATA0, PAD_BT_MODEM_TO_BASEBAND_RX_DATA1,
	PAD_BT_MODEM_TO_BASEBAND_RX_DATA2, PAD_BT_MODEM_TO_BASEBAND_RX_VALID};
static const unsigned camera_pins[] = {
	PAD_GPIO0, PAD_GPIO1, PAD_GPIO2, PAD_GPIO3, PAD_GPIO4, PAD_GPIO5,
	PAD_GPIO6, PAD_GPIO7, PAD_GPIO8, PAD_I2C6_SDAT, PAD_I2S0_BCLK_IN,
	PAD_I2S0_DIN0, PAD_I2S0_DIN1, PAD_I2S0_DIN2, PAD_I2S0_DIN3};
static const unsigned cpu0_trace_pins[] = {
	PAD_GPIO0, PAD_GPIO1, PAD_GPIO2, PAD_GPIO3, PAD_GPIO4, PAD_GPIO5,
	PAD_GPIO6, PAD_GPIO7, PAD_I2S0_BCLK, PAD_I2S0_BCLK_IN, PAD_I2S0_DIN0,
	PAD_I2S0_DIN1, PAD_I2S0_DIN2, PAD_I2S0_DIN3, PAD_I2S0_DOUT0,
	PAD_I2S0_DOUT1, PAD_I2S0_DOUT2, PAD_I2S0_DOUT3, PAD_I2S0_LRCLK,
	PAD_I2S0_LRCLK_IN, PAD_I2S0_MCLK};
static const unsigned cpu1_trace_pins[] = {
	PAD_GPIO0, PAD_GPIO1, PAD_GPIO2, PAD_GPIO3, PAD_GPIO4, PAD_GPIO5,
	PAD_GPIO6, PAD_GPIO7, PAD_I2S0_BCLK, PAD_I2S0_BCLK_IN, PAD_I2S0_DIN0,
	PAD_I2S0_DIN1, PAD_I2S0_DIN2, PAD_I2S0_DIN3, PAD_I2S0_DOUT0,
	PAD_I2S0_DOUT1, PAD_I2S0_DOUT2, PAD_I2S0_DOUT3, PAD_I2S0_LRCLK,
	PAD_I2S0_LRCLK_IN, PAD_I2S0_MCLK};
static const unsigned cpu2_pins[] = {
	PAD_I2C7_SCLK, PAD_I2C7_SDAT, PAD_UART0_CTS, PAD_UART0_RTS,
	PAD_UART0_RXD, PAD_UART0_TXD};
static const unsigned ets_pins[] = {PAD_GPIO8};
static const unsigned gpio0_pins[] = {PAD_GPIO0};
static const unsigned gpio1_pins[] = {PAD_GPIO1};
static const unsigned gpio2_pins[] = {PAD_GPIO2};
static const unsigned gpio3_pins[] = {PAD_GPIO3};
static const unsigned gpio4_pins[] = {PAD_GPIO4};
static const unsigned gpio5_pins[] = {PAD_GPIO5};
static const unsigned gpio6_pins[] = {PAD_GPIO6};
static const unsigned gpio7_pins[] = {PAD_GPIO7};
static const unsigned gpio8_pins[] = {PAD_GPIO8};
static const unsigned gpio9_pins[] = {PAD_GPIO9};
static const unsigned gpio10_pins[] = {PAD_GPIO10};
static const unsigned gpio11_pins[] = {PAD_GPIO11};
static const unsigned gpio12_pins[] = {PAD_GPIO12};
static const unsigned gpio13_pins[] = {PAD_GPIO13};
static const unsigned gpio14_pins[] = {PAD_GPIO14};
static const unsigned gpio15_pins[] = {PAD_GPIO15};
static const unsigned gpio16_pins[] = {PAD_I2C6_SDAT};
static const unsigned gpio17_pins[] = {PAD_I2C6_SCLK};
static const unsigned gpio18_pins[] = {PAD_I2C7_SDAT};
static const unsigned gpio19_pins[] = {PAD_I2C7_SCLK};
static const unsigned gpio20_pins[] = {PAD_UART0_TXD};
static const unsigned gpio21_pins[] = {PAD_UART0_RXD};
static const unsigned gpio22_pins[] = {PAD_UART0_CTS};
static const unsigned gpio23_pins[] = {PAD_UART0_RTS};
static const unsigned gpio24_pins[] = {PAD_I2S0_MCLK};
static const unsigned gpio25_pins[] = {PAD_I2S0_LRCLK};
static const unsigned gpio26_pins[] = {PAD_I2S0_BCLK};
static const unsigned gpio27_pins[] = {PAD_I2S0_DOUT0};
static const unsigned gpio28_pins[] = {PAD_I2S0_DOUT1};
static const unsigned gpio29_pins[] = {PAD_I2S0_DOUT2};
static const unsigned gpio30_pins[] = {PAD_I2S0_DOUT3};
static const unsigned gpio31_pins[] = {PAD_I2S0_LRCLK_IN};
static const unsigned i2c6_pins[] = {PAD_I2C6_SCLK, PAD_I2C6_SDAT};
static const unsigned i2c7_pins[] = {PAD_I2C7_SCLK, PAD_I2C7_SDAT};
static const unsigned i2s0_pins[] = {
	PAD_I2S0_BCLK, PAD_I2S0_BCLK_IN, PAD_I2S0_DIN0, PAD_I2S0_DIN1,
	PAD_I2S0_DIN2, PAD_I2S0_DIN3, PAD_I2S0_DOUT0, PAD_I2S0_DOUT1,
	PAD_I2S0_DOUT2, PAD_I2S0_DOUT3,	PAD_I2S0_LRCLK, PAD_I2S0_LRCLK_IN,
	PAD_I2S0_MCLK};
static const unsigned ltc_spi_pins[] = {
	PAD_LTC_SPI_BIDIR_OEN, PAD_LTC_SPI_MISO,};
static const unsigned ohot_plug_0_pins[] = {PAD_GPIO9};
static const unsigned ohot_plug_1_pins[] = {PAD_GPIO15};
static const unsigned pdi1_pins[] = {
	PAD_WLAN_FASTWR2, PAD_WLAN_FASTWR3, PAD_WLAN_FASTWR4,
	PAD_WLAN_FASTWR5};
static const unsigned rf_spi_pins[] = {
	PAD_RF_SPI_BIDIR_OEN, PAD_RF_SPI_CLK, PAD_RF_SPI_MISO,
	PAD_RF_SPI_MOSI, PAD_RF_SPI_SS};
static const unsigned uart0_pins[] = {
	PAD_UART0_CTS, PAD_UART0_RTS, PAD_UART0_RXD, PAD_UART0_TXD};
static const unsigned usb0_drvvbus_0_pins[] = {
	PAD_BT_BASEBAND_TO_MODEM_TX_DATA0};
static const unsigned usb0_drvvbus_1_pins[] = {PAD_GPIO11};
static const unsigned usb0_port_0_pins[] = {PAD_BT_BASEBAND_TO_MODEM_TX_DATA1};
static const unsigned usb0_port_1_pins[] = {PAD_GPIO12};
static const unsigned usb1_drvvbus_0_pins[] = {
	PAD_BT_BASEBAND_TO_MODEM_TX_DATA2};
static const unsigned usb1_drvvbus_1_pins[] = {PAD_GPIO13};
static const unsigned usb1_port_0_pins[] = {PAD_LTC_SPI_MISO};
static const unsigned usb1_port_1_pins[] = {PAD_GPIO14};
static const unsigned usb_bc_isel0_0_pins[] = {PAD_I2C6_SDAT};
static const unsigned usb_bc_isel0_1_pins[] = {PAD_WLAN_TX_ON1};
static const unsigned usb_bc_isel1_0_pins[] = {PAD_I2C6_SCLK};
static const unsigned usb_bc_isel1_1_pins[] = {PAD_WLAN_RX_ON1};
static const unsigned usb_bc_isel2_0_pins[] = {PAD_GPIO9};
static const unsigned usb_bc_isel2_1_pins[] = {PAD_WLAN_RF_SHDN1};
static const unsigned wlan_pins[] = {
	PAD_WLAN_ANT_SEL0_0, PAD_WLAN_ANT_SEL1_1, PAD_WLAN_FASTWR0,
	PAD_WLAN_FASTWR1, PAD_WLAN_FASTWR2, PAD_WLAN_FASTWR3,
	PAD_WLAN_FASTWR4, PAD_WLAN_FASTWR5, PAD_WLAN_PA_ON0,
	PAD_WLAN_PA_ON1, PAD_WLAN_RF_SHDN0,
	PAD_WLAN_RF_SHDN1, PAD_WLAN_RF_STBY0, PAD_WLAN_RF_STBY1,
	PAD_WLAN_RX_HP0, PAD_WLAN_RX_HP1, PAD_WLAN_RX_ON0, PAD_WLAN_RX_ON1,
	PAD_WLAN_TX_ON0, PAD_WLAN_TX_ON1};

static const char * const afe_out_clk_groups[] = {"afe_out_clk_0_grp",
						  "afe_out_clk_1_grp"};
static const char * const bt_baseband_to_modem_groups[] = {
	"bt_baseband_to_modem_grp"};
static const char * const bt_modem_to_baseband_rx_groups[] = {
	"bt_modem_to_baseband_rx_grp"};
static const char * const camera_groups[] = {"camera_grp"};
static const char * const cpu0_trace_groups[] = {"cpu0_trace_grp"};
static const char * const cpu1_trace_groups[] = {"cpu1_trace_grp"};
static const char * const cpu2_groups[] = {"cpu2_grp"};
static const char * const ets_groups[] = {"ets_grp"};
static const char * const gpio0_groups[] = {"gpio0_grp"};
static const char * const gpio1_groups[] = {"gpio1_grp"};
static const char * const gpio2_groups[] = {"gpio2_grp"};
static const char * const gpio3_groups[] = {"gpio3_grp"};
static const char * const gpio4_groups[] = {"gpio4_grp"};
static const char * const gpio5_groups[] = {"gpio5_grp"};
static const char * const gpio6_groups[] = {"gpio6_grp"};
static const char * const gpio7_groups[] = {"gpio7_grp"};
static const char * const gpio8_groups[] = {"gpio8_grp"};
static const char * const gpio9_groups[] = {"gpio9_grp"};
static const char * const gpio10_groups[] = {"gpio10_grp"};
static const char * const gpio11_groups[] = {"gpio11_grp"};
static const char * const gpio12_groups[] = {"gpio12_grp"};
static const char * const gpio13_groups[] = {"gpio13_grp"};
static const char * const gpio14_groups[] = {"gpio14_grp"};
static const char * const gpio15_groups[] = {"gpio15_grp"};
static const char * const gpio16_groups[] = {"gpio16_grp"};
static const char * const gpio17_groups[] = {"gpio17_grp"};
static const char * const gpio18_groups[] = {"gpio18_grp"};
static const char * const gpio19_groups[] = {"gpio19_grp"};
static const char * const gpio20_groups[] = {"gpio20_grp"};
static const char * const gpio21_groups[] = {"gpio21_grp"};
static const char * const gpio22_groups[] = {"gpio22_grp"};
static const char * const gpio23_groups[] = {"gpio23_grp"};
static const char * const gpio24_groups[] = {"gpio24_grp"};
static const char * const gpio25_groups[] = {"gpio25_grp"};
static const char * const gpio26_groups[] = {"gpio26_grp"};
static const char * const gpio27_groups[] = {"gpio27_grp"};
static const char * const gpio28_groups[] = {"gpio28_grp"};
static const char * const gpio29_groups[] = {"gpio29_grp"};
static const char * const gpio30_groups[] = {"gpio30_grp"};
static const char * const gpio31_groups[] = {"gpio31_grp"};
static const char * const i2c6_groups[] = {"i2c6_grp"};
static const char * const i2c7_groups[] = {"i2c7_grp"};
static const char * const i2s0_groups[] = {"i2s0_grp"};
static const char * const ltc_spi_groups[] = {"ltc_spi_grp"};
static const char * const ohot_plug_groups[] = {
	"ohot_plug_0_grp", "ohot_plug_1_grp"};
static const char * const pdi1_groups[] = {"pdi1_grp"};
static const char * const rf_spi_groups[] = {"rf_spi_grp"};
static const char * const uart0_groups[] = {"uart0_grp"};
static const char * const usb0_drvvbus_groups[] = {
	"usb0_drvvbus_0_grp", "usb0_drvvbus_1_grp"};
static const char * const usb0_port_groups[] = {
	"usb0_port_0_grp", "usb0_port_1_grp"};
static const char * const usb1_drvvbus_groups[] = {
	"usb1_drvvbus_0_grp", "usb1_drvvbus_1_grp"};
static const char * const usb1_port_groups[] = {
	"usb1_port_0_grp", "usb1_port_1_grp"};
static const char * const usb_bc_isel0_groups[] = {
	"usb_bc_isel0_0_grp", "usb_bc_isel0_1_grp"};
static const char * const usb_bc_isel1_groups[] = {
	"usb_bc_isel1_0_grp", "usb_bc_isel1_1_grp"};
static const char * const usb_bc_isel2_groups[] = {
	"usb_bc_isel2_0_grp", "usb_bc_isel2_1_grp"};
static const char * const wlan_groups[] = {"wlan_grp"};

static struct img_group palma_groups[] = {
	IMG_GROUP("afe_out_clk_0_grp", afe_out_clk_0_pins, 1),
	IMG_GROUP("afe_out_clk_1_grp", afe_out_clk_1_pins, 1),
	IMG_GROUP("bt_baseband_to_modem_grp",
		  bt_baseband_to_modem_pins, 0),
	IMG_GROUP("bt_modem_to_baseband_rx_grp",
		  bt_modem_to_baseband_rx_pins, 0),
	IMG_GROUP("camera_grp", camera_pins, 3),
	IMG_GROUP("cpu0_trace_grp", cpu0_trace_pins, 2),
	IMG_GROUP("cpu1_trace_grp", cpu1_trace_pins, 1),
	IMG_GROUP("cpu2_grp", cpu2_pins, 2),
	IMG_GROUP("ets_grp", ets_pins, 2),
	IMG_GROUP("gpio0_grp", gpio0_pins, 0),
	IMG_GROUP("gpio1_grp", gpio1_pins, 0),
	IMG_GROUP("gpio2_grp", gpio2_pins, 0),
	IMG_GROUP("gpio3_grp", gpio3_pins, 0),
	IMG_GROUP("gpio4_grp", gpio4_pins, 0),
	IMG_GROUP("gpio5_grp", gpio5_pins, 0),
	IMG_GROUP("gpio6_grp", gpio6_pins, 0),
	IMG_GROUP("gpio7_grp", gpio7_pins, 0),
	IMG_GROUP("gpio8_grp", gpio8_pins, 0),
	IMG_GROUP("gpio9_grp", gpio9_pins, 0),
	IMG_GROUP("gpio10_grp", gpio10_pins, 0),
	IMG_GROUP("gpio11_grp", gpio11_pins, 0),
	IMG_GROUP("gpio12_grp", gpio12_pins, 0),
	IMG_GROUP("gpio13_grp", gpio13_pins, 0),
	IMG_GROUP("gpio14_grp", gpio14_pins, 0),
	IMG_GROUP("gpio15_grp", gpio15_pins, 0),
	IMG_GROUP("gpio16_grp", gpio16_pins, 1),
	IMG_GROUP("gpio17_grp", gpio17_pins, 1),
	IMG_GROUP("gpio18_grp", gpio18_pins, 1),
	IMG_GROUP("gpio19_grp", gpio19_pins, 1),
	IMG_GROUP("gpio20_grp", gpio20_pins, 1),
	IMG_GROUP("gpio21_grp", gpio21_pins, 1),
	IMG_GROUP("gpio22_grp", gpio22_pins, 1),
	IMG_GROUP("gpio23_grp", gpio23_pins, 1),
	IMG_GROUP("gpio24_grp", gpio24_pins, 2),
	IMG_GROUP("gpio25_grp", gpio25_pins, 2),
	IMG_GROUP("gpio26_grp", gpio26_pins, 2),
	IMG_GROUP("gpio27_grp", gpio27_pins, 2),
	IMG_GROUP("gpio28_grp", gpio28_pins, 2),
	IMG_GROUP("gpio29_grp", gpio29_pins, 2),
	IMG_GROUP("gpio30_grp", gpio30_pins, 2),
	IMG_GROUP("gpio31_grp", gpio31_pins, 2),
	IMG_GROUP("i2c6_grp", i2c6_pins, 0),
	IMG_GROUP("i2c7_grp", i2c7_pins, 0),
	IMG_GROUP("i2s0_grp", i2s0_pins, 0),
	IMG_GROUP("ltc_spi_grp", ltc_spi_pins, 0),
	IMG_GROUP("ohot_plug_0_grp", ohot_plug_0_pins, 1),
	IMG_GROUP("ohot_plug_1_grp", ohot_plug_1_pins, 1),
	IMG_GROUP("pdi1_grp", pdi1_pins, 1),
	IMG_GROUP("rf_spi_grp", rf_spi_pins, 0),
	IMG_GROUP("uart0_grp", uart0_pins, 0),
	IMG_GROUP("usb0_drvvbus_0_grp", usb0_drvvbus_0_pins, 1),
	IMG_GROUP("usb0_drvvbus_1_grp", usb0_drvvbus_1_pins, 1),
	IMG_GROUP("usb0_port_0_grp", usb0_port_0_pins, 1),
	IMG_GROUP("usb0_port_1_grp", usb0_port_1_pins, 1),
	IMG_GROUP("usb1_drvvbus_0_grp", usb1_drvvbus_0_pins, 1),
	IMG_GROUP("usb1_drvvbus_1_grp", usb1_drvvbus_1_pins, 1),
	IMG_GROUP("usb1_port_0_grp", usb1_port_0_pins, 1),
	IMG_GROUP("usb1_port_1_grp", usb1_port_1_pins, 1),
	IMG_GROUP("usb_bc_isel0_0_grp", usb_bc_isel0_0_pins, 2),
	IMG_GROUP("usb_bc_isel0_1_grp", usb_bc_isel0_1_pins, 1),
	IMG_GROUP("usb_bc_isel1_0_grp", usb_bc_isel1_0_pins, 2),
	IMG_GROUP("usb_bc_isel1_1_grp", usb_bc_isel1_1_pins, 1),
	IMG_GROUP("usb_bc_isel2_0_grp", usb_bc_isel2_0_pins, 2),
	IMG_GROUP("usb_bc_isel2_1_grp", usb_bc_isel2_1_pins, 1),
	IMG_GROUP("wlan_grp", wlan_pins, 0),
};

static const unsigned palma_muxed_pads[] = {
	PAD_I2S0_MCLK, PAD_I2S0_LRCLK, PAD_I2S0_BCLK, PAD_I2S0_DOUT0,
	PAD_I2S0_DOUT1, PAD_I2S0_DOUT2, PAD_I2S0_DOUT3, PAD_I2S0_LRCLK_IN,
	PAD_I2S0_BCLK_IN, PAD_I2S0_DIN0, PAD_I2S0_DIN1, PAD_I2S0_DIN2,
	PAD_I2S0_DIN3, PAD_GPIO0, PAD_GPIO1, PAD_GPIO2, PAD_GPIO3, PAD_GPIO4,
	PAD_GPIO5, PAD_GPIO6, PAD_GPIO7, PAD_GPIO8, PAD_GPIO9, PAD_GPIO11,
	PAD_GPIO12, PAD_GPIO13, PAD_GPIO14, PAD_GPIO15, PAD_I2C6_SDAT,
	PAD_I2C6_SCLK, PAD_I2C7_SDAT, PAD_I2C7_SCLK, PAD_UART0_TXD,
	PAD_UART0_RXD, PAD_UART0_CTS, PAD_UART0_RTS, PAD_WLAN_TX_ON0,
	PAD_WLAN_RX_ON0, PAD_WLAN_RF_SHDN0, PAD_WLAN_RF_STBY0, PAD_WLAN_PA_ON0,
	PAD_WLAN_RX_HP0, PAD_WLAN_FASTWR0, PAD_WLAN_FASTWR2,
	PAD_WLAN_FASTWR3, PAD_WLAN_FASTWR4, PAD_WLAN_FASTWR5,
	PAD_WLAN_TX_ON1, PAD_WLAN_RX_ON1, PAD_WLAN_RF_SHDN1, PAD_WLAN_RF_STBY1,
	PAD_WLAN_PA_ON1, PAD_WLAN_RX_HP1, PAD_WLAN_FASTWR1, PAD_WLAN_ANT_SEL0_0,
	PAD_WLAN_ANT_SEL1_1, PAD_BT_MODEM_TO_BASEBAND_RX_VALID,
	PAD_BT_MODEM_TO_BASEBAND_RX_DATA0, PAD_BT_MODEM_TO_BASEBAND_RX_DATA1,
	PAD_BT_MODEM_TO_BASEBAND_RX_DATA2, PAD_BT_BASEBAND_TO_MODEM_RATE0,
	PAD_BT_BASEBAND_TO_MODEM_RATE1, PAD_BT_BASEBAND_TO_MODEM_TX_EN,
	PAD_BT_BASEBAND_TO_MODEM_RX_EN, PAD_BT_BASEBAND_TO_MODEM_SYNC_P,
	PAD_BT_BASEBAND_TO_MODEM_TX_EDR, PAD_BT_BASEBAND_TO_MODEM_TX_VALID,
	PAD_BT_BASEBAND_TO_MODEM_TX_DATA0, PAD_BT_BASEBAND_TO_MODEM_TX_DATA1,
	PAD_BT_BASEBAND_TO_MODEM_TX_DATA2, PAD_LTC_SPI_MISO,
	PAD_LTC_SPI_BIDIR_OEN, PAD_RF_SPI_CLK, PAD_RF_SPI_SS, PAD_RF_SPI_MOSI,
	PAD_RF_SPI_MISO, PAD_RF_SPI_BIDIR_OEN, };

static const struct img_pmx_func palma_functions[] = {
	IMG_FUNCTIONS(i2s0),
	IMG_FUNCTIONS(gpio0),
	IMG_FUNCTIONS(gpio1),
	IMG_FUNCTIONS(gpio2),
	IMG_FUNCTIONS(gpio3),
	IMG_FUNCTIONS(gpio4),
	IMG_FUNCTIONS(gpio5),
	IMG_FUNCTIONS(gpio6),
	IMG_FUNCTIONS(gpio7),
	IMG_FUNCTIONS(gpio8),
	IMG_FUNCTIONS(i2c6),
	IMG_FUNCTIONS(i2c7),
	IMG_FUNCTIONS(uart0),
	IMG_FUNCTIONS(wlan),
	IMG_FUNCTIONS(bt_modem_to_baseband_rx),
	IMG_FUNCTIONS(bt_baseband_to_modem),
	IMG_FUNCTIONS(ltc_spi),
	IMG_FUNCTIONS(rf_spi),
	IMG_FUNCTIONS(gpio9),
	IMG_FUNCTIONS(gpio10),
	IMG_FUNCTIONS(gpio11),
	IMG_FUNCTIONS(gpio12),
	IMG_FUNCTIONS(gpio13),
	IMG_FUNCTIONS(gpio14),
	IMG_FUNCTIONS(gpio15),
	IMG_FUNCTIONS(cpu1_trace),
	IMG_FUNCTIONS(gpio16),
	IMG_FUNCTIONS(gpio17),
	IMG_FUNCTIONS(gpio18),
	IMG_FUNCTIONS(gpio19),
	IMG_FUNCTIONS(gpio20),
	IMG_FUNCTIONS(gpio21),
	IMG_FUNCTIONS(gpio22),
	IMG_FUNCTIONS(gpio23),
	IMG_FUNCTIONS(gpio24),
	IMG_FUNCTIONS(gpio25),
	IMG_FUNCTIONS(gpio26),
	IMG_FUNCTIONS(gpio27),
	IMG_FUNCTIONS(gpio28),
	IMG_FUNCTIONS(gpio29),
	IMG_FUNCTIONS(gpio30),
	IMG_FUNCTIONS(gpio31),
	IMG_FUNCTIONS(usb_bc_isel0),
	IMG_FUNCTIONS(usb_bc_isel1),
	IMG_FUNCTIONS(usb_bc_isel2),
	IMG_FUNCTIONS(pdi1),
	IMG_FUNCTIONS(usb0_drvvbus),
	IMG_FUNCTIONS(usb0_port),
	IMG_FUNCTIONS(usb1_drvvbus),
	IMG_FUNCTIONS(usb1_port),
	IMG_FUNCTIONS(afe_out_clk),
	IMG_FUNCTIONS(ohot_plug),
	IMG_FUNCTIONS(cpu0_trace),
	IMG_FUNCTIONS(ets),
	IMG_FUNCTIONS(cpu2),
	IMG_FUNCTIONS(camera),
};

static void palma_set_mux(struct img_pinctrl *pctl, unsigned pin, int mux_src)
{
	u32 reg = pread(pctl, PADS_SRC_SEL_REG(pin));

	reg &= ~(PADS_SRC_SEL_MASK(pin) << PADS_SRC_SEL_SHIFT(pin));
	reg |= mux_src << PADS_SRC_SEL_SHIFT(pin);
	pwrite(pctl, PADS_SRC_SEL_REG(pin), reg);
}

static int palma_pinconf_get(struct img_pinctrl *pctl, unsigned int pin,
			     enum pin_config_param param)
{
	u32 val, arg = 0;

	switch (param) {
	case PIN_CONFIG_INPUT_SCHMITT_ENABLE:
		val = pread(pctl, PADS_SCHMITT_EN_REG(pin));
		val >>= PADS_SCHMITT_EN_SHIFT(pin);
		arg = val & PADS_SCHMITT_EN_MASK;
		break;
	case PIN_CONFIG_BIAS_HIGH_IMPEDANCE:
		val = pread(pctl, PADS_PU_PD_REG(pin));
		val >>= PADS_PU_PD_SHIFT(pin);
		arg = (val & PADS_PU_PD_MASK) == PADS_PU_PD_HIGHZ;
		break;
	case PIN_CONFIG_BIAS_PULL_UP:
		val = pread(pctl, PADS_PU_PD_REG(pin));
		val >>= PADS_PU_PD_SHIFT(pin);
		arg = (val & PADS_PU_PD_MASK) == PADS_PU_PD_UP;
		break;
	case PIN_CONFIG_BIAS_PULL_DOWN:
		val = pread(pctl, PADS_PU_PD_REG(pin));
		val >>= PADS_PU_PD_SHIFT(pin);
		arg = (val & PADS_PU_PD_MASK) == PADS_PU_PD_DOWN;
		break;
	case PIN_CONFIG_BIAS_BUS_HOLD:
		val = pread(pctl, PADS_PU_PD_REG(pin));
		val >>= PADS_PU_PD_SHIFT(pin);
		arg = (val & PADS_PU_PD_MASK) == PADS_PU_PD_BUS_HOLD;
		break;
	case PIN_CONFIG_SLEW_RATE:
		val = pread(pctl, PADS_SLEW_RATE_REG(pin));
		val >>= PADS_SLEW_RATE_SHIFT(pin);
		arg = val & PADS_SLEW_RATE_MASK;
		break;
	case PIN_CONFIG_DRIVE_STRENGTH:
		val = pread(pctl, PADS_DRIVE_STR_REG(pin));
		val >>= PADS_DRIVE_STR_SHIFT(pin);
		switch (val & PADS_DRIVE_STR_MASK) {
		case PADS_DRIVE_STRENGTH_2MA:
			arg = 2;
			break;
		case PADS_DRIVE_STRENGTH_4MA:
			arg = 4;
			break;
		case PADS_DRIVE_STRENGTH_8MA:
			arg = 8;
			break;
		case PADS_DRIVE_STRENGTH_12MA:
		default:
			arg = 12;
			break;
		}
		break;
	default:
		dev_dbg(pctl->dev, "Property %u not supported\n", param);
		return -ENOTSUPP;
	}

	return arg;
}

static int palma_pinconf_set(struct img_pinctrl *pctl, unsigned int pin,
			     enum pin_config_param param, u32 arg)
{
	u32 drv, reg;

	switch (param) {
	case PIN_CONFIG_INPUT_SCHMITT_ENABLE:
		reg = pread(pctl, PADS_SCHMITT_EN_REG(pin));
		reg &= ~(PADS_SCHMITT_EN_MASK
		       << PADS_SCHMITT_EN_SHIFT(pin));
		reg |= arg << PADS_SCHMITT_EN_SHIFT(pin);
		pwrite(pctl, PADS_SCHMITT_EN_REG(pin), reg);
		break;
	case PIN_CONFIG_BIAS_HIGH_IMPEDANCE:
		reg = pread(pctl, PADS_PU_PD_REG(pin));
		reg &= ~(PADS_PU_PD_MASK << PADS_PU_PD_SHIFT(pin));
		reg |= PADS_PU_PD_HIGHZ << PADS_PU_PD_SHIFT(pin);
		pwrite(pctl, PADS_PU_PD_REG(pin), reg);
		break;
	case PIN_CONFIG_BIAS_PULL_UP:
		reg = pread(pctl, PADS_PU_PD_REG(pin));
		reg &= ~(PADS_PU_PD_MASK << PADS_PU_PD_SHIFT(pin));
		reg |= PADS_PU_PD_UP << PADS_PU_PD_SHIFT(pin);
		pwrite(pctl, PADS_PU_PD_REG(pin), reg);
		break;
	case PIN_CONFIG_BIAS_PULL_DOWN:
		reg = pread(pctl, PADS_PU_PD_REG(pin));
		reg &= ~(PADS_PU_PD_MASK << PADS_PU_PD_SHIFT(pin));
		reg |= PADS_PU_PD_DOWN << PADS_PU_PD_SHIFT(pin);
		pwrite(pctl, PADS_PU_PD_REG(pin), reg);
		break;
	case PIN_CONFIG_BIAS_BUS_HOLD:
		reg = pread(pctl, PADS_PU_PD_REG(pin));
		reg &= ~(PADS_PU_PD_MASK << PADS_PU_PD_SHIFT(pin));
		reg |= PADS_PU_PD_BUS_HOLD << PADS_PU_PD_SHIFT(pin);
		pwrite(pctl, PADS_PU_PD_REG(pin), reg);
		break;
	case PIN_CONFIG_SLEW_RATE:
		reg = pread(pctl, PADS_SLEW_RATE_REG(pin));
		reg &= ~(PADS_SLEW_RATE_MASK
		       << PADS_SLEW_RATE_SHIFT(pin));
		reg |= arg << PADS_SLEW_RATE_SHIFT(pin);
			pwrite(pctl, PADS_SLEW_RATE_REG(pin), reg);
			break;
	case PIN_CONFIG_DRIVE_STRENGTH:
		switch (arg) {
		case 2:
			drv = PADS_DRIVE_STRENGTH_2MA;
			break;
		case 4:
			drv = PADS_DRIVE_STRENGTH_4MA;
			break;
		case 8:
			drv = PADS_DRIVE_STRENGTH_8MA;
			break;
		case 12:
			drv = PADS_DRIVE_STRENGTH_12MA;
			break;
		default:
			dev_err(pctl->dev,
				"Drive strength %umA not supported\n",
				arg);
			return -EINVAL;
		}
		reg = pread(pctl, PADS_DRIVE_STR_REG(pin));
		reg &= ~(PADS_DRIVE_STR_MASK
		       << PADS_DRIVE_STR_SHIFT(pin));
		reg |= (drv << PADS_DRIVE_STR_SHIFT(pin));
		pwrite(pctl, PADS_DRIVE_STR_REG(pin), reg);
		break;
	default:
		dev_err(pctl->dev, "Property %u not supported\n",
			param);
		return -ENOTSUPP;
	}
	return 0;
}

static void palma_pinconf_save_ctx(struct img_pinctrl *pctl, void *soc_ctx)
{
	struct img_palma_pinctrl_ctx *palma_ctx = soc_ctx;
	int i;

	for (i = 0; i < ARRAY_SIZE(palma_ctx->schmitt_en); i++) {
		palma_ctx->schmitt_en[i] =
			pread(pctl, PADS_CR_SCHMITT_EN0 + 0x4 * i);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->pu_pd); i++) {
		palma_ctx->pu_pd[i] =
			pread(pctl, PADS_CR_PU_PD0 + 0x4 * i);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->slew_rate); i++) {
		palma_ctx->slew_rate[i] =
			pread(pctl, PADS_CR_SLEW_RATE0 + 0x4 * i);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->drive_str); i++) {
		palma_ctx->drive_str[i] =
			pread(pctl, PADS_CR_DRIVE_STR0 + 0x4 * i);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->src_sel); i++) {
		palma_ctx->src_sel[i] =
			pread(pctl, PADS_CR_SRC_SEL0 + 0x4 * i);
	}
}

static void palma_pinconf_restore_ctx(struct img_pinctrl *pctl, void *soc_ctx)
{
	struct img_palma_pinctrl_ctx *palma_ctx = soc_ctx;
	int i;

	for (i = 0; i < ARRAY_SIZE(palma_ctx->schmitt_en); i++) {
		pwrite(pctl, PADS_CR_SCHMITT_EN0 + 0x4 * i,
		       palma_ctx->schmitt_en[i]);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->pu_pd); i++) {
		pwrite(pctl, PADS_CR_PU_PD0 + 0x4 * i,
		       palma_ctx->pu_pd[i]);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->slew_rate); i++) {
		pwrite(pctl, PADS_CR_SLEW_RATE0 + 0x4 * i,
		       palma_ctx->slew_rate[i]);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->drive_str); i++) {
		pwrite(pctl, PADS_CR_DRIVE_STR0 + 0x4 * i,
		       palma_ctx->drive_str[i]);
	}
	for (i = 0; i < ARRAY_SIZE(palma_ctx->src_sel); i++) {
		pwrite(pctl, PADS_CR_SRC_SEL0 + 0x4 * i,
		       palma_ctx->src_sel[i]);
	}
}

static struct img_pinctrl_soc_funcs palma_soc_funcs = {
	.set_mux = palma_set_mux,
	.set_pinconf = palma_pinconf_set,
	.get_pinconf = palma_pinconf_get,
	.save_ctx = palma_pinconf_save_ctx,
	.restore_ctx = palma_pinconf_restore_ctx,
};

static struct img_pinctrl_soc_data palma_pinctrl_info = {
	.pins = palma_pins,
	.npins = ARRAY_SIZE(palma_pins),
	.groups = palma_groups,
	.ngroups = ARRAY_SIZE(palma_groups),
	.functions = palma_functions,
	.nfunctions = ARRAY_SIZE(palma_functions),
	.muxed_pads = palma_muxed_pads,
	.nmuxed_pads = ARRAY_SIZE(palma_muxed_pads),
	.funcs = &palma_soc_funcs,
};

static int img_palma_pinctrl_probe(struct platform_device *pdev)
{
	struct img_palma_pinctrl_ctx *palma_ctx;

	palma_ctx = devm_kzalloc(&pdev->dev, sizeof(*palma_ctx), GFP_KERNEL);
	if (!palma_ctx)
		return -ENOMEM;

	return img_pinctrl_probe(pdev, &palma_pinctrl_info, palma_ctx);
}

#ifdef CONFIG_PM_SLEEP
static int img_palma_pinctrl_suspend(struct device *dev)
{
	return img_pinctrl_suspend(dev);
}

static int img_palma_pinctrl_resume(struct device *dev)
{
	return img_pinctrl_resume(dev);
}
#endif

static SIMPLE_DEV_PM_OPS(img_palma_pinctrl_pm_ops,
			 img_palma_pinctrl_suspend, img_palma_pinctrl_resume);

static const struct of_device_id img_palma_pinctrl_of_match[] = {
	{ .compatible = "img,cashew-palma-pinctrl", },
	{ /* Sentinel */ },
};

static struct platform_driver img_pinctrl_driver = {
	.driver = {
		.name = "cashew-palma-pin-controller",
		.of_match_table = img_palma_pinctrl_of_match,
		.pm = &img_palma_pinctrl_pm_ops,
		.suppress_bind_attrs = true,
	},
	.probe = img_palma_pinctrl_probe,
};

static int __init img_pinctrl_register(void)
{
	return platform_driver_register(&img_pinctrl_driver);
}
arch_initcall(img_pinctrl_register);

