/*
 * Cashew SoC pinctrl driver
 *
 * Copyright (C) 2016 Imagination Technologies Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 */

#ifndef _PINCTRL_CASHEW_H_
#define _PINCTRL_CASHEW_H_

#include <linux/gpio.h>
#include <linux/pinctrl/pinconf.h>
#include <linux/pinctrl/pinctrl.h>

/* Common register values */
#define PADS_PU_PD_HIGHZ		0x0
#define PADS_PU_PD_UP			0x1
#define PADS_PU_PD_DOWN			0x2
#define PADS_PU_PD_BUS_HOLD		0x3

#define PADS_DRIVE_STRENGTH_2MA		0x0
#define PADS_DRIVE_STRENGTH_4MA		0x1
#define PADS_DRIVE_STRENGTH_8MA		0x2
#define PADS_DRIVE_STRENGTH_12MA	0x3

/* GPIO block register interface */
#define GPIO_IO_MODE			0x00
#define GPIO_BIT_EN			0x04
#define GPIO_IRQ_POLARITY		0x08
#define GPIO_IRQ_TYPE			0x0c
#define GPIO_IRQ_TYPE_LEVEL	0x0
#define GPIO_IRQ_TYPE_EDGE	0x1
#define GPIO_IRQ_EDGE			0x10
#define GPIO_IRQ_EDGE_SINGLE	0x0
#define GPIO_IRQ_EDGE_DUAL	0x1
#define GPIO_IRQ_STATUS			0x14
#define GPIO_IRQ_MASK			0x18
#define GPIO_IRQ_ENABLE			0x1c
#define GPIO_INPUT			0x20
#define GPIO_OUTPUT			0x24

#define GPIO_BLOCK_LENGTH		0x30

struct img_group {
	const char *name;
	const unsigned int *pins;
	const unsigned int npins;
	const int mux_id;
};

#define IMG_GROUP(_name, _pins, _mux)			\
	{						\
		.name = _name,				\
		.pins = _pins,				\
		.npins = ARRAY_SIZE(_pins),		\
		.mux_id = _mux,				\
	}

struct img_pmx_func {
	const char *name;
	const char *const *groups;
	const unsigned ngroups;
};

#define IMG_FUNCTIONS(_name)				\
	{						\
		.name = #_name,				\
		.groups = _name##_groups,		\
		.ngroups = ARRAY_SIZE(_name##_groups),	\
	}

struct img_gpio_ctx {
	u32 io_mode;
	u32 bit_en;
	u32 irq_mask;
	u32 irq_polarity;
	u32 irq_type;
	u32 irq_edge;
	u32 output;
};

struct img_gpio_bank {
	struct img_pinctrl *pctl;
	unsigned int irq;
	void __iomem *base;
	unsigned int npins;
	struct gpio_chip gpio_chip;
	struct irq_chip irq_chip;
	struct img_gpio_ctx suspend_ctx;
};

struct img_pinctrl {
	struct device *dev;
	void __iomem *base;
	struct pinctrl_dev *pctldev;
	struct img_gpio_bank *gpio_banks;
	int num_banks;
	const struct img_pinctrl_soc_data *data;
	void *soc_ctx;
};

struct img_pinctrl_soc_funcs {
	void (*set_mux)(struct img_pinctrl *pctl, unsigned int pin,
			int mux_sel);
	int (*set_pinconf)(struct img_pinctrl *pctl, unsigned int pin,
			   enum pin_config_param param, u32 arg);
	int (*get_pinconf)(struct img_pinctrl *pctl, unsigned int pin,
			   enum pin_config_param param);
	void (*save_ctx)(struct img_pinctrl *pctl, void *ctx);
	void (*restore_ctx)(struct img_pinctrl *pctl, void *ctx);
};

struct img_pinctrl_soc_data {
	struct device *dev;
	const struct pinctrl_pin_desc *pins;
	unsigned int npins;
	const struct img_group *groups;
	unsigned int ngroups;
	const struct img_pmx_func *functions;
	unsigned int nfunctions;
	const unsigned int *muxed_pads;
	unsigned int nmuxed_pads;
	struct img_pinctrl_soc_funcs *funcs;
};

int img_pinctrl_probe(struct platform_device *pdev,
		      struct img_pinctrl_soc_data *data,
		      void *soc_ctx);
int img_pinctrl_suspend(struct device *dev);
int img_pinctrl_resume(struct device *dev);
void pwrite(struct img_pinctrl *pctl, u32 reg, u32 val);
u32 pread(struct img_pinctrl *pctl, u32 reg);
#endif /* _PINCTRL_CASHEW_H_ */
