#ifndef VPP_API_UAPI_H
#define VPP_API_UAPI_H
#include <linux/videodev2.h>
#include <linux/version.h>
#undef PDEBUG             /* undef it, just in case */
#ifdef VPP_DEBUG
#	ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
//#     define PDEBUG(fmt, args...) printk( KERN_DEBUG fmt, ## args)
#     define PDEBUG(fmt, args...) printk(  fmt, ## args)
#   else
     /* This one for user space. Do not print. */
/*#    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)*/
#     define PDEBUG(fmt, args...)
#   endif
#else
#   define PDEBUG(fmt, args...) //printk( KERN_DEBUG "vpp_debug not defined: " fmt, ## args)/* not debugging: nothing */
#endif

#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 1, 0))
	#define V4L2_COLORSPACE_DEFAULT 0
#endif

enum {
	CABLE_CONNECT = 0,
	CABLE_DISCONNECT,
	VIDEO_CHANGED,
	AUDIO_CHANGED,
	CLOCK_CHANGED,
	VPP_DUMMY_EVENT,
	VIDEO_IN_INVALID_EVENT,
};

/* v4l2_events */
#define V4L2_EVENT_VIDEO_IN_CLASS	(V4L2_EVENT_PRIVATE_START + 2 *1000)
#define VIDEO_IN_CABLE_CONNECT (V4L2_EVENT_VIDEO_IN_CLASS + CABLE_CONNECT)
#define VIDEO_IN_CABLE_DISCONNECT (V4L2_EVENT_VIDEO_IN_CLASS + CABLE_DISCONNECT)
#define VIDEO_IN_VIDEO_CHANGED (V4L2_EVENT_VIDEO_IN_CLASS + VIDEO_CHANGED)
#define VIDEO_IN_AUDIO_CHANGED (V4L2_EVENT_VIDEO_IN_CLASS + AUDIO_CHANGED)
#define VIDEO_IN_CLOCK_CHANGED (V4L2_EVENT_VIDEO_IN_CLASS + CLOCK_CHANGED)
#define VIDEO_IN_VPP_DUMMY_EVENT (V4L2_EVENT_VIDEO_IN_CLASS + VPP_DUMMY_EVENT)


//#define __VPP_TEST_TARGET_IS_SIMULATOR__
/* Hue is between -30 and +30 degrees.  Saturation and contrast are between 0 and 2. All in x.25 signed fixed point format */
#define CSC_HSC_FRACTIONAL_BITS							25
/*Brightness is between -50 and 50, in x.10 signed fixed point format*/
#define	CSC_OUTPUT_OFFSET_FRACTIONAL_BITS				10		/* Bits of fractional data in user specified value for Brightness */
#define	VPP_DEFAULT_BRIGHTNESS		  	(0 << CSC_OUTPUT_OFFSET_FRACTIONAL_BITS)
#define	VPP_DEFAULT_CONTRAST		  	(1 << CSC_HSC_FRACTIONAL_BITS)
#define	VPP_DEFAULT_HUE			  		(0 << CSC_HSC_FRACTIONAL_BITS)
#define	VPP_DEFAULT_SATURATION		  	(1 << CSC_HSC_FRACTIONAL_BITS)
#define VPP_MAX_OUTPUT_WIDTH   4096//(VPP_DIF_FRAME_STRIDE_DIFFRSTRIDE_LSBMASK + 1)
#define VPP_MAX_OUTPUT_HEIGHT  4096//(VPP_DIF_FRAME_HEIGHT_DIFFRHEIGHT_LSBMASK + 1)
#define VPP_CSC_MATRIX_FRACTIONAL_BITS	7
#define VPP_CSC_MATRIX_DECIMAL_BITS		4
#define CSC_MATRIX_COEFFS_MIN	(1 << (VPP_CSC_MATRIX_FRACTIONAL_BITS +VPP_CSC_MATRIX_DECIMAL_BITS - 1))	/* 4.7 signed fixed point format, minimum is -1024 */
#define CSC_MATRIX_COEFFS_MAX	CSC_MATRIX_COEFFS_MIN - 1	/* 3.8 signed fixed point format, maximum is 1023 */

#define VPP_MAX_NO_OF_PLANES_PER_FRAME	4
enum VPP_BUFFERS{
	VPP_BUF_ALL = 0, // can be Y_EVEN_INTERLACED, Y_PROGRESSIVE_PLANAR, PROGRESSIVE_NONE_PLANAR
						// When only one buffer the output format should be used to find out how the buffer address registers are set.
	VPP_BUF_ODD_INTERLACED_OR_CHROMA_PROGRESSIVE, // can be Y_ODD_INTERLACED or UV_PROGRESSIVE_PLANAR
	VPP_BUF_INTERLACED_PLANAR_CHROMA_EVEN,// This is only used for INTERLACED_PLANAR_CHROMA_EVEN
	VPP_BUF_INTERLACED_PLANAR_CHROMA_ODD,

	VPP_BUF_INVALID
};

//#define V4L2_CID_USER_BASE_INDEX  0x0F000
// V4L2_CID_USER_BASE + V4L2_CID_USER_BASE_INDEX
enum {
	VPP_CTRL_CSC_MULT = 0,
	VPP_CTRL_CSC_IN_OFFSET,
	VPP_CTRL_CSC_OUT_OFFSET,
	VPP_CTRL_HSCALER_COEFFS,
	VPP_CTRL_VSCALER_COEFFS,
	VPP_CTRL_INPUT_MODE,
	VPP_CTRL_INPUT_POLARITIES,
	VPP_CTRL_INPUT_COLORSPACE,
	VPP_CTRL_INPUT_COLORSPACE_RANGE,
	VPP_CTRL_SWAP_INTERNAL_UV_LANES,
	VPP_CTRL_TILING_METHOD,
#ifdef __VPP_TARGET_IS_SIMULATOR__
	VPP_CTRL_VP_KICK,
#endif
	VPP_CTRL_INVALID
};

#define IMG_VPP_CID_BASE			(V4L2_CID_USER_BASE | 0x0f000)  //0x00980900 | 0x0f000 =  0x0098f900
//A 3x3 matrix
#define IMG_VPP_CID_CSC_MULT		(IMG_VPP_CID_BASE + VPP_CTRL_CSC_MULT) //0x0098f900
//A 1x3 array
#define IMG_VPP_CID_CSC_IN_OFFSET	(IMG_VPP_CID_BASE + VPP_CTRL_CSC_IN_OFFSET)
//A 1x3 array
#define IMG_VPP_CID_CSC_OUT_OFFSET	(IMG_VPP_CID_BASE + VPP_CTRL_CSC_OUT_OFFSET)
//A 1x17 array
#define IMG_VPP_CID_HSCALER_COEFFS			(IMG_VPP_CID_BASE + VPP_CTRL_HSCALER_COEFFS)
//a 1x9 array
#define IMG_VPP_CID_VSCALER_COEFFS			(IMG_VPP_CID_BASE + VPP_CTRL_VSCALER_COEFFS)
#define IMG_VPP_CID_INPUT_MODE				(IMG_VPP_CID_BASE + VPP_CTRL_INPUT_MODE)
#define IMG_VPP_CID_INPUT_POLARITIES		(IMG_VPP_CID_BASE + VPP_CTRL_INPUT_POLARITIES)
#define IMG_VPP_CID_INPUT_COLORSPACE		(IMG_VPP_CID_BASE + VPP_CTRL_INPUT_COLORSPACE)
#define IMG_VPP_CID_INPUT_COLORSPACE_RANGE	(IMG_VPP_CID_BASE + VPP_CTRL_INPUT_COLORSPACE_RANGE)
#define IMG_VPP_CID_SWAP_INTERNAL_UV_LANES		(IMG_VPP_CID_BASE + VPP_CTRL_SWAP_INTERNAL_UV_LANES)
#define IMG_VPP_CID_TILING_METHOD 		(IMG_VPP_CID_BASE + VPP_CTRL_TILING_METHOD)
#ifdef __VPP_TARGET_IS_SIMULATOR__
#define IMG_VPP_CID_VP_KICK 		(IMG_VPP_CID_BASE + VPP_CTRL_VP_KICK)
#endif
#define VPP_H_TAPS					8
#define VPP_H_INTERPOLATION_POINTS	16
#define VPP_V_TAPS					4
#define VPP_V_INTERPOLATION_POINTS	16
/*!
******************************************************************************

 This type defines the bus mode. NOTE : Modes 8 and 9 - the ten bit modes
 are not directly programmable into registers. They are programmed as the
 equivalent 8 bit mode with an additional ten bit bit set in the second
 configuration register.

******************************************************************************/
typedef enum
{
	/*! Unknown Mode						*/
	VPP_B1BMODE_MODE_UNKNOWN,
	/*! 24 Bit RGB							*/
	VPP_B1BMODE_MODE_1_444_DVI_RGB_3X8,
	/*! 24 Bit YCbCr						*/
	VPP_B1BMODE_MODE_2_444_YCBCR_3X8,
	/*! 16 Bit YCbCr						*/
	VPP_B1BMODE_MODE_3_422_YCBCR_2X8,
	/*! 20 Bit YCbCr						*/
	VPP_B1BMODE_MODE_4_422_YCBCR_2X10,
	/*! 8 Bit YCbCr							*/
	VPP_B1BMODE_MODE_5_422_YCBCR_1X8,
	/*! 10 Bit YCbCr Connection Type A		*/
	VPP_B1BMODE_MODE_6_422_YCBCR_1X10A,
	/*! 10 Bit YCbCr Connection Type B		*/
	VPP_B1BMODE_MODE_7_422_YCBCR_1X10B,
	/*! 30 Bit RGB							*/
	VPP_B1BMODE_MODE_1_444_DVI_RGB_3X10,
	/*! 30 Bit YCbCr						*/
	VPP_B1BMODE_MODE_2_444_YCBCR_3X10,
	/*! place holder do not move			*/
	VPP_NO_OF_B1BMODE_MODE,
} VPP_eB1BMode;

typedef enum
{
	VPP_POLARITY_FIELD_HIGH_DENABLE_LOW_VYSNC_LOW_HSYNC_LOW = 0,
	VPP_POLARITY_FIELD_HIGH_DENABLE_LOW_VYSNC_LOW_HSYNC_HIGH,
	VPP_POLARITY_FIELD_HIGH_DENABLE_LOW_VYSNC_HIGH_HSYNC_LOW,
	VPP_POLARITY_FIELD_HIGH_DENABLE_LOW_VYSNC_HIGH_HSYNC_HIGH,
	VPP_POLARITY_FIELD_HIGH_DENABLE_HIGH_VYSNC_LOW_HSYNC_LOW,
	VPP_POLARITY_FIELD_HIGH_DENABLE_HIGH_VYSNC_LOW_HSYNC_HIGH,
	VPP_POLARITY_FIELD_HIGH_DENABLE_HIGH_VYSNC_HIGH_HSYNC_LOW,
	VPP_POLARITY_FIELD_HIGH_DENABLE_HIGH_VYSNC_HIGH_HSYNC_HIGH,
	VPP_POLARITY_FIELD_LOW_DENABLE_LOW_VYSNC_LOW_HSYNC_LOW,
	VPP_POLARITY_FIELD_LOW_DENABLE_LOW_VYSNC_LOW_HSYNC_HIGH,
	VPP_POLARITY_FIELD_LOW_DENABLE_LOW_VYSNC_HIGH_HSYNC_LOW,
	VPP_POLARITY_FIELD_LOW_DENABLE_LOW_VYSNC_HIGH_HSYNC_HIGH,
	VPP_POLARITY_FIELD_LOW_DENABLE_HIGH_VYSNC_LOW_HSYNC_LOW,
	VPP_POLARITY_FIELD_LOW_DENABLE_HIGH_VYSNC_LOW_HSYNC_HIGH,
	VPP_POLARITY_FIELD_LOW_DENABLE_HIGH_VYSNC_HIGH_HSYNC_LOW,
	VPP_POLARITY_FIELD_LOW_DENABLE_HIGH_VYSNC_HIGH_HSYNC_HIGH,
	VPP_POLARITY_INVALID,
} VPP_eSignalPolarities;

typedef enum {
	VPP_TILING_NONE = 0,
	VPP_TILING_256_16,
	VPP_TILING_512_8,
	VPP_TILING_INVALID,
}e_vpp_tiling;

#ifdef __VPP_TARGET_IS_SIMULATOR__
typedef enum {
	VPP_KICK_AUTO = 0,
	VPP_KICK_MANUAL,
	VPP_KICK_ON,
	VPP_KICK_OFF,
	VPP_KICK_INVALID,
}e_vpp_kick;
#endif

/*!
******************************************************************************

 This type defines the video stream format.

******************************************************************************/
typedef enum
{
	/*! If input is 444 (Modes 1,2,8 and 9) */
	VPP_VID_STREAM_FORMAT_444,
	/*! 4:2:2 Input Data					*/
	VPP_VID_STREAM_FORMAT_422,
	/*! 4:4:4 656 with embedded TRS			*/
	VPP_VID_STREAM_FORMAT_656_EMBED_TRS,
	/*! 4:4:4 656 with HVF					*/
	VPP_VID_STREAM_FORMAT_656_HVF

} VPP_eVideoStreamFormat;

/*!
******************************************************************************

 This type defines whether the source has field adjust.

******************************************************************************/
typedef enum
{
	/*! Unknown Field Adjust					*/
	VPP_FIELD_ADJUST_UNKNOWN,
	/*! No Field Adjust							*/
	VPP_FIELD_ADJUST_NONE,
	/*! Field Adjust on Field 0					*/
	VPP_FIELD_ADJUST_ON_FIELD_0,
	/*! Field Adjust on Field 1					*/
	VPP_FIELD_ADJUST_ON_FIELD_1,

} VPP_eFieldAdjust;




#endif //VPP_API_UAPI_H
