/*
 * PDP Desktop Graphics Framebuffer
 *
 * Copyright (c) 2011 Imagination Technologies Ltd.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */

#ifndef _UAPI_VIDEO_PDPFB_H
#define _UAPI_VIDEO_PDPFB_H

#ifndef __PDPFB_USE_OSAL__
	#include <linux/ioctl.h>
	#include <linux/types.h>
#endif

#define PDP_GAMMA_TABLE_ENTRIES		33

struct pdpfb_geom {
	__u16 x, y;
	__u16 w, h;	/* 0 for no scaling */
};

struct pdpfb_ckey {
	__u32 ckey;	/* rgb101010 */
	__u32 mask;	/* mask of ckey */
};

struct pdpfb_lumakey_coefs {
	__u32 cr, cg, cb;			/* RGB coefficients, in 10.2 unsigned fixed point format */
	__u32 output_offset;		/* Y output offset, in 9.1 unsigned fixed point format. Should always have the fraction bit set for rounding */
};

struct pdpfb_lumakey {
	__u32 enable;
	__u32 preset;				/* 0 for custom coefficients */
	struct pdpfb_lumakey_coefs coefs;
	__u32 luma_key_alpha_mult;	/* multiply incoming alpha with luma key alpha */
	__u16 luma_key_min;			/* minimum luma value (10 bit) */
	__u16 luma_key_max;			/* maximum luma value (10 bit) - luma values between luma_key_min and luma_key_max have alpha of 'transparent', otherwise 'opaque' */
	__u16 alpha_opaque;			/* alpha value for 'opaque' (10 bit) */
	__u16 alpha_trans;			/* alpha value for 'transparent' (10 bit) */
};

struct pdpfb_blend_config {
	__u32 blend_mode;
	__u32 porter_mode;
	__u32 blend_type;
};

/* YUV->RGB colour space conversion */
struct pdpfb_csc_coefs {
	__s32 ry, rv, ru;
	__s32 gy, gv, gu;
	__s32 by, bv, bu;
};

struct pdpfb_csc {
	__u32 enable;
	__u32 cosited;
};

struct pdpfb_csc_config {
	__u32 preset;	/* 0 for custom coefficients */

	struct pdpfb_csc_coefs coefs;
};

/* ProcAmp */
struct pdpfb_procamp_hsbc {
	__s32 hue;					/* Between -30 and +30 degrees (specified in x.25 signed fixed point format)	*/
	__s32 saturation;			/* Between 0 and 2 (specified in x.25 signed fixed point format)				*/
	__s32 brightness;			/* Between -50 and 50 (specified in x.25 signed fixed point format)				*/
	__s32 contrast;				/* Between 0 and 2 (specified in x.25 signed fixed point format)				*/
};

struct pdpfb_procamp_coefs {
	__s32 c11, c12, c13;
	__s32 c21, c22, c23;
	__s32 c31, c32, c33;
};

struct pdpfb_procamp_offsets {
	__s32 r, g, b;
};

struct pdpfb_procamp {
	__u32 enable;
	__u32 custom;
	struct pdpfb_procamp_hsbc hsbc;
	__u32 output_range;
	struct pdpfb_procamp_coefs coefs;				/* In 4.10 signed fixed point format */
	struct pdpfb_procamp_offsets input_offsets;		/* In 10.0 signed integer format */
	struct pdpfb_procamp_offsets output_offsets;	/* In 11.1 signed fixed point format */
};

struct pdpfb_gamma_config {
	__u32 custom;
	__u32 input_transfer_function;
	__u32 output_transfer_function;
	__u32 coefs[PDP_GAMMA_TABLE_ENTRIES];			/* In RGB101010 format */
};

/* Planar pixel formats */
struct pdpfb_planar {
	/* offsets of planes inside framebuffer */
	__u32 y_offset;
	__u32 u_offset;
	__u32 v_offset;
	/* line strides for each plane */
	__u32 y_line_length;
	__u32 u_line_length;
	__u32 v_line_length;
};

/* User provided memory */
struct pdpfb_usermem {
	__u32 addr;					/* Must be 16-byte aligned */
	__u32 len;
	__u16 flags;
};

struct pdpfb_usermem_file {
	char name[256];				/* Name of file containing the image data */
	__u32 offset;				/* File offset of start of image data */
};

struct pdpfb_bgnd_col {
	__u32 rgb;					/* RGB101010 */
	__u16 alpha;				/* 10-bit alpha */
};

struct pdpfb_disp_mode {
	__u32 xres;
	__u32 yres;
	__u32 refresh;				/* Refresh rate in Hz */
};

#define PDP_CKEYMODE_DISABLE	0x0
#define PDP_CKEYMODE_PREVIOUS	0x1	/* compare ckey with previous plane */
#define PDP_CKEYMODE_CURRENT	0x2	/* compare ckey with current plane */

#define PDP_BLENDMODE_NOALPHA	0x0
#define PDP_BLENDMODE_INVERT	0x1
#define PDP_BLENDMODE_GLOBAL	0x2
#define PDP_BLENDMODE_PIXEL		0x3

#define PDP_PORTERMODE_FOREGROUND_OVER_BACKGROUND	0x0
#define PDP_PORTERMODE_FOREGROUND_IN_BACKGROUND		0x1
#define PDP_PORTERMODE_FOREGROUND_OUT_BACKGROUND	0x2
#define PDP_PORTERMODE_FOREGROUND_ATOP_BACKGROUND	0x3
#define PDP_PORTERMODE_FOREGROUND_XOR_BACKGROUND	0x4
#define PDP_PORTERMODE_FOREGROUND_PLUS_BACKGROUND	0x5
#define PDP_PORTERMODE_BACKGROUND_OVER_FOREGROUND	0x6
#define PDP_PORTERMODE_BACKGROUND_IN_FOREGROUND		0x7
#define PDP_PORTERMODE_BACKGROUND_OUT_FOREGROUND	0x8
#define PDP_PORTERMODE_BACKGROUND_ATOP_FOREGROUND	0x9
#define PDP_PORTERMODE_BACKGROUND_XOR_FOREGROUND	0xA
#define PDP_PORTERMODE_CLEAR						0xB
#define PDP_PORTERMODE_FOREGROUND					0xC

#define PDP_BLENDTYPE_NORMAL		0x0
#define PDP_BLENDTYPE_CLIP_TO_SELF	0x1

/* non standard pixel formats */
#define PDP_PIXFMT_420_PL8			1	/* YV12 */
#define PDP_PIXFMT_420_PL8IVU		2	/* 4:2:0 8-bit planar, interleaved chroma (v lsb) */
#define PDP_PIXFMT_420_PL8IUV		3	/* 4:2:0 8-bit planar, interleaved chroma (u lsb) */
#define PDP_PIXFMT_420_PL10IVU		4	/* 4:2:0 10-bit planar, interleaved chroma (v lsb) */
#define PDP_PIXFMT_420_PL10IUV		5	/* 4:2:0 10-bit planar, interleaved chroma (u lsb) */
#define PDP_PIXFMT_422_PL10IVU		6	/* 4:2:2 10-bit planar, interleaved chroma (v lsb) */
#define PDP_PIXFMT_422_PL10IUV		7	/* 4:2:2 10-bit planar, interleaved chroma (u lsb) */
#define PDP_PIXFMT_422_UY0VY1_8888	8	/* U lsb */
#define PDP_PIXFMT_422_VY0UY1_8888	9	/* V lsb */
#define PDP_PIXFMT_422_Y0UY1V_8888	10	/* Y0 lsb */
#define PDP_PIXFMT_422_Y0VY1U_8888	11	/* Y0 lsb */
#define PDP_PIXFMT_AYUV8888			12	/* A lsb */
#define PDP_PIXFMT_YUV101010		13	/* Y lsb */
#define PDP_PIXFMT_MAX				14

#define PDP_CSCPRESET_HDTV			0x1
#define PDP_CSCPRESET_SDTV			0x2	/* default */
#define PDP_CSCPRESET_LEGACYHDTV	0x3
#define PDP_CSCPRESET_LEGACYSDTV	0x4

#define PDP_PROCAMP_OUTPUT_RANGE_0_1023		0x0	/* 0 - 1023 */
#define PDP_PROCAMP_OUTPUT_RANGE_64_940		0x1	/* 64 - 940 */
#define PDP_PROCAMP_OUTPUT_RANGE_192_832	0x2	/* 192 - 832 */

#define PDP_PROCAMP_HSBC_FRACTIONAL_BITS	25

#define PDP_GAMMA_1_0			0	/* 1.0 */
#define PDP_GAMMA_1_4			1	/* 1.4 */
#define PDP_GAMMA_1_8			2	/* 1.8 */
#define PDP_GAMMA_2_0			3	/* 2.0 */
#define PDP_GAMMA_2_2			4	/* 2.2 */
#define PDP_GAMMA_709			5	/* BT709 */
#define PDP_GAMMA_240M			6	/* SMPTE240M */
#define PDP_GAMMA_sRGB			7	/* sRGB */
#define PDP_GAMMA_2_8			8	/* 2.8 */

#define PDP_LUMAKEYPRESET_HDTV		0x1
#define PDP_LUMAKEYPRESET_SDTV		0x2

/* PDPIO_SETUSERMEM flags */
#define PDP_USERMEM_ALLPLANES	0x1
#define PDP_USERMEM_VIRTUAL		0x2

#define PDP_3D_MODE_DISABLE			0 /* Disable alternate 3D mode					*/
#define PDP_3D_MODE_BGND_WITH_POS0	1 /* Alternate background with plane in pos0	*/
#define PDP_3D_MODE_POS0_WITH_POS1	2 /* Alternate plane in pos0 with plane in pos1 */
#define PDP_3D_MODE_POS1_WITH_POS2	3 /* Alternate plane in pos1 with plane in pos2 */
#define PDP_3D_MODE_POS2_WITH_POS3	4 /* Alternate plane in pos2 with plane in pos3 */
#define PDP_3D_MODE_POS3_WITH_POS4	5 /* Alternate plane in pos3 with plane in pos4 */

#define PDP_SPLIT_SCALE_DISABLE		0 /* No split scaling							*/
#define PDP_SPLIT_SCALE_LEFT_PLANE	1 /* Left half of a split scaling operation		*/
#define PDP_SPLIT_SCALE_RIGHT_PLANE	2 /* Right half of a split scaling operation	*/

#define PDPIO 0xF2

/* general (0x00 - 0x0F) */
#define PDPIO_GETBGND			_IOR(PDPIO, 0x00, struct pdpfb_bgnd_col)
#define PDPIO_SETBGND			_IOW(PDPIO, 0x00, struct pdpfb_bgnd_col)
#define PDPIO_GETDISPMODE		_IOR(PDPIO, 0x01, struct pdpfb_disp_mode)
#define PDPIO_SETDISPMODE		_IOWR(PDPIO, 0x01, struct pdpfb_disp_mode)
#define PDPIO_GETCSCCONFIG		_IOR(PDPIO, 0x02, struct pdpfb_csc_config)
#define PDPIO_SETCSCCONFIG		_IOWR(PDPIO, 0x02, struct pdpfb_csc_config)
#define PDPIO_GETPROCAMP		_IOR(PDPIO, 0x03, struct pdpfb_procamp)
#define PDPIO_SETPROCAMP		_IOWR(PDPIO, 0x03, struct pdpfb_procamp)
#define PDPIO_GETGAMMACONFIG	_IOR(PDPIO, 0x04, struct pdpfb_gamma_config)
#define PDPIO_SETGAMMACONFIG	_IOWR(PDPIO, 0x04, struct pdpfb_gamma_config)
#define PDPIO_SETUSERMEM		_IOW(PDPIO, 0x05, struct pdpfb_usermem)
#define PDPIO_GET3DMODE			_IOR(PDPIO, 0x06, __u32)
#define PDPIO_SET3DMODE			_IOW(PDPIO, 0x06, __u32)
#define PDPIO_GETOUTPUTDEPTH	_IOR(PDPIO, 0x07, __u32)
#define PDPIO_SETOUTPUTDEPTH	_IOW(PDPIO, 0x07, __u32)
#define PDPIO_GETSIGNATURE		_IOR(PDPIO, 0x08, __u32)
#define PDPIO_SETSIMULATOR		_IOR(PDPIO, 0x09, int)
#define PDPIO_SETUSERMEMFILE	_IOW(PDPIO, 0x0A, struct pdpfb_usermem_file)

/* general plane (0x10 - 0x1F) */
#define PDPIO_GETEN				_IOR(PDPIO, 0x10, int)
#define PDPIO_SETEN				_IOW(PDPIO, 0x10, int)
#define PDPIO_GETPLANEPOS		_IOR(PDPIO, 0x11, int)
#define PDPIO_SETPLANEPOS		_IOWR(PDPIO, 0x11, int)
#define PDPIO_GETGEOM			_IOR(PDPIO, 0x12, struct pdpfb_geom)
#define PDPIO_SETGEOM			_IOWR(PDPIO, 0x12, struct pdpfb_geom)
#define PDPIO_GETCKEYMODE		_IOR(PDPIO, 0x13, __u32)
#define PDPIO_SETCKEYMODE		_IOW(PDPIO, 0x13, __u32)
#define PDPIO_GETCKEY			_IOR(PDPIO, 0x14, struct pdpfb_ckey)
#define PDPIO_SETCKEY			_IOWR(PDPIO, 0x14, struct pdpfb_ckey)
#define PDPIO_GETBLENDCONFIG	_IOR(PDPIO, 0x15, struct pdpfb_blend_config)
#define PDPIO_SETBLENDCONFIG	_IOW(PDPIO, 0x15, struct pdpfb_blend_config)
#define PDPIO_GETGALPHA			_IOR(PDPIO, 0x16, __u32)
#define PDPIO_SETGALPHA			_IOWR(PDPIO, 0x16, __u32)
#define PDPIO_GETCSC			_IOR(PDPIO, 0x17, struct pdpfb_csc)
#define PDPIO_SETCSC			_IOWR(PDPIO, 0x17, struct pdpfb_csc)
#define PDPIO_GETGAMMAEN		_IOR(PDPIO, 0x18, int)
#define PDPIO_SETGAMMAEN		_IOW(PDPIO, 0x18, int)
#define PDPIO_GETDITHERINGEN	_IOR(PDPIO, 0x19, int)
#define PDPIO_SETDITHERINGEN	_IOW(PDPIO, 0x19, int)
#define PDPIO_GETBILINEAREN		_IOW(PDPIO, 0x1A, int)
#define PDPIO_SETBILINEAREN		_IOR(PDPIO, 0x1A, int)

/* graphics planes (0x20 - 0x2F) */

/* video planes (0x30 - 0x3F) */
#define PDPIO_GETPLANAR			_IOR(PDPIO, 0x31, struct pdpfb_planar)
#define PDPIO_SETPLANAR			_IOWR(PDPIO, 0x31, struct pdpfb_planar)
#define PDPIO_GETLUMAKEY		_IOR(PDPIO, 0x32, struct pdpfb_lumakey)
#define PDPIO_SETLUMAKEY		_IOW(PDPIO, 0x32, struct pdpfb_lumakey)
#define PDPIO_GETCLIPGEOM		_IOR(PDPIO, 0x33, struct pdpfb_geom)
#define PDPIO_SETCLIPGEOM		_IOW(PDPIO, 0x33, struct pdpfb_geom)
#define PDPIO_GETSPLITSCALE		_IOR(PDPIO, 0x34, int)
#define PDPIO_SETSPLITSCALE		_IOW(PDPIO, 0x34, int)

#endif /* _UAPI_VIDEO_PDPFB_H */
