/*
 * Cashew Event Timer Internal Header
 *
 * Copyright (C) 2015 Imagination Technologies Ltd.
 *
 * Author: Sifan Naeem <sifan.naeem@imgtec.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 */

#ifndef __CASHEW_EVT_INTERNAL_H__
#define __CASHEW_EVT_INTERNAL_H__

enum cashew_evt_state {
	CASHEW_EVT_STATE_IDLE = 0,
	CASHEW_EVT_STATE_ACTIVE_FIRST,
	CASHEW_EVT_STATE_ACTIVE_SECOND,
	CASHEW_EVT_STATE_ACTIVE_THIRD,
	CASHEW_EVT_STATE_COMPLETE
};

struct cashew_evt_callback {
	u64 trigger_time;
	u32 cyc;
	void (*callback)(struct cashew_evt *, void *);
	void *context;
};

struct cashew_evt_measurement {
	enum cashew_evt_state state;
	void (*callback)(void *);
	void *context;
};

struct cashew_evt_ctx {
	u32 counter;
	u32 timer_enable;
	u32 sources;
	u32 event_ctl;
	u32 int_enable;
	u32 time[CASHEW_EVT_NUM_ENABLES];
	u32 source[CASHEW_EVT_SOURCE_MAX];
	u32 et_event[CASHEW_EVT_SOURCE_MAX];
};

struct cashew_evt {
	struct list_head list;
	spinlock_t lock;
	struct device *dev;
	struct device_node *np;
	void __iomem *base;
	struct clk *audio_pll;
	struct clk *clk_sys;
	struct clk *clk_ref_a;
	struct clk *clk_ref_b;
	const char *ref_names[2];
	struct clk *clk_ref_internal;
	struct cyclecounter cc;
	struct timecounter tc;
	struct notifier_block evt_clk_notifier;
	struct hrtimer poll_timer;
	ktime_t quarter_rollover;
	unsigned long sys_rate;
	struct cashew_evt_callback trigger_cbs[CASHEW_EVT_NUM_ENABLES];
	struct cashew_evt_measurement sample_rates[CASHEW_EVT_MAX_SOURCES];
	struct cashew_evt_measurement phase_difference;
	struct cashew_evt_ctx suspend_ctx;
};

/* Call with lock held */
u64 _cashew_evt_get_time(struct cashew_evt *evt);
/* Call without lock held */
u64 cashew_evt_get_time(struct cashew_evt *evt);
int cashew_evt_time_to_reg(struct cashew_evt *evt, u64 time,
			   u32 *reg, u64 min_time_delta);
int cashew_evt_init(struct cashew_evt *evt);
void cashew_evt_deinit(struct cashew_evt *evt);
void cashew_evt_clk_rate_change(struct cashew_evt *evt);
void cashew_evt_restart(struct cashew_evt *evt);
void cashew_evt_stop(struct cashew_evt *evt);

#endif
