/*
 * Imagination Technologies Cashew Event Timer Header
 *
 * Copyright (C) 2015 Imagination Technologies Ltd.
 *
 * Author: Sifan Naeem <sifan.naeem@imgtec.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 */

#ifndef __IMG_CASHEW_EVT_H__
#define __IMG_CASHEW_EVT_H__

struct cashew_evt;

enum cashew_evt_enable {
	CASHEW_EVT_ENABLE_ETS = 0,
	CASHEW_EVT_ENABLE_SPDIF_OUT,
	CASHEW_EVT_ENABLE_I2S_OUT,
	CASHEW_EVT_ENABLE_PARALLEL_IN,
	CASHEW_EVT_ENABLE_IRQ_0,
	CASHEW_EVT_ENABLE_IRQ_1,
	CASHEW_EVT_NUM_ENABLES
};

enum cashew_evt_type {
	CASHEW_EVT_TYPE_PULSE = 1,
	CASHEW_EVT_TYPE_LEVEL
};

enum cashew_evt_source {
	CASHEW_EVT_SOURCE_GPIO_0 = 0,
	CASHEW_EVT_SOURCE_GPIO_8,
	CASHEW_EVT_SOURCE_SPDIF_IN,
	CASHEW_EVT_SOURCE_SPDIF_OUT,
	CASHEW_EVT_SOURCE_I2S_IN_0,
	CASHEW_EVT_SOURCE_I2S_OUT_0,
	CASHEW_EVT_NUM_SOURCES
};

/* Number of timestamps that can be logged simultaneously. */
#define	CASHEW_EVT_NUM_TIMESTAMPS	12
/* Maximum number of event source inputs. */
#define	CASHEW_EVT_SOURCE_MAX		12
/* Number of ets sources. */
#define	CASHEW_EVT_NUM_EVENTS		30

#define	CASHEW_EVT_MAX_SOURCES		2

extern struct cashew_evt *cashew_evt_get(struct device_node *np);

extern void cashew_evt_get_time_ts(struct cashew_evt *evt,
				   struct timespec *ts);

/* Call this outside of an event callback */
extern void cashew_evt_disable_event(struct cashew_evt *evt,
				     enum cashew_evt_enable event);

/* Call this inside of an event callback */
extern void _cashew_evt_disable_event(struct cashew_evt *evt,
				      enum cashew_evt_enable event);

extern int cashew_evt_set_event(struct cashew_evt *evt,
		enum cashew_evt_enable event,
		enum cashew_evt_type type,
		struct timespec *ts,
		void (*event_trigger_callback)(struct cashew_evt *, void *),
		void *context);

extern int cashew_evt_set_source(struct cashew_evt *evt, int id,
				 enum cashew_evt_source source);

extern int cashew_evt_get_source(struct cashew_evt *evt,
				 int id, enum cashew_evt_source *source);

extern int cashew_evt_get_sample_rate(struct cashew_evt *evt, int id,
				      u32 *val, u32 *sys_freq,
				      void (*callback)(void *), void *context);

extern int cashew_evt_get_phase_diff(struct cashew_evt *evt, u32 *val,
				     u32 *sys_freq, void (*callback)(void *),
				     void *context);

extern void cashew_evt_abort_measurements(struct cashew_evt *evt);

extern int cashew_evt_read_time_stamp(struct cashew_evt *evt, u32 *val,
				      u32 *event, int index);

extern int cashew_evt_set_event_source(struct cashew_evt *evt, int event,
				       int index);
#endif
