/*
 * clk_connections.h clk-cashew
 *
 * Copyright (C) 2015 Imagination Technologies Ltd.
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 */

#include "cashew_clk_regs.h"
#define MUXC MUX

static struct cashew_pll_rate_table pll0_rates[] = {
	{
		.fref		= 19200000,
		.fout_min	= 1200000000,
		.fout		= 1200000000,
		.fout_max	= 1200000000,
		.refdiv		= 1,
		.fbdiv		= 62,
		.frac		= 0x800000,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 1200000000,
		.fout		= 1200000000,
		.fout_max	= 1200000000,
		.refdiv		= 1,
		.fbdiv		= 62,
		.frac		= 0x800000,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 1200000000,
		.fout		= 1200000000,
		.fout_max	= 1200000000,
		.refdiv		= 1,
		.fbdiv		= 23,
		.frac		= 0x13B13B,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
};

static struct cashew_pll_rate_table pll1_rates[] = {
	{
		.fref		= 19200000,
		.fout_min	= 2000000000,
		.fout		= 2000000000,
		.fout_max	= 2000000000,
		.refdiv		= 1,
		.fbdiv		= 104,
		.frac		= 0x2AAAAB,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},	{
		.fref		= 38400000,
		.fout_min	= 2000000000,
		.fout		= 2000000000,
		.fout_max	= 2000000000,
		.refdiv		= 1,
		.fbdiv		= 52,
		.frac		= 0x155555,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 2000000000,
		.fout		= 2000000000,
		.fout_max	= 2000000000,
		.refdiv		= 1,
		.fbdiv		= 38,
		.frac		= 0x762762,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
};

static struct cashew_pll_rate_table pll2_rates[] = {
	{
		.fref		= 38400000,
		.fout_min	= 1600000000,
		.fout		= 1600000000,
		.fout_max	= 1600000000,
		.refdiv		= 1,
		.fbdiv		= 41,
		.frac		= 0xAAAAAB,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
		{
		.fref		= 19200000,
		.fout_min	= 1600000000,
		.fout		= 1600000000,
		.fout_max	= 1600000000,
		.refdiv		= 1,
		.fbdiv		= 83,
		.frac		= 0x555555,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 1600000000,
		.fout		= 1600000000,
		.fout_max	= 1600000000,
		.refdiv		= 1,
		.fbdiv		= 30,
		.frac		= 0xC4EC4F,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 19200000,
		.fout_min	= 800000000,
		.fout		= 800000000,
		.fout_max	= 800000000,
		.refdiv		= 1,
		.fbdiv		= 41,
		.frac		= 0xAAAAAB,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 800000000,
		.fout		= 800000000,
		.fout_max	= 800000000,
		.refdiv		= 1,
		.fbdiv		= 41,
		.frac		= 0xAAAAAB,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 800000000,
		.fout		= 800000000,
		.fout_max	= 800000000,
		.refdiv		= 1,
		.fbdiv		= 30,
		.frac		= 0xC4EC4F,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
};


static struct cashew_pll_rate_table pll3_rates[] = {
	{
		.fref		= 19200000,
		.fout_min	= 1200000000,
		.fout		= 1200000000,
		.fout_max	= 1200000000,
		.refdiv		= 1,
		.fbdiv		= 62,
		.frac		= 0x800000,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},

	{
		.fref		= 38400000,
		.fout_min	= 1200000000,
		.fout		= 1200000000,
		.fout_max	= 1200000000,
		.refdiv		= 1,
		.fbdiv		= 62,
		.frac		= 0x800000,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 1200000000,
		.fout		= 1200000000,
		.fout_max	= 1200000000,
		.refdiv		= 1,
		.fbdiv		= 46,
		.frac		= 0x276276,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
};

static struct cashew_pll_rate_table pll5_rates[] = {
	{
		.fref		= 26000000,
		.fout_min	= 520020000,
		.fout		= 520020000,
		.fout_max	= 520020000,
		.refdiv		= 1,
		.fbdiv		= 40,
		.frac		= 25811,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 26000000,
		.fout_min	= 2400000000UL,
		.fout		= 2400000000UL,
		.fout_max	= 2400000000UL,
		.refdiv		= 1,
		.fbdiv		= 92,
		.frac		= 0x4EC4EC,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 520020000,
		.fout		= 520020000,
		.fout_max	= 520020000,
		.refdiv		= 1,
		.fbdiv		= 20,
		.frac		= 0x326A,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 520020000,
		.fout		= 520020000,
		.fout_max	= 520020000,
		.refdiv		= 1,
		.fbdiv		= 27,
		.frac		= 0x15999A,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 2400000000UL,
		.fout		= 2400000000UL,
		.fout_max	= 2400000000UL,
		.refdiv		= 1,
		.fbdiv		= 62,
		.frac		= 0x800000,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 720000000,
		.fout		= 720000000,
		.fout_max	= 720000000,
		.refdiv		= 1,
		.fbdiv		= 37,
		.frac		= 0x800000,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 720000000,
		.fout		= 720000000,
		.fout_max	= 720000000,
		.refdiv		= 1,
		.fbdiv		= 27,
		.frac		= 0xB13B14,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 768000000,
		.fout		= 768000000,
		.fout_max	= 768000000,
		.refdiv		= 1,
		.fbdiv		= 29,
		.frac		= 0x89D89E,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 768000000,
		.fout		= 768000000,
		.fout_max	= 768000000,
		.refdiv		= 1,
		.fbdiv		= 40,
		.frac		= 0x0,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 27000000,
		.fout_min	= 520020000,
		.fout		= 520020000,
		.fout_max	= 520020000,
		.refdiv		= 1,
		.fbdiv		= 38,
		.frac		= 8724152,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 27000000,
		.fout_min	= 2400000000UL,
		.fout		= 2400000000UL,
		.fout_max	= 2400000000UL,
		.refdiv		= 1,
		.fbdiv		= 88,
		.frac		= 0xE38E39,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 27000000,
		.fout_min	= 2400000000UL,
		.fout		= 2400000000UL,
		.fout_max	= 2400000000UL,
		.refdiv		= 1,
		.fbdiv		= 88,
		.frac		= 0xE38E39,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 19200000,
		.fout_min	= 2400000000UL,
		.fout		= 2400000000UL,
		.fout_max	= 2400000000UL,
		.refdiv		= 1,
		.fbdiv		= 125,
		.frac		= 0,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 19200000,
		.fout_min	= 520020000,
		.fout		= 520020000,
		.fout_max	= 520020000,
		.refdiv		= 1,
		.fbdiv		= 27,
		.frac		= 0x15999A,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
};


static struct cashew_pll_rate_table pll6_rates[] = {
	{
		.fref		= 38400000,
		.fout_min	= 1843200000/2,
		.fout		= 1843200000/2,
		.fout_max	= 1843200000/2,
		.refdiv		= 0x1,
		.fbdiv		= 48,
		.frac		= 0,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 135475200,
		.fout		= 135475200,
		.fout_max	= 135475200,
		.refdiv		= 0x1,
		.fbdiv		= 28,
		.frac		= 395810,
		.postdiv1	= 0x4,
		.postdiv2	= 0x2,
	},
	{
		.fref		= 38400000,
		.fout_min	= 147456000,
		.fout		= 147456000,
		.fout_max	= 147456000,
		.refdiv		= 0x1,
		.fbdiv		= 30,
		.frac		= 12079596,
		.postdiv1	= 0x4,
		.postdiv2	= 0x2,
	},
	{
		.fref		= 38400000,
		.fout_min	= 1843200000,
		.fout		= 1843200000,
		.fout_max	= 1843200000,
		.refdiv		= 0x1,
		.fbdiv		= 48,
		.frac		= 0,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 1843200000,
		.fout		= 1843200000,
		.fout_max	= 1843200000,
		.refdiv		= 0x1,
		.fbdiv		= 48,
		.frac		= 0,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 1693440000,
		.fout		= 1693440000,
		.fout_max	= 1693440000,
		.refdiv		= 0x1,
		.fbdiv		= 44,
		.frac		= 0x19999A,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 19200000,
		.fout_min	= 1843200000,
		.fout		= 1843200000,
		.fout_max	= 1843200000,
		.refdiv		= 0x1,
		.fbdiv		= 96,
		.frac		= 0,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 26000000,
		.fout_min	= 1693440000,
		.fout		= 1693440000,
		.fout_max	= 1693440000,
		.refdiv		= 0x1,
		.fbdiv		= 65,
		.frac		= 0x21deeb,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},	{
		.fref		= 26000000,
		.fout_min	= 1843200000,
		.fout		= 1843200000,
		.fout_max	= 1843200000,
		.refdiv		= 0x1,
		.fbdiv		= 70,
		.frac		= 0xe46e47,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 1843200000,
		.fout		= 1843200000,
		.fout_max	= 1843200000,
		.refdiv		= 0x1,
		.fbdiv		= 35,
		.frac		= 0x723723,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 921600000,
		.fout		= 921600000,
		.fout_max	= 921600000,
		.refdiv		= 0x1,
		.fbdiv		= 35,
		.frac		= 0x723723,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 594000000,
		.fout_min	= 2970000000UL,
		.fout		= 2970000000UL,
		.fout_max	= 2970000000UL,
		.refdiv		= 0x8,
		.fbdiv		= 40,
		.frac		= 0,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
};

static struct cashew_pll_rate_table pll7_rates[] = {
	{
		.fref		= 38400000,
		.fout_min	= 1600000000,
		.fout		= 1600000000,
		.fout_max	= 1600000000,
		.refdiv		= 1,
		.fbdiv		= 41,
		.frac		= 0xAAAAAB,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 1600000000,
		.fout		= 1600000000,
		.fout_max	= 1600000000,
		.refdiv		= 1,
		.fbdiv		= 41,
		.frac		= 0xAAAAAB,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 52000000,
		.fout_min	= 1600000000,
		.fout		= 1600000000,
		.fout_max	= 1600000000,
		.refdiv		= 1,
		.fbdiv		= 30,
		.frac		= 0xC4EC4F,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
	{
		.fref		= 240000000,
		.fout_min	= 768000000,
		.fout		= 768000000,
		.fout_max	= 768000000,
		.refdiv		= 5,
		.fbdiv		= 32,
		.frac		= 0,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 252000000,
		.fout_min	= 672000000,
		.fout		= 672000000,
		.fout_max	= 672000000,
		.refdiv		= 6,
		.fbdiv		= 32,
		.frac		= 0,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
	{
		.fref		= 288000000,
		.fout_min	= 768000000,
		.fout		= 768000000,
		.fout_max	= 768000000,
		.refdiv		= 6,
		.fbdiv		= 32,
		.frac		= 0,
		.postdiv1	= 2,
		.postdiv2	= 1,
	},
};

static struct cashew_pll_rate_table pll8_rates[] = {
	{
		.fref		= 38400000,
		.fout_min	= 640000000,
		.fout		= 640000000,
		.fout_max	= 640000000,
		.refdiv		= 1,
		.fbdiv		= 66,
		.frac		= 11184811,
		.postdiv1	= 4,
		.postdiv2	= 1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 640000000,
		.fout		= 640000000,
		.fout_max	= 640000000,
		.refdiv		= 1,
		.fbdiv		= 66,
		.frac		= 11184811,
		.postdiv1	= 4,
		.postdiv2	= 1,
	},
	{
		.fref		= 19200000,
		.fout_min	= 640000000,
		.fout		= 640000000,
		.fout_max	= 640000000,
		.refdiv		= 1,
		.fbdiv		= 33,
		.frac		= 0x555555,
		.postdiv1	= 1,
		.postdiv2	= 1,
	},
};

static struct cashew_pll_rate_table pdp_pll_rates[] = {
	{
		/*To provide  594 MHz */
		.fref		= 74250000,
		.fout_min	= 594000000,
		.fout		= 594000000,
		.fout_max	= 594000000,
		.refdiv		= 0x1,
		.fbdiv		= 0x20,
		.frac		= 0x0,
		.postdiv1	= 0x4,
		.postdiv2	= 0x1,
	},
	{
		/* To provide  593.4066MHz */
		.fref		= 74250000,
		.fout_min	= 593406500,
		.fout		= 593406600,
		.fout_max	= 593406700,
		.refdiv		= 0x1,
		.fbdiv		= 0x1F,
		.frac		= 0xF7D0F1,
		.postdiv1	= 0x4,
		.postdiv2	= 0x1,
	},
	{
		/*To provide  594 MHz */
		.fref		= 26000000,
		.fout_min	= 593999900,
		.fout		= 594000000,
		.fout_max	= 594000100,
		.refdiv		= 0x1,
		.fbdiv		= 0x2D,
		.frac		= 0xB13B14,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		/* To provide  495MHz */
		.fref		= 26000000,
		.fout_min	= 494999900,
		.fout		= 495000000,
		.fout_max	= 495000100,
		.refdiv		= 0x1,
		.fbdiv		= 0x26,
		.frac		= 0x13B13B,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		/* To provide  371.25MHz */
		.fref		= 26000000,
		.fout_min	= 371259900,
		.fout		= 371250000,
		.fout_max	= 371250100,
		.refdiv		= 0x1,
		.fbdiv		= 0x1C,
		.frac		= 0x8EC4EC,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		/*To provide  594 MHz */
		.fref		= 52000000,
		.fout_min	= 593999900,
		.fout		= 594000000,
		.fout_max	= 594000100,
		.refdiv		= 0x1,
		.fbdiv		= 0x2D,
		.frac		= 0xB13B14,
		.postdiv1	= 0x4,
		.postdiv2	= 0x1,
	},
	{
		/* To provide  495MHz */
		.fref		= 52000000,
		.fout_min	= 494999900,
		.fout		= 495000000,
		.fout_max	= 495000100,
		.refdiv		= 0x1,
		.fbdiv		= 0x26,
		.frac		= 0x13B13B,
		.postdiv1	= 0x4,
		.postdiv2	= 0x1,
	},
	{
		/* To provide  371.25MHz */
		.fref		= 52000000,
		.fout_min	= 371259900,
		.fout		= 371250000,
		.fout_max	= 371250100,
		.refdiv		= 0x1,
		.fbdiv		= 0x1C,
		.frac		= 0x8EC4EC,
		.postdiv1	= 0x4,
		.postdiv2	= 0x1,
	},
	/* Some valid paramaters for fref = 38.4MHz */
	{
		.fref		= 38400000,
		.fout_min	= 594000000,
		.fout		= 594000000,
		.fout_max	= 594000000,
		.refdiv		= 0x1,
		.fbdiv		= 0x1E,
		.frac		= 0xF00000,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 38400000,
		.fout_min	= 495000000,
		.fout		= 495000000,
		.fout_max	= 495000000,
		.refdiv		= 0x1,
		.fbdiv		= 0x19,
		.frac		= 0xC80000,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 19200000,
		.fout_min	= 594000000,
		.fout		= 594000000,
		.fout_max	= 594000000,
		.refdiv		= 0x1,
		.fbdiv		= 0x3D,
		.frac		= 0xE00000,
		.postdiv1	= 0x2,
		.postdiv2	= 0x1,
	},
	{
		.fref		= 19200000,
		.fout_min	= 495000000,
		.fout		= 495000000,
		.fout_max	= 495000000,
		.refdiv		= 0x1,
		.fbdiv		= 0x19,
		.frac		= 0xC80000,
		.postdiv1	= 0x1,
		.postdiv2	= 0x1,
	},
};


PNAME(cs_pll2_pll2refclk_0_en) = {"cr_top_cs_pll2refclk_0_en", "pll2" };
PNAME(cs_pll1_pll1refclk_0_en) = {"cr_top_cs_pll1refclk_0_en", "pll1" };
PNAME(cs_pll0_pll0refclk_0_en) = {"cr_top_cs_pll0refclk_0_en", "pll0"};
PNAME(cs_pll3_pll3refclk_0_en) = {"cr_top_cs_pll3refclk_0_en", "pll3"};
PNAME(mux_pll1bypass_pll2bypass) = {"cr_top_cs_pll1bypassclk_0_en", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_pll4bypass_pll5bypass) = {"cr_top_cs_pll4bypassclk_0_en", "cr_top_cs_pll5bypassclk_0_en"};
PNAME(mux_pll0bypass_pll1bypass) = {"cr_top_cs_pll0bypassclk_0_en", "cr_top_cs_pll1bypassclk_0_en"};
PNAME(cs_xtal1_xtal2) = {"xtal1", "xtal2"};
PNAME(mux_xtal1_xtal2) = {"xtal1", "xtal2"};
PNAME(mux_xtal1_xtal0) = {"xtal1", "xtal0"};
PNAME(mux_xtal0_xtal1) = {"xtal0", "xtal1"};

/*AV*/

PNAME(cs_pll7_pll7refclk_gate) = {"cr_top_pll7refclk_gate_en", "pll7"};
PNAME(cs_pll6_pll6refclk_gate) = {"cr_top_pll6refclk_gate_en", "pll6"};
PNAME(cs_pll5_pll5refclk_gate) = {"cr_top_pll5refclk_gate_en", "pll5"};
PNAME(cs_pll4_pll4refclk_gate) = {"cr_top_pll4refclk_gate_en", "pll4"};
PNAME(mux_pll7refclk_0_lvds) = {"cr_top_mx_pll7refclk_0_sw", "lvds_clk"};
PNAME(mux_xtal2_xtal1) = {"xtal2", "xtal1"};
PNAME(mux_pll6refclk_0_hdmi_rx_tmds) = {"cr_top_mx_pll6refclk_0_sw", "hdmi_rx_tmds_clk"};

PNAME(cs_xtal1_mx_nochspreclk_2) = { "cr_top_mx_nochspreclk_2_sw", "xtal1"};
PNAME(mux_nochspreclk_1_pll3bypass) = { "cr_top_mx_nochspreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_nochspreclk_0_pll2bypass) = { "cr_top_mx_nochspreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(cs_xtal1_mx_nocmspreclk_2) = { "cr_top_mx_nocmspreclk_2_sw", "xtal1"};
PNAME(mux_nocmspreclk_1_pll3bypass) = { "cr_top_mx_nocmspreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_nocmspreclk_0_pll2bypass) = { "cr_top_mx_nocmspreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(cs_xtal1_mx_noclspreclk_2) = { "cr_top_mx_noclspreclk_2_sw", "xtal1"};
PNAME(mux_noclspreclk_1_pll3bypass) = { "cr_top_mx_noclspreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_noclspreclk_0_pll2bypass) = { "cr_top_mx_noclspreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(cs_xtal1_mx_cpu1preclk_2) = { "cr_top_mx_cpu1preclk_2_sw", "xtal1"};
PNAME(mux_cpu1preclk_1_pll3bypass) = { "cr_top_mx_cpu1preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_cpu1preclk_0_pll2bypass) = { "cr_top_mx_cpu1preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(cs_xtal1_mx_cpu2preclk_2) = { "cr_top_mx_cpu2preclk_2_sw", "xtal1"};
PNAME(mux_cpu2preclk_1_pll3bypass) = { "cr_top_mx_cpu2preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_cpu2preclk_0_pll2bypass) = { "cr_top_mx_cpu2preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_vxecorepreclk_1_pll3bypass) = { "cr_top_mx_vxecorepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_vxecorepreclk_0_pll2bypass) = { "cr_top_mx_vxecorepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_vxdcorepreclk_1_pll3bypass) = { "cr_top_mx_vxdcorepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_vxdcorepreclk_0_pll2bypass) = { "cr_top_mx_vxdcorepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_gpucorepreclk_1_pll3bypass) = { "cr_top_mx_gpucorepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_gpucorepreclk_0_pll2bypass) = { "cr_top_mx_gpucorepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_ispcorepreclk_1_pll3bypass) = { "cr_top_mx_ispcorepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_ispcorepreclk_0_pll2bypass) = { "cr_top_mx_ispcorepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_seccorepreclk_1_pll3bypass) = { "cr_top_mx_seccorepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_seccorepreclk_0_pll2bypass) = { "cr_top_mx_seccorepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_vppcorepreclk_1_pll3bypass) = { "cr_top_mx_vppcorepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_vppcorepreclk_0_pll2bypass) = { "cr_top_mx_vppcorepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_pdp0corepreclk_1_pll3bypass) = { "cr_top_mx_pdp0corepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_pdp0corepreclk_0_pll2bypass) = { "cr_top_mx_pdp0corepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_pdp1corepreclk_1_pll3bypass) = { "cr_top_mx_pdp1corepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_pdp1corepreclk_0_pll2bypass) = { "cr_top_mx_pdp1corepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_rpufastpreclk_1_pll3bypass) = { "cr_top_mx_rpufastpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_rpufastpreclk_0_pll2bypass) = { "cr_top_mx_rpufastpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_rpucorepreclk_1_pll3bypass) = { "cr_top_mx_rpucorepreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_rpucorepreclk_0_pll2bypass) = { "cr_top_mx_rpucorepreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_hdmirxcfgpreclk_1_pll3bypass) = { "cr_top_mx_hdmirxcfgpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_hdmirxcfgpreclk_0_pll2bypass) = { "cr_top_mx_hdmirxcfgpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_mipidsipreclk_1_pll3bypass) = { "cr_top_mx_mipidsipreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_mipidsipreclk_0_pll2bypass) = { "cr_top_mx_mipidsipreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_mipitxcfgpreclk_1_pll3bypass) = { "cr_top_mx_mipitxcfgpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_mipitxcfgpreclk_0_pll2bypass) = { "cr_top_mx_mipitxcfgpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_mipirx0cfgpreclk_1_pll3bypass) = { "cr_top_mx_mipirx0cfgpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_mipirx0cfgpreclk_0_pll2bypass) = { "cr_top_mx_mipirx0cfgpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_mipirx1cfgpreclk_1_pll3bypass) = { "cr_top_mx_mipirx1cfgpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_mipirx1cfgpreclk_0_pll2bypass) = { "cr_top_mx_mipirx1cfgpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c0preclk_1_pll3bypass) = { "cr_top_mx_i2c0preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c0preclk_0_pll2bypass) = { "cr_top_mx_i2c0preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c1preclk_1_pll3bypass) = { "cr_top_mx_i2c1preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c1preclk_0_pll2bypass) = { "cr_top_mx_i2c1preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c2preclk_1_pll3bypass) = { "cr_top_mx_i2c2preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c2preclk_0_pll2bypass) = { "cr_top_mx_i2c2preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c3preclk_1_pll3bypass) = { "cr_top_mx_i2c3preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c3preclk_0_pll2bypass) = { "cr_top_mx_i2c3preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c4preclk_1_pll3bypass) = { "cr_top_mx_i2c4preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c4preclk_0_pll2bypass) = { "cr_top_mx_i2c4preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c5preclk_1_pll3bypass) = { "cr_top_mx_i2c5preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c5preclk_0_pll2bypass) = { "cr_top_mx_i2c5preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c6preclk_1_pll3bypass) = { "cr_top_mx_i2c6preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c6preclk_0_pll2bypass) = { "cr_top_mx_i2c6preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_i2c7preclk_1_pll3bypass) = { "cr_top_mx_i2c7preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_i2c7preclk_0_pll2bypass) = { "cr_top_mx_i2c7preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_spim0preclk_1_pll3bypass) = { "cr_top_mx_spim0preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_spim0preclk_0_pll2bypass) = { "cr_top_mx_spim0preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_spim1preclk_1_pll3bypass) = { "cr_top_mx_spim1preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_spim1preclk_0_pll2bypass) = { "cr_top_mx_spim1preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sdhost0preclk_1_pll3bypass) = { "cr_top_mx_sdhost0preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sdhost0preclk_0_pll2bypass) = { "cr_top_mx_sdhost0preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sdhost1preclk_1_pll3bypass) = { "cr_top_mx_sdhost1preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sdhost1preclk_0_pll2bypass) = { "cr_top_mx_sdhost1preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sdhost2preclk_1_pll3bypass) = { "cr_top_mx_sdhost2preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sdhost2preclk_0_pll2bypass) = { "cr_top_mx_sdhost2preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_ethernetpreclk_1_pll3bypass) = { "cr_top_mx_ethernetpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_ethernetpreclk_0_pll2bypass) = { "cr_top_mx_ethernetpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_ufcpreclk_1_pll3bypass) = { "cr_top_mx_ufcpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_ufcpreclk_0_pll2bypass) = { "cr_top_mx_ufcpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sysout0preclk_1_pll3bypass) = { "cr_top_mx_sysout0preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sysout0preclk_0_pll2bypass) = { "cr_top_mx_sysout0preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sysout1preclk_1_pll3bypass) = { "cr_top_mx_sysout1preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sysout1preclk_0_pll2bypass) = { "cr_top_mx_sysout1preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sys0preclk_1_pll3bypass) = { "cr_top_mx_sys0preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sys0preclk_0_pll2bypass) = { "cr_top_mx_sys0preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sys1preclk_1_pll3bypass) = { "cr_top_mx_sys1preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sys1preclk_0_pll2bypass) = { "cr_top_mx_sys1preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sys2preclk_1_pll3bypass) = { "cr_top_mx_sys2preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sys2preclk_0_pll2bypass) = { "cr_top_mx_sys2preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_sys3preclk_1_pll3bypass) = { "cr_top_mx_sys3preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_sys3preclk_0_pll2bypass) = { "cr_top_mx_sys3preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_isppixelpreclk_1_pll7bypass) = { "cr_top_mx_isppixelpreclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_isppixelpreclk_0_pll6bypass) = { "cr_top_mx_isppixelpreclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_pixel0preclk_1_pll7bypass) = { "cr_top_mx_pixel0preclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_pixel0preclk_0_pll6bypass) = { "cr_top_mx_pixel0preclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_pixel1preclk_1_pll7bypass) = { "cr_top_mx_pixel1preclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_pixel1preclk_0_pll6bypass) = { "cr_top_mx_pixel1preclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_pixel1x7preclk_1_pll7bypass) = { "cr_top_mx_pixel1x7preclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_pixel1x7preclk_0_pll6bypass) = { "cr_top_mx_pixel1x7preclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_hdmitx0esmpreclk_1_pll7bypass) = { "cr_top_mx_hdmitx0esmpreclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_hdmitx0esmpreclk_0_pll6bypass) = { "cr_top_mx_hdmitx0esmpreclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_hdmitx1esmpreclk_1_pll7bypass) = { "cr_top_mx_hdmitx1esmpreclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_hdmitx1esmpreclk_0_pll6bypass) = { "cr_top_mx_hdmitx1esmpreclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_hdmirxesmpreclk_1_pll7bypass) = { "cr_top_mx_hdmirxesmpreclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_hdmirxesmpreclk_0_pll6bypass) = { "cr_top_mx_hdmirxesmpreclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_hdmirxaudiopreclk_1_pll7bypass) = { "cr_top_mx_hdmirxaudiopreclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_hdmirxaudiopreclk_0_pll6bypass) = { "cr_top_mx_hdmirxaudiopreclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_i2s0audiopremclk_1_pll7bypass) = { "cr_top_mx_i2s0audiopremclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_i2s0audiopremclk_0_pll6bypass) = { "cr_top_mx_i2s0audiopremclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_i2s1audiopremclk_1_pll7bypass) = { "cr_top_mx_i2s1audiopremclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_i2s1audiopremclk_0_pll6bypass) = { "cr_top_mx_i2s1audiopremclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_spdifpreclk_1_pll7bypass) = { "cr_top_mx_spdifpreclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_spdifpreclk_0_pll6bypass) = { "cr_top_mx_spdifpreclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_auxadcpreclk_1_pll7bypass) = { "cr_top_mx_auxadcpreclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_auxadcpreclk_0_pll6bypass) = { "cr_top_mx_auxadcpreclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(cs_xtal1_mx_timerpreclk_2) = { "cr_top_mx_timerpreclk_2_sw", "xtal1"};
PNAME(mux_timerpreclk_1_pll3bypass) = { "cr_top_mx_timerpreclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_timerpreclk_0_pll2bypass) = { "cr_top_mx_timerpreclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};
PNAME(mux_avout0preclk_1_pll7bypass) = { "cr_top_mx_avout0preclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_avout0preclk_0_pll6bypass) = { "cr_top_mx_avout0preclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_avout1preclk_1_pll7bypass) = { "cr_top_mx_avout1preclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_avout1preclk_0_pll6bypass) = { "cr_top_mx_avout1preclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_av0preclk_1_pll7bypass) = { "cr_top_mx_av0preclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_av0preclk_0_pll6bypass) = { "cr_top_mx_av0preclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(mux_av1preclk_1_pll7bypass) = { "cr_top_mx_av1preclk_1_sw", "cr_top_cs_pll7bypassclk_0_en"};
PNAME(mux_av1preclk_0_pll6bypass) = { "cr_top_mx_av1preclk_0_sw", "cr_top_cs_pll6bypassclk_0_en"};
PNAME(cs_xtal1_mx_noccip3preclk_2) = { "cr_top_mx_noccip3preclk_2_sw", "xtal1"};
PNAME(mux_noccip3preclk_1_pll3bypass) = { "cr_top_mx_noccip3preclk_1_sw", "cr_top_cs_pll3bypassclk_0_en"};
PNAME(mux_noccip3preclk_0_pll2bypass) = { "cr_top_mx_noccip3preclk_0_sw", "cr_top_cs_pll2bypassclk_0_en"};

static struct cashew_device_clock  cashew_sys_devices[] __initdata = {
/*------CR_TOP_CLOCK_SYS_PRIMARY 0x0000------------*/
/*0x0004  CR_TOP_CLOCK_NOC_HS	g=1 d=2 m=4 */
{
.dev_gate = GATE(CS_NOCHSPRESWCLK_0_EN, "cr_top_cs_nochspreswclk_0_en", "cr_top_nochsclk_div_0", CLOCK_NOC_HS_OFFSET, CS_NOCHSPRESWCLK_0_EN_SHIFT),
.dev_ddiv = DDIV(NOCHSCLK_DIV_0, "cr_top_nochsclk_div_0", "cr_top_cs_nochspreswclk_0_sw", CLOCK_NOC_HS_OFFSET, NOCHSPRESWCLK_DIV_0_LENGTH, NOCHSCLK_DIV_0_LENGTH, NOCHSPRESWCLK_DIV_0_SHIFT, NOCHSCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUXC(CS_NOCHSPRESWCLK_0_SW, "cr_top_cs_nochspreswclk_0_sw", cs_xtal1_mx_nochspreclk_2, CLOCK_NOC_HS_OFFSET, CS_NOCHSPRESWCLK_0_SW_SHIFT),
	MUX(MX_NOCHSPRECLK_2_SW, "cr_top_mx_nochspreclk_2_sw", mux_nochspreclk_1_pll3bypass, CLOCK_NOC_HS_OFFSET, MX_NOCHSPRECLK_2_SW_SHIFT),
	MUX(MX_NOCHSPRECLK_1_SW, "cr_top_mx_nochspreclk_1_sw", mux_nochspreclk_0_pll2bypass, CLOCK_NOC_HS_OFFSET, MX_NOCHSPRECLK_1_SW_SHIFT),
	MUX(MX_NOCHSPRECLK_0_SW, "cr_top_mx_nochspreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_NOC_HS_OFFSET,  MX_NOCHSPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	4,
},

/*0x0008  CR_TOP_CLOCK_NOC_MS	g=1 d=2 m=4 */
{
.dev_gate = GATE(CS_NOCMSPRESWCLK_0_EN, "cr_top_cs_nocmspreswclk_0_en", "cr_top_nocmsclk_div_0", CLOCK_NOC_MS_OFFSET, CS_NOCMSPRESWCLK_0_EN_SHIFT),
.dev_ddiv = DDIV(NOCMSCLK_DIV_0, "cr_top_nocmsclk_div_0", "cr_top_cs_nocmspreswclk_0_sw", CLOCK_NOC_MS_OFFSET, NOCMSPRESWCLK_DIV_0_LENGTH, NOCMSCLK_DIV_0_LENGTH, NOCMSPRESWCLK_DIV_0_SHIFT, NOCMSCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUXC(CS_NOCMSPRESWCLK_0_SW, "cr_top_cs_nocmspreswclk_0_sw", cs_xtal1_mx_nocmspreclk_2, CLOCK_NOC_MS_OFFSET, CS_NOCMSPRESWCLK_0_SW_SHIFT),
	MUX(MX_NOCMSPRECLK_2_SW, "cr_top_mx_nocmspreclk_2_sw", mux_nocmspreclk_1_pll3bypass, CLOCK_NOC_MS_OFFSET, MX_NOCMSPRECLK_2_SW_SHIFT),
	MUX(MX_NOCMSPRECLK_1_SW, "cr_top_mx_nocmspreclk_1_sw", mux_nocmspreclk_0_pll2bypass, CLOCK_NOC_MS_OFFSET, MX_NOCMSPRECLK_1_SW_SHIFT),
	MUX(MX_NOCMSPRECLK_0_SW, "cr_top_mx_nocmspreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_NOC_MS_OFFSET,  MX_NOCMSPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	4,
},

/*0x000C  CR_TOP_CLOCK_NOC_LS	g=1 d=2 m=4 */
{
.dev_gate = GATE(CS_NOCLSPRESWCLK_0_EN, "cr_top_cs_noclspreswclk_0_en", "cr_top_noclsclk_div_0", CLOCK_NOC_LS_OFFSET, CS_NOCLSPRESWCLK_0_EN_SHIFT),
.dev_ddiv = DDIV(NOCLSCLK_DIV_0, "cr_top_noclsclk_div_0", "cr_top_cs_noclspreswclk_0_sw", CLOCK_NOC_LS_OFFSET, NOCLSPRESWCLK_DIV_0_LENGTH, NOCLSCLK_DIV_0_LENGTH, NOCLSPRESWCLK_DIV_0_SHIFT, NOCLSCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUXC(CS_NOCLSPRESWCLK_0_SW, "cr_top_cs_noclspreswclk_0_sw", cs_xtal1_mx_noclspreclk_2, CLOCK_NOC_LS_OFFSET, CS_NOCLSPRESWCLK_0_SW_SHIFT),
	MUX(MX_NOCLSPRECLK_2_SW, "cr_top_mx_noclspreclk_2_sw", mux_noclspreclk_1_pll3bypass, CLOCK_NOC_LS_OFFSET, MX_NOCLSPRECLK_2_SW_SHIFT),
	MUX(MX_NOCLSPRECLK_1_SW, "cr_top_mx_noclspreclk_1_sw", mux_noclspreclk_0_pll2bypass, CLOCK_NOC_LS_OFFSET, MX_NOCLSPRECLK_1_SW_SHIFT),
	MUX(MX_NOCLSPRECLK_0_SW, "cr_top_mx_noclspreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_NOC_LS_OFFSET,  MX_NOCLSPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	4,
},

/*0x0010  CR_TOP_CLOCK_CPU0	g=1 d=2 m=0 */
{
.dev_gate = GATE(CPU0PRECLK_GATE_EN, "cr_top_cpu0preclk_gate_en", "cr_top_cpu0clk_div_0", CLOCK_CPU0_OFFSET, CPU0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(CPU0CLK_DIV_0, "cr_top_cpu0clk_div_0", "cr_top_cs_pll0bypassclk_0_en", CLOCK_CPU0_OFFSET, CPU0PRECLK_DIV_0_LENGTH, CPU0CLK_DIV_0_LENGTH, CPU0PRECLK_DIV_0_SHIFT, CPU0CLK_DIV_0_SHIFT),

.no_of_muxes =	0,
},

/*0x0014  CR_TOP_CLOCK_CPU1	g=2 d=2 m=4 */

{
.dev_gate =  GATE(CS_CPU1PRESWCLK_0_EN, "cr_top_cs_cpu1preswclk_0_en", "cr_top_cpu1clk_div_0", CLOCK_CPU1_OFFSET, CS_CPU1PRESWCLK_0_EN_SHIFT),
.dev_ddiv = DDIV(CPU1CLK_DIV_0, "cr_top_cpu1clk_div_0", "cr_top_cs_cpu1preswclk_0_sw", CLOCK_CPU1_OFFSET, CPU1PRESWCLK_DIV_0_LENGTH, CPU1CLK_DIV_0_LENGTH, CPU1PRESWCLK_DIV_0_SHIFT, CPU1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUXC(CS_CPU1PRESWCLK_0_SW, "cr_top_cs_cpu1preswclk_0_sw", cs_xtal1_mx_cpu1preclk_2, CLOCK_CPU1_OFFSET, CS_CPU1PRESWCLK_0_SW_SHIFT),
	MUX(MX_CPU1PRECLK_2_SW, "cr_top_mx_cpu1preclk_2_sw", mux_cpu1preclk_1_pll3bypass, CLOCK_CPU1_OFFSET, MX_CPU1PRECLK_2_SW_SHIFT),
	MUX(MX_CPU1PRECLK_1_SW, "cr_top_mx_cpu1preclk_1_sw", mux_cpu1preclk_0_pll2bypass, CLOCK_CPU1_OFFSET, MX_CPU1PRECLK_1_SW_SHIFT),
	MUX(MX_CPU1PRECLK_0_SW, "cr_top_mx_cpu1preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_CPU1_OFFSET,  MX_CPU1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	4,
},
{
.dev_gate = GATE(CPU1NOCCLK_GATE_EN, "cr_top_cpu1nocclk_gate_en", "cr_top_cs_cpu1preswclk_0_en", CLOCK_CPU1_OFFSET, CPU1NOCCLK_GATE_EN_SHIFT),
.no_of_muxes =	0,
},

/*0x0018  CR_TOP_CLOCK_CPU2	g=1 d=2 m=4 */
{
.dev_gate = GATE(CS_CPU2PRESWCLK_0_EN, "cr_top_cs_cpu2preswclk_0_en", "cr_top_cpu2clk_div_0", CLOCK_CPU2_OFFSET, CS_CPU2PRESWCLK_0_EN_SHIFT),
.dev_ddiv = DDIV(CPU2CLK_DIV_0, "cr_top_cpu2clk_div_0", "cr_top_cs_cpu2preswclk_0_sw", CLOCK_CPU2_OFFSET, CPU2PRESWCLK_DIV_0_LENGTH, CPU2CLK_DIV_0_LENGTH, CPU2PRESWCLK_DIV_0_SHIFT, CPU2CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUXC(CS_CPU2PRESWCLK_0_SW, "cr_top_cs_cpu2preswclk_0_sw", cs_xtal1_mx_cpu2preclk_2, CLOCK_CPU2_OFFSET, CS_CPU2PRESWCLK_0_SW_SHIFT),
	MUX(MX_CPU2PRECLK_2_SW, "cr_top_mx_cpu2preclk_2_sw", mux_cpu2preclk_1_pll3bypass, CLOCK_CPU2_OFFSET, MX_CPU2PRECLK_2_SW_SHIFT),
	MUX(MX_CPU2PRECLK_1_SW, "cr_top_mx_cpu2preclk_1_sw", mux_cpu2preclk_0_pll2bypass, CLOCK_CPU2_OFFSET, MX_CPU2PRECLK_1_SW_SHIFT),
	MUX(MX_CPU2PRECLK_0_SW, "cr_top_mx_cpu2preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_CPU2_OFFSET,  MX_CPU2PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	4,
},

/*0x001C  CR_TOP_CLOCK_VXE	g=1 d=2 m=3 */
{
.dev_gate = GATE(VXECOREPRECLK_GATE_EN, "cr_top_vxecorepreclk_gate_en", "cr_top_vxecoreclk_div_0", CLOCK_VXE_OFFSET, VXECOREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(VXECORECLK_DIV_0, "cr_top_vxecoreclk_div_0", "cr_top_mx_vxecorepreclk_2_sw", CLOCK_VXE_OFFSET, VXECOREPRECLK_DIV_0_LENGTH, VXECORECLK_DIV_0_LENGTH, VXECOREPRECLK_DIV_0_SHIFT, VXECORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_VXECOREPRECLK_2_SW, "cr_top_mx_vxecorepreclk_2_sw", mux_vxecorepreclk_1_pll3bypass, CLOCK_VXE_OFFSET, MX_VXECOREPRECLK_2_SW_SHIFT),
	MUX(MX_VXECOREPRECLK_1_SW, "cr_top_mx_vxecorepreclk_1_sw", mux_vxecorepreclk_0_pll2bypass, CLOCK_VXE_OFFSET, MX_VXECOREPRECLK_1_SW_SHIFT),
	MUX(MX_VXECOREPRECLK_0_SW, "cr_top_mx_vxecorepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_VXE_OFFSET,  MX_VXECOREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0020  CR_TOP_CLOCK_VXD	g=1 d=2 m=3 */
{
.dev_gate = GATE(VXDCOREPRECLK_GATE_EN, "cr_top_vxdcorepreclk_gate_en", "cr_top_vxdcoreclk_div_0", CLOCK_VXD_OFFSET, VXDCOREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(VXDCORECLK_DIV_0, "cr_top_vxdcoreclk_div_0", "cr_top_mx_vxdcorepreclk_2_sw", CLOCK_VXD_OFFSET, VXDCOREPRECLK_DIV_0_LENGTH, VXDCORECLK_DIV_0_LENGTH, VXDCOREPRECLK_DIV_0_SHIFT, VXDCORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_VXDCOREPRECLK_2_SW, "cr_top_mx_vxdcorepreclk_2_sw", mux_vxdcorepreclk_1_pll3bypass, CLOCK_VXD_OFFSET, MX_VXDCOREPRECLK_2_SW_SHIFT),
	MUX(MX_VXDCOREPRECLK_1_SW, "cr_top_mx_vxdcorepreclk_1_sw", mux_vxdcorepreclk_0_pll2bypass, CLOCK_VXD_OFFSET, MX_VXDCOREPRECLK_1_SW_SHIFT),
	MUX(MX_VXDCOREPRECLK_0_SW, "cr_top_mx_vxdcorepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_VXD_OFFSET,  MX_VXDCOREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0024  CR_TOP_CLOCK_GPU	g=1 d=2 m=3 */
{
.dev_gate = GATE(GPUCOREPRECLK_GATE_EN, "cr_top_gpucorepreclk_gate_en", "cr_top_gpucoreclk_div_0", CLOCK_GPU_OFFSET, GPUCOREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(GPUCORECLK_DIV_0, "cr_top_gpucoreclk_div_0", "cr_top_mx_gpucorepreclk_2_sw", CLOCK_GPU_OFFSET, GPUCOREPRECLK_DIV_0_LENGTH, GPUCORECLK_DIV_0_LENGTH, GPUCOREPRECLK_DIV_0_SHIFT, GPUCORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_GPUCOREPRECLK_2_SW, "cr_top_mx_gpucorepreclk_2_sw", mux_gpucorepreclk_1_pll3bypass, CLOCK_GPU_OFFSET, MX_GPUCOREPRECLK_2_SW_SHIFT),
	MUX(MX_GPUCOREPRECLK_1_SW, "cr_top_mx_gpucorepreclk_1_sw", mux_gpucorepreclk_0_pll2bypass, CLOCK_GPU_OFFSET, MX_GPUCOREPRECLK_1_SW_SHIFT),
	MUX(MX_GPUCOREPRECLK_0_SW, "cr_top_mx_gpucorepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_GPU_OFFSET,  MX_GPUCOREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0028  CR_TOP_CLOCK_ISP	g=1 d=2 m=3 */
{
.dev_gate = GATE(ISPCOREPRECLK_GATE_EN, "cr_top_ispcorepreclk_gate_en", "cr_top_ispcoreclk_div_0", CLOCK_ISP_OFFSET, ISPCOREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(ISPCORECLK_DIV_0, "cr_top_ispcoreclk_div_0", "cr_top_mx_ispcorepreclk_2_sw", CLOCK_ISP_OFFSET, ISPCOREPRECLK_DIV_0_LENGTH, ISPCORECLK_DIV_0_LENGTH, ISPCOREPRECLK_DIV_0_SHIFT, ISPCORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_ISPCOREPRECLK_2_SW, "cr_top_mx_ispcorepreclk_2_sw", mux_ispcorepreclk_1_pll3bypass, CLOCK_ISP_OFFSET, MX_ISPCOREPRECLK_2_SW_SHIFT),
	MUX(MX_ISPCOREPRECLK_1_SW, "cr_top_mx_ispcorepreclk_1_sw", mux_ispcorepreclk_0_pll2bypass, CLOCK_ISP_OFFSET, MX_ISPCOREPRECLK_1_SW_SHIFT),
	MUX(MX_ISPCOREPRECLK_0_SW, "cr_top_mx_ispcorepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_ISP_OFFSET,  MX_ISPCOREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x002C  CR_TOP_CLOCK_SEC	g=1 d=2 m=3 */
{
.dev_gate = GATE(SECCOREPRECLK_GATE_EN, "cr_top_seccorepreclk_gate_en", "cr_top_seccoreclk_div_0", CLOCK_SEC_OFFSET, SECCOREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SECCORECLK_DIV_0, "cr_top_seccoreclk_div_0", "cr_top_mx_seccorepreclk_2_sw", CLOCK_SEC_OFFSET, SECCOREPRECLK_DIV_0_LENGTH, SECCORECLK_DIV_0_LENGTH, SECCOREPRECLK_DIV_0_SHIFT, SECCORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SECCOREPRECLK_2_SW, "cr_top_mx_seccorepreclk_2_sw", mux_seccorepreclk_1_pll3bypass, CLOCK_SEC_OFFSET, MX_SECCOREPRECLK_2_SW_SHIFT),
	MUX(MX_SECCOREPRECLK_1_SW, "cr_top_mx_seccorepreclk_1_sw", mux_seccorepreclk_0_pll2bypass, CLOCK_SEC_OFFSET, MX_SECCOREPRECLK_1_SW_SHIFT),
	MUX(MX_SECCOREPRECLK_0_SW, "cr_top_mx_seccorepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SEC_OFFSET,  MX_SECCOREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0030  CR_TOP_CLOCK_VPP	g=1 d=2 m=3 */
{
.dev_gate = GATE(VPPCOREPRECLK_GATE_EN, "cr_top_vppcorepreclk_gate_en", "cr_top_vppcoreclk_div_0", CLOCK_VPP_OFFSET, VPPCOREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(VPPCORECLK_DIV_0, "cr_top_vppcoreclk_div_0", "cr_top_mx_vppcorepreclk_2_sw", CLOCK_VPP_OFFSET, VPPCOREPRECLK_DIV_0_LENGTH, VPPCORECLK_DIV_0_LENGTH, VPPCOREPRECLK_DIV_0_SHIFT, VPPCORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_VPPCOREPRECLK_2_SW, "cr_top_mx_vppcorepreclk_2_sw", mux_vppcorepreclk_1_pll3bypass, CLOCK_VPP_OFFSET, MX_VPPCOREPRECLK_2_SW_SHIFT),
	MUX(MX_VPPCOREPRECLK_1_SW, "cr_top_mx_vppcorepreclk_1_sw", mux_vppcorepreclk_0_pll2bypass, CLOCK_VPP_OFFSET, MX_VPPCOREPRECLK_1_SW_SHIFT),
	MUX(MX_VPPCOREPRECLK_0_SW, "cr_top_mx_vppcorepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_VPP_OFFSET,  MX_VPPCOREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0034  CR_TOP_CLOCK_PDP0	g=1 d=2 m=3 */
{
.dev_gate = GATE(PDP0COREPRECLK_GATE_EN, "cr_top_pdp0corepreclk_gate_en", "cr_top_pdp0coreclk_div_0", CLOCK_PDP0_OFFSET, PDP0COREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(PDP0CORECLK_DIV_0, "cr_top_pdp0coreclk_div_0", "cr_top_mx_pdp0corepreclk_2_sw", CLOCK_PDP0_OFFSET, PDP0COREPRECLK_DIV_0_LENGTH, PDP0CORECLK_DIV_0_LENGTH, PDP0COREPRECLK_DIV_0_SHIFT, PDP0CORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_PDP0COREPRECLK_2_SW, "cr_top_mx_pdp0corepreclk_2_sw", mux_pdp0corepreclk_1_pll3bypass, CLOCK_PDP0_OFFSET, MX_PDP0COREPRECLK_2_SW_SHIFT),
	MUX(MX_PDP0COREPRECLK_1_SW, "cr_top_mx_pdp0corepreclk_1_sw", mux_pdp0corepreclk_0_pll2bypass, CLOCK_PDP0_OFFSET, MX_PDP0COREPRECLK_1_SW_SHIFT),
	MUX(MX_PDP0COREPRECLK_0_SW, "cr_top_mx_pdp0corepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_PDP0_OFFSET,  MX_PDP0COREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0038  CR_TOP_CLOCK_PDP1	g=1 d=2 m=3 */
{
.dev_gate = GATE(PDP1COREPRECLK_GATE_EN, "cr_top_pdp1corepreclk_gate_en", "cr_top_pdp1coreclk_div_0", CLOCK_PDP1_OFFSET, PDP1COREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(PDP1CORECLK_DIV_0, "cr_top_pdp1coreclk_div_0", "cr_top_mx_pdp1corepreclk_2_sw", CLOCK_PDP1_OFFSET, PDP1COREPRECLK_DIV_0_LENGTH, PDP1CORECLK_DIV_0_LENGTH, PDP1COREPRECLK_DIV_0_SHIFT, PDP1CORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_PDP1COREPRECLK_2_SW, "cr_top_mx_pdp1corepreclk_2_sw", mux_pdp1corepreclk_1_pll3bypass, CLOCK_PDP1_OFFSET, MX_PDP1COREPRECLK_2_SW_SHIFT),
	MUX(MX_PDP1COREPRECLK_1_SW, "cr_top_mx_pdp1corepreclk_1_sw", mux_pdp1corepreclk_0_pll2bypass, CLOCK_PDP1_OFFSET, MX_PDP1COREPRECLK_1_SW_SHIFT),
	MUX(MX_PDP1COREPRECLK_0_SW, "cr_top_mx_pdp1corepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_PDP1_OFFSET,  MX_PDP1COREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x003C  CR_TOP_CLOCK_RPU_FAST	g=1 d=2 m=3 */
{
.dev_gate = GATE(RPUFASTPRECLK_GATE_EN, "cr_top_rpufastpreclk_gate_en", "cr_top_rpufastclk_div_0", CLOCK_RPU_FAST_OFFSET, RPUFASTPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(RPUFASTCLK_DIV_0, "cr_top_rpufastclk_div_0", "cr_top_mx_rpufastpreclk_2_sw", CLOCK_RPU_FAST_OFFSET, RPUFASTPRECLK_DIV_0_LENGTH, RPUFASTCLK_DIV_0_LENGTH, RPUFASTPRECLK_DIV_0_SHIFT, RPUFASTCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_RPUFASTPRECLK_2_SW, "cr_top_mx_rpufastpreclk_2_sw", mux_rpufastpreclk_1_pll3bypass, CLOCK_RPU_FAST_OFFSET, MX_RPUFASTPRECLK_2_SW_SHIFT),
	MUX(MX_RPUFASTPRECLK_1_SW, "cr_top_mx_rpufastpreclk_1_sw", mux_rpufastpreclk_0_pll2bypass, CLOCK_RPU_FAST_OFFSET, MX_RPUFASTPRECLK_1_SW_SHIFT),
	MUX(MX_RPUFASTPRECLK_0_SW, "cr_top_mx_rpufastpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_RPU_FAST_OFFSET,  MX_RPUFASTPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0040  CR_TOP_CLOCK_RPU_CORE	g=1 d=2 m=3 */
{
.dev_gate = GATE(RPUCOREPRECLK_GATE_EN, "cr_top_rpucorepreclk_gate_en", "cr_top_rpucoreclk_div_0", CLOCK_RPU_CORE_OFFSET, RPUCOREPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(RPUCORECLK_DIV_0, "cr_top_rpucoreclk_div_0", "cr_top_mx_rpucorepreclk_2_sw", CLOCK_RPU_CORE_OFFSET, RPUCOREPRECLK_DIV_0_LENGTH, RPUCORECLK_DIV_0_LENGTH, RPUCOREPRECLK_DIV_0_SHIFT, RPUCORECLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_RPUCOREPRECLK_2_SW, "cr_top_mx_rpucorepreclk_2_sw", mux_rpucorepreclk_1_pll3bypass, CLOCK_RPU_CORE_OFFSET, MX_RPUCOREPRECLK_2_SW_SHIFT),
	MUX(MX_RPUCOREPRECLK_1_SW, "cr_top_mx_rpucorepreclk_1_sw", mux_rpucorepreclk_0_pll2bypass, CLOCK_RPU_CORE_OFFSET, MX_RPUCOREPRECLK_1_SW_SHIFT),
	MUX(MX_RPUCOREPRECLK_0_SW, "cr_top_mx_rpucorepreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_RPU_CORE_OFFSET,  MX_RPUCOREPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0044  CR_TOP_CLOCK_RPU_SLEEP	g=1 d=1 m=1 */
{
.dev_gate = GATE(RPUSLEEPCLK_GATE_EN, "cr_top_rpusleepclk_gate_en", "cr_top_rpusleepclk_div_0", CLOCK_RPU_SLEEP_OFFSET, RPUSLEEPCLK_GATE_EN_SHIFT),
.dev_div = DIVC(RPUSLEEPCLK_DIV_0, "cr_top_rpusleepclk_div_0", "cr_top_mx_rpusleepclk_0_sw", CLOCK_RPU_SLEEP_OFFSET, RPUSLEEPCLK_DIV_0_SHIFT, RPUSLEEPCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_RPUSLEEPCLK_0_SW, "cr_top_mx_rpusleepclk_0_sw", mux_xtal0_xtal1, CLOCK_RPU_SLEEP_OFFSET, MX_RPUSLEEPCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x0048  CR_TOP_CLOCK_HDMI_RX_CFG	g=1 d=2 m=3 */
{
.dev_gate = GATE(HDMIRXCFGPRECLK_GATE_EN, "cr_top_hdmirxcfgpreclk_gate_en", "cr_top_hdmirxcfgclk_div_0", CLOCK_HDMI_RX_CFG_OFFSET, HDMIRXCFGPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(HDMIRXCFGCLK_DIV_0, "cr_top_hdmirxcfgclk_div_0", "cr_top_mx_hdmirxcfgpreclk_2_sw", CLOCK_HDMI_RX_CFG_OFFSET, HDMIRXCFGPRECLK_DIV_0_LENGTH, HDMIRXCFGCLK_DIV_0_LENGTH, HDMIRXCFGPRECLK_DIV_0_SHIFT, HDMIRXCFGCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_HDMIRXCFGPRECLK_2_SW, "cr_top_mx_hdmirxcfgpreclk_2_sw", mux_hdmirxcfgpreclk_1_pll3bypass, CLOCK_HDMI_RX_CFG_OFFSET, MX_HDMIRXCFGPRECLK_2_SW_SHIFT),
	MUX(MX_HDMIRXCFGPRECLK_1_SW, "cr_top_mx_hdmirxcfgpreclk_1_sw", mux_hdmirxcfgpreclk_0_pll2bypass, CLOCK_HDMI_RX_CFG_OFFSET, MX_HDMIRXCFGPRECLK_1_SW_SHIFT),
	MUX(MX_HDMIRXCFGPRECLK_0_SW, "cr_top_mx_hdmirxcfgpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_HDMI_RX_CFG_OFFSET,  MX_HDMIRXCFGPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x004C  CR_TOP_CLOCK_HDMI_RX_CEC	g=1 d=1 m=1 */
{
.dev_gate = GATE(HDMIRXCECCLK_GATE_EN, "cr_top_hdmirxcecclk_gate_en", "cr_top_hdmirxcecclk_div_0", CLOCK_HDMI_RX_CEC_OFFSET, HDMIRXCECCLK_GATE_EN_SHIFT),
.dev_div = DIVC(HDMIRXCECCLK_DIV_0, "cr_top_hdmirxcecclk_div_0", "cr_top_mx_hdmirxcecclk_0_sw", CLOCK_HDMI_RX_CEC_OFFSET, HDMIRXCECCLK_DIV_0_SHIFT, HDMIRXCECCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_HDMIRXCECCLK_0_SW, "cr_top_mx_hdmirxcecclk_0_sw", mux_xtal0_xtal1, CLOCK_HDMI_RX_CEC_OFFSET, MX_HDMIRXCECCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x0050  CR_TOP_CLOCK_HDMI_RX_IMODEDET	g=1 d=2 m=1 */
{
.dev_gate = GATE(HDMIRXIMODEDETPRECLK_GATE_EN, "cr_top_hdmirximodedetpreclk_gate_en", "cr_top_hdmirximodedetclk_div_0", CLOCK_HDMI_RX_IMODEDET_OFFSET, HDMIRXIMODEDETPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(HDMIRXIMODEDETCLK_DIV_0, "cr_top_hdmirximodedetclk_div_0", "cr_top_mx_hdmirximodedetpreclk_0_sw", CLOCK_HDMI_RX_IMODEDET_OFFSET, HDMIRXIMODEDETPRECLK_DIV_0_LENGTH, HDMIRXIMODEDETCLK_DIV_0_LENGTH, HDMIRXIMODEDETPRECLK_DIV_0_SHIFT, HDMIRXIMODEDETCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_HDMIRXIMODEDETPRECLK_0_SW, "cr_top_mx_hdmirximodedetpreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_HDMI_RX_IMODEDET_OFFSET, MX_HDMIRXIMODEDETPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x0054  CR_TOP_CLOCK_HDMI_TX_CEC	g=1 d=1 m=1 */
{
.dev_gate = GATE(HDMITXCECCLK_GATE_EN, "cr_top_hdmitxcecclk_gate_en", "cr_top_hdmitxcecclk_div_0", CLOCK_HDMI_TX_CEC_OFFSET, HDMITXCECCLK_GATE_EN_SHIFT),
.dev_div = DIVC(HDMITXCECCLK_DIV_0, "cr_top_hdmitxcecclk_div_0", "cr_top_mx_hdmitxcecclk_0_sw", CLOCK_HDMI_TX_CEC_OFFSET, HDMITXCECCLK_DIV_0_SHIFT, HDMITXCECCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_HDMITXCECCLK_0_SW, "cr_top_mx_hdmitxcecclk_0_sw", mux_xtal0_xtal1, CLOCK_HDMI_TX_CEC_OFFSET, MX_HDMITXCECCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x0058  CR_TOP_CLOCK_HDMI_ISFR	g=1 d=2 m=1 */
{
.dev_gate = GATE(HDMIISFRPRECLK_GATE_EN, "cr_top_hdmiisfrpreclk_gate_en", "cr_top_hdmiisfrclk_div_0", CLOCK_HDMI_ISFR_OFFSET, HDMIISFRPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(HDMIISFRCLK_DIV_0, "cr_top_hdmiisfrclk_div_0", "cr_top_mx_hdmiisfrpreclk_0_sw", CLOCK_HDMI_ISFR_OFFSET, HDMIISFRPRECLK_DIV_0_LENGTH, HDMIISFRCLK_DIV_0_LENGTH, HDMIISFRPRECLK_DIV_0_SHIFT, HDMIISFRCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_HDMIISFRPRECLK_0_SW, "cr_top_mx_hdmiisfrpreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_HDMI_ISFR_OFFSET, MX_HDMIISFRPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x005C  CR_TOP_CLOCK_MIPI_REF	g=1 d=1 m=0 */
{
.dev_gate = GATE(MIPIREFCLK_GATE_EN, "cr_top_mipirefclk_gate_en", "cr_top_mipirefclk_div_0", CLOCK_MIPI_REF_OFFSET, MIPIREFCLK_GATE_EN_SHIFT),
.dev_div = DIVC(MIPIREFCLK_DIV_0, "cr_top_mipirefclk_div_0", "xtal1", CLOCK_MIPI_REF_OFFSET, MIPIREFCLK_DIV_0_SHIFT, MIPIREFCLK_DIV_0_LENGTH, 0),
.no_of_muxes =	0,
},

/*0x0060  CR_TOP_CLOCK_MIPI_DSI	g=1 d=2 m=3 */
{
.dev_gate = GATE(MIPIDSIPRECLK_GATE_EN, "cr_top_mipidsipreclk_gate_en", "cr_top_mipidsiclk_div_0", CLOCK_MIPI_DSI_OFFSET, MIPIDSIPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(MIPIDSICLK_DIV_0, "cr_top_mipidsiclk_div_0", "cr_top_mx_mipidsipreclk_2_sw", CLOCK_MIPI_DSI_OFFSET, MIPIDSIPRECLK_DIV_0_LENGTH, MIPIDSICLK_DIV_0_LENGTH, MIPIDSIPRECLK_DIV_0_SHIFT, MIPIDSICLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_MIPIDSIPRECLK_2_SW, "cr_top_mx_mipidsipreclk_2_sw", mux_mipidsipreclk_1_pll3bypass, CLOCK_MIPI_DSI_OFFSET, MX_MIPIDSIPRECLK_2_SW_SHIFT),
	MUX(MX_MIPIDSIPRECLK_1_SW, "cr_top_mx_mipidsipreclk_1_sw", mux_mipidsipreclk_0_pll2bypass, CLOCK_MIPI_DSI_OFFSET, MX_MIPIDSIPRECLK_1_SW_SHIFT),
	MUX(MX_MIPIDSIPRECLK_0_SW, "cr_top_mx_mipidsipreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_MIPI_DSI_OFFSET,  MX_MIPIDSIPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0064  CR_TOP_CLOCK_MIPI_TX_CFG	g=1 d=2 m=3 */
{
.dev_gate = GATE(MIPITXCFGPRECLK_GATE_EN, "cr_top_mipitxcfgpreclk_gate_en", "cr_top_mipitxcfgclk_div_0", CLOCK_MIPI_TX_CFG_OFFSET, MIPITXCFGPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(MIPITXCFGCLK_DIV_0, "cr_top_mipitxcfgclk_div_0", "cr_top_mx_mipitxcfgpreclk_2_sw", CLOCK_MIPI_TX_CFG_OFFSET, MIPITXCFGPRECLK_DIV_0_LENGTH, MIPITXCFGCLK_DIV_0_LENGTH, MIPITXCFGPRECLK_DIV_0_SHIFT, MIPITXCFGCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_MIPITXCFGPRECLK_2_SW, "cr_top_mx_mipitxcfgpreclk_2_sw", mux_mipitxcfgpreclk_1_pll3bypass, CLOCK_MIPI_TX_CFG_OFFSET, MX_MIPITXCFGPRECLK_2_SW_SHIFT),
	MUX(MX_MIPITXCFGPRECLK_1_SW, "cr_top_mx_mipitxcfgpreclk_1_sw", mux_mipitxcfgpreclk_0_pll2bypass, CLOCK_MIPI_TX_CFG_OFFSET, MX_MIPITXCFGPRECLK_1_SW_SHIFT),
	MUX(MX_MIPITXCFGPRECLK_0_SW, "cr_top_mx_mipitxcfgpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_MIPI_TX_CFG_OFFSET,  MX_MIPITXCFGPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0068  CR_TOP_CLOCK_MIPI_RX0_CFG	g=1 d=2 m=3 */
{
.dev_gate = GATE(MIPIRX0CFGPRECLK_GATE_EN, "cr_top_mipirx0cfgpreclk_gate_en", "cr_top_mipirx0cfgclk_div_0", CLOCK_MIPI_RX0_CFG_OFFSET, MIPIRX0CFGPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(MIPIRX0CFGCLK_DIV_0, "cr_top_mipirx0cfgclk_div_0", "cr_top_mx_mipirx0cfgpreclk_2_sw", CLOCK_MIPI_RX0_CFG_OFFSET, MIPIRX0CFGPRECLK_DIV_0_LENGTH, MIPIRX0CFGCLK_DIV_0_LENGTH, MIPIRX0CFGPRECLK_DIV_0_SHIFT, MIPIRX0CFGCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_MIPIRX0CFGPRECLK_2_SW, "cr_top_mx_mipirx0cfgpreclk_2_sw", mux_mipirx0cfgpreclk_1_pll3bypass, CLOCK_MIPI_RX0_CFG_OFFSET, MX_MIPIRX0CFGPRECLK_2_SW_SHIFT),
	MUX(MX_MIPIRX0CFGPRECLK_1_SW, "cr_top_mx_mipirx0cfgpreclk_1_sw", mux_mipirx0cfgpreclk_0_pll2bypass, CLOCK_MIPI_RX0_CFG_OFFSET, MX_MIPIRX0CFGPRECLK_1_SW_SHIFT),
	MUX(MX_MIPIRX0CFGPRECLK_0_SW, "cr_top_mx_mipirx0cfgpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_MIPI_RX0_CFG_OFFSET,  MX_MIPIRX0CFGPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x006C  CR_TOP_CLOCK_MIPI_RX1_CFG	g=1 d=2 m=3 */
{
.dev_gate = GATE(MIPIRX1CFGPRECLK_GATE_EN, "cr_top_mipirx1cfgpreclk_gate_en", "cr_top_mipirx1cfgclk_div_0", CLOCK_MIPI_RX1_CFG_OFFSET, MIPIRX1CFGPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(MIPIRX1CFGCLK_DIV_0, "cr_top_mipirx1cfgclk_div_0", "cr_top_mx_mipirx1cfgpreclk_2_sw", CLOCK_MIPI_RX1_CFG_OFFSET, MIPIRX1CFGPRECLK_DIV_0_LENGTH, MIPIRX1CFGCLK_DIV_0_LENGTH, MIPIRX1CFGPRECLK_DIV_0_SHIFT, MIPIRX1CFGCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_MIPIRX1CFGPRECLK_2_SW, "cr_top_mx_mipirx1cfgpreclk_2_sw", mux_mipirx1cfgpreclk_1_pll3bypass, CLOCK_MIPI_RX1_CFG_OFFSET, MX_MIPIRX1CFGPRECLK_2_SW_SHIFT),
	MUX(MX_MIPIRX1CFGPRECLK_1_SW, "cr_top_mx_mipirx1cfgpreclk_1_sw", mux_mipirx1cfgpreclk_0_pll2bypass, CLOCK_MIPI_RX1_CFG_OFFSET, MX_MIPIRX1CFGPRECLK_1_SW_SHIFT),
	MUX(MX_MIPIRX1CFGPRECLK_0_SW, "cr_top_mx_mipirx1cfgpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_MIPI_RX1_CFG_OFFSET,  MX_MIPIRX1CFGPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0070  CR_TOP_CLOCK_UART0	g=1 d=2 m=1 */
{
.dev_gate = GATE(UART0PRECLK_GATE_EN, "cr_top_uart0preclk_gate_en", "cr_top_uart0clk_div_0", CLOCK_UART0_OFFSET, UART0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(UART0CLK_DIV_0, "cr_top_uart0clk_div_0", "cr_top_mx_uart0preclk_0_sw", CLOCK_UART0_OFFSET, UART0PRECLK_DIV_0_LENGTH, UART0CLK_DIV_0_LENGTH, UART0PRECLK_DIV_0_SHIFT, UART0CLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_UART0PRECLK_0_SW, "cr_top_mx_uart0preclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_UART0_OFFSET, MX_UART0PRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x0074  CR_TOP_CLOCK_UART1	g=1 d=2 m=1 */
{
.dev_gate = GATE(UART1PRECLK_GATE_EN, "cr_top_uart1preclk_gate_en", "cr_top_uart1clk_div_0", CLOCK_UART1_OFFSET, UART1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(UART1CLK_DIV_0, "cr_top_uart1clk_div_0", "cr_top_mx_uart1preclk_0_sw", CLOCK_UART1_OFFSET, UART1PRECLK_DIV_0_LENGTH, UART1CLK_DIV_0_LENGTH, UART1PRECLK_DIV_0_SHIFT, UART1CLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_UART1PRECLK_0_SW, "cr_top_mx_uart1preclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_UART1_OFFSET, MX_UART1PRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x0078  CR_TOP_CLOCK_I2C0	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C0PRECLK_GATE_EN, "cr_top_i2c0preclk_gate_en", "cr_top_i2c0clk_div_0", CLOCK_I2C0_OFFSET, I2C0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C0CLK_DIV_0, "cr_top_i2c0clk_div_0", "cr_top_mx_i2c0preclk_2_sw", CLOCK_I2C0_OFFSET, I2C0PRECLK_DIV_0_LENGTH, I2C0CLK_DIV_0_LENGTH, I2C0PRECLK_DIV_0_SHIFT, I2C0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C0PRECLK_2_SW, "cr_top_mx_i2c0preclk_2_sw", mux_i2c0preclk_1_pll3bypass, CLOCK_I2C0_OFFSET, MX_I2C0PRECLK_2_SW_SHIFT),
	MUX(MX_I2C0PRECLK_1_SW, "cr_top_mx_i2c0preclk_1_sw", mux_i2c0preclk_0_pll2bypass, CLOCK_I2C0_OFFSET, MX_I2C0PRECLK_1_SW_SHIFT),
	MUX(MX_I2C0PRECLK_0_SW, "cr_top_mx_i2c0preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C0_OFFSET,  MX_I2C0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x007C  CR_TOP_CLOCK_I2C1	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C1PRECLK_GATE_EN, "cr_top_i2c1preclk_gate_en", "cr_top_i2c1clk_div_0", CLOCK_I2C1_OFFSET, I2C1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C1CLK_DIV_0, "cr_top_i2c1clk_div_0", "cr_top_mx_i2c1preclk_2_sw", CLOCK_I2C1_OFFSET, I2C1PRECLK_DIV_0_LENGTH, I2C1CLK_DIV_0_LENGTH, I2C1PRECLK_DIV_0_SHIFT, I2C1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C1PRECLK_2_SW, "cr_top_mx_i2c1preclk_2_sw", mux_i2c1preclk_1_pll3bypass, CLOCK_I2C1_OFFSET, MX_I2C1PRECLK_2_SW_SHIFT),
	MUX(MX_I2C1PRECLK_1_SW, "cr_top_mx_i2c1preclk_1_sw", mux_i2c1preclk_0_pll2bypass, CLOCK_I2C1_OFFSET, MX_I2C1PRECLK_1_SW_SHIFT),
	MUX(MX_I2C1PRECLK_0_SW, "cr_top_mx_i2c1preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C1_OFFSET,  MX_I2C1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0080  CR_TOP_CLOCK_I2C2	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C2PRECLK_GATE_EN, "cr_top_i2c2preclk_gate_en", "cr_top_i2c2clk_div_0", CLOCK_I2C2_OFFSET, I2C2PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C2CLK_DIV_0, "cr_top_i2c2clk_div_0", "cr_top_mx_i2c2preclk_2_sw", CLOCK_I2C2_OFFSET, I2C2PRECLK_DIV_0_LENGTH, I2C2CLK_DIV_0_LENGTH, I2C2PRECLK_DIV_0_SHIFT, I2C2CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C2PRECLK_2_SW, "cr_top_mx_i2c2preclk_2_sw", mux_i2c2preclk_1_pll3bypass, CLOCK_I2C2_OFFSET, MX_I2C2PRECLK_2_SW_SHIFT),
	MUX(MX_I2C2PRECLK_1_SW, "cr_top_mx_i2c2preclk_1_sw", mux_i2c2preclk_0_pll2bypass, CLOCK_I2C2_OFFSET, MX_I2C2PRECLK_1_SW_SHIFT),
	MUX(MX_I2C2PRECLK_0_SW, "cr_top_mx_i2c2preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C2_OFFSET,  MX_I2C2PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0084  CR_TOP_CLOCK_I2C3	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C3PRECLK_GATE_EN, "cr_top_i2c3preclk_gate_en", "cr_top_i2c3clk_div_0", CLOCK_I2C3_OFFSET, I2C3PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C3CLK_DIV_0, "cr_top_i2c3clk_div_0", "cr_top_mx_i2c3preclk_2_sw", CLOCK_I2C3_OFFSET, I2C3PRECLK_DIV_0_LENGTH, I2C3CLK_DIV_0_LENGTH, I2C3PRECLK_DIV_0_SHIFT, I2C3CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C3PRECLK_2_SW, "cr_top_mx_i2c3preclk_2_sw", mux_i2c3preclk_1_pll3bypass, CLOCK_I2C3_OFFSET, MX_I2C3PRECLK_2_SW_SHIFT),
	MUX(MX_I2C3PRECLK_1_SW, "cr_top_mx_i2c3preclk_1_sw", mux_i2c3preclk_0_pll2bypass, CLOCK_I2C3_OFFSET, MX_I2C3PRECLK_1_SW_SHIFT),
	MUX(MX_I2C3PRECLK_0_SW, "cr_top_mx_i2c3preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C3_OFFSET,  MX_I2C3PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0088  CR_TOP_CLOCK_I2C4	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C4PRECLK_GATE_EN, "cr_top_i2c4preclk_gate_en", "cr_top_i2c4clk_div_0", CLOCK_I2C4_OFFSET, I2C4PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C4CLK_DIV_0, "cr_top_i2c4clk_div_0", "cr_top_mx_i2c4preclk_2_sw", CLOCK_I2C4_OFFSET, I2C4PRECLK_DIV_0_LENGTH, I2C4CLK_DIV_0_LENGTH, I2C4PRECLK_DIV_0_SHIFT, I2C4CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C4PRECLK_2_SW, "cr_top_mx_i2c4preclk_2_sw", mux_i2c4preclk_1_pll3bypass, CLOCK_I2C4_OFFSET, MX_I2C4PRECLK_2_SW_SHIFT),
	MUX(MX_I2C4PRECLK_1_SW, "cr_top_mx_i2c4preclk_1_sw", mux_i2c4preclk_0_pll2bypass, CLOCK_I2C4_OFFSET, MX_I2C4PRECLK_1_SW_SHIFT),
	MUX(MX_I2C4PRECLK_0_SW, "cr_top_mx_i2c4preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C4_OFFSET,  MX_I2C4PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x008C  CR_TOP_CLOCK_I2C5	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C5PRECLK_GATE_EN, "cr_top_i2c5preclk_gate_en", "cr_top_i2c5clk_div_0", CLOCK_I2C5_OFFSET, I2C5PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C5CLK_DIV_0, "cr_top_i2c5clk_div_0", "cr_top_mx_i2c5preclk_2_sw", CLOCK_I2C5_OFFSET, I2C5PRECLK_DIV_0_LENGTH, I2C5CLK_DIV_0_LENGTH, I2C5PRECLK_DIV_0_SHIFT, I2C5CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C5PRECLK_2_SW, "cr_top_mx_i2c5preclk_2_sw", mux_i2c5preclk_1_pll3bypass, CLOCK_I2C5_OFFSET, MX_I2C5PRECLK_2_SW_SHIFT),
	MUX(MX_I2C5PRECLK_1_SW, "cr_top_mx_i2c5preclk_1_sw", mux_i2c5preclk_0_pll2bypass, CLOCK_I2C5_OFFSET, MX_I2C5PRECLK_1_SW_SHIFT),
	MUX(MX_I2C5PRECLK_0_SW, "cr_top_mx_i2c5preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C5_OFFSET,  MX_I2C5PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0090  CR_TOP_CLOCK_I2C6	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C6PRECLK_GATE_EN, "cr_top_i2c6preclk_gate_en", "cr_top_i2c6clk_div_0", CLOCK_I2C6_OFFSET, I2C6PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C6CLK_DIV_0, "cr_top_i2c6clk_div_0", "cr_top_mx_i2c6preclk_2_sw", CLOCK_I2C6_OFFSET, I2C6PRECLK_DIV_0_LENGTH, I2C6CLK_DIV_0_LENGTH, I2C6PRECLK_DIV_0_SHIFT, I2C6CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C6PRECLK_2_SW, "cr_top_mx_i2c6preclk_2_sw", mux_i2c6preclk_1_pll3bypass, CLOCK_I2C6_OFFSET, MX_I2C6PRECLK_2_SW_SHIFT),
	MUX(MX_I2C6PRECLK_1_SW, "cr_top_mx_i2c6preclk_1_sw", mux_i2c6preclk_0_pll2bypass, CLOCK_I2C6_OFFSET, MX_I2C6PRECLK_1_SW_SHIFT),
	MUX(MX_I2C6PRECLK_0_SW, "cr_top_mx_i2c6preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C6_OFFSET,  MX_I2C6PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0094  CR_TOP_CLOCK_I2C7	g=1 d=2 m=3 */
{
.dev_gate = GATE(I2C7PRECLK_GATE_EN, "cr_top_i2c7preclk_gate_en", "cr_top_i2c7clk_div_0", CLOCK_I2C7_OFFSET, I2C7PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2C7CLK_DIV_0, "cr_top_i2c7clk_div_0", "cr_top_mx_i2c7preclk_2_sw", CLOCK_I2C7_OFFSET, I2C7PRECLK_DIV_0_LENGTH, I2C7CLK_DIV_0_LENGTH, I2C7PRECLK_DIV_0_SHIFT, I2C7CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2C7PRECLK_2_SW, "cr_top_mx_i2c7preclk_2_sw", mux_i2c7preclk_1_pll3bypass, CLOCK_I2C7_OFFSET, MX_I2C7PRECLK_2_SW_SHIFT),
	MUX(MX_I2C7PRECLK_1_SW, "cr_top_mx_i2c7preclk_1_sw", mux_i2c7preclk_0_pll2bypass, CLOCK_I2C7_OFFSET, MX_I2C7PRECLK_1_SW_SHIFT),
	MUX(MX_I2C7PRECLK_0_SW, "cr_top_mx_i2c7preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_I2C7_OFFSET,  MX_I2C7PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0098  CR_TOP_CLOCK_SPIM0	g=1 d=2 m=3 */
{
.dev_gate = GATE(SPIM0PRECLK_GATE_EN, "cr_top_spim0preclk_gate_en", "cr_top_spim0clk_div_0", CLOCK_SPIM0_OFFSET, SPIM0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SPIM0CLK_DIV_0, "cr_top_spim0clk_div_0", "cr_top_mx_spim0preclk_2_sw", CLOCK_SPIM0_OFFSET, SPIM0PRECLK_DIV_0_LENGTH, SPIM0CLK_DIV_0_LENGTH, SPIM0PRECLK_DIV_0_SHIFT, SPIM0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SPIM0PRECLK_2_SW, "cr_top_mx_spim0preclk_2_sw", mux_spim0preclk_1_pll3bypass, CLOCK_SPIM0_OFFSET, MX_SPIM0PRECLK_2_SW_SHIFT),
	MUX(MX_SPIM0PRECLK_1_SW, "cr_top_mx_spim0preclk_1_sw", mux_spim0preclk_0_pll2bypass, CLOCK_SPIM0_OFFSET, MX_SPIM0PRECLK_1_SW_SHIFT),
	MUX(MX_SPIM0PRECLK_0_SW, "cr_top_mx_spim0preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SPIM0_OFFSET,  MX_SPIM0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x009C  CR_TOP_CLOCK_SPIM1	g=1 d=2 m=3 */
{
.dev_gate = GATE(SPIM1PRECLK_GATE_EN, "cr_top_spim1preclk_gate_en", "cr_top_spim1clk_div_0", CLOCK_SPIM1_OFFSET, SPIM1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SPIM1CLK_DIV_0, "cr_top_spim1clk_div_0", "cr_top_mx_spim1preclk_2_sw", CLOCK_SPIM1_OFFSET, SPIM1PRECLK_DIV_0_LENGTH, SPIM1CLK_DIV_0_LENGTH, SPIM1PRECLK_DIV_0_SHIFT, SPIM1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SPIM1PRECLK_2_SW, "cr_top_mx_spim1preclk_2_sw", mux_spim1preclk_1_pll3bypass, CLOCK_SPIM1_OFFSET, MX_SPIM1PRECLK_2_SW_SHIFT),
	MUX(MX_SPIM1PRECLK_1_SW, "cr_top_mx_spim1preclk_1_sw", mux_spim1preclk_0_pll2bypass, CLOCK_SPIM1_OFFSET, MX_SPIM1PRECLK_1_SW_SHIFT),
	MUX(MX_SPIM1PRECLK_0_SW, "cr_top_mx_spim1preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SPIM1_OFFSET,  MX_SPIM1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00A0  CR_TOP_CLOCK_SDHOST0	g=2 d=3 m=4 */
{
.dev_gate = GATE(SDHOST0PRECLK_GATE_EN, "cr_top_sdhost0preclk_gate_en", "cr_top_sdhost0clk_div_0", CLOCK_SDHOST0_OFFSET, SDHOST0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SDHOST0CLK_DIV_0, "cr_top_sdhost0clk_div_0", "cr_top_mx_sdhost0preclk_2_sw", CLOCK_SDHOST0_OFFSET, SDHOST0PRECLK_DIV_0_LENGTH, SDHOST0CLK_DIV_0_LENGTH, SDHOST0PRECLK_DIV_0_SHIFT, SDHOST0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SDHOST0PRECLK_2_SW, "cr_top_mx_sdhost0preclk_2_sw", mux_sdhost0preclk_1_pll3bypass, CLOCK_SDHOST0_OFFSET, MX_SDHOST0PRECLK_2_SW_SHIFT),
	MUX(MX_SDHOST0PRECLK_1_SW, "cr_top_mx_sdhost0preclk_1_sw", mux_sdhost0preclk_0_pll2bypass, CLOCK_SDHOST0_OFFSET, MX_SDHOST0PRECLK_1_SW_SHIFT),
	MUX(MX_SDHOST0PRECLK_0_SW, "cr_top_mx_sdhost0preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SDHOST0_OFFSET,  MX_SDHOST0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},
{
.dev_gate = GATE(SDHOST0LPCLK_GATE_EN, "cr_top_sdhost0lpclk_gate_en", "cr_top_sdhost0lpclk_div_0", CLOCK_SDHOST0_OFFSET, SDHOST0LPCLK_GATE_EN_SHIFT),
.dev_div = DIVC(SDHOST0LPCLK_DIV_0, "cr_top_sdhost0lpclk_div_0", "cr_top_mx_sdhost0lpclk_0_sw", CLOCK_SDHOST0_OFFSET, SDHOST0LPCLK_DIV_0_SHIFT, SDHOST0LPCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_SDHOST0LPCLK_0_SW, "cr_top_mx_sdhost0lpclk_0_sw", mux_xtal0_xtal1, CLOCK_SDHOST0_OFFSET, MX_SDHOST0LPCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00A4  CR_TOP_CLOCK_SDHOST1	g=2 d=3 m=4 */
{
.dev_gate = GATE(SDHOST1PRECLK_GATE_EN, "cr_top_sdhost1preclk_gate_en", "cr_top_sdhost1clk_div_0", CLOCK_SDHOST1_OFFSET, SDHOST1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SDHOST1CLK_DIV_0, "cr_top_sdhost1clk_div_0", "cr_top_mx_sdhost1preclk_2_sw", CLOCK_SDHOST1_OFFSET, SDHOST1PRECLK_DIV_0_LENGTH, SDHOST1CLK_DIV_0_LENGTH, SDHOST1PRECLK_DIV_0_SHIFT, SDHOST1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SDHOST1PRECLK_2_SW, "cr_top_mx_sdhost1preclk_2_sw", mux_sdhost1preclk_1_pll3bypass, CLOCK_SDHOST1_OFFSET, MX_SDHOST1PRECLK_2_SW_SHIFT),
	MUX(MX_SDHOST1PRECLK_1_SW, "cr_top_mx_sdhost1preclk_1_sw", mux_sdhost1preclk_0_pll2bypass, CLOCK_SDHOST1_OFFSET, MX_SDHOST1PRECLK_1_SW_SHIFT),
	MUX(MX_SDHOST1PRECLK_0_SW, "cr_top_mx_sdhost1preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SDHOST1_OFFSET,  MX_SDHOST1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},
{
.dev_gate = GATE(SDHOST1LPCLK_GATE_EN, "cr_top_sdhost1lpclk_gate_en", "cr_top_sdhost1lpclk_div_0", CLOCK_SDHOST1_OFFSET, SDHOST1LPCLK_GATE_EN_SHIFT),
.dev_div = DIVC(SDHOST1LPCLK_DIV_0, "cr_top_sdhost1lpclk_div_0", "cr_top_mx_sdhost1lpclk_0_sw", CLOCK_SDHOST1_OFFSET, SDHOST1LPCLK_DIV_0_SHIFT, SDHOST1LPCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_SDHOST1LPCLK_0_SW, "cr_top_mx_sdhost1lpclk_0_sw", mux_xtal0_xtal1, CLOCK_SDHOST1_OFFSET, MX_SDHOST1LPCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00A8  CR_TOP_CLOCK_SDHOST2	g=2 d=3 m=4 */
{
.dev_gate = GATE(SDHOST2PRECLK_GATE_EN, "cr_top_sdhost2preclk_gate_en", "cr_top_sdhost2clk_div_0", CLOCK_SDHOST2_OFFSET, SDHOST2PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SDHOST2CLK_DIV_0, "cr_top_sdhost2clk_div_0", "cr_top_mx_sdhost2preclk_2_sw", CLOCK_SDHOST2_OFFSET, SDHOST2PRECLK_DIV_0_LENGTH, SDHOST2CLK_DIV_0_LENGTH, SDHOST2PRECLK_DIV_0_SHIFT, SDHOST2CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SDHOST2PRECLK_2_SW, "cr_top_mx_sdhost2preclk_2_sw", mux_sdhost2preclk_1_pll3bypass, CLOCK_SDHOST2_OFFSET, MX_SDHOST2PRECLK_2_SW_SHIFT),
	MUX(MX_SDHOST2PRECLK_1_SW, "cr_top_mx_sdhost2preclk_1_sw", mux_sdhost2preclk_0_pll2bypass, CLOCK_SDHOST2_OFFSET, MX_SDHOST2PRECLK_1_SW_SHIFT),
	MUX(MX_SDHOST2PRECLK_0_SW, "cr_top_mx_sdhost2preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SDHOST2_OFFSET,  MX_SDHOST2PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},
{
.dev_gate = GATE(SDHOST2LPCLK_GATE_EN, "cr_top_sdhost2lpclk_gate_en", "cr_top_sdhost2lpclk_div_0", CLOCK_SDHOST2_OFFSET, SDHOST2LPCLK_GATE_EN_SHIFT),
.dev_div = DIVC(SDHOST2LPCLK_DIV_0, "cr_top_sdhost2lpclk_div_0", "cr_top_mx_sdhost2lpclk_0_sw", CLOCK_SDHOST2_OFFSET, SDHOST2LPCLK_DIV_0_SHIFT, SDHOST2LPCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_SDHOST2LPCLK_0_SW, "cr_top_mx_sdhost2lpclk_0_sw", mux_xtal0_xtal1, CLOCK_SDHOST2_OFFSET, MX_SDHOST2LPCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00AC  CR_TOP_CLOCK_ETHERNET	g=1 d=2 m=3 */
{
.dev_gate = GATE(ETHERNETPRECLK_GATE_EN, "cr_top_ethernetpreclk_gate_en", "cr_top_ethernetclk_div_0", CLOCK_ETHERNET_OFFSET, ETHERNETPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(ETHERNETCLK_DIV_0, "cr_top_ethernetclk_div_0", "cr_top_mx_ethernetpreclk_2_sw", CLOCK_ETHERNET_OFFSET, ETHERNETPRECLK_DIV_0_LENGTH, ETHERNETCLK_DIV_0_LENGTH, ETHERNETPRECLK_DIV_0_SHIFT, ETHERNETCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_ETHERNETPRECLK_2_SW, "cr_top_mx_ethernetpreclk_2_sw", mux_ethernetpreclk_1_pll3bypass, CLOCK_ETHERNET_OFFSET, MX_ETHERNETPRECLK_2_SW_SHIFT),
	MUX(MX_ETHERNETPRECLK_1_SW, "cr_top_mx_ethernetpreclk_1_sw", mux_ethernetpreclk_0_pll2bypass, CLOCK_ETHERNET_OFFSET, MX_ETHERNETPRECLK_1_SW_SHIFT),
	MUX(MX_ETHERNETPRECLK_0_SW, "cr_top_mx_ethernetpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_ETHERNET_OFFSET,  MX_ETHERNETPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00B0  CR_TOP_CLOCK_USB0	g=1 d=1 m=1 */
{
.dev_gate = GATE(USB0CLK_GATE_EN, "cr_top_usb0clk_gate_en", "cr_top_usb0clk_div_0", CLOCK_USB0_OFFSET, USB0CLK_GATE_EN_SHIFT),
.dev_div = DIVC(USB0CLK_DIV_0, "cr_top_usb0clk_div_0", "cr_top_mx_usb0clk_0_sw", CLOCK_USB0_OFFSET, USB0CLK_DIV_0_SHIFT, USB0CLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_USB0CLK_0_SW, "cr_top_mx_usb0clk_0_sw", mux_xtal1_xtal2, CLOCK_USB0_OFFSET, MX_USB0CLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00B4  CR_TOP_CLOCK_USB0_SUSPEND	g=1 d=1 m=1 */
{
.dev_gate = GATE(USB0SUSPENDCLK_GATE_EN, "cr_top_usb0suspendclk_gate_en", "cr_top_usb0suspendclk_div_0", CLOCK_USB0_SUSPEND_OFFSET, USB0SUSPENDCLK_GATE_EN_SHIFT),
.dev_div = DIVC(USB0SUSPENDCLK_DIV_0, "cr_top_usb0suspendclk_div_0", "cr_top_mx_usb0suspendclk_0_sw", CLOCK_USB0_SUSPEND_OFFSET, USB0SUSPENDCLK_DIV_0_SHIFT, USB0SUSPENDCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_USB0SUSPENDCLK_0_SW, "cr_top_mx_usb0suspendclk_0_sw", mux_xtal0_xtal1, CLOCK_USB0_SUSPEND_OFFSET, MX_USB0SUSPENDCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00B8  CR_TOP_CLOCK_USB1	g=1 d=1 m=1 */
{
.dev_gate = GATE(USB1CLK_GATE_EN, "cr_top_usb1clk_gate_en", "cr_top_usb1clk_div_0", CLOCK_USB1_OFFSET, USB1CLK_GATE_EN_SHIFT),
.dev_div = DIVC(USB1CLK_DIV_0, "cr_top_usb1clk_div_0", "cr_top_mx_usb1clk_0_sw", CLOCK_USB1_OFFSET, USB1CLK_DIV_0_SHIFT, USB1CLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_USB1CLK_0_SW, "cr_top_mx_usb1clk_0_sw", mux_xtal1_xtal2, CLOCK_USB1_OFFSET, MX_USB1CLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00BC  CR_TOP_CLOCK_USB1_SUSPEND	g=1 d=1 m=1 */
{
.dev_gate = GATE(USB1SUSPENDCLK_GATE_EN, "cr_top_usb1suspendclk_gate_en", "cr_top_usb1suspendclk_div_0", CLOCK_USB1_SUSPEND_OFFSET, USB1SUSPENDCLK_GATE_EN_SHIFT),
.dev_div = DIVC(USB1SUSPENDCLK_DIV_0, "cr_top_usb1suspendclk_div_0", "cr_top_mx_usb1suspendclk_0_sw", CLOCK_USB1_SUSPEND_OFFSET, USB1SUSPENDCLK_DIV_0_SHIFT, USB1SUSPENDCLK_DIV_0_LENGTH, 0),
.dev_muxes = {MUXC(MX_USB1SUSPENDCLK_0_SW, "cr_top_mx_usb1suspendclk_0_sw", mux_xtal0_xtal1, CLOCK_USB1_SUSPEND_OFFSET, MX_USB1SUSPENDCLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00C0  CR_TOP_CLOCK_UFC	g=1 d=2 m=3 */
{
.dev_gate = GATE(UFCPRECLK_GATE_EN, "cr_top_ufcpreclk_gate_en", "cr_top_ufcclk_div_0", CLOCK_UFC_OFFSET, UFCPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(UFCCLK_DIV_0, "cr_top_ufcclk_div_0", "cr_top_mx_ufcpreclk_2_sw", CLOCK_UFC_OFFSET, UFCPRECLK_DIV_0_LENGTH, UFCCLK_DIV_0_LENGTH, UFCPRECLK_DIV_0_SHIFT, UFCCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_UFCPRECLK_2_SW, "cr_top_mx_ufcpreclk_2_sw", mux_ufcpreclk_1_pll3bypass, CLOCK_UFC_OFFSET, MX_UFCPRECLK_2_SW_SHIFT),
	MUX(MX_UFCPRECLK_1_SW, "cr_top_mx_ufcpreclk_1_sw", mux_ufcpreclk_0_pll2bypass, CLOCK_UFC_OFFSET, MX_UFCPRECLK_1_SW_SHIFT),
	MUX(MX_UFCPRECLK_0_SW, "cr_top_mx_ufcpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_UFC_OFFSET,  MX_UFCPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00C4  CR_TOP_CLOCK_PWM	g=1 d=2 m=1 */
{
.dev_gate = GATE(PWMPRECLK_GATE_EN, "cr_top_pwmpreclk_gate_en", "cr_top_pwmclk_div_0", CLOCK_PWM_OFFSET, PWMPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(PWMCLK_DIV_0, "cr_top_pwmclk_div_0", "cr_top_mx_pwmpreclk_0_sw", CLOCK_PWM_OFFSET, PWMPRECLK_DIV_0_LENGTH, PWMCLK_DIV_0_LENGTH, PWMPRECLK_DIV_0_SHIFT, PWMCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_PWMPRECLK_0_SW, "cr_top_mx_pwmpreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_PWM_OFFSET, MX_PWMPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00C8  CR_TOP_CLOCK_PDM	g=1 d=2 m=1 */
{
.dev_gate = GATE(PDMPRECLK_GATE_EN, "cr_top_pdmpreclk_gate_en", "cr_top_pdmclk_div_0", CLOCK_PDM_OFFSET, PDMPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(PDMCLK_DIV_0, "cr_top_pdmclk_div_0", "cr_top_mx_pdmpreclk_0_sw", CLOCK_PDM_OFFSET, PDMPRECLK_DIV_0_LENGTH, PDMCLK_DIV_0_LENGTH, PDMPRECLK_DIV_0_SHIFT, PDMCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_PDMPRECLK_0_SW, "cr_top_mx_pdmpreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_PDM_OFFSET, MX_PDMPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00CC  CR_TOP_CLOCK_OTP	g=1 d=2 m=1 */
{
.dev_gate = GATE(OTPPRECLK_GATE_EN, "cr_top_otppreclk_gate_en", "cr_top_otpclk_div_0", CLOCK_OTP_OFFSET, OTPPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(OTPCLK_DIV_0, "cr_top_otpclk_div_0", "cr_top_mx_otppreclk_0_sw", CLOCK_OTP_OFFSET, OTPPRECLK_DIV_0_LENGTH, OTPCLK_DIV_0_LENGTH, OTPPRECLK_DIV_0_SHIFT, OTPCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_OTPPRECLK_0_SW, "cr_top_mx_otppreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_OTP_OFFSET, MX_OTPPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00D0  CR_TOP_CLOCK_TEMP	g=1 d=2 m=1 */
{
.dev_gate = GATE(TEMPPRECLK_GATE_EN, "cr_top_temppreclk_gate_en", "cr_top_tempclk_div_0", CLOCK_TEMP_OFFSET, TEMPPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(TEMPCLK_DIV_0, "cr_top_tempclk_div_0", "cr_top_mx_temppreclk_0_sw", CLOCK_TEMP_OFFSET, TEMPPRECLK_DIV_0_LENGTH, TEMPCLK_DIV_0_LENGTH, TEMPPRECLK_DIV_0_SHIFT, TEMPCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_TEMPPRECLK_0_SW, "cr_top_mx_temppreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_TEMP_OFFSET, MX_TEMPPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00D4  CR_TOP_CLOCK_VOLT	g=1 d=2 m=1 */
{
.dev_gate = GATE(VOLTPRECLK_GATE_EN, "cr_top_voltpreclk_gate_en", "cr_top_voltclk_div_0", CLOCK_VOLT_OFFSET, VOLTPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(VOLTCLK_DIV_0, "cr_top_voltclk_div_0", "cr_top_mx_voltpreclk_0_sw", CLOCK_VOLT_OFFSET, VOLTPRECLK_DIV_0_LENGTH, VOLTCLK_DIV_0_LENGTH, VOLTPRECLK_DIV_0_SHIFT, VOLTCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_VOLTPRECLK_0_SW, "cr_top_mx_voltpreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_VOLT_OFFSET, MX_VOLTPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00D8  CR_TOP_CLOCK_ETS	g=1 d=2 m=1 */
{
.dev_gate = GATE(ETSPRECLK_GATE_EN, "cr_top_etspreclk_gate_en", "cr_top_etsclk_div_0", CLOCK_ETS_OFFSET, ETSPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(ETSCLK_DIV_0, "cr_top_etsclk_div_0", "cr_top_mx_etspreclk_0_sw", CLOCK_ETS_OFFSET, ETSPRECLK_DIV_0_LENGTH, ETSCLK_DIV_0_LENGTH, ETSPRECLK_DIV_0_SHIFT, ETSCLK_DIV_0_SHIFT),
.dev_muxes = {MUX(MX_ETSPRECLK_0_SW, "cr_top_mx_etspreclk_0_sw", mux_pll1bypass_pll2bypass, CLOCK_ETS_OFFSET, MX_ETSPRECLK_0_SW_SHIFT),},

.no_of_muxes =	1,
},

/*0x00DC  CR_TOP_CLOCK_SYS_OUT0	g=1 d=2 m=3 */
{
.dev_gate = GATE(SYSOUT0PRECLK_GATE_EN, "cr_top_sysout0preclk_gate_en", "cr_top_sysout0clk_div_0", CLOCK_SYS_OUT0_OFFSET, SYSOUT0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SYSOUT0CLK_DIV_0, "cr_top_sysout0clk_div_0", "cr_top_mx_sysout0preclk_2_sw", CLOCK_SYS_OUT0_OFFSET, SYSOUT0PRECLK_DIV_0_LENGTH, SYSOUT0CLK_DIV_0_LENGTH, SYSOUT0PRECLK_DIV_0_SHIFT, SYSOUT0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SYSOUT0PRECLK_2_SW, "cr_top_mx_sysout0preclk_2_sw", mux_sysout0preclk_1_pll3bypass, CLOCK_SYS_OUT0_OFFSET, MX_SYSOUT0PRECLK_2_SW_SHIFT),
	MUX(MX_SYSOUT0PRECLK_1_SW, "cr_top_mx_sysout0preclk_1_sw", mux_sysout0preclk_0_pll2bypass, CLOCK_SYS_OUT0_OFFSET, MX_SYSOUT0PRECLK_1_SW_SHIFT),
	MUX(MX_SYSOUT0PRECLK_0_SW, "cr_top_mx_sysout0preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SYS_OUT0_OFFSET,  MX_SYSOUT0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00E0  CR_TOP_CLOCK_SYS_OUT1	g=1 d=2 m=3 */
{
.dev_gate = GATE(SYSOUT1PRECLK_GATE_EN, "cr_top_sysout1preclk_gate_en", "cr_top_sysout1clk_div_0", CLOCK_SYS_OUT1_OFFSET, SYSOUT1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SYSOUT1CLK_DIV_0, "cr_top_sysout1clk_div_0", "cr_top_mx_sysout1preclk_2_sw", CLOCK_SYS_OUT1_OFFSET, SYSOUT1PRECLK_DIV_0_LENGTH, SYSOUT1CLK_DIV_0_LENGTH, SYSOUT1PRECLK_DIV_0_SHIFT, SYSOUT1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SYSOUT1PRECLK_2_SW, "cr_top_mx_sysout1preclk_2_sw", mux_sysout1preclk_1_pll3bypass, CLOCK_SYS_OUT1_OFFSET, MX_SYSOUT1PRECLK_2_SW_SHIFT),
	MUX(MX_SYSOUT1PRECLK_1_SW, "cr_top_mx_sysout1preclk_1_sw", mux_sysout1preclk_0_pll2bypass, CLOCK_SYS_OUT1_OFFSET, MX_SYSOUT1PRECLK_1_SW_SHIFT),
	MUX(MX_SYSOUT1PRECLK_0_SW, "cr_top_mx_sysout1preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SYS_OUT1_OFFSET,  MX_SYSOUT1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00E4  CR_TOP_CLOCK_SYS0	g=1 d=2 m=3 */
{
.dev_gate = GATE(SYS0PRECLK_GATE_EN, "cr_top_sys0preclk_gate_en", "cr_top_sys0clk_div_0", CLOCK_SYS0_OFFSET, SYS0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SYS0CLK_DIV_0, "cr_top_sys0clk_div_0", "cr_top_mx_sys0preclk_2_sw", CLOCK_SYS0_OFFSET, SYS0PRECLK_DIV_0_LENGTH, SYS0CLK_DIV_0_LENGTH, SYS0PRECLK_DIV_0_SHIFT, SYS0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SYS0PRECLK_2_SW, "cr_top_mx_sys0preclk_2_sw", mux_sys0preclk_1_pll3bypass, CLOCK_SYS0_OFFSET, MX_SYS0PRECLK_2_SW_SHIFT),
	MUX(MX_SYS0PRECLK_1_SW, "cr_top_mx_sys0preclk_1_sw", mux_sys0preclk_0_pll2bypass, CLOCK_SYS0_OFFSET, MX_SYS0PRECLK_1_SW_SHIFT),
	MUX(MX_SYS0PRECLK_0_SW, "cr_top_mx_sys0preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SYS0_OFFSET,  MX_SYS0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00E8  CR_TOP_CLOCK_SYS1	g=1 d=2 m=3 */
{
.dev_gate = GATE(SYS1PRECLK_GATE_EN, "cr_top_sys1preclk_gate_en", "cr_top_sys1clk_div_0", CLOCK_SYS1_OFFSET, SYS1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SYS1CLK_DIV_0, "cr_top_sys1clk_div_0", "cr_top_mx_sys1preclk_2_sw", CLOCK_SYS1_OFFSET, SYS1PRECLK_DIV_0_LENGTH, SYS1CLK_DIV_0_LENGTH, SYS1PRECLK_DIV_0_SHIFT, SYS1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SYS1PRECLK_2_SW, "cr_top_mx_sys1preclk_2_sw", mux_sys1preclk_1_pll3bypass, CLOCK_SYS1_OFFSET, MX_SYS1PRECLK_2_SW_SHIFT),
	MUX(MX_SYS1PRECLK_1_SW, "cr_top_mx_sys1preclk_1_sw", mux_sys1preclk_0_pll2bypass, CLOCK_SYS1_OFFSET, MX_SYS1PRECLK_1_SW_SHIFT),
	MUX(MX_SYS1PRECLK_0_SW, "cr_top_mx_sys1preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SYS1_OFFSET,  MX_SYS1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00EC  CR_TOP_CLOCK_SYS2	g=1 d=2 m=3 */
{
.dev_gate = GATE(SYS2PRECLK_GATE_EN, "cr_top_sys2preclk_gate_en", "cr_top_sys2clk_div_0", CLOCK_SYS2_OFFSET, SYS2PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SYS2CLK_DIV_0, "cr_top_sys2clk_div_0", "cr_top_mx_sys2preclk_2_sw", CLOCK_SYS2_OFFSET, SYS2PRECLK_DIV_0_LENGTH, SYS2CLK_DIV_0_LENGTH, SYS2PRECLK_DIV_0_SHIFT, SYS2CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SYS2PRECLK_2_SW, "cr_top_mx_sys2preclk_2_sw", mux_sys2preclk_1_pll3bypass, CLOCK_SYS2_OFFSET, MX_SYS2PRECLK_2_SW_SHIFT),
	MUX(MX_SYS2PRECLK_1_SW, "cr_top_mx_sys2preclk_1_sw", mux_sys2preclk_0_pll2bypass, CLOCK_SYS2_OFFSET, MX_SYS2PRECLK_1_SW_SHIFT),
	MUX(MX_SYS2PRECLK_0_SW, "cr_top_mx_sys2preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SYS2_OFFSET,  MX_SYS2PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00F0  CR_TOP_CLOCK_SYS3	g=1 d=2 m=3 */
{
.dev_gate = GATE(SYS3PRECLK_GATE_EN, "cr_top_sys3preclk_gate_en", "cr_top_sys3clk_div_0", CLOCK_SYS3_OFFSET, SYS3PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SYS3CLK_DIV_0, "cr_top_sys3clk_div_0", "cr_top_mx_sys3preclk_2_sw", CLOCK_SYS3_OFFSET, SYS3PRECLK_DIV_0_LENGTH, SYS3CLK_DIV_0_LENGTH, SYS3PRECLK_DIV_0_SHIFT, SYS3CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SYS3PRECLK_2_SW, "cr_top_mx_sys3preclk_2_sw", mux_sys3preclk_1_pll3bypass, CLOCK_SYS3_OFFSET, MX_SYS3PRECLK_2_SW_SHIFT),
	MUX(MX_SYS3PRECLK_1_SW, "cr_top_mx_sys3preclk_1_sw", mux_sys3preclk_0_pll2bypass, CLOCK_SYS3_OFFSET, MX_SYS3PRECLK_1_SW_SHIFT),
	MUX(MX_SYS3PRECLK_0_SW, "cr_top_mx_sys3preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_SYS3_OFFSET,  MX_SYS3PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x012C  CR_TOP_CLOCK_TIMER	g=1 d=2 m=4 */
{
.dev_gate = GATE(CS_TIMERPRESWCLK_0_EN, "cr_top_cs_timerpreswclk_0_en", "cr_top_timerclk_div_0", CLOCK_TIMER_OFFSET, CS_TIMERPRESWCLK_0_EN_SHIFT),
.dev_ddiv = DDIV(TIMERCLK_DIV_0, "cr_top_timerclk_div_0", "cr_top_cs_timerpreswclk_0_sw", CLOCK_TIMER_OFFSET, TIMERPRESWCLK_DIV_0_LENGTH, TIMERCLK_DIV_0_LENGTH, TIMERPRESWCLK_DIV_0_SHIFT, TIMERCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUXC(CS_TIMERPRESWCLK_0_SW, "cr_top_cs_timerpreswclk_0_sw", cs_xtal1_mx_timerpreclk_2, CLOCK_TIMER_OFFSET, CS_TIMERPRESWCLK_0_SW_SHIFT),
	MUX(MX_TIMERPRECLK_2_SW, "cr_top_mx_timerpreclk_2_sw", mux_timerpreclk_1_pll3bypass, CLOCK_TIMER_OFFSET, MX_TIMERPRECLK_2_SW_SHIFT),
	MUX(MX_TIMERPRECLK_1_SW, "cr_top_mx_timerpreclk_1_sw", mux_timerpreclk_0_pll2bypass, CLOCK_TIMER_OFFSET, MX_TIMERPRECLK_1_SW_SHIFT),
	MUX(MX_TIMERPRECLK_0_SW, "cr_top_mx_timerpreclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_TIMER_OFFSET,  MX_TIMERPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	4,
},

/*0x0140  CR_TOP_CLOCK_NOC_CIP3	g=1 d=2 m=4 */
{
.dev_gate = GATE(CS_NOCCIP3PRESWCLK_0_EN, "cr_top_cs_noccip3preswclk_0_en", "cr_top_noccip3clk_div_0", CLOCK_NOC_CIP3_OFFSET - CLOCK_AV_PRIMARY_OFFSET, CS_NOCCIP3PRESWCLK_0_EN_SHIFT),
.dev_ddiv = DDIV(NOCCIP3CLK_DIV_0, "cr_top_noccip3clk_div_0", "cr_top_cs_noccip3preswclk_0_sw", CLOCK_NOC_CIP3_OFFSET - CLOCK_AV_PRIMARY_OFFSET, NOCCIP3PRESWCLK_DIV_0_LENGTH, NOCCIP3CLK_DIV_0_LENGTH, NOCCIP3PRESWCLK_DIV_0_SHIFT, NOCCIP3CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUXC(CS_NOCCIP3PRESWCLK_0_SW, "cr_top_cs_noccip3preswclk_0_sw", cs_xtal1_mx_noccip3preclk_2, CLOCK_NOC_CIP3_OFFSET - CLOCK_AV_PRIMARY_OFFSET, CS_NOCCIP3PRESWCLK_0_SW_SHIFT),
	MUX(MX_NOCCIP3PRECLK_2_SW, "cr_top_mx_noccip3preclk_2_sw", mux_noccip3preclk_1_pll3bypass, CLOCK_NOC_CIP3_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_NOCCIP3PRECLK_2_SW_SHIFT),
	MUX(MX_NOCCIP3PRECLK_1_SW, "cr_top_mx_noccip3preclk_1_sw", mux_noccip3preclk_0_pll2bypass, CLOCK_NOC_CIP3_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_NOCCIP3PRECLK_1_SW_SHIFT),
	MUX(MX_NOCCIP3PRECLK_0_SW, "cr_top_mx_noccip3preclk_0_sw", mux_pll0bypass_pll1bypass, CLOCK_NOC_CIP3_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_NOCCIP3PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	4,
},
};

/*------CR_TOP_CLOCK_AV_PRIMARY 0x00F4------------*/
static struct cashew_device_clock  cashew_av_devices[] __initdata = {
/*Note:
The following clocks are sys clocks but have been put in av clock area due to register address
NOC_CIP3
TIMER
be careful with plls and particularly test them.
*/
/*------CR_TOP_CLOCK_AV_PRIMARY 0x00F4------------*/
/*0x00F8  CR_TOP_CLOCK_ISPPIXEL	g=1 d=2 m=3 */
{
.dev_gate = GATE(ISPPIXELPRECLK_GATE_EN, "cr_top_isppixelpreclk_gate_en", "cr_top_isppixelclk_div_0", CLOCK_ISPPIXEL_OFFSET - CLOCK_AV_PRIMARY_OFFSET, ISPPIXELPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(ISPPIXELCLK_DIV_0, "cr_top_isppixelclk_div_0", "cr_top_mx_isppixelpreclk_2_sw", CLOCK_ISPPIXEL_OFFSET - CLOCK_AV_PRIMARY_OFFSET, ISPPIXELPRECLK_DIV_0_LENGTH, ISPPIXELCLK_DIV_0_LENGTH, ISPPIXELPRECLK_DIV_0_SHIFT, ISPPIXELCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_ISPPIXELPRECLK_2_SW, "cr_top_mx_isppixelpreclk_2_sw", mux_isppixelpreclk_1_pll7bypass, CLOCK_ISPPIXEL_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_ISPPIXELPRECLK_2_SW_SHIFT),
	MUX(MX_ISPPIXELPRECLK_1_SW, "cr_top_mx_isppixelpreclk_1_sw", mux_isppixelpreclk_0_pll6bypass, CLOCK_ISPPIXEL_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_ISPPIXELPRECLK_1_SW_SHIFT),
	MUX(MX_ISPPIXELPRECLK_0_SW, "cr_top_mx_isppixelpreclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_ISPPIXEL_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_ISPPIXELPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x00FC  CR_TOP_CLOCK_PIXEL0	g=1 d=2 m=3 */
{
.dev_gate = GATE(PIXEL0PRECLK_GATE_EN, "cr_top_pixel0preclk_gate_en", "cr_top_pixel0clk_div_0", CLOCK_PIXEL0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, PIXEL0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(PIXEL0CLK_DIV_0, "cr_top_pixel0clk_div_0", "cr_top_mx_pixel0preclk_2_sw", CLOCK_PIXEL0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, PIXEL0PRECLK_DIV_0_LENGTH, PIXEL0CLK_DIV_0_LENGTH, PIXEL0PRECLK_DIV_0_SHIFT, PIXEL0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_PIXEL0PRECLK_2_SW, "cr_top_mx_pixel0preclk_2_sw", mux_pixel0preclk_1_pll7bypass, CLOCK_PIXEL0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_PIXEL0PRECLK_2_SW_SHIFT),
	MUX(MX_PIXEL0PRECLK_1_SW, "cr_top_mx_pixel0preclk_1_sw", mux_pixel0preclk_0_pll6bypass, CLOCK_PIXEL0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_PIXEL0PRECLK_1_SW_SHIFT),
	MUX(MX_PIXEL0PRECLK_0_SW, "cr_top_mx_pixel0preclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_PIXEL0_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_PIXEL0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0100  CR_TOP_CLOCK_PIXEL1	g=1 d=2 m=3 */
{
.dev_gate = GATE(PIXEL1PRECLK_GATE_EN, "cr_top_pixel1preclk_gate_en", "cr_top_pixel1clk_div_0", CLOCK_PIXEL1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, PIXEL1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(PIXEL1CLK_DIV_0, "cr_top_pixel1clk_div_0", "cr_top_mx_pixel1preclk_2_sw", CLOCK_PIXEL1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, PIXEL1PRECLK_DIV_0_LENGTH, PIXEL1CLK_DIV_0_LENGTH, PIXEL1PRECLK_DIV_0_SHIFT, PIXEL1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_PIXEL1PRECLK_2_SW, "cr_top_mx_pixel1preclk_2_sw", mux_pixel1preclk_1_pll7bypass, CLOCK_PIXEL1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_PIXEL1PRECLK_2_SW_SHIFT),
	MUX(MX_PIXEL1PRECLK_1_SW, "cr_top_mx_pixel1preclk_1_sw", mux_pixel1preclk_0_pll6bypass, CLOCK_PIXEL1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_PIXEL1PRECLK_1_SW_SHIFT),
	MUX(MX_PIXEL1PRECLK_0_SW, "cr_top_mx_pixel1preclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_PIXEL1_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_PIXEL1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0104  CR_TOP_CLOCK_PIXEL1X7	g=1 d=2 m=3 */
{
.dev_gate = GATE(PIXEL1X7PRECLK_GATE_EN, "cr_top_pixel1x7preclk_gate_en", "cr_top_pixel1x7clk_div_0", CLOCK_PIXEL1X7_OFFSET - CLOCK_AV_PRIMARY_OFFSET, PIXEL1X7PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(PIXEL1X7CLK_DIV_0, "cr_top_pixel1x7clk_div_0", "cr_top_mx_pixel1x7preclk_2_sw", CLOCK_PIXEL1X7_OFFSET - CLOCK_AV_PRIMARY_OFFSET, PIXEL1X7PRECLK_DIV_0_LENGTH, PIXEL1X7CLK_DIV_0_LENGTH, PIXEL1X7PRECLK_DIV_0_SHIFT, PIXEL1X7CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_PIXEL1X7PRECLK_2_SW, "cr_top_mx_pixel1x7preclk_2_sw", mux_pixel1x7preclk_1_pll7bypass, CLOCK_PIXEL1X7_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_PIXEL1X7PRECLK_2_SW_SHIFT),
	MUX(MX_PIXEL1X7PRECLK_1_SW, "cr_top_mx_pixel1x7preclk_1_sw", mux_pixel1x7preclk_0_pll6bypass, CLOCK_PIXEL1X7_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_PIXEL1X7PRECLK_1_SW_SHIFT),
	MUX(MX_PIXEL1X7PRECLK_0_SW, "cr_top_mx_pixel1x7preclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_PIXEL1X7_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_PIXEL1X7PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0108  CR_TOP_CLOCK_HDMI_TX0_ESM	g=1 d=2 m=3 */
{
.dev_gate = GATE(HDMITX0ESMPRECLK_GATE_EN, "cr_top_hdmitx0esmpreclk_gate_en", "cr_top_hdmitx0esmclk_div_0", CLOCK_HDMI_TX0_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMITX0ESMPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(HDMITX0ESMCLK_DIV_0, "cr_top_hdmitx0esmclk_div_0", "cr_top_mx_hdmitx0esmpreclk_2_sw", CLOCK_HDMI_TX0_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMITX0ESMPRECLK_DIV_0_LENGTH, HDMITX0ESMCLK_DIV_0_LENGTH, HDMITX0ESMPRECLK_DIV_0_SHIFT, HDMITX0ESMCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_HDMITX0ESMPRECLK_2_SW, "cr_top_mx_hdmitx0esmpreclk_2_sw", mux_hdmitx0esmpreclk_1_pll7bypass, CLOCK_HDMI_TX0_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMITX0ESMPRECLK_2_SW_SHIFT),
	MUX(MX_HDMITX0ESMPRECLK_1_SW, "cr_top_mx_hdmitx0esmpreclk_1_sw", mux_hdmitx0esmpreclk_0_pll6bypass, CLOCK_HDMI_TX0_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMITX0ESMPRECLK_1_SW_SHIFT),
	MUX(MX_HDMITX0ESMPRECLK_0_SW, "cr_top_mx_hdmitx0esmpreclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_HDMI_TX0_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_HDMITX0ESMPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x010C  CR_TOP_CLOCK_HDMI_TX1_ESM	g=1 d=2 m=3 */
{
.dev_gate = GATE(HDMITX1ESMPRECLK_GATE_EN, "cr_top_hdmitx1esmpreclk_gate_en", "cr_top_hdmitx1esmclk_div_0", CLOCK_HDMI_TX1_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMITX1ESMPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(HDMITX1ESMCLK_DIV_0, "cr_top_hdmitx1esmclk_div_0", "cr_top_mx_hdmitx1esmpreclk_2_sw", CLOCK_HDMI_TX1_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMITX1ESMPRECLK_DIV_0_LENGTH, HDMITX1ESMCLK_DIV_0_LENGTH, HDMITX1ESMPRECLK_DIV_0_SHIFT, HDMITX1ESMCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_HDMITX1ESMPRECLK_2_SW, "cr_top_mx_hdmitx1esmpreclk_2_sw", mux_hdmitx1esmpreclk_1_pll7bypass, CLOCK_HDMI_TX1_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMITX1ESMPRECLK_2_SW_SHIFT),
	MUX(MX_HDMITX1ESMPRECLK_1_SW, "cr_top_mx_hdmitx1esmpreclk_1_sw", mux_hdmitx1esmpreclk_0_pll6bypass, CLOCK_HDMI_TX1_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMITX1ESMPRECLK_1_SW_SHIFT),
	MUX(MX_HDMITX1ESMPRECLK_0_SW, "cr_top_mx_hdmitx1esmpreclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_HDMI_TX1_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_HDMITX1ESMPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0110  CR_TOP_CLOCK_HDMI_RX_ESM	g=1 d=2 m=3 */
{
.dev_gate = GATE(HDMIRXESMPRECLK_GATE_EN, "cr_top_hdmirxesmpreclk_gate_en", "cr_top_hdmirxesmclk_div_0", CLOCK_HDMI_RX_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMIRXESMPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(HDMIRXESMCLK_DIV_0, "cr_top_hdmirxesmclk_div_0", "cr_top_mx_hdmirxesmpreclk_2_sw", CLOCK_HDMI_RX_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMIRXESMPRECLK_DIV_0_LENGTH, HDMIRXESMCLK_DIV_0_LENGTH, HDMIRXESMPRECLK_DIV_0_SHIFT, HDMIRXESMCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_HDMIRXESMPRECLK_2_SW, "cr_top_mx_hdmirxesmpreclk_2_sw", mux_hdmirxesmpreclk_1_pll7bypass, CLOCK_HDMI_RX_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMIRXESMPRECLK_2_SW_SHIFT),
	MUX(MX_HDMIRXESMPRECLK_1_SW, "cr_top_mx_hdmirxesmpreclk_1_sw", mux_hdmirxesmpreclk_0_pll6bypass, CLOCK_HDMI_RX_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMIRXESMPRECLK_1_SW_SHIFT),
	MUX(MX_HDMIRXESMPRECLK_0_SW, "cr_top_mx_hdmirxesmpreclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_HDMI_RX_ESM_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_HDMIRXESMPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0114  CR_TOP_CLOCK_HDMI_RX_AUDIO	g=1 d=2 m=3 */
{
.dev_gate = GATE(HDMIRXAUDIOPRECLK_GATE_EN, "cr_top_hdmirxaudiopreclk_gate_en", "cr_top_hdmirxaudioclk_div_0", CLOCK_HDMI_RX_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMIRXAUDIOPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(HDMIRXAUDIOCLK_DIV_0, "cr_top_hdmirxaudioclk_div_0", "cr_top_mx_hdmirxaudiopreclk_2_sw", CLOCK_HDMI_RX_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, HDMIRXAUDIOPRECLK_DIV_0_LENGTH, HDMIRXAUDIOCLK_DIV_0_LENGTH, HDMIRXAUDIOPRECLK_DIV_0_SHIFT, HDMIRXAUDIOCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_HDMIRXAUDIOPRECLK_2_SW, "cr_top_mx_hdmirxaudiopreclk_2_sw", mux_hdmirxaudiopreclk_1_pll7bypass, CLOCK_HDMI_RX_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMIRXAUDIOPRECLK_2_SW_SHIFT),
	MUX(MX_HDMIRXAUDIOPRECLK_1_SW, "cr_top_mx_hdmirxaudiopreclk_1_sw", mux_hdmirxaudiopreclk_0_pll6bypass, CLOCK_HDMI_RX_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_HDMIRXAUDIOPRECLK_1_SW_SHIFT),
	MUX(MX_HDMIRXAUDIOPRECLK_0_SW, "cr_top_mx_hdmirxaudiopreclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_HDMI_RX_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_HDMIRXAUDIOPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0118  CR_TOP_CLOCK_I2S0_AUDIO	g=1 d=3 m=3 */
{
.dev_gate = GATE(I2S0AUDIOPREMCLK_GATE_EN, "cr_top_i2s0audiopremclk_gate_en", "cr_top_i2s0audiomclk_div_0", CLOCK_I2S0_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, I2S0AUDIOPREMCLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2S0AUDIOMCLK_DIV_0, "cr_top_i2s0audiomclk_div_0", "cr_top_mx_i2s0audiopremclk_2_sw", CLOCK_I2S0_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, I2S0AUDIOPREMCLK_DIV_0_LENGTH, I2S0AUDIOMCLK_DIV_0_LENGTH, I2S0AUDIOPREMCLK_DIV_0_SHIFT, I2S0AUDIOMCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2S0AUDIOPREMCLK_2_SW, "cr_top_mx_i2s0audiopremclk_2_sw", mux_i2s0audiopremclk_1_pll7bypass, CLOCK_I2S0_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_I2S0AUDIOPREMCLK_2_SW_SHIFT),
	MUX(MX_I2S0AUDIOPREMCLK_1_SW, "cr_top_mx_i2s0audiopremclk_1_sw", mux_i2s0audiopremclk_0_pll6bypass, CLOCK_I2S0_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_I2S0AUDIOPREMCLK_1_SW_SHIFT),
	MUX(MX_I2S0AUDIOPREMCLK_0_SW, "cr_top_mx_i2s0audiopremclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_I2S0_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_I2S0AUDIOPREMCLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

{
.dev_div = DIVC(I2S0AUDIOACLK_DIV_0, "cr_top_i2s0audioaclk_div_0", "cr_top_i2s0audiopremclk_gate_en", CLOCK_I2S0_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, I2S0AUDIOACLK_DIV_0_SHIFT, I2S0AUDIOACLK_DIV_0_LENGTH, 0),
.no_of_muxes =	0,
},

/*0x011C  CR_TOP_CLOCK_I2S1_AUDIO	g=1 d=3 m=3 */
{
.dev_gate = GATE(I2S1AUDIOPREMCLK_GATE_EN, "cr_top_i2s1audiopremclk_gate_en", "cr_top_i2s1audiomclk_div_0", CLOCK_I2S1_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, I2S1AUDIOPREMCLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(I2S1AUDIOMCLK_DIV_0, "cr_top_i2s1audiomclk_div_0", "cr_top_mx_i2s1audiopremclk_2_sw", CLOCK_I2S1_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, I2S1AUDIOPREMCLK_DIV_0_LENGTH, I2S1AUDIOMCLK_DIV_0_LENGTH, I2S1AUDIOPREMCLK_DIV_0_SHIFT, I2S1AUDIOMCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_I2S1AUDIOPREMCLK_2_SW, "cr_top_mx_i2s1audiopremclk_2_sw", mux_i2s1audiopremclk_1_pll7bypass, CLOCK_I2S1_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_I2S1AUDIOPREMCLK_2_SW_SHIFT),
	MUX(MX_I2S1AUDIOPREMCLK_1_SW, "cr_top_mx_i2s1audiopremclk_1_sw", mux_i2s1audiopremclk_0_pll6bypass, CLOCK_I2S1_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_I2S1AUDIOPREMCLK_1_SW_SHIFT),
	MUX(MX_I2S1AUDIOPREMCLK_0_SW, "cr_top_mx_i2s1audiopremclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_I2S1_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_I2S1AUDIOPREMCLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

{
.dev_div = DIVC(I2S1AUDIOACLK_DIV_0, "cr_top_i2s1audioaclk_div_0", "cr_top_i2s1audiopremclk_gate_en", CLOCK_I2S1_AUDIO_OFFSET - CLOCK_AV_PRIMARY_OFFSET, I2S1AUDIOACLK_DIV_0_SHIFT, I2S1AUDIOACLK_DIV_0_LENGTH, 0),
.no_of_muxes =	0,
},

/*0x0120  CR_TOP_CLOCK_SPDIF	g=1 d=2 m=3 */
{
.dev_gate = GATE(SPDIFPRECLK_GATE_EN, "cr_top_spdifpreclk_gate_en", "cr_top_spdifclk_div_0", CLOCK_SPDIF_OFFSET - CLOCK_AV_PRIMARY_OFFSET, SPDIFPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(SPDIFCLK_DIV_0, "cr_top_spdifclk_div_0", "cr_top_mx_spdifpreclk_2_sw", CLOCK_SPDIF_OFFSET - CLOCK_AV_PRIMARY_OFFSET, SPDIFPRECLK_DIV_0_LENGTH, SPDIFCLK_DIV_0_LENGTH, SPDIFPRECLK_DIV_0_SHIFT, SPDIFCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_SPDIFPRECLK_2_SW, "cr_top_mx_spdifpreclk_2_sw", mux_spdifpreclk_1_pll7bypass, CLOCK_SPDIF_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_SPDIFPRECLK_2_SW_SHIFT),
	MUX(MX_SPDIFPRECLK_1_SW, "cr_top_mx_spdifpreclk_1_sw", mux_spdifpreclk_0_pll6bypass, CLOCK_SPDIF_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_SPDIFPRECLK_1_SW_SHIFT),
	MUX(MX_SPDIFPRECLK_0_SW, "cr_top_mx_spdifpreclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_SPDIF_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_SPDIFPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0124  CR_TOP_CLOCK_AUXADC	g=1 d=2 m=3 */
{
.dev_gate = GATE(AUXADCPRECLK_GATE_EN, "cr_top_auxadcpreclk_gate_en", "cr_top_auxadcclk_div_0", CLOCK_AUXADC_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AUXADCPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(AUXADCCLK_DIV_0, "cr_top_auxadcclk_div_0", "cr_top_mx_auxadcpreclk_2_sw", CLOCK_AUXADC_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AUXADCPRECLK_DIV_0_LENGTH, AUXADCCLK_DIV_0_LENGTH, AUXADCPRECLK_DIV_0_SHIFT, AUXADCCLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_AUXADCPRECLK_2_SW, "cr_top_mx_auxadcpreclk_2_sw", mux_auxadcpreclk_1_pll7bypass, CLOCK_AUXADC_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AUXADCPRECLK_2_SW_SHIFT),
	MUX(MX_AUXADCPRECLK_1_SW, "cr_top_mx_auxadcpreclk_1_sw", mux_auxadcpreclk_0_pll6bypass, CLOCK_AUXADC_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AUXADCPRECLK_1_SW_SHIFT),
	MUX(MX_AUXADCPRECLK_0_SW, "cr_top_mx_auxadcpreclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_AUXADC_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_AUXADCPRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0128  CR_TOP_CLOCK_ETS_AV	g=1 d=2 m=0 */
{
.dev_gate = GATE(ETSAVPRECLK_GATE_EN, "cr_top_etsavpreclk_gate_en", "cr_top_etsavclk_div_0", CLOCK_ETS_AV_OFFSET - CLOCK_AV_PRIMARY_OFFSET, ETSAVPRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(ETSAVCLK_DIV_0, "cr_top_etsavclk_div_0", "cr_top_cs_pll6bypassclk_0_en", CLOCK_ETS_AV_OFFSET - CLOCK_AV_PRIMARY_OFFSET, ETSAVPRECLK_DIV_0_LENGTH, ETSAVCLK_DIV_0_LENGTH, ETSAVPRECLK_DIV_0_SHIFT, ETSAVCLK_DIV_0_SHIFT),

.no_of_muxes =	0,
},

/*0x0130  CR_TOP_CLOCK_AV_OUT0	g=1 d=2 m=3 */
{
.dev_gate = GATE(AVOUT0PRECLK_GATE_EN, "cr_top_avout0preclk_gate_en", "cr_top_avout0clk_div_0", CLOCK_AV_OUT0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AVOUT0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(AVOUT0CLK_DIV_0, "cr_top_avout0clk_div_0", "cr_top_mx_avout0preclk_2_sw", CLOCK_AV_OUT0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AVOUT0PRECLK_DIV_0_LENGTH, AVOUT0CLK_DIV_0_LENGTH, AVOUT0PRECLK_DIV_0_SHIFT, AVOUT0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_AVOUT0PRECLK_2_SW, "cr_top_mx_avout0preclk_2_sw", mux_avout0preclk_1_pll7bypass, CLOCK_AV_OUT0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AVOUT0PRECLK_2_SW_SHIFT),
	MUX(MX_AVOUT0PRECLK_1_SW, "cr_top_mx_avout0preclk_1_sw", mux_avout0preclk_0_pll6bypass, CLOCK_AV_OUT0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AVOUT0PRECLK_1_SW_SHIFT),
	MUX(MX_AVOUT0PRECLK_0_SW, "cr_top_mx_avout0preclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_AV_OUT0_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_AVOUT0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0134  CR_TOP_CLOCK_AV_OUT1	g=1 d=2 m=3 */
{
.dev_gate = GATE(AVOUT1PRECLK_GATE_EN, "cr_top_avout1preclk_gate_en", "cr_top_avout1clk_div_0", CLOCK_AV_OUT1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AVOUT1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(AVOUT1CLK_DIV_0, "cr_top_avout1clk_div_0", "cr_top_mx_avout1preclk_2_sw", CLOCK_AV_OUT1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AVOUT1PRECLK_DIV_0_LENGTH, AVOUT1CLK_DIV_0_LENGTH, AVOUT1PRECLK_DIV_0_SHIFT, AVOUT1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_AVOUT1PRECLK_2_SW, "cr_top_mx_avout1preclk_2_sw", mux_avout1preclk_1_pll7bypass, CLOCK_AV_OUT1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AVOUT1PRECLK_2_SW_SHIFT),
	MUX(MX_AVOUT1PRECLK_1_SW, "cr_top_mx_avout1preclk_1_sw", mux_avout1preclk_0_pll6bypass, CLOCK_AV_OUT1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AVOUT1PRECLK_1_SW_SHIFT),
	MUX(MX_AVOUT1PRECLK_0_SW, "cr_top_mx_avout1preclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_AV_OUT1_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_AVOUT1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x0138  CR_TOP_CLOCK_AV0	g=1 d=2 m=3 */
{
.dev_gate = GATE(AV0PRECLK_GATE_EN, "cr_top_av0preclk_gate_en", "cr_top_av0clk_div_0", CLOCK_AV0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AV0PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(AV0CLK_DIV_0, "cr_top_av0clk_div_0", "cr_top_mx_av0preclk_2_sw", CLOCK_AV0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AV0PRECLK_DIV_0_LENGTH, AV0CLK_DIV_0_LENGTH, AV0PRECLK_DIV_0_SHIFT, AV0CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_AV0PRECLK_2_SW, "cr_top_mx_av0preclk_2_sw", mux_av0preclk_1_pll7bypass, CLOCK_AV0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AV0PRECLK_2_SW_SHIFT),
	MUX(MX_AV0PRECLK_1_SW, "cr_top_mx_av0preclk_1_sw", mux_av0preclk_0_pll6bypass, CLOCK_AV0_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AV0PRECLK_1_SW_SHIFT),
	MUX(MX_AV0PRECLK_0_SW, "cr_top_mx_av0preclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_AV0_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_AV0PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},

/*0x013C  CR_TOP_CLOCK_AV1	g=1 d=2 m=3 */
{
.dev_gate = GATE(AV1PRECLK_GATE_EN, "cr_top_av1preclk_gate_en", "cr_top_av1clk_div_0", CLOCK_AV1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AV1PRECLK_GATE_EN_SHIFT),
.dev_ddiv = DDIV(AV1CLK_DIV_0, "cr_top_av1clk_div_0", "cr_top_mx_av1preclk_2_sw", CLOCK_AV1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, AV1PRECLK_DIV_0_LENGTH, AV1CLK_DIV_0_LENGTH, AV1PRECLK_DIV_0_SHIFT, AV1CLK_DIV_0_SHIFT),
.dev_muxes = {
	MUX(MX_AV1PRECLK_2_SW, "cr_top_mx_av1preclk_2_sw", mux_av1preclk_1_pll7bypass, CLOCK_AV1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AV1PRECLK_2_SW_SHIFT),
	MUX(MX_AV1PRECLK_1_SW, "cr_top_mx_av1preclk_1_sw", mux_av1preclk_0_pll6bypass, CLOCK_AV1_OFFSET - CLOCK_AV_PRIMARY_OFFSET, MX_AV1PRECLK_1_SW_SHIFT),
	MUX(MX_AV1PRECLK_0_SW, "cr_top_mx_av1preclk_0_sw", mux_pll4bypass_pll5bypass, CLOCK_AV1_OFFSET - CLOCK_AV_PRIMARY_OFFSET,  MX_AV1PRECLK_0_SW_SHIFT),
	},
	.no_of_muxes =	3,
},
};

static    struct    cashew_primary_clock  system_primary __initdata = {
	.prim_gates = {
		GATE(CS_PLL3BYPASSCLK_0_EN, "cr_top_cs_pll3bypassclk_0_en", "cr_top_cs_pll3bypassclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 15),
		GATE(CS_PLL2BYPASSCLK_0_EN, "cr_top_cs_pll2bypassclk_0_en", "cr_top_cs_pll2bypassclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 13),
		GATE(CS_PLL1BYPASSCLK_0_EN, "cr_top_cs_pll1bypassclk_0_en", "cr_top_cs_pll1bypassclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 11),
		GATE(CS_PLL0BYPASSCLK_0_EN, "cr_top_cs_pll0bypassclk_0_en", "cr_top_cs_pll0bypassclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 9),
		GATE(CS_PLL3REFCLK_0_EN, "cr_top_cs_pll3refclk_0_en", "cr_top_cs_pll3refclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 7),
		GATE(CS_PLL2REFCLK_0_EN, "cr_top_cs_pll2refclk_0_en", "cr_top_cs_pll2refclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 5),
		GATE(CS_PLL1REFCLK_0_EN, "cr_top_cs_pll1refclk_0_en", "cr_top_cs_pll1refclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 3),
		GATE(CS_PLL0REFCLK_0_EN, "cr_top_cs_pll0refclk_0_en", "cr_top_cs_pll0refclk_0_sw", CLOCK_SYS_PRIMARY_OFFSET, 1),
		},
	.no_of_gates = 8,
	.prim_muxes = {
		MUXC(CS_PLL3BYPASSCLK_0_SW, "cr_top_cs_pll3bypassclk_0_sw", cs_pll3_pll3refclk_0_en, CLOCK_SYS_PRIMARY_OFFSET, 14),
		MUXC(CS_PLL2BYPASSCLK_0_SW, "cr_top_cs_pll2bypassclk_0_sw", cs_pll2_pll2refclk_0_en, CLOCK_SYS_PRIMARY_OFFSET, 12),
		MUXC(CS_PLL1BYPASSCLK_0_SW, "cr_top_cs_pll1bypassclk_0_sw", cs_pll1_pll1refclk_0_en, CLOCK_SYS_PRIMARY_OFFSET, 10),
		MUXC(CS_PLL0BYPASSCLK_0_SW, "cr_top_cs_pll0bypassclk_0_sw", cs_pll0_pll0refclk_0_en, CLOCK_SYS_PRIMARY_OFFSET, 8),
		MUXC(CS_PLL3REFCLK_0_SW, "cr_top_cs_pll3refclk_0_sw", cs_xtal1_xtal2, CLOCK_SYS_PRIMARY_OFFSET, 6),
		MUXC(CS_PLL2REFCLK_0_SW, "cr_top_cs_pll2refclk_0_sw", cs_xtal1_xtal2, CLOCK_SYS_PRIMARY_OFFSET, 4),
		MUXC(CS_PLL1REFCLK_0_SW, "cr_top_cs_pll1refclk_0_sw", cs_xtal1_xtal2, CLOCK_SYS_PRIMARY_OFFSET, 2),
		MUXC(CS_PLL0REFCLK_0_SW, "cr_top_cs_pll0refclk_0_sw", cs_xtal1_xtal2, CLOCK_SYS_PRIMARY_OFFSET, 0),
		},
	.no_of_muxes = 8,
	.prim_plls = {
		PLL(CLK_PLL_0, "pll0", "cr_top_cs_pll0refclk_0_en", PLL_GF40LP_FRAC, 0x0, pll0_rates),
		PLL(CLK_PLL_1, "pll1", "cr_top_cs_pll1refclk_0_en", PLL_GF40LP_FRAC, 0x0C, pll1_rates),
		PLL(CLK_PLL_2, "pll2", "cr_top_cs_pll2refclk_0_en", PLL_GF40LP_FRAC, 0x18, pll2_rates),
		PLL(CLK_PLL_3, "pll3", "cr_top_cs_pll3refclk_0_en", PLL_GF40LP_FRAC, 0x24, pll3_rates),
		},
	.no_of_plls = 4,
};

static    struct    cashew_primary_clock    av_primary __initdata = {
	.prim_gates = {
		GATE(CS_PLL7BYPASSCLK_0_EN, "cr_top_cs_pll7bypassclk_0_en", "cr_top_cs_pll7bypassclk_0_sw", 0x0, 17),
		GATE(CS_PLL6BYPASSCLK_0_EN, "cr_top_cs_pll6bypassclk_0_en", "cr_top_cs_pll6bypassclk_0_sw", 0x0, 15),
		GATE(CS_PLL5BYPASSCLK_0_EN, "cr_top_cs_pll5bypassclk_0_en", "cr_top_cs_pll5bypassclk_0_sw", 0x0, 13),
		GATE(CS_PLL4BYPASSCLK_0_EN, "cr_top_cs_pll4bypassclk_0_en", "cr_top_cs_pll4bypassclk_0_sw", 0x0, 11),
		GATE(PLL7REFCLK_GATE_EN, "cr_top_pll7refclk_gate_en", "cr_top_mx_pll7refclk_1_sw", 0x0, 9),
		GATE(PLL6REFCLK_GATE_EN, "cr_top_pll6refclk_gate_en", "cr_top_mx_pll6refclk_1_sw", 0x0, 6),
		GATE(PLL5REFCLK_GATE_EN, "cr_top_pll5refclk_gate_en", "cr_top_mx_pll5refclk_0_sw", 0x0, 3),
		GATE(PLL4REFCLK_GATE_EN, "cr_top_pll4refclk_gate_en", "cr_top_mx_pll4refclk_0_sw", 0x0, 1),
		},
	.no_of_gates = 8,
	.prim_muxes = {
		MUXC(CS_PLL7BYPASSCLK_0_SW, "cr_top_cs_pll7bypassclk_0_sw", cs_pll7_pll7refclk_gate, 0x0, 16),
		MUXC(CS_PLL6BYPASSCLK_0_SW, "cr_top_cs_pll6bypassclk_0_sw", cs_pll6_pll6refclk_gate, 0x0, 14),
		MUXC(CS_PLL5BYPASSCLK_0_SW, "cr_top_cs_pll5bypassclk_0_sw", cs_pll5_pll5refclk_gate, 0x0, 12),
		MUXC(CS_PLL4BYPASSCLK_0_SW, "cr_top_cs_pll4bypassclk_0_sw", cs_pll4_pll4refclk_gate, 0x0, 10),
		MUX(MX_PLL7REFCLK_1_SW, "cr_top_mx_pll7refclk_1_sw", mux_pll7refclk_0_lvds, 0x0, 8),
		MUX(MX_PLL7REFCLK_0_SW, "cr_top_mx_pll7refclk_0_sw", mux_xtal2_xtal1, 0x0, 7),
		MUX(MX_PLL6REFCLK_1_SW, "cr_top_mx_pll6refclk_1_sw", mux_pll6refclk_0_hdmi_rx_tmds, 0x0, 5),
		MUX(MX_PLL6REFCLK_0_SW, "cr_top_mx_pll6refclk_0_sw", mux_xtal2_xtal1, 0x0, 4),
		MUX(MX_PLL5REFCLK_0_SW, "cr_top_mx_pll5refclk_0_sw", mux_xtal2_xtal1, 0x0, 2),
		MUX(MX_PLL4REFCLK_0_SW, "cr_top_mx_pll4refclk_0_sw", mux_xtal2_xtal1, 0x0, 0),
		},
	.no_of_muxes = 10,
	.prim_plls = {
		PLL(CLK_PLL_4, "pll4", "cr_top_pll4refclk_gate_en", PLL_GF40LP_FRAC, 0x0, pdp_pll_rates),
		PLL(CLK_PLL_5, "pll5", "cr_top_pll5refclk_gate_en", PLL_GF40LP_FRAC, 0x0C, pll5_rates),
		PLL(CLK_PLL_6, "pll6", "cr_top_pll6refclk_gate_en", PLL_GF40LP_FRAC_AUDIO, 0x18, pll6_rates),
		PLL(CLK_PLL_7, "pll7", "cr_top_pll7refclk_gate_en", PLL_GF40LP_FRAC, 0x24, pll7_rates),
		},
	.no_of_plls = 4,
};

struct cashew_cv prim_av_cvs[2] = {
		{
			.id = LVDS_CLK,
			.name = "lvds_clk",
			.parent = "xtal_dummy",
			.max_rate = 288000000,
			.min_rate = 240000000,
		},
		{
			.id = HDMI_RX_TMDS_CLK,
			.name = "hdmi_rx_tmds_clk",
			.parent = "xtal_dummy",
			.max_rate = 600000000,
			.min_rate = 25000000,
		},
};

PNAME(mux_xtal1xtal2_xtal3) = { "cr_top_mx_pll8refclk_0_sw", "xtal3"};
PNAME(mux_pll8refclk_gate_en_pll8) = {"cr_top_pll8refclk_gate_en", "pll8"};
static    struct    cashew_primary_clock    afe_primary  __initdata = {
	.prim_gates = {
		GATE(PLL8REFCLK_GATE_EN, "cr_top_pll8refclk_gate_en", "cr_top_mx_pll8refclk_1_sw", 0x0, 2),
		GATE(CS_PLL8BYPASSCLK_0_EN, "cr_top_cs_pll8bypassclk_0_en", "cr_top_cs_pll8bypassclk_0_sw", 0x0, 4),
	},

	.no_of_gates = 2,
	.prim_muxes = {
		MUXC(CS_PLL8BYPASSCLK_0_SW, "cr_top_cs_pll8bypassclk_0_sw", mux_pll8refclk_gate_en_pll8, 0x0, 3),
		MUX(MX_PLL8REFCLK_1_SW, "cr_top_mx_pll8refclk_1_sw", mux_xtal1xtal2_xtal3, 0x0, 1),
		MUX(MX_PLL8REFCLK_0_SW, "cr_top_mx_pll8refclk_0_sw", mux_xtal1_xtal2, 0x0, 0),
		},

	.no_of_muxes = 3,
	.prim_plls = {
		PLL(CLK_PLL_8, "pll8", "cr_top_pll8refclk_gate_en", PLL_GF40LP_FRAC, 0x0, pll8_rates),
		},

	.no_of_plls = 1,
};

static struct cashew_device_clock  cashew_afe_devices[] __initdata = {
{
.dev_gate = GATE(UCCP_AFEDACCLK_GATE_EN, "cr_uccp_afedacclk_gate_en", "cr_uccp_afedacclk_div_0", 0x04, 0),
.dev_div = DIVC(UCCP_AFEDACCLK_DIV_0, "cr_uccp_afedacclk_div_0", "cr_top_cs_pll8bypassclk_0_en", 0x04, 3, 1, 0),
.no_of_muxes = 0,
},

{
.dev_gate = GATE(UCCP_AFEADCCLK_GATE_EN, "cr_uccp_afeadcclk_gate_en", "cr_uccp_afeadcclk_div_0", 0x04, 4),
.dev_div = DIVC(UCCP_AFEADCCLK_DIV_0, "cr_uccp_afeadcclk_div_0", "cr_top_cs_pll8bypassclk_0_en", 0x04, 3, 5, 0),
.no_of_muxes = 0,
},

{
.dev_gate = GATE(UCCP_AFEOUTCLK_GATE_EN, "cr_uccp_afeoutclk_gate_en", "cr_uccp_afeoutclk_div_0", 0x04, 8),
.dev_div = DIVC(UCCP_AFEOUTCLK_DIV_0, "cr_uccp_afeoutclk_div_0", "cr_top_cs_pll8bypassclk_0_en", 0x04, 3, 9, 0),
.no_of_muxes = 0,
},

};

static struct cashew_device_clock  cashew_aon_devices[] __initdata = {
{
.dev_gate = GATE(CS_RTCCLK_0_EN, "cr_top_cs_rtcclk_0_en", "cr_top_rtcclk_div_0", 0x0, 1),
.dev_div = DIVC(RTCCLK_DIV_0, "cr_top_rtcclk_div_0", "cr_top_cs_rtcclk_0_sw", 0x0, 2, 12, 0),
.dev_muxes = {MUXC(CS_RTCCLK_0_SW, "cr_top_cs_rtcclk_0_sw", mux_xtal1_xtal0, 0, 0),},
.no_of_muxes = 1,
},

{
.dev_gate = GATE(EFUSECLK_GATE_EN, "cr_top_efuseclk_gate_en", "cr_top_efuseclk_div_0", 0x0, 14),
.dev_div = DIVC(EFUSECLK_DIV_0, "cr_top_efuseclk_div_0", "xtal1", 0x0, 15, 8, 0),
.no_of_muxes = 0,
},

{
.dev_gate = GATE(MEMREPAIRCLK_GATE_EN, "cr_top_memrepairclk_gate_en", "memrepair_fixed_div", 0x0, 23),
.no_of_muxes = 0,
},

};

/* The following gates are used for sys_clk of some of the peripherals.
 * They are added to the list of clocks for those devices in device tree.
 * They are enabled/disabled by the associated drivers and are completely
 * separate from other clocks of the same driver.
 */
static struct cashew_gate cashew_perip_sys_gates[] __initdata = {
GATE(ETSSYSCLKS_GATE_EN, "cr_perip_etssysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 26),
GATE(SPDIFSYSCLKS_GATE_EN, "cr_perip_spdifsysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 25),
GATE(SPDIFINSYSCLKS_GATE_EN, "cr_perip_spdifinsysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 24),
GATE(SCB0SYSCLKS_GATE_EN, "cr_perip_scb0sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 23),
GATE(SCB1SYSCLKS_GATE_EN, "cr_perip_scb1sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 22),
GATE(SCB2SYSCLKS_GATE_EN, "cr_perip_scb2sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 21),
GATE(SCB3SYSCLKS_GATE_EN, "cr_perip_scb3sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 20),
GATE(SCB4SYSCLKS_GATE_EN, "cr_perip_scb4sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 19),
GATE(SCB5SYSCLKS_GATE_EN, "cr_perip_scb5sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 18),
GATE(SCB6SYSCLKS_GATE_EN, "cr_perip_scb6sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 17),
GATE(SCB7SYSCLKS_GATE_EN, "cr_perip_scb7sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 16),
GATE(SPISSYSCLKS_GATE_EN, "cr_perip_spissysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 15),
GATE(I2SOUT0SYSCLKS_GATE_EN, "cr_perip_i2sout0sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 14),
GATE(I2SOUT1SYSCLKS_GATE_EN, "cr_perip_i2sout1sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 13),
GATE(I2SIN0SYSCLKS_GATE_EN, "cr_perip_i2sin0sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 12),
GATE(I2SIN1SYSCLKS_GATE_EN, "cr_perip_i2sin1sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 11),
GATE(PWMSYSCLKS_GATE_EN, "cr_perip_pwmsysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 10),
GATE(RNGSYSCLKS_GATE_EN, "cr_perip_rngsysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 9),
GATE(GPIO0SYSCLKS_GATE_EN, "cr_perip_gpio0sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 8),
GATE(GPIO1SYSCLKS_GATE_EN, "cr_perip_gpio1sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 7),
GATE(MDCSYSCLKS_GATE_EN, "cr_perip_mdcsysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 6),
GATE(UART0SYSCLKS_GATE_EN, "cr_perip_uart0sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 5),
GATE(UART1SYSCLKS_GATE_EN, "cr_perip_uart1sysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 4),
GATE(RSHFSYSCLKS_GATE_EN, "cr_perip_rshfsysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 3),
GATE(ETHPKTSNIFFERSYSCLKS_GATE_EN, "cr_perip_ethpktsniffersysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 2),
GATE(TIMERSYSCLKS_GATE_EN, "cr_perip_timersysclks_gate_en", "cr_top_cs_noclspreswclk_0_en", CR_PERIP_CLKEN_OFFSET, 1),
};

/*

static struct cashew_fixed_factor cashew_ffs[] __initdata = {
	FIXED_FACTOR(CLK_WIFI_DIV4, "wifi_div4", "wifi_pll", 4),
	FIXED_FACTOR(CLK_WIFI_DIV8, "wifi_div8", "wifi_pll", 8),
};
*/

